package com.tofly.scada.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.scada.common.BaseController;
import com.tofly.scada.entity.dto.DeviceQuery;
import com.tofly.scada.entity.dto.ScadaQuery;
import com.tofly.scada.service.ScadaService;
import com.tofly.scada.service.StatisticsScadaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author HaiQiu
 * @date 2022/4/25
 */
@Api(tags = "SCADA数据操作")
@RequestMapping("/scada")
@RestController
public class ScadaController extends BaseController {

    @Autowired
    private ScadaService scadaService;

    @Autowired
    private StatisticsScadaService statisticsScadaService;

    @ToFlyAppLog(title = "查询最新SCADA所有数据")
    @ApiOperation(value = "查询最新SCADA所有数据")
    @PostMapping("/all")
    public ResultRespone getData() {
        return decide(scadaService.getListData());
    }

    @ToFlyAppLog(title = "根据设备指标查询最新SCADA所有数据")
    @ApiOperation(value = "根据设备指标查询最新SCADA所有数据")
    @PostMapping("/app/data")
    public ResultRespone getAppData(@RequestBody(required = false) DeviceQuery deviceQuery) {
        return decide(scadaService.getAppData(deviceQuery));
    }


    @ToFlyAppLog(title = "分页查询统计SCADA数据")
    @ApiOperation(value = "分页查询统计SCADA数据")
    @GetMapping("/page")
    public ResultRespone getMonthsData(Page page, ScadaQuery scadaQuery,
                                       @ApiParam("true 分页，false不分页（默认分页）")
                                       @RequestParam(required = false) Boolean isPage) {
        return decide(statisticsScadaService.getPageData(page, scadaQuery,isPage));
    }

    @ToFlyAppLog(title = "趋势分析统计SCADA数据")
    @ApiOperation(value = "趋势分析统计SCADA数据")
    @GetMapping("/trend/page")
    public ResultRespone geTrendData(Page page, ScadaQuery scadaQuery,
                                     @ApiParam("true 分页，false不分页（默认分页）")
                                     @RequestParam(required = false) Boolean isPage) {
        if (isPage == null) {
            isPage = true;
        }
        return decide(statisticsScadaService.geTrendData(page, scadaQuery, isPage));
    }


    @ToFlyAppLog(title = "手动更新范围数据")
    @ApiOperation(value = "手动更新范围数据")
    @PutMapping("/data/fetch")
    public ResultRespone fetchData(@ApiParam(value = "code编码，非必传,多个逗号隔开") @RequestParam(required = false) String code,
                                   @ApiParam(value = "开始时间:2022-01-01 00:00:00",required = true) @RequestParam String start,
                                   @ApiParam(value = "结束时间:2022-10-01 14:00:00",required = true) @RequestParam String end,
                                   @ApiParam(value = "统计类型0小时，1按天，2月份，3年份",required = true) @RequestParam Integer type) {
        scadaService.fetchData(code, start, end, type);
        return decide("更新正在进行中...请稍后再查看结果");
    }

    @ToFlyAppLog(title = "手动更新小时数据")
    @ApiOperation(value = "手动更新小时数据")
    @PutMapping("/hours/fetch")
    public ResultRespone fetch24HoursData(@ApiParam("需要更新的时间数据:2022-05-05 11:00:00,不传默认更新当前时间往前的一个小时，" +
            "时间不允许更新当前及其未来时间数据") @RequestParam(required = false) String syncTime) {
        return decide(scadaService.fetch24HoursData(syncTime));
    }

    @ToFlyAppLog(title = "手动更新月份数据数据")
    @ApiOperation(value = "手动更新月份数据数据")
    @PutMapping("/months/fetch")
    public ResultRespone fetchMonthsData(@ApiParam("需要更新的时间数据:2022-05-05,不传默认更新当前时间往后的一个月，" +
            "时间不允许更新未来时间数据，默认更新上个月数据") @RequestParam(required = false) String syncTime) {
        scadaService.fetchMonthsData(syncTime);
        return decide("更新成功");
    }


    @ToFlyAppLog(title = "手动更新天数数据数据")
    @ApiOperation(value = "手动更新天数数据数据")
    @PutMapping("/days/fetch")
    public ResultRespone fetchDaysData(@ApiParam("需要更新的时间数据:2022-05-05,不传默认更新当前时间往后的一天，" +
            "时间不允许更新未来时间数据，默认更新昨天数据") @RequestParam(required = false) String syncTime) {
        scadaService.fetchDaysData(syncTime);
        return decide("更新成功");
    }


}
