package com.tofly.scada.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.scada.common.BaseController;
import com.tofly.scada.entity.StatisticsScadaNew;
import com.tofly.scada.entity.dto.StatisticsScadaNewDto;
import com.tofly.scada.entity.vo.StatisticsScadaNewVo;
import com.tofly.scada.service.StatisticsScadaNewService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * (StatisticsScadaNew)表控制层
 *
 * @author HaiQiu
 * @since 2022-10-28 16:57:10
 */
@Api(tags = "用水量统计")
@RestController
@RequestMapping("statisticsScadaNew")
public class StatisticsScadaNewController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private StatisticsScadaNewService statisticsScadaNewService;

    /**
     * 分页查询数据
     *
     * @param page               分页对象
     * @param statisticsScadaNew 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询数据")
    @GetMapping("/page")
    public ResultRespone pageList(Page<StatisticsScadaNewVo> page, StatisticsScadaNewVo statisticsScadaNew) {
        if(StringUtil.isNotEmpty(statisticsScadaNew.getTotalAll()) && statisticsScadaNew.getTotalAll().equals("1")){
            return decide(statisticsScadaNewService.pageListToal(statisticsScadaNew));
        }
        return decide(statisticsScadaNewService.pageList(page, statisticsScadaNew));
    }

    /**
     * 查询所有数据
     *
     * @param statisticsScadaNew 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/list")
    public ResultRespone allList(StatisticsScadaNewVo statisticsScadaNew) {
        return decide(statisticsScadaNewService.allList(statisticsScadaNew));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id", value = "数据ID")
    @GetMapping("{id}")
    public ResultRespone get(@PathVariable Serializable id) {
        return decide(statisticsScadaNewService.get(id));
    }

    /**
     * 新增数据
     *
     * @param statisticsScadaNewDto 实体
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultRespone insert(@Validated @RequestBody StatisticsScadaNewDto statisticsScadaNewDto) {
        statisticsScadaNewService.insert(statisticsScadaNewDto);
        return decide("新增成功");
    }

    /**
     * 修改数据
     *
     * @param statisticsScadaNewDto 实体
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultRespone edit(@Validated @RequestBody StatisticsScadaNewDto statisticsScadaNewDto) {
        statisticsScadaNewService.edit(statisticsScadaNewDto);
        return decide("修改成功");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList", value = "数据ID数组集合")
    @DeleteMapping
    public ResultRespone delete(@RequestParam("idList") List<Long> idList) {
        statisticsScadaNewService.delete(idList);
        return decide("删除成功");
    }

    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
        statisticsScadaNewService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultRespone importData(@RequestParam(name = "file") MultipartFile file) {
        statisticsScadaNewService.importData(file);
        return decide("导入成功");
    }


    /**
     * 数据导出
     *
     * @param page               分页
     * @param statisticsScadaNew 查询模型
     * @param isAll              是否导出全部数据，true是，false否
     */
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page page, StatisticsScadaNewVo statisticsScadaNew, Boolean isAll) {
        statisticsScadaNewService.export(page, statisticsScadaNew, isAll);
    }
}

