package com.tofly.scada.entity;


import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.haiqiu.tools.excel.annotation.Excel;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 用水量统计(StatisticsScadaNew)表实体类
 *
 * @author HaiQiu
 * @since 2022-10-31 13:22:32
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@TableName(value = "STATISTICS_SCADA_NEW", resultMap = "StatisticsScadaNewMap")
@ApiModel(value = "StatisticsScadaNew对象", description = "用水量统计")
@KeySequence(value = "STATISTICS_SCADA_NEW_SEQ")
public class StatisticsScadaNew extends Model<StatisticsScadaNew> {

    private static final long serialVersionUID = -79151294626137593L;

    /**
     * 主键ID
     */
    @Excel(name = "序号",column = "A")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.INPUT)
    private Long id;

    /**
     * 指标编码
     */
    @ApiModelProperty(value = "指标编码")
    private String code;

    /**
     * 累计最新值
     */
    @ApiModelProperty(value = "累计最新值")
    private String value;

    /**
     * 统计类型：0小时统计 1天统计 2月统计 3年统计
     */
    @ApiModelProperty(value = "统计类型：0小时统计 1天统计 2月统计 3年统计")
    private Integer statisticsType;

    /**
     * 统计开始时间
     */
    @ApiModelProperty(value = "统计开始时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date startTime;

    /**
     * 统计结束时间
     */
    @Excel(name = "日期",column = "C")
    @ApiModelProperty(value = "统计结束时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date endTime;

    /**
     * 最大值
     */
    @ApiModelProperty(value = "最大值")
    private String max;

    /**
     * 最小值
     */
    @ApiModelProperty(value = "最小值")
    private String min;

    /**
     * 平均值
     */
    @ApiModelProperty(value = "平均值")
    private String ave;

    /**
     * 叠加值
     */
    @ApiModelProperty(value = "叠加值")
    private String fold;

    /**
     * 指标归属(字典码表关联 例如:01 瞬时流量)
     */
    @ApiModelProperty(value = "指标归属(字典码表关联 例如:01 瞬时流量)")
    private String dataType;

    /**
     * 合格率
     */
    @ApiModelProperty(value = "合格率")
    private String pass;

    /**
     * 超标次数
     */
    @ApiModelProperty(value = "超标次数")
    private String errorCount;

    /**
     * 不合格超标时长
     */
    @ApiModelProperty(value = "不合格超标时长")
    private String errorTimeTotal;

    /**
     * 超标总时长
     */
    @ApiModelProperty(value = "超标总时长")
    private String timeTotal;

    /**
     * 差值
     */
    @Excel(name = "当日供水量",column = "D")
    @ApiModelProperty(value = "差值")
    private String difference;

    /**
     * 创建时间
     */
    @TableField(fill = FieldFill.INSERT)
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    /**
     * 修改时间
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @ApiModelProperty(value = "修改时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;

}

