package com.tofly.scada.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.thread.ThreadUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.scada.common.FlowConstant;
import com.tofly.scada.common.MessageTopicEnum;
import com.tofly.scada.common.statisticsScadaNewCode;
import com.tofly.scada.entity.*;
import com.tofly.scada.entity.dto.DeviceQuery;
import com.tofly.scada.entity.dto.ScadaQuery;
import com.tofly.scada.entity.vo.*;
import com.tofly.scada.mapper.AllocationMapper;
import com.tofly.scada.mapper.DeviceArchiveManageMapper;
import com.tofly.scada.mapper.ScadaMapper;
import com.tofly.scada.mapper.ScadaReportMapper;
import com.tofly.scada.service.ScadaService;
import com.tofly.scada.service.StatisticsScadaNewService;
import com.tofly.scada.service.StatisticsScadaService;
import com.tofly.scada.service.TfScadaReportListService;
import com.tofly.scada.util.SendTidingsUtil;
import com.tofly.scada.util.TimeUtils;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hpsf.Array;
import org.apache.poi.hpsf.Decimal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author HaiQiu
 * @date 2022/4/25
 */
@Configuration
@Slf4j
public class ScadaServiceImpl extends ServiceImpl<ScadaMapper, Scada> implements ScadaService {

    private static final Pattern pattern = Pattern.compile("-?[0-9]+.?[0-9]*");
    /**
     * 统计量放行记录的code
     */
    @Autowired
    private statisticsScadaNewCode statisticsScadaNewCode;

    private final Logger logger = LoggerFactory.getLogger(ScadaServiceImpl.class);


    public static final String SCADA_HISTORY = "SCADA_HISTORY";
    @Autowired
    private ScadaMapper scadaMapper;

    @Autowired
    private StatisticsScadaService statisticsScadaService;

    @Autowired
    private AllocationMapper allocationMapper;

    @Autowired
    private DeviceArchiveManageMapper deviceArchiveManageMapper;

    @Autowired
    private ScadaReportMapper reportMapper;

    @Autowired
    private SendTidingsUtil sendTidingsUtil;

    @Autowired
    private ScadaReportServiceImpl scadaReportService;

    @Autowired
    private TfScadaReportListService tfScadaReportListService;

    @Autowired
    private StatisticsScadaNewService statisticsScadaNewService;


    /**
     * 根据当前时间动态自动创建月份表，创建完成之后返回表名
     * <p>
     * 往后往前推迟月份，1即创建下一月的表，-1表示创建上个月表
     *
     * @return 月份表名
     */
    //每月最后28日凌晨2点执行创建下个月的数据表
    /*@Scheduled(cron = "0 0 2 28 * ? ")*/
    public void createMonths() {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        cal.add(Calendar.MONTH, 1);
        SimpleDateFormat format = new SimpleDateFormat("yyyy_MM");
        String date = format.format(cal.getTime());
        String tableName = "SCADA_MONTH_" + date;
        logger.info("开始检测Scada月份表状态");
        if (scadaMapper.verifyTableMonthsIsExits(tableName) <= 0) {
            logger.info("Scada月份表状态：不存在，开始创建");
            scadaMapper.createTableMonths(tableName);
            logger.info("Scada月份表状态：创建完成");
        }
        logger.info("结束检测Scada月份表状态");
    }

    public String createMonths(Date time) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(time == null ? new Date() : time);
        SimpleDateFormat format = new SimpleDateFormat("yyyy_MM");
        String date = format.format(cal.getTime());
        String tableName = "SCADA_MONTH_" + date;
        logger.info("开始检测Scada月份表状态");
        if (scadaMapper.verifyTableMonthsIsExits(tableName) <= 0) {
            logger.info("Scada月份表状态：不存在，开始创建");
            scadaMapper.createTableMonths(tableName);
            logger.info("Scada月份表状态：创建完成");
        }
        logger.info("结束检测Scada月份表状态");
        return tableName;
    }


    /**
     * 同步24小时数据，支持自定义时间添加
     */
    //每小时执行一次（整点整分）
   /* @Scheduled(cron = "0 0 * * *  ? ")*/
    public void insert24H() {
        logger.info("同步scada历史数据到24小时表：正在进行时间调整");
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        Date date = new Date();
        calendar.setTime(date);
        //延迟一个小时
        calendar.add(Calendar.HOUR_OF_DAY, -1);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        //拉长时间到59分59秒
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        String end = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        logger.info("同步scada历史数据到24小时表：正在进行查询历史数据");
        //查询历史数据
//        List<Scada> scadas = getScadasDatas(code, startTime, endTime);
        List<String> codes = getCodes(null);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_ZERO, true);
    }

   /* @Async*/
    public void insert24H(String code, Date time) {
        logger.info("同步scada历史数据到24小时表：正在进行时间调整");
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        Date date = time == null ? new Date() : time;
        calendar.setTime(date);
        //延迟一个小时
//        calendar.add(Calendar.HOUR_OF_DAY, -1);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        //拉长时间到59分59秒
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        String end = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        logger.info("同步scada历史数据到24小时表：正在进行查询历史数据");
        //查询历史数据
//        List<Scada> scadas = getScadasDatas(code, startTime, endTime);
        List<String> codes = getCodes(code);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_ZERO, false);
    }


    /**
     * 同步每天数据，每天凌晨一点半同步昨天的数据
     */
   /* @Scheduled(cron = "0 30 1 * * ? ")*/
    public void syncDaysData() {
        logger.info("同步scada历史数据到表：正在进行时间调整");
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        Date date = new Date();
        calendar.setTime(date);
        //延迟一天
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        //拉长时间到59分59秒
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        String end = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        logger.info("同步scada历史数据到24小时表：正在进行查询历史数据");
        //查询历史数据
//        List<Scada> scadas = getScadasDatas(code, startTime, endTime);
        List<String> codes = getCodes(null);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_ONE, true);

    }


   /* @Async*/
    public void syncDaysData(String code, Date time) {
        logger.info("同步scada历史数据到表：正在进行时间调整");
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(time == null ? new Date() : time);
        //延迟一天
//        calendar.add(Calendar.DAY_OF_MONTH, -1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        //拉长时间到59分59秒
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        String end = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        logger.info("同步scada历史数据到24小时表：正在进行查询历史数据");
        //查询历史数据
//        List<Scada> scadas = getScadasDatas(code, startTime, endTime);
        List<String> codes = getCodes(code);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_ONE, false);

    }


    /**
     * 每月1号凌晨1点同步更新历史数据与上个月表数据，支持自定义时间添加
     */
   /* @Scheduled(cron = "0 0 1 1 * ? ")*/
    public void syncDataIntoMonths() {
        logger.info("同步scada历史数据到月份表：正在进行时间调整");
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        //倒退一个月开始时间
        calendar.add(Calendar.MONTH, -1);
        calendar.set(Calendar.DATE, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        //倒退一个月最后一天时间
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        String end = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        logger.info("同步scada历史数据到月份表：正在进行历史数据查询....");
        //查询历史数据
//        List<Scada> scadas = getScadasDatas(code, startTime, endTime);
        List<String> codes = getCodes(null);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_TWO, true);
        logger.info("同步scada历史数据到月份表：操作完成");
    }

   /* @Async*/
    public void syncDataIntoMonths(String code, Date time) {
        logger.info("同步scada历史数据到月份表：正在进行时间调整");
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(time == null ? new Date() : time);
        //倒退一个月开始时间
//        calendar.add(Calendar.MONTH, -1);
        calendar.set(Calendar.DATE, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        //倒退一个月最后一天时间
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        String end = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        logger.info("同步scada历史数据到月份表：正在进行历史数据查询....");
        //查询历史数据
//        List<Scada> scadas = getScadasDatas(code, startTime, endTime);
        List<String> codes = getCodes(code);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_TWO, false);
        logger.info("同步scada历史数据到月份表：操作完成");
    }


//    /**
//     * 普通进程跑
//     *
//     * @param startTime
//     * @param endTime
//     * @param codes
//     * @param DateType
//     */
//    public void saveStatisticsByCode(Date startTime, Date endTime, List<String> codes, Integer DateType) {
//        if (!CollectionUtils.isEmpty(codes)) {
//            logger.info("同步scada历史数据到表：正在进行数据分组");
//            //有数据就分组
//            logger.info("同步scada历史数据到表：正在进行分组循环计算");
//            //分组循环计算添加
//            for (String code : codes) {
//                //当前线程名称
//                String name = Thread.currentThread().getName();
//                logger.info("当前线程：{}----统计添加数据：{}中。。。。", name, code);
//                QueryWrapper<Scada> scadaQueryWrapper = new QueryWrapper<>();
//                scadaQueryWrapper.between("SCADA_TIME", startTime, endTime)
//                        .in("CODE", code);
//                List<Scada> scadaList = scadaMapper.selectList(scadaQueryWrapper);
//                if (CollectionUtils.isEmpty(scadaList)) {
//                    return;
//                }
//
//                String defaultValue = "0";
//
//                BigDecimal sum = BigDecimal.ZERO;
//                BigDecimal average = BigDecimal.ZERO;
//                BigDecimal maxVal = new BigDecimal(scadaList.get(0).getValue() == null ? "0" : scadaList.get(0).getValue());
//                BigDecimal minVal = new BigDecimal(scadaList.get(0).getValue() == null ? "0" : scadaList.get(0).getValue());
//
//                for (String s : statisticsScadaNewCode.getLlj()) {
//                    if (scadaList.get(0).getCode().contains(s)) {
//                        Iterator<Scada> iterator = scadaList.iterator();
//                        while (iterator.hasNext()) {
//                            Scada next = iterator.next();
//                            for (String s1 : statisticsScadaNewCode.getLlj()) {
//                                if (next.getCode().contains(s1) && !next.getValue().equals(defaultValue)) {
//                                    maxVal = new BigDecimal(next.getValue());
//                                    minVal = maxVal;
//                                    break;
//                                }
//                            }
//                        }
//                    }
//                }
//
//
//                for (Scada scada : scadaList) {
//                    for (String s : statisticsScadaNewCode.getLlj()) {
//                        if (scada.getCode().contains(s) && scada.getValue().equals(defaultValue)) {
//                            continue;
//                        }
//                    }
//                    BigDecimal decimal = new BigDecimal(scada.getValue());
//                    sum = decimal.add(sum);
//                    if (decimal.compareTo(maxVal) > 0) {
//                        maxVal = decimal;
//                    }
//                    if (decimal.compareTo(minVal) < 0) {
//                        minVal = decimal;
//                    }
//                }
//                average = sum.divide(BigDecimal.valueOf(scadaList.size()), RoundingMode.HALF_UP);
//
//                StatisticsScada statisticsScada = new StatisticsScada();
//                statisticsScada.setCode(scadaList.get(0).getCode());
//                statisticsScada.setValue(scadaList.get(scadaList.size() - 1).getValue());
//                statisticsScada.setAve(average.toString());
//                statisticsScada.setMax(maxVal.toString());
//                statisticsScada.setMin(minVal.toString());
//                statisticsScada.setFold(sum.toString());
//                statisticsScada.setStartTime(startTime);
//                statisticsScada.setEndTime(endTime);
//                statisticsScada.setStatisticsType(DateType);
//
//                //查询指标报警上下限计算合格率相关
//                QueryWrapper<Allocation> allocationQueryWrapper = new QueryWrapper<>();
//                allocationQueryWrapper.eq("VARIABLE_CODE", scadaList.get(0).getCode());
//                List<Allocation> allocations = allocationMapper.selectList(allocationQueryWrapper);
//                if (!CollectionUtils.isEmpty(allocations) && StringUtils.hasText(allocations.get(0).getType())) {
//                    Set<Long> reportIdSets = getReportIdSets(allocations);
//                    if (!CollectionUtils.isEmpty(reportIdSets)) {
//                        logger.info("计算指标报警上下限计算合格率相关....");
//                        QueryWrapper<ScadaReport> scadaReportQueryWrapper = new QueryWrapper<>();
//                        scadaReportQueryWrapper.in("ID", reportIdSets);
//                        List<ScadaReport> scadaReports = reportMapper.selectList(scadaReportQueryWrapper);
//                        if (!CollectionUtils.isEmpty(scadaReports)) {
//                            //超标次数
//                            int errorCount = 0;
//                            //不合格超标时长
//                            long errorTimeTotal = 0;
//                            //超标总时长
//                            long timeTotal = (scadaList.get(scadaList.size() - 1).getScadaTime().getTime()
//                                    - scadaList.get(0).getScadaTime().getTime()) / 1000;
//                            //错误时间
//                            Date errorTime = null;
//                            for (Scada scada : scadaList) {
//                                for (String s : statisticsScadaNewCode.getLlj()) {
//                                    if (scada.getCode().contains(s) && scada.getCode().equals("0")) {
//                                        continue;
//                                    }
//                                }
//                                for (ScadaReport report : scadaReports) {
//                                    if (Double.parseDouble(scada.getValue()) < Double.parseDouble(report.getReportLower())
//                                            || Double.parseDouble(scada.getValue()) > Double.parseDouble(report.getReportUpper())) {
//                                        errorCount = errorCount + 1;
//                                        errorTime = scada.getScadaTime();
//                                        break;
//                                    } else if (errorTime != null) {
//                                        errorTimeTotal = errorTimeTotal +
//                                                (scada.getScadaTime().getTime() - errorTime.getTime()) / 1000;
//                                    }
//                                }
//                            }
//                            //合格率
//                            float pass = 0;
//                            if (timeTotal != 0) {
//                                BigDecimal num1 = new BigDecimal(String.valueOf(errorTimeTotal));
//                                BigDecimal num2 = new BigDecimal(String.valueOf(timeTotal));
//                                pass = num1.divide(num2, RoundingMode.HALF_UP).floatValue();
//                            }
//
//                            statisticsScada.setErrorCount(String.valueOf(errorCount));
//                            statisticsScada.setErrorTimeTotal(String.valueOf(errorTimeTotal == 0 ? timeTotal : errorTimeTotal));
//                            statisticsScada.setPass(String.valueOf(pass));
//                            statisticsScada.setTimeTotal(String.valueOf(timeTotal));
//                        }
//                    }
//                    statisticsScada.setDataType(String.valueOf(allocations.get(0).getType()));
//                }
//
//                //删除旧数据
//
//                int delete = statisticsScadaService.delete(DateType, Collections.singletonList(statisticsScada.getCode()),
//                        statisticsScada.getStartTime(), statisticsScada.getEndTime());
//                logger.info("成功删除{}条旧数据", delete);
//                //添加数据
//                logger.info("统计添加数据.....：{}", statisticsScada);
//                try {
//                    statisticsScadaService.save(statisticsScada);
//                    logger.info("统计添加数据完成：{}", statisticsScada);
//                } catch (Exception e) {
//                    e.printStackTrace();
//                    log.error("统计数据添加出现错误----数据：{}---错误:{}", statisticsScada, e.getMessage());
//                }
//                logger.info("统计添加数据完成：{}", statisticsScada);
//            }
//        } else {
//            logger.info("获取历史数据为空");
//        }
//    }


//    /**
//     * 设置统计数据
//     *
//     * @param startTime 开始时间
//     * @param endTime   结束时间
//     * @param scadas    数据集合
//     */
//    @Async
//    public void saveStatisticsData(Date startTime, Date endTime, List<Scada> scadas, Integer DateType) {
//        if (!CollectionUtils.isEmpty(scadas)) {
//            logger.info("同步scada历史数据到表：正在进行数据分组");
//            //有数据就分组
//            Map<String, List<Scada>> listMap = scadas.stream().filter(scada -> scada.getCode() != null && scada.getValue() != null)
//                    .collect(Collectors.groupingBy(Scada::getCode));
//            logger.info("同步scada历史数据到表：正在进行分组循环计算");
//            ExecutorService executorService = Executors.newFixedThreadPool(listMap.entrySet().size());
//            CountDownLatch countDownLatch = new CountDownLatch(listMap.entrySet().size());
//            //分组循环计算添加
//            for (Map.Entry<String, List<Scada>> entity : listMap.entrySet()) {
//                //开启多线程
//                executorService.submit(new Runnable() {
//                    @Override
//                    public void run() {
//                        //当前线程名称
//                        String name = Thread.currentThread().getName();
//                        logger.info("当前线程：{}----统计添加数据中。。。。", name);
//                        List<Scada> scadaList = entity.getValue();
//
////                double average = scadaList.stream().mapToLong(value -> Long.parseLong(value.getValue())).average().getAsDouble();
////                double max = scadaList.stream().mapToLong(value -> Long.parseLong(value.getValue())).max().getAsLong();
////                double min = scadaList.stream().mapToLong(value -> Long.parseLong(value.getValue())).min().getAsLong();
////                double sum = scadaList.stream().mapToLong(value -> Long.parseLong(value.getValue())).sum();
//                        BigDecimal sum = BigDecimal.ZERO;
//                        BigDecimal average = BigDecimal.ZERO;
//                        BigDecimal maxVal = new BigDecimal(scadaList.get(0).getValue());
//                        BigDecimal minVal = new BigDecimal(scadaList.get(0).getValue());
//                        for (Scada scada : scadaList) {
//                            BigDecimal decimal = new BigDecimal(scada.getValue());
//                            sum = decimal.add(sum);
//                            if (decimal.compareTo(maxVal) > 0) {
//                                maxVal = decimal;
//                            }
//                            if (decimal.compareTo(minVal) < 0) {
//                                minVal = decimal;
//                            }
//                        }
//                        average = sum.divide(BigDecimal.valueOf(scadaList.size()), RoundingMode.HALF_UP);
//
//                        StatisticsScada statisticsScada = new StatisticsScada();
//                        statisticsScada.setCode(scadaList.get(0).getCode());
//                        statisticsScada.setValue(scadaList.get(scadaList.size() - 1).getValue());
//                        statisticsScada.setAve(average.toString());
//                        statisticsScada.setMax(maxVal.toString());
//                        statisticsScada.setMin(minVal.toString());
//                        statisticsScada.setFold(sum.toString());
//                        statisticsScada.setStartTime(startTime);
//                        statisticsScada.setEndTime(endTime);
//                        statisticsScada.setStatisticsType(DateType);
//
//                        //查询指标报警上下限计算合格率相关
//                        QueryWrapper<Allocation> allocationQueryWrapper = new QueryWrapper<>();
//                        allocationQueryWrapper.eq("VARIABLE_CODE", scadaList.get(0).getCode());
//                        List<Allocation> allocations = allocationMapper.selectList(allocationQueryWrapper);
//                        if (!CollectionUtils.isEmpty(allocations) && StringUtils.hasText(allocations.get(0).getType())) {
//                            Set<Long> reportIdSets = getReportIdSets(allocations);
//                            if (!CollectionUtils.isEmpty(reportIdSets)) {
//                                logger.info("计算指标报警上下限计算合格率相关....");
//                                QueryWrapper<ScadaReport> scadaReportQueryWrapper = new QueryWrapper<>();
//                                scadaReportQueryWrapper.in("ID", reportIdSets);
//                                List<ScadaReport> scadaReports = reportMapper.selectList(scadaReportQueryWrapper);
//                                if (!CollectionUtils.isEmpty(scadaReports)) {
//                                    //超标次数
//                                    int errorCount = 0;
//                                    //不合格超标时长
//                                    long errorTimeTotal = 0;
//                                    //超标总时长
//                                    long timeTotal = (scadaList.get(scadaList.size() - 1).getScadaTime().getTime()
//                                            - scadaList.get(0).getScadaTime().getTime()) / 1000;
//                                    //错误时间
//                                    Date errorTime = null;
//                                    for (Scada scada : scadaList) {
//                                        for (ScadaReport report : scadaReports) {
//                                            if (Double.parseDouble(scada.getValue()) < Double.parseDouble(report.getReportLower())
//                                                    || Double.parseDouble(scada.getValue()) > Double.parseDouble(report.getReportUpper())) {
//                                                errorCount = errorCount + 1;
//                                                errorTime = scada.getScadaTime();
//                                                break;
//                                            } else if (errorTime != null) {
//                                                errorTimeTotal = errorTimeTotal +
//                                                        (scada.getScadaTime().getTime() - errorTime.getTime()) / 1000;
//                                            }
//                                        }
//                                    }
//                                    //合格率
//                                    float pass = 0;
//                                    if (timeTotal != 0) {
//                                        BigDecimal num1 = new BigDecimal(String.valueOf(errorTimeTotal));
//                                        BigDecimal num2 = new BigDecimal(String.valueOf(timeTotal));
//                                        pass = num1.divide(num2, RoundingMode.HALF_UP).floatValue();
//                                    }
//
//                                    statisticsScada.setErrorCount(String.valueOf(errorCount));
//                                    statisticsScada.setErrorTimeTotal(String.valueOf(errorTimeTotal == 0 ? timeTotal : errorTimeTotal));
//                                    statisticsScada.setPass(String.valueOf(pass));
//                                    statisticsScada.setTimeTotal(String.valueOf(timeTotal));
//                                }
//                            }
//                            statisticsScada.setDataType(String.valueOf(allocations.get(0).getType()));
//                        }
//
//                        //删除旧数据
//
//                        int delete = statisticsScadaService.delete(DateType, Collections.singletonList(statisticsScada.getCode()),
//                                statisticsScada.getStartTime(), statisticsScada.getEndTime());
//                        logger.info("成功删除{}条旧数据", delete);
//                        //添加数据
//                        logger.info("统计添加数据完成：{}", statisticsScada);
//                        try {
//                            statisticsScadaService.save(statisticsScada);
//                        } catch (Exception e) {
//                            e.printStackTrace();
//                            log.error("统计数据添加出现错误----数据：{}---错误:{}", statisticsScada, e.getMessage());
//                        }
//                        logger.info("统计添加数据完成：{}", statisticsScada);
//                        // 闭锁
//                        countDownLatch.countDown();
//                    }
//                });
//            }
//        } else {
//            logger.info("获取历史数据为空");
//        }
//    }


    /**
     * 截取数组长度批量插入
     *
     * @param scadas    数据集合
     * @param tableName 表名
     * @return 成功条数
     */
    public int insertMouthBatch(List<Scada> scadas, String tableName) {
        int size = scadas.size();
        int limit = 1000;
        int startIndex = 0;
        int endIndex;
        if (size <= limit) {
            //添加新数据
            int insertMouth = scadaMapper.insertMouth(scadas, tableName);
            log.info("新增数据：{}条", insertMouth);
        } else {
            while (startIndex < size) {
                //结束索引范围
                endIndex = startIndex + limit;
                //最后索引不能超过本身长度
                endIndex = endIndex > size ? size : endIndex;
                //截取数据
                List<Scada> scadaList = scadas.subList(startIndex, endIndex);
                //下次索引位置
                startIndex = endIndex;
                //添加新数据
                int insertMouth = scadaMapper.insertMouth(scadaList, tableName);
                log.info("新增数据：{}条", insertMouth);
            }
        }
        return size;
    }


    /**
     * 分code进行多线程存储
     *
     * @param startTime
     * @param endTime
     * @param codes
     * @param dateType  统计类型
     * @param isAuto    是否为自动定时任务，true定时任务，false不是定时任务
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveStatisticsCode(Date startTime, Date endTime, List<String> codes, Integer dateType, boolean isAuto) {
        String start = DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss");
        String end = DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss");
        //待删除的code码
        List<String> deleteCodes;
        if (!CollectionUtils.isEmpty(codes)) {
            // 获取对应时间范围内数据参数
            List<StatisticsScadaCalVo1> scadaLists = new ArrayList<>();
            if (isAuto) {
                List<String> codeByDbs = scadaMapper.selectHistoryMaxAndMinByScadaTime2(codes, startTime, endTime, dateType);
                deleteCodes = codeByDbs;
                if (!CollectionUtils.isEmpty(codeByDbs)) {
                    //删除旧数据
                    int delete = statisticsScadaService.delete(dateType, codeByDbs,
                            startTime, endTime);
                    logger.info("成功删除{}条旧数据", delete);
                    // 获取对应时间范围内数据参数
                    scadaLists = scadaMapper.selectHistoryMaxAndMinByScadaTime1(codeByDbs, startTime, endTime, dateType);
                }
            } else {
                deleteCodes = codes;
                //删除旧数据
                int delete = statisticsScadaService.delete(dateType, codes,
                        startTime, endTime);
                logger.info("成功删除{}条旧数据", delete);
                scadaLists = scadaMapper.selectHistoryMaxAndMinByScadaTime1(codes, startTime,
                        endTime, dateType);
            }
            if (!CollectionUtils.isEmpty(scadaLists)) {
                logger.info("计算scada统计数据：正在进行数据大小{}", scadaLists.size());
                // 存储新增统计数据
                List<StatisticsScada> saveList = new ArrayList<>();
                if (CollectionUtils.isEmpty(scadaLists)) {
                    return;
                }
                scadaLists.forEach(k -> {
                    StatisticsScada statisticsScada = new StatisticsScada();
                    statisticsScada.setCode(k.getCode());
                    String maxValue = k.getMaxValue() == null ? "0" : k.getMaxValue();
                    statisticsScada.setValue(maxValue);
                    statisticsScada.setAve(k.getAvgValue());
                    statisticsScada.setMin(k.getMinValue() == null ? "0" : k.getMinValue());
                    statisticsScada.setMax(maxValue);
                    statisticsScada.setTimeTotal(k.getRunTime());
                    statisticsScada.setErrorTimeTotal(k.getErrTime());
                    statisticsScada.setErrorCount("0");
                    statisticsScada.setFold(k.getDifferenceValue());
                    List<Date> dates = getDateListByType(getDateType(k.getDateTimeStr(), dateType), dateType);
                    if (!CollectionUtils.isEmpty(dates) && dates.size() == 2) {
                        statisticsScada.setStartTime(dates.get(0));
                        statisticsScada.setEndTime(dates.get(1));
                    }
                    statisticsScada.setStatisticsType(dateType);
                    statisticsScada.setDataType(k.getAllocationType());
                    //添加数据
                    saveList.add(statisticsScada);
                });
                // 新增
                logger.info("新增scada统计数据大小{}", saveList.size());
                statisticsScadaService.saveBatch(saveList);

                //TODO 新增加统计表
                List<StatisticsScadaNew> statisticsScadaNews = new ArrayList<>();
                for (StatisticsScada statisticsScada : saveList) {
                    //TODO 新增加统计表
                    for (String s : statisticsScadaNewCode.getLlj()) {
                        if (statisticsScada.getCode().contains(s)) {
                            StatisticsScadaNew aNew = new StatisticsScadaNew();
                            BeanUtils.copyProperties(statisticsScada, aNew);
                            aNew.setId(null);
                            if (statisticsScada.getMax().equals(statisticsScada.getMin())) {
                                aNew.setDifference("0");
                            } else if (statisticsScada.getMax().contains("-") || statisticsScada.getMin().contains("-")) {
                                String m1 = statisticsScada.getMax();
                                String m2 = statisticsScada.getMin();
                                BigDecimal abs = new BigDecimal(m1).subtract(new BigDecimal(m2)).abs();
                                aNew.setDifference(abs.toString());
                            } else {
                                BigDecimal max = new BigDecimal(statisticsScada.getMax());
                                BigDecimal min = new BigDecimal(statisticsScada.getMin());
                                BigDecimal subtract = max.subtract(min);
                                aNew.setDifference(subtract.toString());
                            }
                            statisticsScadaNews.add(aNew);
                        }
                    }
                }
                if (!CollectionUtils.isEmpty(statisticsScadaNews)) {
                    //删除旧数据
                    statisticsScadaNewService.remove(Wrappers.<StatisticsScadaNew>lambdaQuery()
                            .in(StatisticsScadaNew::getCode, deleteCodes)
                            .between(StatisticsScadaNew::getStartTime, startTime, endTime));
                    //添加新数据
                    log.info("新增加统计表大小{}", statisticsScadaNews.size());
                    boolean b = statisticsScadaNewService.saveBatch(statisticsScadaNews);
                    if (b) {
                        log.info("新增加统计表大小{}----------成功------------", statisticsScadaNews.size());
                    }
                }

            } else {
                logger.info("获取历史数据为空");
            }
        }


//        if (!CollectionUtils.isEmpty(codes)) {
//            logger.info("同步scada历史数据到表" + start + "-----" + end + "：正在进行数据分组");
//            //有数据就分组
//            logger.info("同步scada历史数据到表：正在进行分组循环计算");
//            int num = codes.size() > 2 ? codes.size() / 2 : codes.size();
//            // 初始化线程池数量
//            CountDownLatch countDownLatch = ThreadUtil.newCountDownLatch(num);
//
//            //分组循环计算添加
//            for (String code : codes) {
//                ThreadUtil.execute(() -> {
//                    try {
//                        //查询最大值最小值
//                        List<Scada> scadaLists = new ArrayList<>();
//                        if (isAuto) {
//                            List<Scada> scadaListByCodeAndScadaTime = scadaMapper.selectHistoryMaxAndMinByTime(SCADA_HISTORY, code, start, end, dateType);
//                            if (!CollectionUtils.isEmpty(scadaListByCodeAndScadaTime)) {
//                                for (Scada scadaByScadaTimeStr : scadaListByCodeAndScadaTime) {
//                                    List<Scada> scadaListTemp = scadaMapper.selectHistoryMaxAndMinByScadaTimeToOne(SCADA_HISTORY, scadaByScadaTimeStr.getCode(), scadaByScadaTimeStr.getScadaTimeStr(), dateType);
//                                    if (!CollectionUtils.isEmpty(scadaListTemp)) {
//                                        scadaLists.addAll(scadaListTemp);
//                                    }
//                                }
//                            }
//                        } else {
//                            scadaLists = scadaMapper.selectHistoryMaxAndMinByScadaTime(SCADA_HISTORY, code, start, end);
//                        }
//
//                        if (CollectionUtils.isEmpty(scadaLists)) {
//                            return;
//                        }
//                        Map<String, List<Scada>> collect = scadaLists.stream().collect(Collectors.groupingBy(scada -> scada.getCode() + "_" + scada.getScadaTimeStr()));
//                        if (CollectionUtils.isEmpty(collect)) {
//                            return;
//                        }
//                        for (Map.Entry<String, List<Scada>> entry : collect.entrySet()) {
//                            List<Scada> scadaList = entry.getValue();
//                            StatisticsScada statisticsScada = new StatisticsScada();
//                            statisticsScada.setCode(scadaList.get(0).getCode());
//                            statisticsScada.setValue(scadaList.get(0).getValue() == null ? "0" : scadaList.get(0).getValue());
//                            statisticsScada.setMin(scadaList.get(0).getValue() == null ? "0" : scadaList.get(0).getValue());
//                            statisticsScada.setMax(scadaList.get(1).getValue() == null ? "0" : scadaList.get(1).getValue());
//                            List<Date> dates = getDateListByType(scadaList.get(0).getScadaTime(), dateType);
//                            if (!CollectionUtils.isEmpty(dates) && dates.size() == 2) {
//                                statisticsScada.setStartTime(dates.get(0));
//                                statisticsScada.setEndTime(dates.get(1));
//                            }
//                            statisticsScada.setStatisticsType(dateType);
//
//
//                            //TODO 新增加统计表
//                            StatisticsScadaNew aNew = new StatisticsScadaNew();
//                            BeanUtils.copyProperties(statisticsScada, aNew);
//                            aNew.setId(scadaList.get(0).getId());
//                            if (statisticsScada.getMax().equals(statisticsScada.getMin())) {
//                                aNew.setDifference("0");
//                            } else if (statisticsScada.getMax().contains("-") || statisticsScada.getMin().contains("-")) {
//                                String m1 = statisticsScada.getMax();
//                                String m2 = statisticsScada.getMin();
//                                BigDecimal abs = new BigDecimal(m1).subtract(new BigDecimal(m2)).abs();
//                                aNew.setDifference(abs.toString());
//                            } else {
//                                BigDecimal max = new BigDecimal(statisticsScada.getMax());
//                                BigDecimal min = new BigDecimal(statisticsScada.getMin());
//                                BigDecimal subtract = max.subtract(min);
//                                aNew.setDifference(subtract.toString());
//                            }
//
//                            //TODO 新增加统计表
//                            for (String s : statisticsScadaNewCode.getLlj()) {
//                                if (aNew.getCode().contains(s)) {
//                                    statisticsScadaNewService.removeById(scadaList.get(0).getId());
//                                    log.info("新增加统计表{}", aNew);
//                                    statisticsScadaNewService.save(aNew);
//                                }
//                            }
//
//                            //删除旧数据
//                            int delete = statisticsScadaService.delete(dateType, statisticsScada.getCode(),
//                                    DateUtil.format(statisticsScada.getStartTime(), "yyyy-MM-dd HH:mm:ss"),
//                                    DateUtil.format(statisticsScada.getEndTime(), "yyyy-MM-dd HH:mm:ss"));
//                            logger.info("成功删除{}条旧数据", delete);
//                            //添加数据
//                            logger.info("统计添加数据.....：{}", statisticsScada);
//                            try {
//                                statisticsScadaService.save(statisticsScada);
//                                logger.info("统计添加数据完成：{}", statisticsScada);
//                            } catch (Exception e) {
//                                e.printStackTrace();
//                                log.error("统计数据添加出现错误----数据：{}---错误:{}", statisticsScada, e.getMessage());
//                            }
//                            logger.info("统计添加数据完成：{}", statisticsScada);
//
//                        }//循环到这里结束
//                        Thread.sleep(6000);
//                    } catch (InterruptedException e) {
//                        e.printStackTrace();
//                        throw new RuntimeException(e);
//                    }
//                });
//                //调用线程计数器-1
//                countDownLatch.countDown();
//            }
//            //唤醒主线程
//            try {
//                countDownLatch.await();
//            } catch (InterruptedException e) {
//                throw new RuntimeException(e);
//            }
//            logger.info("唤醒主线程---------------同步scada历史数据到表：完成!!!!!!!!!!!!!!!!!!!!!!!!!");


        //分组循环计算添加
//            for (String code : codes) {
//                //查询最大值最小值
//                List<Scada> scadaLists = new ArrayList<>();
//                if (isAuto) {
//                    List<Scada> scadaListByCodeAndScadaTime = scadaMapper.selectHistoryMaxAndMinByTime(SCADA_HISTORY, code, start, end, dateType);
//                    if (!CollectionUtils.isEmpty(scadaListByCodeAndScadaTime)) {
//                        for (Scada scadaByScadaTimeStr : scadaListByCodeAndScadaTime) {
//                            List<Scada> scadaListTemp = scadaMapper.selectHistoryMaxAndMinByScadaTimeToOne(SCADA_HISTORY, scadaByScadaTimeStr.getCode(), scadaByScadaTimeStr.getScadaTimeStr(), dateType);
//                            if (!CollectionUtils.isEmpty(scadaListTemp)) {
//                                scadaLists.addAll(scadaListTemp);
//                            }
//                        }
//                    }
//                } else {
//                    scadaLists = scadaMapper.selectHistoryMaxAndMinByScadaTime(SCADA_HISTORY, code, start, end);
//                }
//
//                if (CollectionUtils.isEmpty(scadaLists)) {
//                    return;
//                }
//                Map<String, List<Scada>> collect = scadaLists.stream().collect(Collectors.groupingBy(scada -> scada.getCode() + "_" + scada.getScadaTimeStr()));
//                if (CollectionUtils.isEmpty(collect)) {
//                    return;
//                }
//                for (Map.Entry<String, List<Scada>> entry : collect.entrySet()) {
//                    List<Scada> scadaList = entry.getValue();
//                    StatisticsScada statisticsScada = new StatisticsScada();
//                    statisticsScada.setCode(scadaList.get(0).getCode());
//                    statisticsScada.setValue(scadaList.get(0).getValue() == null ? "0" : scadaList.get(0).getValue());
//                    statisticsScada.setMin(scadaList.get(0).getValue() == null ? "0" : scadaList.get(0).getValue());
//                    statisticsScada.setMax(scadaList.get(1).getValue() == null ? "0" : scadaList.get(1).getValue());
//                    List<Date> dates = getDateListByType(scadaList.get(0).getScadaTime(), dateType);
//                    if (!CollectionUtils.isEmpty(dates) && dates.size() == 2) {
//                        statisticsScada.setStartTime(dates.get(0));
//                        statisticsScada.setEndTime(dates.get(1));
//                    }
//                    statisticsScada.setStatisticsType(dateType);
//
//
//                    //TODO 新增加统计表
//                    StatisticsScadaNew aNew = new StatisticsScadaNew();
//                    BeanUtils.copyProperties(statisticsScada, aNew);
//                    aNew.setId(scadaList.get(0).getId());
//                    if (statisticsScada.getMax().equals(statisticsScada.getMin())) {
//                        aNew.setDifference("0");
//                    } else if (statisticsScada.getMax().contains("-") || statisticsScada.getMin().contains("-")) {
//                        String m1 = statisticsScada.getMax();
//                        String m2 = statisticsScada.getMin();
//                        BigDecimal abs = new BigDecimal(m1).subtract(new BigDecimal(m2)).abs();
//                        aNew.setDifference(abs.toString());
//                    } else {
//                        BigDecimal max = new BigDecimal(statisticsScada.getMax());
//                        BigDecimal min = new BigDecimal(statisticsScada.getMin());
//                        BigDecimal subtract = max.subtract(min);
//                        aNew.setDifference(subtract.toString());
//                    }
//
//                    //TODO 新增加统计表
//                    for (String s : statisticsScadaNewCode.getLlj()) {
//                        if (aNew.getCode().contains(s)) {
//                            statisticsScadaNewService.removeById(scadaList.get(0).getId());
//                            log.info("新增加统计表{}", aNew);
//                            statisticsScadaNewService.save(aNew);
//                        }
//                    }
//
//                    //删除旧数据
//                    int delete = statisticsScadaService.delete(dateType, statisticsScada.getCode(),
//                            DateUtil.format(statisticsScada.getStartTime(), "yyyy-MM-dd HH:mm:ss"),
//                            DateUtil.format(statisticsScada.getEndTime(), "yyyy-MM-dd HH:mm:ss"));
//                    logger.info("成功删除{}条旧数据", delete);
//                    //添加数据
//                    logger.info("统计添加数据.....：{}", statisticsScada);
//                    try {
//                        statisticsScadaService.save(statisticsScada);
//                        logger.info("统计添加数据完成：{}", statisticsScada);
//                    } catch (Exception e) {
//                        e.printStackTrace();
//                        log.error("统计数据添加出现错误----数据：{}---错误:{}", statisticsScada, e.getMessage());
//                    }
//                    logger.info("统计添加数据完成：{}", statisticsScada);
//
//                }//循环到这里结束
//
//            }
//        } else {
//            logger.info("获取历史数据为空");
//        }
    }

    private Date getDateType(String timeStr, Integer type) {
        if (0 == type) {
            String s = timeStr + ":00:00";
            LocalDateTime localDateTime = TimeUtils.fromString2LocalDateTime(s, "yyyy-MM-dd HH:mm:ss");
            return TimeUtils.convertLDTToDate(localDateTime);
        } else if (1 == type) {
            return TimeUtils.localDateToDate(TimeUtils.fromString2LocalDate(timeStr, "yyyy-MM-dd"));
        } else {
            String s = timeStr + "-01";
            return TimeUtils.localDateToDate(TimeUtils.fromString2LocalDate(s, "yyyy-MM-dd"));
        }
    }

    /**
     * 获取时间的小时，天，月份的开始时间和结束时间
     *
     * @param time 时间
     * @param type 类型
     * @return
     */
    public List<Date> getDateListByType(Date time, Integer type) {
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(time == null ? new Date() : time);
        List<Date> dates = new ArrayList<>();
        if (type.equals(FlowConstant.INTEGER_ZERO)) {
            //一个小时
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.set(Calendar.MILLISECOND, 0);
            Date startTime = calendar.getTime();
            String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
            //拉长时间到59分59秒
            calendar.set(Calendar.MINUTE, 59);
            calendar.set(Calendar.SECOND, 59);
            calendar.set(Calendar.MILLISECOND, 999);
            Date endTime = calendar.getTime();
            String end = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
            dates.add(startTime);
            dates.add(endTime);
        } else if (type.equals(FlowConstant.INTEGER_ONE)) {
            //一天
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.set(Calendar.MILLISECOND, 0);
            Date startTime = calendar.getTime();
            String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
            //拉长时间到59分59秒
            calendar.set(Calendar.HOUR_OF_DAY, 23);
            calendar.set(Calendar.MINUTE, 59);
            calendar.set(Calendar.SECOND, 59);
            calendar.set(Calendar.MILLISECOND, 999);
            Date endTime = calendar.getTime();
            String end = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
            dates.add(startTime);
            dates.add(endTime);
        } else if (type.equals(FlowConstant.INTEGER_TWO)) {
            //一个月
            calendar.set(Calendar.DATE, 1);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.set(Calendar.MILLISECOND, 0);
            Date startTime = calendar.getTime();
            String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
            //倒退一个月最后一天时间
            calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
            calendar.set(Calendar.HOUR_OF_DAY, 23);
            calendar.set(Calendar.MINUTE, 59);
            calendar.set(Calendar.SECOND, 59);
            calendar.set(Calendar.MILLISECOND, 999);
            Date endTime = calendar.getTime();
            dates.add(startTime);
            dates.add(endTime);
        }
        return dates;
    }

    /**
     * 获取历史数据
     *
     * @param start 开始时间
     * @param end   结束时间
     * @return 历史数据
     */
    public List<Scada> getScadasDatas(String code, Date start, Date end) {
        List<Allocation> allocations = allocationMapper.selectList(Wrappers.<Allocation>lambdaQuery()
                .eq(StringUtils.hasText(code), Allocation::getVariableCode, code));
        List<String> codes = allocations.stream().filter(allocation -> allocation.getVariableCode() != null)
                .map(Allocation::getVariableCode).collect(Collectors.toList());

        List<Scada> scadas = new ArrayList<>();
        if (!CollectionUtils.isEmpty(codes)) {
            QueryWrapper<Scada> scadaQueryWrapper = new QueryWrapper<>();
            scadaQueryWrapper.between("SCADA_TIME", start, end)
                    .in("CODE", codes);
            scadas = scadaMapper.selectList(scadaQueryWrapper);
        }
        return scadas;
    }

    public List<String> getCodes(String code) {
        if (StringUtils.hasText(code) && code.contains(",")) {
            String[] codeList = code.split(",");
            List<Allocation> allocations = allocationMapper.selectList(Wrappers.<Allocation>lambdaQuery()
                    .in(!CollectionUtils.isEmpty(Arrays.asList(codeList)), Allocation::getVariableCode, codeList));
            List<String> codes = allocations.stream().filter(allocation -> allocation.getVariableCode() != null)
                    .map(Allocation::getVariableCode).collect(Collectors.toList());

            if (!CollectionUtils.isEmpty(codes)) {
                return codes;
            }
        }
        List<Allocation> allocations = allocationMapper.selectList(Wrappers.<Allocation>lambdaQuery()
                .eq(StringUtils.hasText(code), Allocation::getVariableCode, code));
        List<String> codes = allocations.stream().filter(allocation -> allocation.getVariableCode() != null)
                .map(Allocation::getVariableCode).collect(Collectors.toList());

        if (!CollectionUtils.isEmpty(codes)) {
            return codes;
        }
        return null;
    }

   /* @Async*/
    public void insertMouth(List<Scada> scadas, String tableName) {
        int size = scadas.size();
        int i = 0;
        while (size > 600) {
            scadaMapper.insertMouth(scadas.subList(i, i + 600), tableName);
            i = i + 600;
            size = size - 600;
        }
        if (size > 0) {
            scadaMapper.insertMouth(scadas.subList(i, i + size), tableName);
        }
    }


    @Override
    public List<Scada> getListData() {
        List<Scada> scadaList = scadaMapper.selectMonitoring("SCADA_MONITORING");
        for (Scada scada : scadaList) {
            if (StringUtils.hasText(scada.getValue()) && scada.getValue().startsWith(".")) {
                scada.setValue("0" + scada.getValue());
            }
        }
        return scadaList;
    }

    @Override
    public boolean fetch24HoursData(String syncTime) {
        Date date = null;
        if (StringUtils.isEmpty(syncTime)) {
            date = new Date();
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
        } else {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
            try {
                Date parse = format.parse(syncTime);
                Date now = new Date();
                Calendar cal = Calendar.getInstance();
                cal.setTime(now);
                cal.set(Calendar.MINUTE, 0);
                cal.set(Calendar.SECOND, 0);
                if (parse.getTime() >= now.getTime()) {
                    throw new RuntimeException("时间参数不允许大于等于当前时间：" + format.format(now));
                }
                date = parse;
            } catch (ParseException e) {
                throw new RuntimeException("时间参数格式传入错误：" + e);
            }
        }
        insert24H(null, date);
        return true;
    }

    @Override
    public void fetchMonthsData(String syncTime) {
        Date date = null;
        if (StringUtils.isEmpty(syncTime)) {
            date = new Date();
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
        } else {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date parse = format.parse(syncTime);
                Date now = new Date();
                Calendar cal = Calendar.getInstance();
                cal.setTime(now);
                if (parse.getTime() > now.getTime()) {
                    throw new RuntimeException("时间参数不允许大于当前时间：" + format.format(now));
                }
                date = parse;
            } catch (ParseException e) {
                throw new RuntimeException("时间参数格式传入错误：" + e);
            }
        }
        syncDataIntoMonths(null, date);
    }

    @Override
    public void fetchDaysData(String syncTime) {
        Date date = null;
        if (StringUtils.isEmpty(syncTime)) {
            date = new Date();
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
        } else {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date parse = format.parse(syncTime);
                Date now = new Date();
                Calendar cal = Calendar.getInstance();
                cal.setTime(now);
                if (parse.getTime() > now.getTime()) {
                    throw new RuntimeException("时间参数不允许大于当前时间：" + format.format(now));
                }
                date = parse;
            } catch (ParseException e) {
                throw new RuntimeException("时间参数格式传入错误：" + e);
            }
        }
        syncDaysData(null, date);
    }

    @Override
    public Page<ScadaVo> get24HoursData(Page page, ScadaQuery scadaQuery) {
        QueryWrapper<ScadaQuery> queryWrapper = new QueryWrapper<>();
        if (StringUtils.hasText(scadaQuery.getCode())) {
            queryWrapper.like("hours.CODE", scadaQuery.getCode());
        }
        if (StringUtils.hasText(scadaQuery.getValue())) {
            queryWrapper.like("hours.VALUE", scadaQuery.getValue());
        }
        if (scadaQuery.getStart() != null && scadaQuery.getEnd() != null) {
            queryWrapper.between("hours.SCADA_TIME", scadaQuery.getStart(), scadaQuery.getEnd());
        }
        queryWrapper.orderByDesc("hours.SCADA_TIME");
        return scadaMapper.pageList(page, queryWrapper, "SCADA_HOURS");
    }


    @Override
    public Page getMonthsData(Page page, ScadaQuery scadaQuery) {
        QueryWrapper<ScadaQuery> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("hours.SCADA_TIME");
        if (StringUtils.hasText(scadaQuery.getCode())) {
            queryWrapper.like("hours.CODE", scadaQuery.getCode());
        }
        if (StringUtils.hasText(scadaQuery.getValue())) {
            queryWrapper.like("hours.VALUE", scadaQuery.getValue());
        }
        if (scadaQuery.getStart() != null && scadaQuery.getEnd() != null) {

            if (scadaQuery.getStart().getYear() == scadaQuery.getEnd().getYear() &&
                    scadaQuery.getStart().getMonth() == scadaQuery.getEnd().getMonth()) {
                List<Date> startAndEndByDate = TimeUtils.getMonthsToStartAndEndByDate(scadaQuery.getStart());
                queryWrapper.between("hours.SCADA_TIME", scadaQuery.getStart(), scadaQuery.getEnd());

                Calendar cal = Calendar.getInstance();
                cal.setTime(startAndEndByDate.get(0));
                SimpleDateFormat format = new SimpleDateFormat("yyyy_MM");
                String date = format.format(cal.getTime());
                String tableName = "SCADA_MONTH_" + date;
                return scadaMapper.pageList(page, queryWrapper, tableName);
            }
        }
        return scadaMapper.pageList(page, queryWrapper, SCADA_HISTORY);
    }

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;


    public List<DeviceArchiveManageVo> getAppData_Old(DeviceQuery deviceQuery) {
        QueryWrapper<DeviceArchiveManage> queryWrapper = getQueryWrapper(deviceQuery);
        List<DeviceArchiveManage> deviceArchiveManages = deviceArchiveManageMapper.pageList(queryWrapper);
        if (CollectionUtils.isEmpty(deviceArchiveManages)) {
            return null;
        }
        //查询所有指标
        List<Allocation> allocations = allocationMapper.selectList(null);
        //查询所有报警
        List<ScadaReport> scadaReports = reportMapper.selectList(Wrappers.<ScadaReport>lambdaQuery()
                .eq(ScadaReport::getIsDel,0));
        //查询最新scada数据
        List<Scada> scadas = getListData();

        int allocationsHash = allocations.hashCode();
        int scadaReportsHash = scadaReports.hashCode();
        int scadasHash = scadas.hashCode();

        Integer a = (Integer) redisTemplate.opsForValue().get("allocations");
        Integer sr = (Integer) redisTemplate.opsForValue().get("scadaReports");
        Integer s = (Integer) redisTemplate.opsForValue().get("scadas");
        if (a != null && sr != null && s != null) {
            if (a.equals(allocationsHash) && sr.equals(scadaReportsHash) && s.equals(scadasHash)) {
                String monitorScada = (String) redisTemplate.opsForValue().get("monitor_scada");
                List<DeviceArchiveManageVo> manageVos = JSON.parseArray(monitorScada, DeviceArchiveManageVo.class);
                return manageVos;
            }
        }
        return getDeviceArchiveManageVos(deviceArchiveManages, allocations, scadaReports, scadas, allocationsHash, scadaReportsHash, scadasHash);

//        List<DeviceArchiveManageVo> manageVos = deviceArchiveManages.stream().map(deviceArchiveManage -> {
//            DeviceArchiveManageVo vo = new DeviceArchiveManageVo();
//            BeanUtils.copyProperties(deviceArchiveManage, vo);
//            //查询绑定的指标
//            if (!CollectionUtils.isEmpty(allocations)) {
//                //获取符合设备绑定的指标
//                List<AllocationVo> allocationVos = new HashSet<>(allocations).stream().filter(allocation -> allocation.getDeviceId() != null &&
//                        allocation.getDeviceId().equals(vo.getId())).map(allocation -> {
//                    AllocationVo allocationVo = new AllocationVo();
//                    BeanUtils.copyProperties(allocation, allocationVo);
//                    //查询scada数据
//                    if (!CollectionUtils.isEmpty(scadas)) {
//                        List<Scada> scadaList = scadas.stream()
//                                .filter(scada -> scada.getCode() != null && scada.getCode().equals(allocation.getVariableCode()))
//                                .collect(Collectors.toList());
//                        if (!CollectionUtils.isEmpty(scadaList)) {
//                            allocationVo.setScada(scadaList.get(0));
//                        }
//                    }
//                    //设置报警相关信息
//                    Set<Long> reportIdSets = getReportIdSets(Collections.singletonList(allocation));
//                    if (!CollectionUtils.isEmpty(scadaReports) && !CollectionUtils.isEmpty(reportIdSets)
//                            && allocationVo.getIsAlarm() != null
//                            && allocationVo.getIsAlarm().equals("1")) {
//                        //筛选出报警信息列表
//                        List<ScadaReport> reports = scadaReports.stream().filter(scadaReport -> scadaReport.getId() != null
//                                && reportIdSets.contains(scadaReport.getId())).collect(Collectors.toList());
//                        allocationVo.setScadaReports(reports);
//                    }
//                    return allocationVo;
//                }).collect(Collectors.toList());
//                vo.setAllocations(allocationVos);
//            }
//            return vo;
//        }).collect(Collectors.toList());
//        return manageVos;
    }

    @Override
    public List<DeviceArchiveManageVo> getAppData(DeviceQuery deviceQuery) {
        QueryWrapper<DeviceArchiveManage> queryWrapper = getQueryWrapper(deviceQuery);
        List<DeviceArchiveManage> deviceArchiveManages = deviceArchiveManageMapper.pageList(queryWrapper);
        if (CollectionUtils.isEmpty(deviceArchiveManages)) {
            return null;
        }
        //查询所有指标
        List<Allocation> allocations = allocationMapper.selectList(null);
        //查询所有报警
        List<ScadaReport> scadaReports = reportMapper.selectList(Wrappers.<ScadaReport>lambdaQuery()
                .eq(ScadaReport::getIsDel,0));
        //查询最新scada数据
        List<Scada> scadas = getListData();

        List<DeviceArchiveManageVo> manageVos = deviceArchiveManages.stream().map(deviceArchiveManage -> {
            DeviceArchiveManageVo vo = new DeviceArchiveManageVo();
            BeanUtils.copyProperties(deviceArchiveManage, vo);
            //查询绑定的指标
            if (!CollectionUtils.isEmpty(allocations)) {
                //获取符合设备绑定的指标
                List<AllocationVo> allocationVos = new HashSet<>(allocations).stream().filter(allocation -> allocation.getDeviceId() != null &&
                        allocation.getDeviceId().equals(vo.getId())).map(allocation -> {
                    AllocationVo allocationVo = new AllocationVo();
                    BeanUtils.copyProperties(allocation, allocationVo);
                    Scada scadaBelong = null;
                    //查询scada数据
                    if (!CollectionUtils.isEmpty(scadas)) {
                        List<Scada> scadaList = scadas.stream()
                                .filter(scada -> scada.getCode() != null && scada.getCode().equals(allocation.getVariableCode()))
                                .collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(scadaList)) {
                            scadaBelong = scadaList.get(0);
                            allocationVo.setScada(scadaList.get(0));
                        }
                    }
                    //设置报警相关信息
                    Set<Long> reportIdSets = getReportIdSets(Collections.singletonList(allocation));
                    if (!CollectionUtils.isEmpty(scadaReports) && !CollectionUtils.isEmpty(reportIdSets)) {
                        //筛选出报警信息列表
                        List<ScadaReport> reports = scadaReports.stream().filter(scadaReport -> scadaReport.getId() != null
                                && reportIdSets.contains(scadaReport.getId())).collect(Collectors.toList());
                        //设置指标报警状态为未报警
                        allocationVo.setIsAlarm("0");
                        //判断指标是否报警
                        if (scadaBelong != null && scadaBelong.getValue() != null) {
                            for (ScadaReport scadaReport : reports) {
                                if (scadaReport.getReportLower() == null || scadaReport.getReportUpper() == null) {
                                    continue;
                                }
                                if (!isNumber(scadaReport.getReportLower())
                                        || !isNumber(scadaReport.getReportUpper())
                                        || !isNumber(scadaBelong.getValue())) {
                                    continue;
                                }
                                Double lower = Double.parseDouble(scadaReport.getReportLower());
                                Double upper = Double.parseDouble(scadaReport.getReportUpper());
                                Double value = Double.parseDouble(scadaBelong.getValue());

                                if (value.compareTo(upper) > 0 || value.compareTo(lower) < 0) {
                                   allocationVo.setIsAlarm("1");
                                    break;
                                }
                            }
                        }
                        allocationVo.setScadaReports(reports);
                    }
                    return allocationVo;
                }).collect(Collectors.toList());
                vo.setAllocations(allocationVos);
            }
            if (org.apache.commons.lang3.StringUtils.isNotBlank(deviceQuery.getDeviceTypeIds())) {
                List<String> deviceTypeList = Arrays.stream(deviceQuery.getDeviceTypeIds().split(",")).map(String::trim)
                        .collect(Collectors.toList());
                for (String s : deviceTypeList) {
                    if (StringUtil.isNotEmpty(vo.getDeviceType()) && vo.getDeviceType().equals(s)) {
                        return vo;
                    }
                }
                return null;
            }
            return vo;
        }).filter(Objects::nonNull).collect(Collectors.toList());
        return manageVos;
    }


    /**
     * 判断字符串内容是否为数字类型
     *
     * @param str
     * @return
     */
    public static boolean isNumber(String str) {
        Matcher m = pattern.matcher(str);
        return m.matches();
    }

    /**
     * 获取实时监控组装数据
     *
     * @param deviceArchiveManages
     * @param allocations
     * @param scadaReports
     * @param scadas
     * @param allocationsHash
     * @param scadaReportsHash
     * @param scadasHash
     * @return
     */

    private List<DeviceArchiveManageVo> getDeviceArchiveManageVos(List<DeviceArchiveManage> deviceArchiveManages, List<Allocation> allocations, List<ScadaReport> scadaReports, List<Scada> scadas, int allocationsHash, int scadaReportsHash, int scadasHash) {
        redisTemplate.opsForValue().set("allocations", allocationsHash,60, TimeUnit.MINUTES);
        redisTemplate.opsForValue().set("scadaReports", scadaReportsHash,60, TimeUnit.MINUTES);
        redisTemplate.opsForValue().set("scadas", scadasHash,60, TimeUnit.MINUTES);

        List<DeviceArchiveManageVo> manageVos = new ArrayList<>();
        for (DeviceArchiveManage manage : deviceArchiveManages) {
            DeviceArchiveManageVo vo = new DeviceArchiveManageVo();
            BeanUtils.copyProperties(manage, vo);
            //查询绑定的指标
            if (!CollectionUtils.isEmpty(allocations)) {
                List<AllocationVo> allocationVoList = new ArrayList<>();
                for (Allocation allocation : allocations) {
                    if (allocation.getDeviceId().equals(manage.getId())) {
                        AllocationVo allocationVo = new AllocationVo();
                        BeanUtils.copyProperties(allocation, allocationVo);
                        //查询scada数据
                        if (!CollectionUtils.isEmpty(scadas)) {
                            for (Scada scada : scadas) {
                                if (scada.getCode().equals(allocation.getVariableCode())) {
                                    allocationVo.setScada(scada);
                                }
                            }
                        }
                        //设置报警相关信息
                        if (StringUtils.hasText(allocation.getReportId()) && !CollectionUtils.isEmpty(scadaReports)) {
                            List<Long> ids = Arrays.stream(allocation.getReportId().split(",")).map(Long::parseLong).collect(Collectors.toList());
                            List<ScadaReport> scadaReportList = new ArrayList<>();
                            for (ScadaReport report : scadaReports) {
                                if (ids.contains(report.getId())) {
                                    scadaReportList.add(report);
                                }
                            }
                            allocationVo.setScadaReports(scadaReportList);
                        }
                        allocationVoList.add(allocationVo);
                    }
                }
                vo.setAllocations(allocationVoList);
            }
            manageVos.add(vo);
        }
        redisTemplate.opsForValue().set("monitor_scada", JSON.toJSONString(manageVos),60, TimeUnit.MINUTES);
        return manageVos;
    }


    /**
     * 获取关联指标
     *
     * @param vo
     */
    private void getAllocation(DeviceArchiveManageVo vo) {
        QueryWrapper<Allocation> allocationQueryWrapper = new QueryWrapper<>();
        allocationQueryWrapper.eq("DEVICE_ID", vo.getId());
        List<Allocation> allocations = allocationMapper.selectList(allocationQueryWrapper);
        if (!CollectionUtils.isEmpty(allocations)) {
            List<AllocationVo> allocationVos = allocations.stream().map(allocation -> {
                AllocationVo allocationVo = new AllocationVo();
                BeanUtils.copyProperties(allocation, allocationVo);
                //查询scada数据
                List<Scada> dataByCode = scadaMapper.getListDataByCode(allocation.getVariableCode());
                if (!CollectionUtils.isEmpty(dataByCode)) {
                    allocationVo.setScada(dataByCode.get(0));
                }
                //查询绑定的报警列表
                Set<Long> reportIdSets = getReportIdSets(Collections.singletonList(allocation));
                if (!CollectionUtils.isEmpty(reportIdSets)) {
                    logger.info("获取指标报警相关....");
                    QueryWrapper<ScadaReport> scadaReportQueryWrapper = new QueryWrapper<>();
                    scadaReportQueryWrapper.in("ID", reportIdSets);
                    List<ScadaReport> scadaReports = reportMapper.selectList(scadaReportQueryWrapper);
                    if (!CollectionUtils.isEmpty(scadaReports)) {
                        allocationVo.setScadaReports(scadaReports);
                    }
                }
                //返回数据
                return allocationVo;
            }).collect(Collectors.toList());
            vo.setAllocations(allocationVos);
        }
    }


    /**
     * 设置查询条件
     *
     * @param deviceQuery
     * @return
     */
    private QueryWrapper<DeviceArchiveManage> getQueryWrapper(DeviceQuery deviceQuery) {
        QueryWrapper<DeviceArchiveManage> queryWrapper = new QueryWrapper<>();
        if (!ObjectUtils.isEmpty(deviceQuery)) {
            if (StringUtils.hasText(deviceQuery.getType())) {
                queryWrapper.eq("manage.TYPE", deviceQuery.getType());
            }
            if (StringUtils.hasText(deviceQuery.getDeviceType())) {
                queryWrapper.eq("manage.DEVICE_TYPE", deviceQuery.getDeviceType());
            }
            if (StringUtils.hasText(deviceQuery.getStatus())) {
                queryWrapper.eq("manage.STATUS", deviceQuery.getStatus());
            }
            if (StringUtils.hasText(deviceQuery.getQuery())) {
                queryWrapper.and(q -> {
                    q.like("manage.ADDRESS", deviceQuery.getQuery())
                            .or().like("manage.NAME", deviceQuery.getQuery())
                            .or().like("manage.DESCRIBE", deviceQuery.getQuery());
                });

            }
            if (StringUtils.hasText(deviceQuery.getName())) {
                queryWrapper.like("manage.NAME", deviceQuery.getName());
            }
            if (StringUtils.hasText(deviceQuery.getCode())) {
                queryWrapper.like("allocation.VARIABLE_CODE", deviceQuery.getCode());
            }
            if (deviceQuery.getDeviceId() != null) {
                queryWrapper.eq("manage.ID", deviceQuery.getDeviceId());
            }
            if (deviceQuery.getLatitude() != null && deviceQuery.getLongitude() != null) {
                queryWrapper.eq("manage.LATITUDE", deviceQuery.getLatitude())
                        .eq("manage.LONGITUDE", deviceQuery.getLongitude());
            }
        }
        return queryWrapper;
    }


    @Override
    public void insertDataAndMouth(Scada scada) {
        if (ObjectUtils.isEmpty(scada)) {
            throw new ToflyDeniedException("数据为空");
        }
        if (scada.getId() != null && StringUtils.isEmpty(scada.getValue()) && scada.getScadaTime() == null) {
            scada = getById(scada.getId());
        }
        log.info("获取到同步scada信息数据：-----{}-----", scada.toString());
        if (!ObjectUtils.isEmpty(scada)) {
            //存储到分表月份
            String tableName = createMonths(scada.getScadaTime() == null ? new Date() : scada.getScadaTime());
            log.info("创建月份表成功：-----{}-----", tableName);
            //检测数据是否存在，不存在就插入
            if (CollectionUtils.isEmpty(scadaMapper.selectHistoryById(tableName, scada.getId()))) {
                log.info("scada数据不存在月份表中，正在执行新增数据");
                scadaMapper.insertByOne(scada, tableName);
                log.info("新增数据成功：-----{}-----", scada);
            }
            //检测报警信息
            //sendNotice(scada);
        }
    }

    @Override
    /*@Async*/
    public void insertData(Scada scada) {
        if (ObjectUtils.isEmpty(scada)) {
            throw new ToflyDeniedException("数据为空");
        }

        if (!ObjectUtils.isEmpty(scada)) {
            //存储到分表月份
//            String tableName = createMonths(scada.getScadaTime() == null ? new Date() : scada.getScadaTime());
//            log.info("创建月份表成功：-----{}-----", tableName);
//            //检测数据是否存在，不存在就插入
//            if (CollectionUtils.isEmpty(scadaMapper.selectHistoryById(tableName, scada.getId()))) {
//                log.info("scada数据不存在月份表中，正在执行新增数据");
//                scadaMapper.insertByOne(scada, tableName);
//                log.info("新增数据成功：-----{}-----", scada);
//            }
            //检测报警信息
            sendNotice(scada);
        }
    }

    /**
     * 获取实时表SCADA_MONITORING_REALTIME中的数据根据ID
     *
     * @param scada
     * @return
     */
    private Scada getScada(Scada scada) {
        if (scada != null && scada.getId() != null && StringUtils.isEmpty(scada.getValue()) && scada.getScadaTime() == null) {
            List<Scada> scadas = baseMapper.selectScadaMonitoringRealtimeById("SCADA_MONITORING_REALTIME", scada.getId());
            if (!CollectionUtils.isEmpty(scadas)) {
                scada = scadas.get(0);
            }
        }
        return scada;
    }

    /**
     * 定时任务，每分钟5执行一次
     */
   /* @Scheduled(cron = "0 0/5 * * * ?")*/
   /* @Async*/
    public void timingInsertData() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        Date end = calendar.getTime();
        calendar.add(Calendar.MINUTE, -5);
        Date start = calendar.getTime();

//        List<String> codes = getCodes(null);
//        if (CollectionUtils.isEmpty(codes)) {
//            return;
//        }
        List<Scada> scadas = scadaMapper.selectList(new QueryWrapper<Scada>().lambda()
                .between(Scada::getUpdateTime, start, end));
        log.info("时间{}-----{}----->查询每分钟定时任务实时数据{}", start, end, scadas);
        if (!CollectionUtils.isEmpty(scadas)) {
            //存储到分表月份
            String tableName = createMonths(scadas.get(0).getScadaTime() == null ? new Date() : scadas.get(0).getScadaTime());
            int i = insertMouthDataBatch(scadas, tableName);
            log.info("{}新增合计数据：{}条", tableName, i);
            //监控+报警
            for (Scada scada : scadas) {
                // todo 暂时不使用 实时监控表
//                if (scada.getValue().startsWith(".")) {
//                    scada.setValue("0" + scada.getValue());
//                }
//                Scada monitoring = scadaMapper.selectDataByMonitoring(scada.getCode());
//                log.info("查询存在的实时数据{}", monitoring);
//                if (ObjectUtils.isEmpty(monitoring)) {
//                    scadaMapper.insertMonitoring(scada);
//                    log.info("无实时数据，进行新增{}", scada);
//                } else {
//                    scadaMapper.updateMonitoring(scada);
//                    log.info("有实时数据，进行更新{}", scada);
//                }
                //检测报警信息
                sendNotice(scada, DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
            }
        }
    }

    /**
     * 截取数组长度批量插入
     *
     * @param scadas    数据集合
     * @param tableName 表名
     * @return 成功条数
     */
    public int insertMouthDataBatch(List<Scada> scadas, String tableName) {
        int size = scadas.size();
        int limit = 1000;
        int startIndex = 0;
        int endIndex;
        if (size <= limit) {
            //添加新数据
            int insertMouth = scadaMapper.insertMouth(scadas, tableName);
            log.info("新增数据：{}条", insertMouth);
        } else {
            while (startIndex < size) {
                //结束索引范围
                endIndex = startIndex + limit;
                //最后索引不能超过本身长度
                endIndex = endIndex > size ? size : endIndex;
                //截取数据
                List<Scada> scadaList = scadas.subList(startIndex, endIndex);
                //下次索引位置
                startIndex = endIndex;
                //添加新数据
                int insertMouth = scadaMapper.insertMouth(scadaList, tableName);
                log.info("{}新增数据：{}条", tableName, insertMouth);
            }
        }
        return size;
    }

    @Override
    public void fetchData(String code, String start, String end, Integer type) {
        Date startTime = null;
        Date endTime = null;
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
            startTime = format.parse(start);
            endTime = format.parse(end);
        } catch (Exception e) {
            throw new RuntimeException("时间格式有误");
        }
        //统计类型0小时，1按天，2月份，3年份
        switch (type) {
            case 0: {
                log.info("开始进行统计计算-----{}", type);
                List<Date> dates = findDates(startTime, endTime, 0);
                log.info("获取日期范围....{}", dates);
                if (!CollectionUtils.isEmpty(dates)) {
                    log.info("进入计算....{}", dates);
                    for (Date date : dates) {
                        insert24H(code, date);
                    }
                }
                break;
            }
            case 1: {
                log.info("开始进行统计计算-----{}", type);
                List<Date> dates = findDates(startTime, endTime, 1);
                log.info("获取日期范围....{}", dates);
                if (!CollectionUtils.isEmpty(dates)) {
                    log.info("进入计算....{}", dates);
                    for (Date date : dates) {
                        syncDaysData(code, date);
                    }
                }
                break;
            }
            case 2: {
                log.info("开始进行统计计算-----{}", type);
                List<Date> dates = findDates(startTime, endTime, 2);
                log.info("获取到时间范围....{}", dates);
                if (!CollectionUtils.isEmpty(dates)) {
                    log.info("进入计算....{}", dates);
                    for (Date date : dates) {
                        syncDataIntoMonths(code, date);
                    }
                }
                break;
            }
            case 3: {
                //TODO 暂时不同步年数据
//                List<Date> dates = findDates(startTime, endTime,0);
//                if (!CollectionUtils.isEmpty(dates)){
//                    for (Date date : dates) {
//                        syncDaysData(date);
//                    }
//                }
                break;
            }
        }
    }

    @Override
    public Page<StatisticsScadaVo> selectHistoryByMouthsTables(List<String> mouthTables, Page page,
                                                               QueryWrapper<ScadaQuery> queryWrapper, boolean isPage) {
        if (isPage) {
//            Page<StatisticsScadaVo> statisticsScadaVoPage = baseMapper.selectHistoryByMouthsTables(mouthTables, page, queryWrapper);
            Page<StatisticsScadaVo> statisticsScadaVoPage = baseMapper.selectHistoryByMouthsTables1("SCADA_HISTORY", page, queryWrapper);
            //特殊值处理
            defaultValHandle(statisticsScadaVoPage);
            return statisticsScadaVoPage;
        } else {
            Page<StatisticsScadaVo> pageDb = new Page<>();
//            List<StatisticsScadaVo> statisticsScadaVoPage = baseMapper.selectHistoryByMouthsTablesNoPage(mouthTables, queryWrapper);
            List<StatisticsScadaVo> statisticsScadaVoPage = baseMapper.selectHistoryByMouthsTablesNoPage1("SCADA_HISTORY", queryWrapper);
            pageDb.setRecords(statisticsScadaVoPage);
            //特殊值处理
            defaultValHandle(pageDb);
            return pageDb;
        }
    }

    /**
     * 特殊值处理
     *
     * @param statisticsScadaVoPage 数据集合
     */
    private void defaultValHandle(Page<StatisticsScadaVo> statisticsScadaVoPage) {
        for (StatisticsScadaVo scadaVo : statisticsScadaVoPage.getRecords()) {
            if (!StringUtils.hasText(scadaVo.getValue())) {
                scadaVo.setValue("0");
            } else if (StringUtils.hasText(scadaVo.getValue()) && scadaVo.getValue().startsWith(".")) {
                String value = scadaVo.getValue();
                scadaVo.setValue("0" + value);
            }
        }
    }

    @Override
    public void fetchDataMouth(String code, String startStr, String endStr) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        Date end = DateUtil.parseDateTime(endStr);
        calendar.add(Calendar.MINUTE, -1);
        Date start = DateUtil.parseDateTime(startStr);

        List<String> codes = getCodes(code);
        if (CollectionUtils.isEmpty(codes)) {
            return;
        }
        List<Scada> scadas = scadaMapper.selectList(new QueryWrapper<Scada>().lambda()
                .between(Scada::getScadaTime, start, end)
                .in(Scada::getCode, codes));
        log.info("查询每分钟定时任务实时数据{}", scadas);
        if (!CollectionUtils.isEmpty(scadas)) {
            for (Scada scada : scadas) {
                //实时监控表
                if (scada.getValue().startsWith(".")) {
                    scada.setValue("0" + scada.getValue());
                }
                Scada monitoring = scadaMapper.selectDataByMonitoring(scada.getCode());
                log.info("查询存在的实时数据{}", monitoring);
                if (ObjectUtils.isEmpty(monitoring)) {
                    scadaMapper.insertMonitoring(scada);
                    log.info("无实时数据，进行新增{}", scada);
                } else {
                    scadaMapper.updateMonitoring(scada);
                    log.info("有实时数据，进行更新{}", scada);
                }
                //月份表-警告通知
                insertData(scada);
            }
        }
    }


    /**
     * 获取日期列表
     *
     * @param start 开始时间:2022-01-01 00:00:00
     * @param end   结束时间:2022-10-01 14:00:00
     * @param type  统计类型 //统计类型0小时，1按天，2月份，3年份
     * @return
     */
    public static List<Date> findDates(Date start, Date end, Integer type) {
        Calendar s = Calendar.getInstance();
        s.setTime(start);
//        Calendar e = Calendar.getInstance();
//        e.setTime(end);
//        if (start.getYear() == end.getYear() && start.getMonth() == end.getMonth() && start.getDay() == end.getDay()) {
//            List<Date> dates = new ArrayList<>();
//            dates.add(start);
//            return dates;
//        }
//        if (start.getYear() == end.getYear() && start.getMonth() == end.getMonth() && type == 0) {
//            List<Date> dates = new ArrayList<>();
//            dates.add(start);
//            return dates;
//        }
        //结束时间在开始时间之后
        List<Date> dates = new ArrayList<>();
        dates.add(start);
        dates.add(end);
        while (end.getTime() > s.getTime().getTime()) {
            if (type == 1) {
                s.add(Calendar.DAY_OF_MONTH, 1);
            } else if (type == 2) {
                s.add(Calendar.MONTH, 1);
            } else if (type == 3) {
                s.add(Calendar.DAY_OF_YEAR, 1);
            } else if (type == 0) {
                s.add(Calendar.HOUR_OF_DAY, 1);
            }
            dates.add(s.getTime());
        }
        return dates;
    }


    /**
     * 异步执行同步信息
     *
     * @param data
     */
    public void syncSave(Scada data) {
        if (ObjectUtils.isEmpty(data)) {
            throw new RuntimeException("参数为空");
        }
        System.out.println("主线程 =====> 开始 =====> " + System.currentTimeMillis());

        ExecutorService executorService = Executors.newSingleThreadExecutor();
        executorService.submit(() -> {
            System.out.println("异步线程 =====> 开始 =====> " + System.currentTimeMillis());
            try {
                Thread.sleep(5000);
                syncScadaData(data);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            System.out.println("异步线程 =====> 结束 =====> " + System.currentTimeMillis());
        });
        executorService.shutdown(); // 回收线程池

        try {
            Thread.sleep(2000);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }

        System.out.println("主线程 =====> 结束 =====> " + System.currentTimeMillis());

    }

    private void syncScadaData(Scada scada) {
//        Scada scada = null;
//        if (data.getId() != null) {
//            scada = scadaMapper.selectById(data.getId());
//        } else if (StringUtils.hasText(data.getCode())) {
//            Page<Scada> page = new Page<>(1, 1);
//            QueryWrapper<Scada> scadaQueryWrapper = new QueryWrapper<>();
//            scadaQueryWrapper.eq("CODE", data.getCode())
//                    .orderByDesc("SCADA_TIME");
//            Page<Scada> scadas = scadaMapper.selectPage(page, scadaQueryWrapper);
//            List<Scada> records = scadas.getRecords();
//            if (!CollectionUtils.isEmpty(records)) {
//                scada = records.get(0);
//            }
//        }

        log.info("获取到同步scada信息数据：-----{}-----", scada.toString());
        if (!ObjectUtils.isEmpty(scada)) {
            //检测是否为系统创建的指标
            log.info("检测是否为系统创建的指标");
            Integer selectCount = allocationMapper.selectCount(new QueryWrapper<Allocation>()
                    .eq("VARIABLE_CODE", scada.getCode()));
            if (selectCount == 0) {
                log.info("数据不存在指标绑定配置，无法进行同步更新存储");
                throw new RuntimeException("数据不存在指标绑定配置，无法进行同步更新存储");
            }
            //存储到历史表
            scadaMapper.insert(scada);
            //存储到分表月份
            String tableName = createMonths(scada.getScadaTime() == null ? new Date() : scada.getScadaTime());
            log.info("创建月份表成功：-----{}-----", tableName);
            //检测数据是否存在，不存在就插入
            if (CollectionUtils.isEmpty(scadaMapper.selectHistoryById(tableName, scada.getId()))) {
                log.info("scada数据不存在月份表中，正在执行新增数据");
                scadaMapper.insertByOne(scada, tableName);
                log.info("新增数据成功：-----{}-----", scada);
            }
            //检测报警信息
            sendNotice(scada);
        }
    }

    /**
     * 发送报警信息
     *
     * @param scada scada数据
     */
    @SneakyThrows
    //@Async
    void sendNotice(Scada scada) {
        //Thread.sleep(5000);
        scada = getScada(scada);
        log.info("获取到同步scada信息数据：-----{}-----", scada.toString());

        if (!StringUtils.hasText(scada.getValue())) {
            logger.info("数据：{}，存在value值为空，报警流程结束", scada);
            return;
        }

        logger.info("检测发送告警信息");
        QueryWrapper<Allocation> allocationQueryWrapper = new QueryWrapper<>();
        allocationQueryWrapper.eq("VARIABLE_CODE", scada.getCode())
                .eq("IS_ALARM", "1");
        List<Allocation> allocations = allocationMapper.selectList(allocationQueryWrapper);
        logger.info("匹配到检测指标{}，正在执行报警处理", allocations);
        if (!CollectionUtils.isEmpty(allocations)) {
            logger.info("匹配到检测指标...");
            for (Allocation allocation : allocations) {
                if (!StringUtils.hasText(allocation.getIsAlarm()) || allocation.getIsAlarm().equals("0")) {
                    logger.info("未设置报警开关，正在退出这次匹配");
                    break;
                }
                DeviceArchiveManage deviceArchiveManage=null;
                if(allocation.getDeviceId()!=null){
                   deviceArchiveManage=deviceArchiveManageMapper.selectById(allocation.getDeviceId());
                }

                List<Long> reports = getReportIds(Collections.singletonList(allocation));
                QueryWrapper<ScadaReport> scadaReportQueryWrapper = new QueryWrapper<>();
                scadaReportQueryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO)
                        .in("ID", reports);
                List<ScadaReport> scadaReports = reportMapper.selectList(scadaReportQueryWrapper);
                if (!CollectionUtils.isEmpty(scadaReports)) {
                    logger.info("匹配到告警模版...");
                    //报警值
                    BigDecimal scadaVal = new BigDecimal(scada.getValue());

                    for (ScadaReport scadaReport : scadaReports) {

                        BigDecimal upVal = new BigDecimal(scadaReport.getReportUpper());
                        BigDecimal downVal = new BigDecimal(scadaReport.getReportLower());
                        //当值不在正常的范围里，需要报警处理
                        if (scadaVal.compareTo(upVal) > 0 || scadaVal.compareTo(downVal) < 0) {

                            //报警人处理
                             List<String> list_user=new ArrayList<>();
                             if(scadaReport.getLiaisonPeople()!=null){
                                 list_user.add(scadaReport.getLiaisonPeople()+"");

                             }

                              if(StringUtil.isNotEmpty(scadaReport.getNoticeUser())){
                                  List<String> noticeUser=Arrays.asList(scadaReport.getNoticeUser().split(",")) ;
                                  noticeUser.forEach(a->{
                                     if(!list_user.contains(a)){
                                         list_user.add(a);
                                     }

                                  });

                              }

                            for (String user:list_user
                                 ) {
                                if(StringUtil.isNotEmpty(user)){
                                  MessagepushW messagepush = new MessagepushW();
                                // TODO 报警模版后续处理
                                messagepush.setTopic(MessageTopicEnum.REPORT_MSG.getTopic());
                                messagepush.setMessage(createMsg(scadaReport.getMsgTemplate(), scadaReport,allocation,scada,deviceArchiveManage));
                                messagepush.setType("8");
                                messagepush.setTableName(MessageTopicEnum.REPORT_MSG.getTableName());
                                messagepush.setCreateTime(scada.getScadaTime());

                                sendTidingsUtil.sendMsg(messagepush, Long.parseLong(user));
                                logger.info("发送报警消息：{}", messagepush);
                                }

                            }

                            //保存报警信息列表数据
                            saveReportListData(scada, scadaReports, allocations);
                            logger.info("记录到报警列表：{}", scada);
                        }
                    }
                    log.info("--------报警处理结束----------");
                } else {
                    log.info("--------未匹配到报警模版---报警处理结束-------");
                }
            }
        } else {
            logger.info("未匹配到指标，已完成处理");
        }
    }


    /**
     * 发送报警信息（通过定时任务获取的数据）
     *
     * @param scada scada数据
     */
    @SneakyThrows
   /* @Async*/
    void sendNotice(Scada scada, String date) {
        log.info("获取到定时时间{}任务同步scada信息数据：-----{}-----", date, scada.toString());
        if (!StringUtils.hasText(scada.getValue())) {
            logger.info("数据：{}，存在value值为空，报警流程结束", scada);
            return;
        }
        logger.info("检测发送告警信息");
        QueryWrapper<Allocation> allocationQueryWrapper = new QueryWrapper<>();
        allocationQueryWrapper.eq("VARIABLE_CODE", scada.getCode())
                .eq("IS_ALARM", "1");
        List<Allocation> allocations = allocationMapper.selectList(allocationQueryWrapper);
        logger.info("匹配到检测指标{}，正在执行报警处理", allocations);
        if (!CollectionUtils.isEmpty(allocations)) {
            logger.info("匹配到检测指标...");
            for (Allocation allocation : allocations) {
                if (!StringUtils.hasText(allocation.getIsAlarm()) || allocation.getIsAlarm().equals("0")) {
                    logger.info("未设置报警开关，正在退出这次匹配");
                    break;
                }
                DeviceArchiveManage deviceArchiveManage=null;
                if(allocation.getDeviceId()!=null){
                    deviceArchiveManage=deviceArchiveManageMapper.selectById(allocation.getDeviceId());
                }
                List<Long> reports = getReportIds(Collections.singletonList(allocation));
                QueryWrapper<ScadaReport> scadaReportQueryWrapper = new QueryWrapper<>();
                scadaReportQueryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO)
                        .in("ID", reports);
                List<ScadaReport> scadaReports = reportMapper.selectList(scadaReportQueryWrapper);
                if (!CollectionUtils.isEmpty(scadaReports)) {
                    logger.info("匹配到告警模版...");
                    //报警值
                    BigDecimal scadaVal = new BigDecimal(scada.getValue());

                    for (ScadaReport scadaReport : scadaReports) {

                        BigDecimal upVal = new BigDecimal(scadaReport.getReportUpper());
                        BigDecimal downVal = new BigDecimal(scadaReport.getReportLower());
                        boolean isFlag=false;
                        //当值不在正常的范围里，需要报警处理
                        if(scada.getCode().endsWith(".YLBJ")){
                             //有这个指标就报警
                            isFlag=true;
                        }
                        if(!isFlag && (scadaVal.compareTo(upVal) > 0 || scadaVal.compareTo(downVal) < 0)){

                            isFlag=true;
                        }


                        if (isFlag) {

                            //报警人处理
                            List<String> list_user=new ArrayList<>();
                            if(scadaReport.getLiaisonPeople()!=null){
                                list_user.add(scadaReport.getLiaisonPeople()+"");

                            }

                            if(StringUtil.isNotEmpty(scadaReport.getNoticeUser())){
                                List<String> noticeUser=Arrays.asList(scadaReport.getNoticeUser().split(",")) ;
                                noticeUser.forEach(a->{
                                    if(!list_user.contains(a)){
                                        list_user.add(a);
                                    }

                                });

                            }

                            for (String user:list_user
                            ) {
                                if(StringUtil.isNotEmpty(user)){
                                    MessagepushW messagepush = new MessagepushW();
                                    // TODO 报警模版后续处理
                                    messagepush.setTopic(MessageTopicEnum.REPORT_MSG.getTopic());
                                    messagepush.setMessage(createMsg(scadaReport.getMsgTemplate(), scadaReport,allocation,scada,deviceArchiveManage));
                                    messagepush.setType("8");
                                    messagepush.setTableName(MessageTopicEnum.REPORT_MSG.getTableName());
                                    messagepush.setCreateTime(scada.getScadaTime());

                                    sendTidingsUtil.sendMsg(messagepush, Long.parseLong(user));
                                    logger.info("发送报警消息：{}", messagepush);
                                }

                            }

                            //保存报警信息列表数据
                            saveReportListData(scada, scadaReports, allocations);
                            logger.info("记录到报警列表：{}", scada);
                        }

                    }
                    log.info("--------报警处理结束----------");
                } else {
                    log.info("--------未匹配到报警模版---报警处理结束-------");
                }
            }
        } else {
            logger.info("未匹配到指标，已完成处理");
        }
    }

    private void saveReportListData(Scada scada, List<ScadaReport> scadaReports, List<Allocation> allocations) {
        TfScadaReportList tfScadaReportList = new TfScadaReportList();
        BeanUtils.copyProperties(scada, tfScadaReportList);
        List<String> levelTitles = scadaReports.stream().filter(scadaReport -> StringUtils.hasText(scadaReport.getReportLevel()))
                .map(ScadaReport::getReportLevel).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(levelTitles)) {
            String join = org.apache.commons.lang3.StringUtils.join(levelTitles);
            tfScadaReportList.setReportLevel(join);
        }
        List<String> reportUppers = scadaReports.stream().filter(scadaReport -> StringUtils.hasText(scadaReport.getReportUpper()))
                .map(ScadaReport::getReportUpper).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(reportUppers)) {
            String join = org.apache.commons.lang3.StringUtils.join(reportUppers);
            tfScadaReportList.setReportUpper(join);
        }
        List<String> reportLowers = scadaReports.stream().filter(scadaReport -> StringUtils.hasText(scadaReport.getReportLower()))
                .map(ScadaReport::getReportLower).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(reportLowers)) {
            String join = org.apache.commons.lang3.StringUtils.join(reportLowers);
            tfScadaReportList.setReportLower(join);
        }
        List<String> displayNames = allocations.stream().filter(scadaReport -> StringUtils.hasText(scadaReport.getDisplayName()))
                .map(Allocation::getDisplayName).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(displayNames)) {
            String join = org.apache.commons.lang3.StringUtils.join(displayNames);
            tfScadaReportList.setDisplayName(join);
        }
        tfScadaReportList.setStatus("0");
        tfScadaReportListService.insert(tfScadaReportList);
        log.info("添加列表报警数据成功：{}", tfScadaReportList);
    }


    /**
     * 构建信息内容模版
     *
     * @param template
     * @param scadaReport
     * @return
     */
    public String createMsg(String template, ScadaReport scadaReport,Allocation allocation,Scada scada,DeviceArchiveManage deviceArchiveManage) {
        //设备名称，设备编码，指标编码，指标名称，时间，上限、下限，采集值，超标
        String unit=allocation.getUnit();//单位
        if (template.contains("#TIME#")) {
            String date = DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss");
            template = template.replaceAll("#TIME#", date);
        }
        if (template.contains("#YHXM#")) {
            String user = scadaReportService.getUser(scadaReport.getLiaisonPeople());
            template = template.replace("#YHXM#",user);
        }
        if (template.contains("#LXDH#")) {
            template = template.replace("#LXDH#",scadaReport.getLiaisonPhone());
        }

        //报警上限
        if (template.contains("#BJSX#")) {
            String BJSX="";
           BJSX=scadaReport.getReportUpper()+unit;
           template = template.replace("#BJSX#",BJSX);
        }
        //报警下限
        if (template.contains("#BJXX#")) {
            String BJXX="";
           BJXX=scadaReport.getReportLower()+unit;
           template = template.replace("#BJXX#",BJXX);
        }

        //设备名称
        if (template.contains("#SBMC#")) {
            String SBMC="";
            if(deviceArchiveManage!=null){
               SBMC=deviceArchiveManage.getName();
            }
            template = template.replace("#SBMC#",SBMC);
        }
        //设备编码
        if (template.contains("#SBBM#")) {
            String SBBM="";
            if(deviceArchiveManage!=null){
                SBBM=deviceArchiveManage.getCode();
            }
            template = template.replace( "#SBBM#",SBBM);
        }
        //指标名称
        if (template.contains("#ZBMC#")) {
            String ZBMC="";
            ZBMC=allocation.getDisplayName();
          template = template.replace("#ZBMC#",ZBMC);
        }
        //指标编码
        if (template.contains("#ZBBM#")) {
            String ZBBM="";
            ZBBM=allocation.getVariableCode();
         template = template.replace("#ZBBM#",ZBBM);
        }

        //报警值
        if (template.contains("#BJZ#")) {
            String BJZ="";
            BJZ=scada.getValue()+unit;

            template = template.replace( "#BJZ#",BJZ);
        }
        //超标
        if (template.contains("#BJCB#")) {
            String BJCB="";
            BigDecimal S_BJCB=new BigDecimal(scada.getValue());
            BigDecimal upper=new BigDecimal(scadaReport.getReportUpper());
            BigDecimal lower=new BigDecimal(scadaReport.getReportLower());
            if(S_BJCB.compareTo(upper)>0){
                BigDecimal add = S_BJCB.subtract(upper);
                BJCB="上限："+add+unit;
            }
            if(S_BJCB.compareTo(lower)<0){
                BigDecimal add = lower.subtract(S_BJCB);
                BJCB="下限："+add+unit;
            }

            template = template.replace("#BJCB#",BJCB);
        }

        return template;
    }

    /**
     * 获取报警ID集合
     *
     * @param allocations
     * @return
     */
    private List<Long> getReportIds(List<Allocation> allocations) {
        List<Long> reports = allocations.stream().filter(allocation -> StringUtils.hasText(allocation.getReportId()))
                .map(allocation -> {
                    if (allocation.getReportId().contains(",")) {
                        String[] reportIds = allocation.getReportId().split(",");
                        for (String reportId : reportIds) {
                            return Long.parseLong(reportId);
                        }
                    } else {
                        return Long.parseLong(allocation.getReportId());
                    }
                    return null;
                }).collect(Collectors.toList());
        return reports;
    }

    /**
     * 获取报警ID集合(去重)
     *
     * @param allocations
     * @return
     */
    public static Set<Long> getReportIdSets(List<Allocation> allocations) {
        Set<Long> reports = allocations.stream().filter(allocation -> StringUtils.hasText(allocation.getReportId()))
                .map(allocation -> {
                    if (allocation.getReportId().contains(",")) {
                        String[] reportIds = allocation.getReportId().split(",");
                        for (String reportId : reportIds) {
                            return Long.parseLong(reportId);
                        }
                    } else {
                        return Long.parseLong(allocation.getReportId());
                    }
                    return null;
                }).collect(Collectors.toSet());
        return reports;
    }


}
