package com.tofly.scada.service.impl;


import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.scada.common.FlowConstant;
import com.tofly.scada.entity.vo.StatisticsScadaNewVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.scada.mapper.StatisticsScadaNewMapper;
import com.tofly.scada.entity.StatisticsScadaNew;
import com.tofly.scada.entity.dto.StatisticsScadaNewDto;
import com.tofly.scada.service.StatisticsScadaNewService;
import org.springframework.stereotype.Service;

import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;
import java.io.Serializable;

/**
 * 用水量统计(StatisticsScadaNew)表服务实现类
 *
 * @author HaiQiu
 * @since 2022-10-31 13:22:33
 */
@Slf4j
@Service("statisticsScadaNewService")
public class StatisticsScadaNewServiceImpl extends ServiceImpl<StatisticsScadaNewMapper, StatisticsScadaNew> implements StatisticsScadaNewService {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public StatisticsScadaNew get(Serializable id) {
        return getById(id);
    }

    /**
     * 新增
     *
     * @param statisticsScadaNewDto 实体
     */
    @Override
    public void insert(StatisticsScadaNewDto statisticsScadaNewDto) {
        StatisticsScadaNew statisticsScadaNew = convert(statisticsScadaNewDto);
        if (!save(statisticsScadaNew)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param statisticsScadaNewDto 实体
     */
    @Override
    public void edit(StatisticsScadaNewDto statisticsScadaNewDto) {
        StatisticsScadaNew statisticsScadaNew = convert(statisticsScadaNewDto);
        if (!updateById(statisticsScadaNew)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param statisticsScadaNewDto dto实体
     */
    private StatisticsScadaNew convert(StatisticsScadaNewDto statisticsScadaNewDto) {
        StatisticsScadaNew statisticsScadaNew = new StatisticsScadaNew();
        BeanUtils.copyProperties(statisticsScadaNewDto, statisticsScadaNew);
        return statisticsScadaNew;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        StatisticsScadaNewVo statisticsScadaNew = new StatisticsScadaNewVo();
        ExcelUtils.createTemplate("用水量统计", "用水量统计","用水量统计",
                StatisticsScadaNew.class, Collections.singletonList(statisticsScadaNew));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<StatisticsScadaNew> datas = ExcelUtils.getExcelToList("用水量统计", file, StatisticsScadaNew.class);
        if (!CollectionUtils.isEmpty(datas)) {
            saveBatch(datas);
        }
    }

    /**
     * 分页查询
     *
     * @param page               分页
     * @param statisticsScadaNew 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<StatisticsScadaNewVo> pageList(Page<StatisticsScadaNewVo> page, StatisticsScadaNewVo statisticsScadaNew) {
        QueryWrapper<StatisticsScadaNewVo> wrapper = getWrapper(statisticsScadaNew);
         baseMapper.pageListToal(wrapper);
        return baseMapper.pageList(page,wrapper);
    }

    @Override
    public Map<String,String>pageListToal(StatisticsScadaNewVo statisticsScadaNew) {
        QueryWrapper<StatisticsScadaNewVo> wrapper = getWrapper(statisticsScadaNew);
        return baseMapper.pageListToal(wrapper);
    }

    private QueryWrapper<StatisticsScadaNewVo> getWrapper(StatisticsScadaNewVo statisticsScadaNew) {
        QueryWrapper<StatisticsScadaNewVo> wrapper = new QueryWrapper<>(statisticsScadaNew);
        //设置默认查询类型
        statisticsScadaNew.setStatisticsType(statisticsScadaNew.getStatisticsType() ==null ? FlowConstant.INTEGER_ONE : statisticsScadaNew.getStatisticsType());
        wrapper.eq(StringUtils.hasText(statisticsScadaNew.getCode()),"ssn.CODE", statisticsScadaNew.getCode())
                .ne(StringUtils.hasText(statisticsScadaNew.getDifference()),"ssn.DIFFERENCE", statisticsScadaNew.getDifference())
                .between(statisticsScadaNew.getStartTime()!=null && statisticsScadaNew.getEndTime()!=null,"ssn.END_TIME"
                        , statisticsScadaNew.getStartTime(), statisticsScadaNew.getEndTime())
                .eq("ssn.STATISTICS_TYPE",statisticsScadaNew.getStatisticsType())
                .like(StringUtils.hasText(statisticsScadaNew.getCodeName()),"tia.DISPLAY_NAME",statisticsScadaNew.getCodeName())
                .like(StringUtils.hasText(statisticsScadaNew.getDeviceName()),"tdam.NAME",statisticsScadaNew.getDeviceName())
                .eq(StringUtils.hasText(statisticsScadaNew.getDeviceId()),"tdam.ID",statisticsScadaNew.getDeviceId())
                .orderByAsc("ssn.END_TIME");
        return wrapper;
    }

    /**
     * 数据导出
     *
     * @param page               分页
     * @param statisticsScadaNew 查询模型
     * @param isAll              是否导出全部数据，true是，false否
     */
    @Override
    public void export(Page page, StatisticsScadaNewVo statisticsScadaNew, Boolean isAll) {
       /* if (isAll == null || !isAll) {
            Page<StatisticsScadaNewVo> pageData = pageList(page,statisticsScadaNew);
            List<StatisticsScadaNewVo> scadaNewVos=pageData.getRecords();
             Map<String,String> map= pageListToal(statisticsScadaNew);

            StatisticsScadaNewVo vo=new StatisticsScadaNewVo();
            vo.setDeviceName("合计");
            vo.setDifference(map.get("difference"));
            scadaNewVos.add(vo);
            //导出数据
            ExcelUtils.getListToExcel(StatisticsScadaNewVo.class, scadaNewVos, "用水量统计");
        } else {
            QueryWrapper<StatisticsScadaNewVo> wrapper = getWrapper(statisticsScadaNew);
            List<StatisticsScadaNewVo> scadaNewVos = baseMapper.listAll(wrapper);
            Map<String,String> map= pageListToal(statisticsScadaNew);
            StatisticsScadaNewVo vo=new StatisticsScadaNewVo();
            vo.setDeviceName("合计");
            vo.setDifference(map.get("difference"));
            scadaNewVos.add(vo);
            ExcelUtils.getListToExcel(StatisticsScadaNewVo.class, scadaNewVos, "用水量统计");
        }*/
        QueryWrapper<StatisticsScadaNewVo> wrapper = getWrapper(statisticsScadaNew);
        List<StatisticsScadaNewVo> scadaNewVos = baseMapper.listAll(wrapper);
        Map<String,String> map= pageListToal(statisticsScadaNew);
        StatisticsScadaNewVo vo=new StatisticsScadaNewVo();
        Object object = map.get("difference");
       String difference= String.valueOf(object);
        vo.setDeviceName("合计");
        vo.setDifference(difference);
        scadaNewVos.add(vo);
        ExcelUtils.getListToExcel(StatisticsScadaNewVo.class, scadaNewVos, "用水量统计");
    }

    /**
     * 查询所有数据
     *
     * @param statisticsScadaNew 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<StatisticsScadaNewVo> allList(StatisticsScadaNewVo statisticsScadaNew) {
        return baseMapper.listAll(getWrapper(statisticsScadaNew));
    }
}

