/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.BaseController;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.dto.AddressBookQuery;
import com.tofly.flow.dto.AddressBookDTO;
import com.tofly.flow.service.AddressBookService;
import io.swagger.annotations.*;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Set;

/**
 * 通讯录
 *
 * @author qf
 * @date Thu Apr 21 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/addressbook")
@Api(tags="通讯录接口")
public class AddressBookController extends BaseController {

    private final  AddressBookService addressBookService;

    @ToFlyAppLog(title = "通讯录新增")
    @PostMapping
    @ApiOperation(value = "通讯录新增")
    public  ResultRespone insert(@RequestBody @Validated AddressBookDTO addressBookDTO){
        addressBookService.insert(addressBookDTO);
        return decide(FlowConstant.ADD_SUCCESS);
    }

    @ToFlyAppLog(title = "通讯录修改")
    @PutMapping
    @ApiOperation(value = "通讯录修改")
    public  ResultRespone edit(@RequestBody @Validated(value = AddressBookDTO.update.class) AddressBookDTO addressBookDTO){
        addressBookService.edit(addressBookDTO);
        return decide(FlowConstant.UPDATE_SUCCESS);
    }

    @ToFlyAppLog(title = "通讯录删除")
    @ApiOperation(value = "通讯录批量删除")
    @DeleteMapping
    public ResultRespone delete(@ApiParam(value ="多个ID以数组形式进行传入") @RequestBody Set<Long> ids){
        addressBookService.delete(ids);
        return decide(FlowConstant.DELETE_SUCCESS);
    }

    @ToFlyAppLog(title = "通讯录模糊-分页查询")
    @ApiOperation(value = "通讯录模糊-分页查询")
    @GetMapping("/list")
    public ResultRespone pageList(Page page,AddressBookQuery query){
        return decide(addressBookService.pageList(page,query));
    }

    @ToFlyAppLog(title = "通讯录模糊-查询所有")
    @ApiOperation(value = "通讯录模糊-查询所有")
    @PostMapping("/all")
    public ResultRespone all(AddressBookQuery query){
        return decide(addressBookService.all(query));
    }

    @ToFlyAppLog(title = "通讯录根据ID查询")
    @ApiOperation(value = "通讯录根据ID查询")
    @GetMapping("/get")
    public ResultRespone all(@ApiParam(value = "通讯录ID") @RequestParam Long id){
        return decide(addressBookService.getById(id));
    }
}
