/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.BaseController;
import com.tofly.flow.entity.EmergencyHazard;
import com.tofly.flow.service.EmergencyHazardService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 事故危险源
 *
 * @author xqf
 * @date Mon Apr 25 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/emergencyhazard")
@Api(tags="事故危险源接口")
public class EmergencyHazardController extends BaseController {

    private final  EmergencyHazardService emergencyHazardService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param emergencyHazard 事故危险源
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getEmergencyHazardPage(Page page, @ModelAttribute EmergencyHazard emergencyHazard) {
      return ResultRespone.success(emergencyHazardService.page(page, Wrappers.query(emergencyHazard)));
    }


    /**
     * 通过id查询事故危险源
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(emergencyHazardService.getById(id));
    }

    /**
     * 新增事故危险源
     * @param emergencyHazard 事故危险源
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增事故危险源" )
    @ApiOperation(value = "新增事故危险源")
    @PostMapping
    public ResultRespone save(@RequestBody EmergencyHazard emergencyHazard) {
      return ResultRespone.success(emergencyHazardService.save(emergencyHazard));
    }

    /**
     * 修改事故危险源
     * @param emergencyHazard 事故危险源
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改事故危险源" )
    @ApiOperation(value = "修改事故危险源")
    @PutMapping
    public ResultRespone updateById(@RequestBody EmergencyHazard emergencyHazard) {
      return ResultRespone.success(emergencyHazardService.updateById(emergencyHazard));
    }

    /**
     * 通过id删除事故危险源
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除事故危险源" )
    @ApiOperation(value = "通过ID删除事故危险源")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(emergencyHazardService.removeById(id));
    }

    /**
     * 通过id批量删除事故危险源
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除事故危险源" )
    @ApiOperation(value = "通过ID批量删除事故危险源")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(emergencyHazardService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
