/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.entity.EmergencyMaterial;
import com.tofly.flow.service.EmergencyMaterialService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 事故应用的物资
 *
 * @author xqf
 * @date Mon Apr 25 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/emergencymaterial")
@Api(tags="事故应用的物资接口")
public class EmergencyMaterialController {

    private final  EmergencyMaterialService emergencyMaterialService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param emergencyMaterial 事故应用的物资
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getEmergencyMaterialPage(Page page, @ModelAttribute EmergencyMaterial emergencyMaterial) {
      return ResultRespone.success(emergencyMaterialService.pageList(page, emergencyMaterial));
    }


    /**
     * 通过id查询事故应用的物资
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(emergencyMaterialService.getById(id));
    }

    /**
     * 新增事故应用的物资
     * @param emergencyMaterial 事故应用的物资
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增事故应用的物资" )
    @ApiOperation(value = "新增事故应用的物资")
    @PostMapping
    public ResultRespone save(@RequestBody EmergencyMaterial emergencyMaterial) {
      return ResultRespone.success(emergencyMaterialService.save(emergencyMaterial));
    }

    /**
     * 修改事故应用的物资
     * @param emergencyMaterial 事故应用的物资
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改事故应用的物资" )
    @ApiOperation(value = "修改事故应用的物资")
    @PutMapping
    public ResultRespone updateById(@RequestBody EmergencyMaterial emergencyMaterial) {
      return ResultRespone.success(emergencyMaterialService.updateById(emergencyMaterial));
    }

    /**
     * 通过id删除事故应用的物资
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除事故应用的物资" )
    @ApiOperation(value = "通过ID删除事故应用的物资")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(emergencyMaterialService.removeById(id));
    }

    /**
     * 通过id批量删除事故应用的物资
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除事故应用的物资" )
    @ApiOperation(value = "通过ID批量删除事故应用的物资")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(emergencyMaterialService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
