/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.entity.EmergencyStaff;
import com.tofly.flow.service.AddressBookService;
import com.tofly.flow.service.EmergencyStaffService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 事故人员
 *
 * @author xqf
 * @date Mon Apr 25 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/emergencystaff")
@Api(tags="事故人员接口")
public class EmergencyStaffController {

    private final  EmergencyStaffService emergencyStaffService;

    private final AddressBookService addressBookService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param emergencyStaff 事故人员
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getEmergencyStaffPage(Page page, @ModelAttribute EmergencyStaff emergencyStaff) {
        Page<EmergencyStaff> pageData = emergencyStaffService.page(page, Wrappers.query(emergencyStaff));
        if (pageData.getTotal()>0){
            for (EmergencyStaff record : pageData.getRecords()) {
                if (record.getAddressBookId()!=null){
                    record.setAddressBook(addressBookService.getById(record.getAddressBookId()));
                }
            }
        }
        return ResultRespone.success(pageData);
    }


    /**
     * 通过id查询事故人员
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(emergencyStaffService.getById(id));
    }

    /**
     * 新增事故人员
     * @param emergencyStaff 事故人员
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增事故人员" )
    @ApiOperation(value = "新增事故人员")
    @PostMapping
    public ResultRespone save(@RequestBody EmergencyStaff emergencyStaff) {
      return ResultRespone.success(emergencyStaffService.save(emergencyStaff));
    }

    /**
     * 修改事故人员
     * @param emergencyStaff 事故人员
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改事故人员" )
    @ApiOperation(value = "修改事故人员")
    @PutMapping
    public ResultRespone updateById(@RequestBody EmergencyStaff emergencyStaff) {
      return ResultRespone.success(emergencyStaffService.updateById(emergencyStaff));
    }

    /**
     * 通过id删除事故人员
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除事故人员" )
    @ApiOperation(value = "通过ID删除事故人员")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(emergencyStaffService.removeById(id));
    }

    /**
     * 通过id批量删除事故人员
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除事故人员" )
    @ApiOperation(value = "通过ID批量删除事故人员")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(emergencyStaffService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
