/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.DocumentDto;
import com.tofly.flow.custom.dto.LoadFolderDocumentDto;
import com.tofly.flow.custom.dto.UpdateDocumentDto;
import com.tofly.flow.entity.FolderDocument;
import com.tofly.flow.service.FolderDocumentService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;

/**
 * 文件夹与文档关联表
 *
 * @author xqf
 * @date Thu Apr 07 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/folderdocument")
@Api(tags="文件夹与文档接口")
public class FolderDocumentController {

    private final  FolderDocumentService folderDocumentService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 文件夹与文档关联表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getFolderDocumentPage(Page page, @ModelAttribute LoadFolderDocumentDto dto) {
      return ResultRespone.success(folderDocumentService.LoadFolderDocumentpage(page, dto));
    }


    /**
     * 转存
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/transferred" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "documentType", value = "文件夹ID/文档类型", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "documentId", value = "文档ID", required = true, dataType = "Long")
    })
    public ResultRespone transferred(Long documentType,Long documentId) {
        Object obj =folderDocumentService.transferred(documentType,documentId);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 新增文件夹与文档关联表
     * @param dto 文件夹与文档关联表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增文件夹与文档关联表" )
    @ApiOperation(value = "新增文件夹与文档关联表")
    @PostMapping
    public ResultRespone save(@Valid DocumentDto dto, List<MultipartFile> fileList) {
        Object obj = folderDocumentService.saveFolderDocument(dto,fileList);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }

    }

    /**
     * 修改文件夹与文档关联表
     * @param dto 文件夹与文档关联表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改文件夹与文档关联表" )
    @ApiOperation(value = "修改文件夹与文档关联表")
    @PutMapping
    public ResultRespone updateById( UpdateDocumentDto dto, List<MultipartFile> fileList) {
        Object obj = folderDocumentService.updateFolderDocument(dto,fileList);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 通过id文档管理文档
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id文档管理文档" )
    @ApiOperation(value = "通过id文档管理文档")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID 文档ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
        boolean up= folderDocumentService.removeUpdateById(id);
        if(up){
            return ResultRespone.success(true,FlowConstant.SUCCESS);
        }else {
            return ResultRespone.failed(false,FlowConstant.FAILED);
        }
    }

    /**
     * 通过id批量删除文件夹与文档关联表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除文件夹与文档关联表" )
    @ApiOperation(value = "通过ID批量删除文件夹与文档关联表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        Object obj = folderDocumentService.removeUpdateByIds(ids);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }
}
