/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.dto.FolderTypeDto;
import com.tofly.flow.dto.UpdateFolderTypeDto;
import com.tofly.flow.entity.FolderType;
import com.tofly.flow.service.FolderTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.validation.Valid;
import java.util.Arrays;
/**
 * 文件夹表
 *
 * @author xqf
 * @date Thu Apr 07 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/foldertype")
@Api(tags="文件夹表接口")
public class FolderTypeController {

    private final  FolderTypeService folderTypeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param folderType 文件夹表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getFolderTypePage(Page page, @ModelAttribute FolderType folderType) {
      return ResultRespone.success(folderTypeService.page(page, Wrappers.query(folderType)
              .lambda().eq(FolderType::getIsDel,FlowConstant.SHORT_ZERO).orderByDesc(FolderType::getId)));
    }


    /**
     * 通过id查询文件夹表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(folderTypeService.getById(id));
    }

    /**
     * 新增文件夹表
     * @param dto 文件夹表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增文件夹表" )
    @ApiOperation(value = "新增文件夹表")
    @PostMapping
    public ResultRespone save(@RequestBody @Valid FolderTypeDto dto) {
        Object obj = folderTypeService.saveFolderTypeDto(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 修改文件夹表
     * @param dto 文件夹表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改文件夹表" )
    @ApiOperation(value = "修改文件夹表")
    @PutMapping
    public ResultRespone updateById(@RequestBody @Valid UpdateFolderTypeDto dto) {
        Object obj = folderTypeService.updateFolderTypeDto(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 通过id删除文件夹表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除文件夹表" )
    @ApiOperation(value = "通过ID删除文件夹表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
        Object obj = folderTypeService.removeUpdateById(id);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    @GetMapping("/folderTree" )
    @ApiOperation(value = "文件树")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dataType", value = "0 公共; 1个人私有;2部门私有,不传差全部",  dataType = "Long"),
            @ApiImplicitParam(name = "moduleType", value = "模块类型0文档 默认为0;1公告",  dataType = "Short")
    })
    public ResultRespone folderTree(Long dataType,Short moduleType){
        return ResultRespone.success(folderTypeService.folderTree(dataType,moduleType));
    }


}
