/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.BaseController;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.AssetsDTO;
import com.tofly.flow.custom.dto.AssetsQueryDTO;
import com.tofly.flow.custom.entity.Assets;
import com.tofly.flow.custom.service.AssetsService;
import io.swagger.annotations.*;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;
import java.util.Set;

/**
 * 资产管理
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/assets")
@Api(tags = "资产管理接口")
public class AssetsController extends BaseController {

    private final AssetsService assetsService;

    /**
     * 分页查询
     *
     * @param page   分页对象
     * @param assets 资产管理
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询",hidden = true)
    public ResultRespone getAssetsPage(Page page, @ModelAttribute Assets assets) {
        return decide(assetsService.page(page, Wrappers.query(assets)));
    }


    /**
     * 车辆设备资产管理(固定资产台账)-分页查询
     *
     * @param page   分页对象
     * @param dto 搜索关键对象
     * @return
     */
    @ToFlyAppLog(title = "固定资产台账(车辆设备资产管理)-分页查询")
    @GetMapping("/list")
    @ApiOperation(value = "固定资产台账(车辆设备资产管理)-分页查询")
    public ResultRespone list(Page page, AssetsQueryDTO dto) {
        return decide(assetsService.pageList(page, dto));
    }


    /**
     * 通过id查询资产管理
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id查询资产管理")
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return decide(assetsService.getById(id));
    }

    /**
     * 新增资产管理
     *
     * @param assetsDTO 资产管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增资产管理")
    @ApiOperation(value = "新增资产管理")
    @PostMapping
    public ResultRespone save(@RequestBody @Validated AssetsDTO assetsDTO) {
        assetsService.insert(assetsDTO);
        return decide(FlowConstant.ADD_SUCCESS);
    }

    /**
     * 修改资产管理
     *
     * @param assetsDTO 资产管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改资产管理")
    @ApiOperation(value = "修改资产管理")
    @PutMapping
    public ResultRespone updateById(@RequestBody @Validated(value = AssetsDTO.update.class) AssetsDTO assetsDTO) {
        assetsService.edit(assetsDTO);
        return decide(FlowConstant.UPDATE_SUCCESS);
    }

    /**
     * 通过id删除资产管理
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除资产管理")
    @ApiOperation(value = "通过ID删除资产管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        boolean update = assetsService.update(new Assets(), new UpdateWrapper<Assets>().lambda()
                .eq(Assets::getIsDel, FlowConstant.SHORT_ZERO)
                .set(Assets::getIsDel, FlowConstant.SHORT_ONE)
                .in(Assets::getId, id));
        return decide(update);
    }

    /**
     * 通过id批量删除资产管理
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除资产管理")
    @ApiOperation(value = "通过ID批量删除资产管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个ID数组传递", required = true, dataType = "Array")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(Set<Long> ids) {
        boolean update = assetsService.update(new Assets(), new UpdateWrapper<Assets>().lambda()
                .eq(Assets::getIsDel, FlowConstant.SHORT_ZERO)
                .set(Assets::getIsDel, FlowConstant.SHORT_ONE)
                .in(Assets::getId, ids));
        return decide(update);
    }

    @ToFlyAppLog(title = "固定资产台账(车辆设备资产管理)-导出")
    @ApiOperation(value = "固定资产台账(车辆设备资产管理)-数据导出")
    @GetMapping("/fixed/export")
    public void fixedExport(HttpServletRequest request, HttpServletResponse response,AssetsQueryDTO dto){
        assetsService.fixedExport(request,response,dto);
    }

    @Deprecated
    @ToFlyAppLog(title = "物资台账-导出")
    @ApiOperation(value = "物资台账-数据导出",hidden = true)
    @GetMapping("/export")
    public void export(HttpServletRequest request, HttpServletResponse response,AssetsQueryDTO dto){
        assetsService.export(request,response,dto);
    }

    @ToFlyAppLog(title = "资产统计")
    @ApiOperation(value = "资产统计")
    @GetMapping("/statistics")
    public ResultRespone statistics(){
        return decide(assetsService.statistics());
    }

    @ToFlyAppLog(title = "设备资产类型统计")
    @ApiOperation(value = "设备资产类型统计")
    @GetMapping("/statistics/device")
    public ResultRespone statisticsDevice(){
        return decide(assetsService.statisticsDevice());
    }
}
