/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.custom.entity.AssetsMaintain;
import com.tofly.flow.custom.service.AssetsMaintainService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 资产维护
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/assetsmaintain")
@Api(tags="资产维护接口")
public class AssetsMaintainController {

    private final  AssetsMaintainService assetsMaintainService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param assetsMaintain 资产维护
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getAssetsMaintainPage(Page page, @ModelAttribute AssetsMaintain assetsMaintain) {
      return ResultRespone.success(assetsMaintainService.page(page, Wrappers.query(assetsMaintain)));
    }


    /**
     * 通过id查询资产维护
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(assetsMaintainService.getById(id));
    }

    /**
     * 新增资产维护
     * @param assetsMaintain 资产维护
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增资产维护" )
    @ApiOperation(value = "新增资产维护")
    @PostMapping
    public ResultRespone save(@RequestBody AssetsMaintain assetsMaintain) {
      return ResultRespone.success(assetsMaintainService.save(assetsMaintain));
    }

    /**
     * 修改资产维护
     * @param assetsMaintain 资产维护
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改资产维护" )
    @ApiOperation(value = "修改资产维护")
    @PutMapping
    public ResultRespone updateById(@RequestBody AssetsMaintain assetsMaintain) {
      return ResultRespone.success(assetsMaintainService.updateById(assetsMaintain));
    }

    /**
     * 通过id删除资产维护
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除资产维护" )
    @ApiOperation(value = "通过ID删除资产维护")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(assetsMaintainService.removeById(id));
    }

    /**
     * 通过id批量删除资产维护
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除资产维护" )
    @ApiOperation(value = "通过ID批量删除资产维护")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(assetsMaintainService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
