/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.BaseTypeDto;
import com.tofly.flow.custom.dto.UpdateBaseTypeDto;
import com.tofly.flow.custom.entity.BaseType;
import com.tofly.flow.custom.service.BaseTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * 基础类型表
 *
 * @author xqf
 * @date Mon Mar 21 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/basetype")
@Api(tags="基础类型表接口")
public class BaseTypeController {

    private final  BaseTypeService baseTypeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param baseType 基础类型表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getBaseTypePage(Page page, @ModelAttribute BaseType baseType) {
      return ResultRespone.success(baseTypeService.page(page, Wrappers.query(baseType).lambda()
              .eq(BaseType::getIsDel, FlowConstant.SHORT_ZERO).orderByAsc(BaseType::getId)));
    }


    /**
     * 通过id查询基础类型表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(baseTypeService.getById(id));
    }

    /**
     * 新增基础类型表
     * @param dto 基础类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增基础类型表" )
    @ApiOperation(value = "新增基础类型表")
    @PostMapping
    public ResultRespone save(@RequestBody @Valid BaseTypeDto dto) {
        if (dto.getForce()==null || !dto.getForce()){
            if (baseTypeService.count(Wrappers.<BaseType>lambdaQuery()
                    .eq(BaseType::getTypeCode,dto.getTypeCode())
                    .eq(BaseType::getDataType,dto.getDataType())
                    .eq(BaseType::getIsDel,FlowConstant.SHORT_ZERO))!=0){
                return ResultRespone.success(false,"代码存在重复，是否保存");
            }
        }
        Object obj = baseTypeService.saveBaseType(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 修改基础类型表
     * @param dto 基础类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改基础类型表" )
    @ApiOperation(value = "修改基础类型表")
    @PutMapping
    public ResultRespone updateById(@RequestBody @Valid UpdateBaseTypeDto dto) {
        if (dto.getForce()==null || !dto.getForce()){
            List<BaseType> list = baseTypeService.list(Wrappers.<BaseType>lambdaQuery()
                    .eq(BaseType::getTypeCode, dto.getTypeCode()).eq(BaseType::getIsDel,FlowConstant.SHORT_ZERO));
            if (!CollectionUtils.isEmpty(list)){
                for (BaseType baseType : list) {
                    if (!baseType.getId().equals(dto.getId())){
                        return ResultRespone.success(false,"代码存在重复，是否保存");
                    }
                }
            }
        }
        Object obj = baseTypeService.updateBaseType(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }


    /**
     * 有效无效
     * @param id 主键id
     * @param valid 有效无效值 0无效，1有效
     * @return
     */
    @ToFlyAppLog(title="有效无效" )
    @ApiOperation(value = "有效无效")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "valid", value = "有效无效,0无效，1有效", required = true, dataType = "Long")
    })
    @GetMapping("/valid")
    public ResultRespone isValid(Long id,Short valid) {
        Object obj = baseTypeService.updateValid(id,valid);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }


    /**
     * 通过id删除基础类型表,有关联数据验证
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除基础类型表,有关联数据验证" )
    @ApiOperation(value = "通过ID删除基础类型表,有关联数据验证")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
        if(baseTypeService.removeUpdateById(id)){
            return ResultRespone.success(true,FlowConstant.SUCCESS);
        }else {
            return ResultRespone.failed(false,"该分类下有物资信息无法删除");
        }
    }

    /**
     * 通过id批量删除基础类型表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除基础类型表" )
    @ApiOperation(value = "通过ID批量删除基础类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        Object obj = baseTypeService.removeUpdateByIds(ids);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    @ToFlyAppLog(title="获取类型树" )
    @ApiOperation(value = "获取类型树")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "datatypes", value = "区分类别，多个id以逗号隔开", required = true, dataType = "String"),
            @ApiImplicitParam(name = "type", value = "0子级；1类别下类型；默认0",defaultValue = "0", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "id", value = "类型id",dataType = "Long")
    })
    @GetMapping("/typetree" )
    public ResultRespone getTypeTree(String datatypes,Long id,Integer type){
        Object obj = baseTypeService.getTypeTree(datatypes,id,type);
        if(obj instanceof String){
            return ResultRespone.failed(String.valueOf(obj),FlowConstant.FAILED);

        }else {
            return ResultRespone.success((List)obj,FlowConstant.SUCCESS);
        }
    }



}
