/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ConferenceRoomDto;
import com.tofly.flow.custom.dto.UpdateConferenceRoomDto;
import com.tofly.flow.custom.entity.ConferenceRoom;
import com.tofly.flow.custom.service.ConferenceRoomService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.validation.Valid;
import java.util.Arrays;
/**
 * 会议室
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/conferenceroom")
@Api(tags="会议室接口")
public class ConferenceRoomController {

    private final  ConferenceRoomService conferenceRoomService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param conferenceRoom 会议室
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getConferenceRoomPage(Page page, @ModelAttribute ConferenceRoom conferenceRoom) {
      return ResultRespone.success(conferenceRoomService.page(page, Wrappers.query(conferenceRoom).lambda()
              .eq(ConferenceRoom::getIsDel, FlowConstant.SHORT_ZERO).orderByDesc(ConferenceRoom::getId)));
    }


    /**
     * 通过id查询会议室
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(conferenceRoomService.getById(id));
    }

    /**
     * 新增会议室
     * @param dto 会议室
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增会议室" )
    @ApiOperation(value = "新增会议室")
    @PostMapping
    public ResultRespone save(@RequestBody @Valid ConferenceRoomDto dto) {
        Object obj = conferenceRoomService.saveConferenceRoom(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 修改会议室
     * @param dto 会议室
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改会议室" )
    @ApiOperation(value = "修改会议室")
    @PutMapping
    public ResultRespone updateById(@RequestBody UpdateConferenceRoomDto dto) {
        Object obj = conferenceRoomService.updateConferenceRoom(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 通过id删除会议室
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除会议室" )
    @ApiOperation(value = "通过ID删除会议室")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
        if(conferenceRoomService.removeUpdateById(id)){
            return ResultRespone.success(true,FlowConstant.SUCCESS);
        }else {
            return ResultRespone.failed(false,FlowConstant.FAILED);
        }
    }

    /**
     * 通过id批量删除会议室
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除会议室" )
    @ApiOperation(value = "通过ID批量删除会议室")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        Object obj = conferenceRoomService.removeUpdateByIds(ids);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }
}
