/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.DeviceFactoryDto;
import com.tofly.flow.custom.entity.DeviceFactory;
import com.tofly.flow.custom.service.DeviceFactoryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;

/**
 * 设备厂家信息
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/devicefactory")
@Api(tags = "设备厂家信息接口")
public class DeviceFactoryController {

    private final DeviceFactoryService deviceFactoryService;
    /**
     * 分页查询
     *
     * @param page          分页对象
     * @param deviceFactory 设备厂家信息
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getDeviceFactoryPage(Page page, @ModelAttribute DeviceFactory deviceFactory) {
        return ResultRespone.success(deviceFactoryService.page(page, Wrappers.query(deviceFactory).lambda().orderByDesc(DeviceFactory::getId)));
    }


    /**
     * 通过id查询设备厂家信息
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(deviceFactoryService.getDeviceFactoryById(id));
    }

    /**
     * 新增设备厂家信息
     *
     * @param dto 设备厂家信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增设备厂家信息")
    @ApiOperation(value = "新增设备厂家信息")
    @PostMapping
    public ResultRespone save(@RequestPart DeviceFactoryDto dto, @RequestPart List<MultipartFile> fileList) {
        Object obj = deviceFactoryService.saveDeviceFactory(dto,fileList);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 通过id删除设备厂家信息
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除设备厂家信息")
    @ApiOperation(value = "通过ID删除设备厂家信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        return ResultRespone.success(deviceFactoryService.removeById(id));
    }

    /**
     * 通过id批量删除设备厂家信息
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除设备厂家信息")
    @ApiOperation(value = "通过ID批量删除设备厂家信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(deviceFactoryService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
