/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.DocumentDto;
import com.tofly.flow.custom.dto.UpdateDocumentDto;
import com.tofly.flow.custom.entity.Document;
import com.tofly.flow.custom.service.DocumentService;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;

/**
 * 流程文档、公文、会议
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/document")
@Api(tags="流程文档、公文、会议")
public class DocumentController {

    private final  DocumentService documentService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param document 文档管理
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getDocumentPage(Page page, @ModelAttribute Document document) {
      return ResultRespone.success(documentService.page(page, Wrappers.query(document).lambda()
              .eq(Document::getIsDel, FlowConstant.SHORT_ZERO).orderByDesc(Document::getId)));
    }

    /**
     * 新增流程文档、公文、会议
     * @param dto 新增流程文档、公文、会议
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增流程文档、公文、会议" )
    @ApiOperation(value = "新增流程文档、公文、会议")
    //@PostMapping
    public ResultRespone save(@RequestPart @Valid DocumentDto dto, @RequestPart List<MultipartFile> fileList){
        String str = documentService.check(dto,null);
        if(StringUtils.isNotEmpty(str)){
            return ResultRespone.failed(str,FlowConstant.FAILED);
        }
        Long id = documentService.saveDocument(dto,fileList);
        return ResultRespone.success(id,FlowConstant.SUCCESS);
    }

    /**
     * 修改流程文档、公文、会议
     * @param dto 修改文档、公文、会议
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改流程文档、公文、会议" )
    @ApiOperation(value = "修改流程文档、公文、会议")
    //@PutMapping
    public ResultRespone update(@RequestPart @Valid UpdateDocumentDto dto, @RequestPart List<MultipartFile> fileList){
        String str = documentService.updateCheck(dto);
        if(StringUtils.isNotEmpty(str)){
            return ResultRespone.failed(str,FlowConstant.FAILED);
        }
        Object id = documentService.updateDocument(dto,fileList);
        return ResultRespone.success(id);
    }



    /**
     * 通过id查询文档管理
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(documentService.getDocumentById(id));
    }

    /**
     * 通过id删除文档管理
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除文档管理" )
    @ApiOperation(value = "通过ID删除文档管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(documentService.removeById(id));
    }

    /**
     * 通过id批量删除文档管理
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除文档管理" )
    @ApiOperation(value = "通过ID批量删除文档管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(documentService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
