/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.custom.dto.ImportInfoDTO;
import com.tofly.flow.custom.dto.SearchImportInfoVo;
import com.tofly.flow.custom.entity.ImportInfo;
import com.tofly.flow.custom.service.ImportInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Set;

import static com.tofly.flow.common.FlowConstant.ADD_SUCCESS;
import static com.tofly.flow.common.FlowConstant.DELETE_SUCCESS;

/**
 * 物资导入详情
 *
 * @author xqf
 * @date Fri Apr 08 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/importinfo")
@Api(tags="物资导入详情接口")
public class ImportInfoController {

    private final  ImportInfoService importInfoService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param search 搜索关键字
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getImportInfoPage(Page page, SearchImportInfoVo search) {
      return ResultRespone.success(importInfoService.pageList(page, search));
    }


    /**
     * 通过id查询物资导入详情
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(importInfoService.getById(id));
    }

    /**
     * 新增物资导入详情
     * @param dto 物资导入详情
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增物资导入详情" )
    @ApiOperation(value = "新增物资导入详情",hidden = true)
    @PostMapping
    public ResultRespone save(@RequestBody @Validated ImportInfoDTO dto) {
        importInfoService.insert(dto);
        return ResultRespone.success(ADD_SUCCESS);
    }

    /**
     * 修改物资导入详情
     * @param dto 物资导入详情
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改物资导入详情" )
    @ApiOperation(value = "修改物资导入详情",hidden = true)
    @PutMapping
    public ResultRespone edit(@RequestBody @Validated(ImportInfoDTO.update.class ) ImportInfoDTO dto) {
        importInfoService.edit(dto);
        return ResultRespone.success(ADD_SUCCESS);
    }

    /**
     * 通过id删除物资导入详情
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除物资导入详情" )
    @ApiOperation(value = "通过ID删除物资导入详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
        ImportInfo importInfo = importInfoService.getById(id);
        if (!ObjectUtils.isEmpty(importInfo)){
            importInfo.setIsDel((short) 1);
            boolean b = importInfoService.updateById(importInfo);
            return ResultRespone.success(b ? "删除成功" : "删除失败");
        }
        return ResultRespone.failed("数据不存在");
    }

    /**
     * 通过id批量删除物资导入详情
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除物资导入详情" )
    @ApiOperation(value = "通过ID批量删除物资导入详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以数组形式传入", required = true)
    })
    @DeleteMapping
    public ResultRespone removeById(@RequestBody Set<Long> ids) {
        importInfoService.delete(ids);
        return ResultRespone.success(DELETE_SUCCESS);
    }

    @ToFlyAppLog(title = "物资导入详情-导入模板下载")
    @ApiOperation(value = "物资导入详情-导入模板下载",hidden = true)
    @GetMapping("/template")
    public void template(HttpServletResponse response){
        importInfoService.template(response);
    }

    @ToFlyAppLog(title = "物资导入详情-数据导出")
    @ApiOperation(value = "物资导入详情-数据导出",hidden = true)
    @GetMapping("/export")
    public void export(HttpServletRequest request, HttpServletResponse response){
        importInfoService.export(request,response);
    }


    @ToFlyAppLog(title = "物资导入详情-导入数据")
    @ApiOperation(value = "物资导入详情-导入数据",hidden = true)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "baseTypeId",value = "导入ID",required = true)
    })
    @PostMapping("/import")
    public ResultRespone importMatter(@RequestParam Long importId, MultipartFile file){
        importInfoService.importMatter(importId,file);
        return ResultRespone.success("导入成功");
    }
}
