/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.BaseController;
import com.tofly.flow.custom.dto.AssetsQueryDTO;
import com.tofly.flow.custom.dto.KeepGrainDeviceDTO;
import com.tofly.flow.custom.entity.KeepGrainDevice;
import com.tofly.flow.custom.service.KeepGrainDeviceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Set;

import static com.tofly.flow.common.FlowConstant.*;

/**
 * 仓储仪器设备
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/keepgraindevice")
@Api(tags = "仓储仪器设备接口")
public class KeepGrainDeviceController extends BaseController {

    private final KeepGrainDeviceService keepGrainDeviceService;

    /**
     * 分页查询
     *
     * @param page            分页对象
     * @param keepGrainDevice 仓储仪器设备
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询",hidden = true)
    public ResultRespone getKeepGrainDevicePage(Page page, @ModelAttribute KeepGrainDevice keepGrainDevice) {
        return ResultRespone.success(keepGrainDeviceService.page(page, Wrappers.query(keepGrainDevice)));
    }


    /**
     * 明细-分页查询
     *
     * @param page   分页对象
     * @param dto 搜索关键对象
     * @return
     */
    @GetMapping("/list")
    @ApiOperation(value = "明细-分页查询")
    public ResultRespone list(Page page, AssetsQueryDTO dto) {
        return decide(keepGrainDeviceService.pageList(page, dto));
    }


    /**
     * 通过id查询仓储仪器设备
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询",hidden = true)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(keepGrainDeviceService.getById(id));
    }

    /**
     * 新增仓储仪器设备
     *
     * @param keepGrainDevice 仓储仪器设备
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增仓储仪器设备")
    @ApiOperation(value = "新增仓储仪器设备",hidden = true)
    @PostMapping
    public ResultRespone save(@RequestBody @Validated KeepGrainDeviceDTO keepGrainDevice) {
        keepGrainDeviceService.insert(keepGrainDevice);
        return ResultRespone.success(ADD_SUCCESS);
    }

    /**
     * 修改仓储仪器设备
     *
     * @param keepGrainDevice 仓储仪器设备
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改仓储仪器设备")
    @ApiOperation(value = "修改仓储仪器设备",hidden = true)
    @PutMapping
    public ResultRespone updateById(@RequestBody @Validated(KeepGrainDeviceDTO.update.class) KeepGrainDeviceDTO keepGrainDevice) {
        keepGrainDeviceService.edit(keepGrainDevice);
        return ResultRespone.success(UPDATE_SUCCESS);
    }

    /**
     * 通过id删除仓储仪器设备
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除仓储仪器设备")
    @ApiOperation(value = "通过ID删除仓储仪器设备")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        KeepGrainDevice device = keepGrainDeviceService.getById(id);
        if (!ObjectUtils.isEmpty(device)){
            device.setIsDel((short) 1);
            boolean b = keepGrainDeviceService.updateById(device);
            return ResultRespone.success(b ? "删除成功" : "删除失败");
        }
        return ResultRespone.failed("数据不存在");
    }

    /**
     * 通过id批量删除仓储仪器设备
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除仓储仪器设备")
    @ApiOperation(value = "通过ID批量删除仓储仪器设备")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以数组传入", required = true, dataType = "Array")
    })
    @DeleteMapping
    public ResultRespone removeById(@RequestBody Set<Long> ids) {
        keepGrainDeviceService.delete(ids);
        return ResultRespone.success(DELETE_SUCCESS);
    }


    @ToFlyAppLog(title = "仓储仪器设备导入模板下载")
    @ApiOperation(value = "仓储仪器设备导入模板下载")
    @GetMapping("/template")
    public void template(HttpServletResponse response){
        keepGrainDeviceService.template(response);
    }

    @ToFlyAppLog(title = "固定资产台账-数据导出")
    @ApiOperation(value = "固定资产台账-数据导出",hidden = true)
    @GetMapping("/export")
    public ResultRespone export(HttpServletRequest request, HttpServletResponse response,AssetsQueryDTO dto){
        keepGrainDeviceService.export(request,response,dto);
        return decide("导出成功");
    }


    @ToFlyAppLog(title = "仓储仪器设备导入数据")
    @ApiOperation(value = "仓储仪器设备导入数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "baseTypeId",value = "分类ID",required = true)
    })
    @PostMapping("/import")
    public ResultRespone importMatter(MultipartFile file){
        keepGrainDeviceService.importMatter(file);
        return ResultRespone.success("导入成功");
    }
}
