/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.BaseController;
import com.tofly.flow.custom.dto.AssetsQueryDTO;
import com.tofly.flow.custom.entity.KeepGrainMatter;
import com.tofly.flow.custom.service.KeepGrainMatterService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;
import java.util.Set;

import static com.tofly.flow.common.FlowConstant.DELETE_SUCCESS;

/**
 * 仓储物料信息接口
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/keepgrainmatter")
@Api(tags = "仓储物料信息接口")
public class KeepGrainMatterController extends BaseController {

    private final KeepGrainMatterService keepGrainMatterService;

    /**
     * 分页查询
     *
     * @param page            分页对象
     * @param keepGrainMatter 仓储物料信息
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询",hidden = true)
    public ResultRespone getKeepGrainMatterPage(Page page, @ModelAttribute KeepGrainMatter keepGrainMatter) {
        return ResultRespone.success(keepGrainMatterService.page(page, Wrappers.query(keepGrainMatter)));
    }

    /**
     * 物资台账-分页查询
     *
     * @param page   分页对象
     * @param dto 搜索关键对象
     * @return
     */
    @GetMapping("/list")
    @ApiOperation(value = "物资台账-分页查询")
    public ResultRespone list(Page page, AssetsQueryDTO dto) {
        return decide(keepGrainMatterService.pageList(page, dto));
    }

    @ToFlyAppLog(title = "物资台账导出")
    @GetMapping("/export")
    @ApiOperation(value = "物资台账导出")
    public ResultRespone export(HttpServletRequest request, HttpServletResponse response,AssetsQueryDTO queryDTO){
        keepGrainMatterService.export(request,response,queryDTO);
        return decide("导出成功");
    }


    /**
     * 通过id查询仓储物料信息
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(keepGrainMatterService.getById(id));
    }

    /**
     * 新增仓储物料信息
     *
     * @param keepGrainMatter 仓储物料信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增仓储物料信息")
    @ApiOperation(value = "新增仓储物料信息",hidden = true)
    @PostMapping
    public ResultRespone save(@RequestBody KeepGrainMatter keepGrainMatter) {
        return ResultRespone.success(keepGrainMatterService.save(keepGrainMatter));
    }

    /**
     * 修改仓储物料信息
     *
     * @param keepGrainMatter 仓储物料信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改仓储物料信息")
    @ApiOperation(value = "修改仓储物料信息",hidden = true)
    @PutMapping
    public ResultRespone updateById(@RequestBody KeepGrainMatter keepGrainMatter) {
        return ResultRespone.success(keepGrainMatterService.updateById(keepGrainMatter));
    }

    /**
     * 通过id删除仓储物料信息
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除仓储物料信息")
    @ApiOperation(value = "通过ID删除仓储物料信息",hidden = true)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        return ResultRespone.success(keepGrainMatterService.removeById(id));
    }

    /**
     * 通过id批量删除仓储物料信息
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除仓储物料信息")
    @ApiOperation(value = "通过ID批量删除仓储物料信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，数组方式传入", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(@RequestBody Set<Long> ids) {
        keepGrainMatterService.delete(ids);
        return ResultRespone.success(DELETE_SUCCESS);
    }


    @ToFlyAppLog(title = "仓储物料信息导入模板下载")
    @ApiOperation(value = "仓储物料信息导入模板下载")
    @GetMapping("/template")
    public void template(HttpServletResponse response){
        keepGrainMatterService.template(response);
    }


    @ToFlyAppLog(title = "仓储物料信息导入数据")
    @ApiOperation(value = "仓储物料信息导入数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "baseTypeId",value = "分类ID",required = true)
    })
    @PostMapping("/import")
    public ResultRespone importMatter(MultipartFile file){
        keepGrainMatterService.importMatter(file);
        return ResultRespone.success("导入成功");
    }
}
