/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.custom.dto.ImportInfoDTO;
import com.tofly.flow.custom.entity.MaterialImport;
import com.tofly.flow.custom.service.MaterialImportService;
import io.swagger.annotations.*;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import static com.tofly.flow.common.FlowConstant.ADD_SUCCESS;
import static com.tofly.flow.common.FlowConstant.DELETE_SUCCESS;

/**
 * 物资导入
 *
 * @author xqf
 * @date Fri Apr 08 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/materialimport")
@Api(tags="物资导入接口")
public class MaterialImportController {

    private final  MaterialImportService materialImportService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param search 查询参数
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getMaterialImportPage(Page page,@RequestParam(required = false) @ApiParam("关键查询参数") String search) {
      return ResultRespone.success(materialImportService.pageList(page, search));
    }


    /**
     * 通过id查询物资导入
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询",hidden = true)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(materialImportService.getById(id));
    }

    /**
     * 新增物资导入
     * @param materialImport 物资导入
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增物资导入" )
    @ApiOperation(value = "新增物资导入",hidden = true)
    @PostMapping
    public ResultRespone save(@RequestBody @Validated ImportInfoDTO materialImport) {
        materialImportService.insert(materialImport);
        return ResultRespone.success(ADD_SUCCESS);
    }

    /**
     * 修改物资导入
     * @param materialImport 物资导入
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改物资导入" )
    @ApiOperation(value = "修改物资导入",hidden = true)
    @PutMapping
    public ResultRespone edit(@RequestBody @Validated(ImportInfoDTO.update.class ) ImportInfoDTO materialImport) {
        materialImportService.edit(materialImport);
        return ResultRespone.success(ADD_SUCCESS);
    }

    @ToFlyAppLog(title = "删除物资导入数据校验")
    @ApiOperation(value = "删除物资导入数据校验接口")
    @GetMapping("/check/delete")
    public ResultRespone checkDelete(@ApiParam("导入ID") @RequestParam Long importId){
        return ResultRespone.success(materialImportService.checkDelete(importId));
    }


    /**
     * 通过id批量删除物资导入
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除物资导入" )
    @ApiOperation(value = "通过ID批量删除物资导入")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以数组传入", required = true, dataType = "Array")
    })
    @DeleteMapping
    public ResultRespone removeById(@RequestBody Set<Long> ids) {
        materialImportService.delete(ids);
        return ResultRespone.success(DELETE_SUCCESS);
    }
}
