/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.MatterDTO;
import com.tofly.flow.custom.dto.MatterQuerySmall;
import com.tofly.flow.custom.entity.Matter;
import com.tofly.flow.custom.service.MatterService;
import com.tofly.flow.custom.vo.MatterVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Set;

import static com.tofly.flow.common.FlowConstant.*;

/**
 * 物资信息表
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/matter")
@Api(tags = "物资信息表接口")
public class MatterController {


    private final MatterService matterService;

    /**
     * 分页查询
     *
     * @param page   分页对象
     * @param dto 物资信息表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getMatterPage(Page page, MatterQuerySmall dto) {
        return ResultRespone.success(matterService.pageList(page,dto));
    }

    /**
     * 校验分类是否存在数据
     *
     * @param baseTypeId
     * @return
     */
    @GetMapping("/check")
    @ApiOperation(value = "校验分类是否存在数据")
    public ResultRespone getMatterPage(@RequestParam Long baseTypeId) {
        QueryWrapper<Matter> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_DEL", SHORT_ZERO);
        queryWrapper.eq("MATTER_TYPE",baseTypeId);
        Page<Matter> pageList = matterService.page(new Page<>(1, 1), queryWrapper);
        return ResultRespone.success(!CollectionUtils.isEmpty(pageList.getRecords()));
    }


    /**
     * 通过id查询物资信息表
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@RequestParam Long id) {
        return ResultRespone.success(matterService.getById(id));
    }

    /**
     * 新增物资信息表
     *
     * @param matter 物资信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增物资信息表")
    @ApiOperation(value = "新增物资信息表")
    @PostMapping
    public ResultRespone save(@RequestBody @Validated MatterDTO matter) {
        matterService.insert(matter);
        return ResultRespone.success(ADD_SUCCESS);
    }

    /**
     * 修改物资信息表
     *
     * @param matter 物资信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改物资信息表")
    @ApiOperation(value = "修改物资信息表")
    @PutMapping
    public ResultRespone updateById(@RequestBody @Validated(value = {MatterDTO.update.class}) MatterDTO matter) {
        matterService.updateByMatter(matter);
        return ResultRespone.success(UPDATE_SUCCESS);
    }

    /**
     * 通过id删除物资信息表
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除物资信息表")
    @ApiOperation(value = "通过ID删除物资信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        Matter matter = matterService.getById(id);
        if (!ObjectUtils.isEmpty(matter)){
            matter.setIsDel((short) 1);
            boolean b = matterService.updateById(matter);
            return ResultRespone.success(b ? "删除成功" : "删除失败");
        }
        return ResultRespone.failed("数据不存在");

    }

    /**
     * 通过id批量删除物资信息表
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除物资信息表")
    @ApiOperation(value = "通过ID批量删除物资信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除,传入数组数据", required = true,dataType = "Array")
    })
    @DeleteMapping
    public ResultRespone removeById(@RequestBody Set<Long> ids) {
        matterService.delete(ids);
        return ResultRespone.success(DELETE_SUCCESS);
    }

    @ToFlyAppLog(title = "物资信息导入模板下载")
    @ApiOperation(value = "物资信息导入模板下载")
    @GetMapping("/template")
    public void template(HttpServletResponse response){
        matterService.template(response);
    }

    @ToFlyAppLog(title = "物资信息数据导出")
    @ApiOperation(value = "物资信息数据导出")
    @GetMapping("/export")
    public void export(HttpServletRequest request, HttpServletResponse response,MatterQuerySmall dto){
        matterService.export(request,response,dto);
    }


    @ToFlyAppLog(title = "物资信息导入数据")
    @ApiOperation(value = "物资信息导入数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "baseTypeId",value = "分类ID",required = true)
    })
    @PostMapping("/import")
    public ResultRespone importMatter(@RequestParam Long baseTypeId, MultipartFile file){
        matterService.importMatter(baseTypeId,file);
        return ResultRespone.success("导入成功");
    }
}
