/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProcessChargeDto;
import com.tofly.flow.custom.entity.ProcessCharge;
import com.tofly.flow.custom.service.ProcessChargeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;

/**
 * 工程收费
 *
 * @author xqf
 * @date Thu Jun 09 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/processcharge")
@Api(tags="工程收费接口")
public class ProcessChargeController {

    private final  ProcessChargeService processChargeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param processCharge 工程收费
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getProcessChargePage(Page page, @ModelAttribute ProcessCharge processCharge) {
      return ResultRespone.success(processChargeService.page(page, Wrappers.query(processCharge).lambda()
              .eq(ProcessCharge::getIsDel,FlowConstant.SHORT_ZERO).orderByDesc(ProcessCharge::getId)));
    }


    /**
     * 通过id查询工程收费
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(processChargeService.getProcessChargeById(id));
    }
    
    /**
     * 新增工程收费
     * @param dto 工程收费
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增工程收费" )
    @ApiOperation(value = "新增工程收费")
    @PostMapping
    public ResultRespone save(@RequestPart ProcessChargeDto dto, @RequestPart List<MultipartFile> fileList) {
        Object obj = processChargeService.saveProcessCharge(dto,fileList);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }


    /**
     * 通过id删除工程收费
     * @param processId id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除工程收费" )
    @ApiOperation(value = "通过ID删除工程收费")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "processId", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{processId}" )
    public ResultRespone removeById(@PathVariable Long processId) {
      return ResultRespone.success(processChargeService.removeById(processId));
    }

    /**
     * 通过id批量删除工程收费
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除工程收费" )
    @ApiOperation(value = "通过ID批量删除工程收费")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(processChargeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
