/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.LoadProcessConfigDto;
import com.tofly.flow.custom.dto.ProcessConfigAllDto;
import com.tofly.flow.custom.dto.ProcessConfigDto;
import com.tofly.flow.custom.dto.UpdateProcessConfigAllDto;
import com.tofly.flow.custom.entity.ProcessConfig;
import com.tofly.flow.custom.service.ProcessConfigService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
/**
 * 审批流程配置
 *
 * @author xqf
 * @date Mon Mar 21 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/processconfig")
@Api(tags="审批流程配置接口")
public class ProcessConfigController {

    private final  ProcessConfigService processConfigService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 审批流程配置
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getProcessConfigPage(Page page, @ModelAttribute LoadProcessConfigDto dto) {
      return ResultRespone.success(processConfigService.pageProcessConfig(page, dto));
    }


    /**
     * 通过id查询审批流程配置
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(processConfigService.getProcessConfigById(id));
    }


    /**
     * 通过业务类型查询流程配置是否存在
     * @param businessType id
     * @return ResultRespone
     */
    @GetMapping("/getConfigByType" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "businessType", value = "业务类型ID", required = true, dataType = "Long")
    })
    public ResultRespone getConfigByType( Long businessType) {
        return ResultRespone.success(processConfigService.getConfigByType(businessType));
    }

    /**
     * 新增流程配置名称类型
     * @param dto 审批流程配置名称类型
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增审批流程配置名称类型" )
    @ApiOperation(value = "新增审批流程配置名称类型")
    @PostMapping
    public ResultRespone save(@RequestBody @Valid ProcessConfigDto dto) {
        Object obj = processConfigService.saveProcessConfigName(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }


    /**
     * 新增流程配置信息
     * @param dto 新增流程配置信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增流程配置信息" )
    @ApiOperation(value = "新增流程配置信息")
    @PostMapping("/saveall")
    public ResultRespone saveProcessConfig(@RequestBody @Valid ProcessConfigAllDto dto){
        Object obj = processConfigService.saveProcessConfig(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 修改流程配置信息
     * @param dto 修改流程配置信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改流程配置信息" )
    @ApiOperation(value = "修改流程配置信息")
    @PostMapping("/updateall")
    public ResultRespone updateProcessConfig(@RequestBody @Valid UpdateProcessConfigAllDto dto){
        Object obj = processConfigService.updateProcessConfig(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }




    /**
     * 修改流程是否有效
     * @param id 流程主键
     * @param isValid 是否有效1有效；0无效
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改流程是否有效" )
    @ApiOperation(value = "修改流程是否有效")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "isValid", value = "isValid 是否有效1有效；0无效", required = true, dataType = "Short")
    })
    @GetMapping("/isStart")
    public ResultRespone updateById(Long id,Short isValid) {
        if(processConfigService.updateValidById(id,isValid)){
            return ResultRespone.success(true,FlowConstant.SUCCESS);
        }else {
            return ResultRespone.failed(false,FlowConstant.FAILED);
        }
    }

    /**
     * 通过id删除审批流程配置
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除审批流程配置" )
    @ApiOperation(value = "通过ID删除审批流程配置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
        if(processConfigService.removeUpdateById(id)){
            return ResultRespone.success(true,FlowConstant.SUCCESS);
        }else {
            return ResultRespone.failed(false,FlowConstant.FAILED);
        }
    }

    /**
     * 通过id批量删除审批流程配置
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除审批流程配置" )
    @ApiOperation(value = "通过ID批量删除审批流程配置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        Object obj = processConfigService.removeUpdateByIds(ids);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }
}


