/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.LoadProcessDto;
import com.tofly.flow.custom.dto.ProcessDto;
import com.tofly.flow.custom.dto.ProcessLDto;
import com.tofly.flow.custom.service.ProcessService;
import com.tofly.flow.custom.vo.ProcessVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.List;

/**
 * 流程信息
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/process")
@Api(tags="流程信息接口")
public class ProcessController {

    private final  ProcessService processService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 流程信息
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getProcessPage(Page page, @ModelAttribute LoadProcessDto dto) {
        return ResultRespone.success(processService.getProcessList(page,dto));
    }

    /**
     * 流程工单列表
     * @param page 分页
     * @param dto 工单流程查询条件
     * @return
     */
    @PostMapping("/list" )
    @ApiOperation(value = "流程工单列表")
    public ResultRespone getProcessList(Page page,@RequestBody ProcessLDto dto){
        return ResultRespone.success(processService.getProcesss(page,dto));
    }


    /**
     * 通过id查询流程信息
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone<ProcessVo> getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(processService.getProcessById(id));
    }

    /**
     * 新增流程信息
     * @param dto 流程信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增流程信息" )
    @ApiOperation(value = "新增流程信息")
    @PostMapping
    public ResultRespone save(@RequestPart @Valid ProcessDto dto,@RequestPart List<MultipartFile> fileList) {
        Object obj = processService.saveProcess(dto,fileList);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 通过id停止流程信息
     * @param id id
     * @param processStep 流程步骤
     * @param stepNode 步骤节点
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id停止流程信息" )
    @ApiOperation(value = "通过id停止流程信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "processStep", value = "流程步骤", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "stepNode", value = "步骤节点", required = true, dataType = "Integer")
    })
    @GetMapping("/stop" )
    public ResultRespone stopProcess(Long id,Integer processStep,Integer stepNode){
        Object obj = processService.stopProcess(id,processStep,stepNode);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }



    /**
     * 通过id删除流程信息
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除流程信息" )
    @ApiOperation(value = "通过ID删除流程信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
        boolean up= processService.removeUpdateById(id);
        if(up){
            return ResultRespone.success(true,FlowConstant.SUCCESS);
        }else {
            return ResultRespone.failed(false,FlowConstant.FAILED);
        }
    }

}
