/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.BatchDeviceExWarehouseDto;
import com.tofly.flow.custom.entity.ProcessDevice;
import com.tofly.flow.custom.service.ProcessDeviceService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 流程出入/退库设备
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/processdevice")
@Api(tags="流程出入/退库设备接口")
public class ProcessDeviceController {

    private final  ProcessDeviceService processDeviceService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param processDevice 流程出入/退库设备
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getProcessDevicePage(Page page, @ModelAttribute ProcessDevice processDevice) {
      return ResultRespone.success(processDeviceService.page(page, Wrappers.query(processDevice)));
    }


    /**
     * 通过id查询流程出入/退库设备
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(processDeviceService.getById(id));
    }

    /**
     * 流程设备出库
     * @param dto 流程设备出库
     * @return ResultRespone
     */
    @ToFlyAppLog(title="流程设备出库" )
    @ApiOperation(value = "流程设备出库")
    @PostMapping("/exWarehouse")
    public ResultRespone exWarehouse(@RequestBody BatchDeviceExWarehouseDto dto) {
        Object obj = processDeviceService.deviceExWarehouse(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 修改流程出入/退库设备
     * @param processDevice 流程出入/退库设备
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改流程出入/退库设备" )
    @ApiOperation(value = "修改流程出入/退库设备")
    @PutMapping
    public ResultRespone updateById(@RequestBody ProcessDevice processDevice) {
      return ResultRespone.success(processDeviceService.updateById(processDevice));
    }

    /**
     * 通过id删除流程出入/退库设备
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除流程出入/退库设备" )
    @ApiOperation(value = "通过ID删除流程出入/退库设备")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(processDeviceService.removeById(id));
    }

    /**
     * 通过id批量删除流程出入/退库设备
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除流程出入/退库设备" )
    @ApiOperation(value = "通过ID批量删除流程出入/退库设备")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(processDeviceService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
