/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProcessJournalSaveDto;
import com.tofly.flow.custom.entity.ProcessJournal;
import com.tofly.flow.custom.service.ProcessJournalService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;

/**
 * 工程施工日志
 *
 * @author xqf
 * @date Wed May 25 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/processjournal")
@Api(tags="工程施工日志接口")
public class ProcessJournalController {

    private final  ProcessJournalService processJournalService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param processJournal 工程施工日志
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getProcessJournalPage(Page page, @ModelAttribute ProcessJournal processJournal) {
      return ResultRespone.success(processJournalService.page(page, Wrappers.query(processJournal).lambda()
              .eq(ProcessJournal::getIsDel, FlowConstant.SHORT_ZERO).orderByDesc(ProcessJournal::getProcessId)));
    }


    /**
     * 通过id查询工程施工日志
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(processJournalService.getProcessJournalById(id));
    }

    /**
     * 新增工程施工日志
     * @param dto 工程施工日志
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增工程施工日志" )
    @ApiOperation(value = "新增工程施工日志")
    @PostMapping
    public ResultRespone save(@RequestPart ProcessJournalSaveDto dto, @RequestPart List<MultipartFile> fileList) {
        Object obj = processJournalService.saveProcessJournal(dto,fileList);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }


    /**
     * 通过id删除工程施工日志
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除工程施工日志" )
    @ApiOperation(value = "通过ID删除工程施工日志")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(processJournalService.removeById(id));
    }

    /**
     * 通过id批量删除工程施工日志
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除工程施工日志" )
    @ApiOperation(value = "通过ID批量删除工程施工日志")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(processJournalService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
