/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProcessNodeAllDto;
import com.tofly.flow.custom.dto.ProcessNodeDto;
import com.tofly.flow.custom.entity.ProcessNode;
import com.tofly.flow.custom.service.ProcessNodeService;
import com.tofly.flow.custom.vo.ProcessNodeVo;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;

/**
 * 流程节点处理信息（操作步骤都有记录）
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/processnode")
@Api(tags="流程节点处理信息（操作步骤都有记录）接口")
public class ProcessNodeController {

    private final  ProcessNodeService processNodeService;


    /**
     * 通过id查询流程节点处理信息（操作步骤都有记录）
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone<ProcessNodeVo> getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(processNodeService.getProcessNodeById(id));
    }


    /**
     * 新增流程节点处理信息（操作步骤都有记录）
     * @param dto 流程节点处理信息（操作步骤都有记录）
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增流程节点处理信息（操作步骤都有记录）" )
    @ApiOperation(value = "新增流程节点处理信息（操作步骤都有记录）")
    @PostMapping
    public ResultRespone save(@RequestPart @Valid ProcessNodeAllDto dto, @RequestPart List<MultipartFile> fileList) {
        Object obj = processNodeService.saveProcessNodeAll(dto, fileList);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

}
