/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProcessLDto;
import com.tofly.flow.custom.entity.ProcessSql;
import com.tofly.flow.custom.service.ProcessSqlService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;

/**
 * 流程查询sql
 *
 * @author xqf
 * @date Tue May 24 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/processsql")
@Api(tags = "流程查询sql接口")
public class ProcessSqlController {

    private final ProcessSqlService processSqlService;

    /**
     * 分页查询
     *
     * @param page       分页对象
     * @param processSql 流程查询sql
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getProcessSqlPage(Page page, @ModelAttribute ProcessSql processSql) {
        return ResultRespone.success(processSqlService.page(page, Wrappers.query(processSql)));
    }

    /**
     * 流程工单列表
     * @param page 分页
     * @param dto 工单流程查询条件
     * @return
     */
    @PostMapping("/datas" )
    @ApiOperation(value = "流程工单列表")
    public ResultRespone getProcessList(Page page,@RequestBody ProcessLDto dto){
        Object obj = processSqlService.getProcesssData(page,dto);
        if(obj instanceof Page){
            return ResultRespone.success(obj);
        }else {
            return ResultRespone.failed(obj);
        }

    }

    /**
     * 通过id查询流程查询sql
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(processSqlService.getById(id));
    }

    /**
     * 新增流程查询sql
     *
     * @param processSql 流程查询sql
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增流程查询sql")
    @ApiOperation(value = "新增流程查询sql")
    @PostMapping
    public ResultRespone save(@RequestBody ProcessSql processSql) {
        return ResultRespone.success(processSqlService.save(processSql));
    }

    /**
     * 修改流程查询sql
     *
     * @param processSql 流程查询sql
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改流程查询sql")
    @ApiOperation(value = "修改流程查询sql")
    @PutMapping
    public ResultRespone updateById(@RequestBody ProcessSql processSql) {
        return ResultRespone.success(processSqlService.updateById(processSql));
    }

    /**
     * 通过id删除流程查询sql
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除流程查询sql")
    @ApiOperation(value = "通过ID删除流程查询sql")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        if(processSqlService.removeUpdateById(id)){
            return ResultRespone.success(true, FlowConstant.SUCCESS);
        }else {
            return ResultRespone.failed(false,FlowConstant.FAILED);
        }
    }


}
