/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProcessStuffDto;
import com.tofly.flow.custom.dto.ProcessStuffOutDto;
import com.tofly.flow.custom.entity.ProcessStuff;
import com.tofly.flow.custom.service.ProcessStuffService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 流程中物料出入/退库信息
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/processstuff")
@Api(tags="流程中物料出入/退库信息接口")
public class ProcessStuffController {

    private final  ProcessStuffService processStuffService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param processStuff 流程中物料出入/退库信息
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getProcessStuffPage(Page page, @ModelAttribute ProcessStuff processStuff) {
      return ResultRespone.success(processStuffService.page(page, Wrappers.query(processStuff)
              .lambda().eq(ProcessStuff::getIsDel, FlowConstant.SHORT_ZERO).orderByDesc(ProcessStuff::getId)));
    }


    /**
     * 分页查询
     * @param page 分页对象
     * @param processStuff 流程中物料出入/退库信息
     * @return
     */
    @GetMapping("/inStock" )
    @ApiOperation(value = "分页查询物质入库记录")
    public ResultRespone getProcessInStockPage(Page page, @ModelAttribute ProcessStuff processStuff) {
        processStuff.setDataType(null);
        return ResultRespone.success(processStuffService.page(page, Wrappers.query(processStuff)
                .lambda().eq(ProcessStuff::getIsDel, FlowConstant.SHORT_ZERO).ne(ProcessStuff::getDataType,5).orderByDesc(ProcessStuff::getId)));
    }

    /**
     * 通过id查询流程中物料出入/退库信息
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(processStuffService.getById(id));
    }

    /**
     * 物资出库信息
     * @param dto 查询条件
     * @return ResultRespone
     */
    @ToFlyAppLog(title="物资出库信息" )
    @ApiOperation(value = "物资出库信息")
    @GetMapping("/out" )
    public ResultRespone getOutMatter(Page page,@ModelAttribute ProcessStuffOutDto dto){
        return ResultRespone.success(processStuffService.loadOutMatter(page,dto));
    }


    public ResultRespone saveProcessStuff(){

        return null;
    }


}
