/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProcessSuperviseDto;
import com.tofly.flow.custom.entity.ProcessSupervise;
import com.tofly.flow.custom.service.ProcessSuperviseService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 流程督办记录信息
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/processsupervise")
@Api(tags="流程督办记录信息接口")
public class ProcessSuperviseController {

    private final  ProcessSuperviseService processSuperviseService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param processSupervise 流程督办记录信息
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getProcessSupervisePage(Page page, @ModelAttribute ProcessSupervise processSupervise) {
      return ResultRespone.success(processSuperviseService.page(page, Wrappers.query(processSupervise).lambda()
              .eq(ProcessSupervise::getIsDel,FlowConstant.SHORT_ZERO)
              .orderByAsc(ProcessSupervise::getId)));
    }


    /**
     * 通过id查询流程督办记录信息
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(processSuperviseService.getById(id));
    }

    /**
     * 新增流程督办记录信息
     * @param dto 流程督办记录信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增流程督办记录信息" )
    @ApiOperation(value = "新增流程督办记录信息")
    @PostMapping
    public ResultRespone save(@RequestBody ProcessSuperviseDto dto) {
        Object obj = processSuperviseService.saveProcessSupervise(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 通过id删除流程督办记录信息
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除流程督办记录信息" )
    @ApiOperation(value = "通过ID删除流程督办记录信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
        if(processSuperviseService.removeUpdateById(id)){
            return ResultRespone.success(true,FlowConstant.SUCCESS);
        }else {
            return ResultRespone.failed(false,FlowConstant.FAILED);
        }
    }

    /**
     * 通过id批量删除流程督办记录信息
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除流程督办记录信息" )
    @ApiOperation(value = "通过ID批量删除流程督办记录信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        Object obj = processSuperviseService.removeUpdateByIds(ids);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true,FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }

    }
}
