/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProjectContractAllDto;
import com.tofly.flow.custom.dto.ProjectContractDto;
import com.tofly.flow.custom.entity.ProjectContract;
import com.tofly.flow.custom.service.ProjectContractService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;

/**
 * 流程合同信息
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/projectcontract")
@Api(tags="流程合同信息接口")
public class ProjectContractController {

    private final  ProjectContractService projectContractService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param projectContract 流程合同信息
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getProjectContractPage(Page page, @ModelAttribute ProjectContract projectContract) {
      return ResultRespone.success(projectContractService.page(page, Wrappers.query(projectContract)));
    }


    /**
     * 通过id查询流程合同信息
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(projectContractService.getById(id));
    }

    /**
     * 新增或修改合同信息
     * @param dto 新增或修改合同信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增或修改合同信息" )
    @ApiOperation(value = "新增或修改合同信息")
    @PostMapping
    public ResultRespone saveOrUpdate(@RequestPart @Valid ProjectContractDto dto, @RequestPart List<MultipartFile> fileList) {
        Object obj = projectContractService.saveProjectContract(dto,fileList);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }


    /**
     * 通过id删除流程合同信息
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除流程合同信息" )
    @ApiOperation(value = "通过ID删除流程合同信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
        if(projectContractService.removeUpdateById(id)){
            return ResultRespone.success(true,FlowConstant.SUCCESS);
        }else {
            return ResultRespone.failed(false,FlowConstant.FAILED);
        }
    }

}
