/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.flow.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.StorageRoomDto;
import com.tofly.flow.custom.entity.StorageRoom;
import com.tofly.flow.custom.service.StorageRoomService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 库房管理
 *
 * @author xqf
 * @date Thu Jun 09 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/storageroom")
@Api(tags="库房管理接口")
public class StorageRoomController {

    private final  StorageRoomService storageRoomService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param storageRoom 库房管理
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getStorageRoomPage(Page page, @ModelAttribute StorageRoom storageRoom) {
      return ResultRespone.success(storageRoomService.page(page, Wrappers.query(storageRoom).lambda()
              .eq(StorageRoom::getIsDel,FlowConstant.SHORT_ZERO).orderByDesc(StorageRoom::getId)));
    }


    /**
     * 通过id查询库房管理
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(storageRoomService.getById(id));
    }

    /**
     * 新增库房管理
     * @param dto 库房管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增库房管理" )
    @ApiOperation(value = "新增库房管理")
    @PostMapping
    public ResultRespone save(@RequestBody StorageRoomDto dto) {
        Object obj = storageRoomService.saveStorageRoom(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, FlowConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false,FlowConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false,String.valueOf(obj));
        }
    }

    /**
     * 通过id删除库房管理
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除库房管理" )
    @ApiOperation(value = "通过ID删除库房管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(storageRoomService.removeById(id));
    }

    /**
     * 通过id批量删除库房管理
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除库房管理" )
    @ApiOperation(value = "通过ID批量删除库房管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(storageRoomService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
