/*
 * Author: wangjian
 * 生成entity
 */
package com.tofly.flow.custom.dto;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.*;
import java.util.Date;


/**
 * 资产管理
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@Data
@ApiModel
public class AssetsDTO extends Model<AssetsDTO> {
    private static final long serialVersionUID = 1L;

    /**
     * ID
     */
    @ApiModelProperty(value = "ID")
    @NotNull(message = "ID不能为空",groups = AssetsDTO.update.class)
    private Long id;

    /**
     * 物资模版ID
     */
    @ApiModelProperty(value = "物资模版ID")
    private Long matterId;

    /**
     * 资产编号
     */
    @ApiModelProperty(value = "资产编号")
    private String assetsNumber;
    /**
     * 仓库物资设备ID
     */
    @ApiModelProperty(value = "仓库物资设备ID")
    private Long keepGrainDeviceId;
    /**
     * 牌号（车牌、门牌）
     */
    @ApiModelProperty(value = "牌号（车牌、门牌）")
    private String brand;
    /**
     * 资产类型
     */
    @ApiModelProperty(value = "资产类型")
    private Long assetsType;
    /**
     * 资产名称
     */
    @ApiModelProperty(value = "资产名称")
    @NotEmpty(message = "资产名称不能为空")
    @Size(max = 20,message = "资产名称只能在1-20个字符")
    private String assetsName;
    /**
     * 用途类型1:生成;2维修......
     */
    @ApiModelProperty(value = "用途类型1:生成;2维修......")
    private Short useType;
    /**
     * 资产型号（车型）
     */
    @ApiModelProperty(value = "资产型号（车型）")
    @NotEmpty(message = "规格型号不能为空")
    private String model;
    /**
     * 编号1（发动机号）
     */
    @ApiModelProperty(value = "编号1（发动机号）")
    private String numberOne;
    /**
     * 编号2（车架号）
     */
    @ApiModelProperty(value = "编号2（车架号）")
    private String numberTwo;
    /**
     * 编号3
     */
    @ApiModelProperty(value = "编号3")
    private String numberThree;
    /**
     * 铭牌信息
     */
    @ApiModelProperty(value = "铭牌信息")
    @Size(max = 100,message = "铭牌信息长度只能在1-100个字符")
    private String nameplate;
    /**
     * 生产日期
     */
    @ApiModelProperty(value = "生产日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date produceTime;
    /**
     * 购买日期
     */
    @ApiModelProperty(value = "购买日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date buyTime;
    /**
     * 到期日期
     */
    @ApiModelProperty(value = "到期日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expireDate;
    /**
     * 报废/出售日期
     */
    @ApiModelProperty(value = "报废/出售日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date handleTime;
    /**
     * 价格
     */
    @ApiModelProperty(value = "价格")
    private Double price;
    /**
     * 资产状况（车况）1:优；2:良；3:差
     */
    @ApiModelProperty(value = "资产状况（车况）1:优；2:良；3:差")
    private Short status;
    /**
     * 资产状态0:在库；1:在用；2:报废；3:出售；4:闲置
     */

    @ApiModelProperty(value = "资产状态0:在库；1:在用；2:报废；3:出售；4:闲置")
    @NotEmpty(message = "资产状态不能为空")
    private Short assetsState;

    /**
     * 使用(负责)部门
     */
    @ApiModelProperty(value = "使用(负责)部门")
    private Long useDept;

    /**
     * 管理部门
     */
    @ApiModelProperty(value = "管理部门")
    @NotNull(message = "管理部门不能为空")
    private String deptName;

    /**
     * 生产商
     */
    @ApiModelProperty(value = "生产商")
    private String produce;

    /**
     * 供应商
     */
    @ApiModelProperty(value = "供应商")
    private String supply;
    /**
     * 负责人
     */
    @ApiModelProperty(value = "负责人")
    private Long personLiable;
    /**
     * 附件id用逗号分隔
     */
    @ApiModelProperty(value = "附件id用逗号分隔")
    private String annex;
    /**
     * 位置（地址）
     */
    @ApiModelProperty(value = "位置（地址）")
    private String addre;
    /**
     * 经度
     */
    @ApiModelProperty(value = "经度")
    private Double longitude;
    /**
     * 纬度
     */
    @ApiModelProperty(value = "纬度")
    private Double latitude;
    /**
     * 数量
     */
    @ApiModelProperty(value = "数量")
    private Integer quantity;
    /**
     * 分组
     */
    @ApiModelProperty(value = "分组")
    private String grouping;
    /**
     * 单位
     */
    @ApiModelProperty(value = "单位")
    private String unit;
    /**
     * 是否应急资产设施1:是；0:否
     */
    @ApiModelProperty(value = "是否应急资产设施1:是；0:否")
    private Short isEmergency;
    /**
     * 折旧数值
     */
    @ApiModelProperty(value = "折旧数值")
    @NotNull(message = "折旧数值不能为空")
    private Double depreciation;
    /**
     * 折旧类型1:固定年折旧；2:固定月折旧；3:年折旧率；4:月折旧率
     */
    @ApiModelProperty(value = "折旧类型1:固定年折旧；2:固定月折旧；3:年折旧率；4:月折旧率")
    @NotNull(message = "折旧类型不能为空")
    @Min(value = 1,message = "折旧类型1:固定年折旧；2:固定月折旧；3:年折旧率；4:月折旧率")
    @Max(value = 4,message = "折旧类型1:固定年折旧；2:固定月折旧；3:年折旧率；4:月折旧率")
    private Short depreciationType;
    /**
     * 是否公用 0:否；1:是
     */
    @ApiModelProperty(value = "是否公用 0:否；1:是")
    private Short isPublic;
    /**
     * 是否有效1有效；0无效
     */
    @ApiModelProperty(value = "是否有效1有效；0无效")
    private Short isValid;
    /**
     * 是否删除1删除；0未删除
     */
    @ApiModelProperty(value = "是否删除1删除；0未删除")
    private Short isDel;
    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;
    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人")
    @TableField(value = "CREATE_USER", fill = FieldFill.INSERT)
    private Long createUser;
    @ApiModelProperty(hidden = true)
    private String createUserName;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;
    /**
     * 修改人
     */
    @ApiModelProperty(value = "修改人")
    @TableField(value = "UPDATE_USER", fill = FieldFill.INSERT_UPDATE)
    private Long updateUser;
    @ApiModelProperty(hidden = true)
    private String updateUserName;
    /**
     * 修改时间
     */
    @ApiModelProperty(value = "修改时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "UPDATE_TIME", fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;


    /**
     * 入库时间
     */
    @ApiModelProperty(value = "入库时间")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date inDate;

    public interface update{}

}
