package com.tofly.flow.custom.dto;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;

/**
 * @author HaiQiu
 * @date 2022/4/8
 */
@Data
@EqualsAndHashCode
public class ImportInfoDTO implements Serializable {

    /**
     * ID
     */
    @TableId(value = "ID", type = IdType.INPUT)
    @ApiModelProperty(value = "ID")
    @NotNull(groups = MatterDTO.update.class,message = "ID不能为空")
    private Long id;
    /**
     * 导入ID
     */
    @ApiModelProperty(value = "导入ID")
    @NotNull(message = "导入ID不能为空")
    @Min(value = 0,message = "导入ID不符合范围")
    private Long importId;

    /**
     * 物资模版ID
     */
    @ApiModelProperty(value = "物资模版ID")
    private Long matterId;
    /**
     * 物资编码
     */
    @ApiModelProperty(value = "物资编码")
    @NotEmpty(message = "物资编码不能为空")
    @Size(max = 20,message = "物资编码长度只能在1-20个字符")
    private String matterNumber;
    /**
     * 物资名称
     */
    @ApiModelProperty(value = "物资名称")
    @NotEmpty(message = "物资名称不能为空")
    @Size(max = 20,message = "物资名称长度只能在1-20个字符")
    private String matterName;
    /**
     * 唯一编号
     */
    @ApiModelProperty(value = "唯一编号")
    @NotEmpty(message = "唯一编号不能为空")
    @Size(max = 20,message = "唯一编号长度只能在1-20个字符")
    private String uniqueNumber;
    /**
     * 规格型号
     */
    @ApiModelProperty(value = "规格型号")
    @NotEmpty(message = "规格型号不能为空")
    @Size(max = 50,message = "规格型号长度只能在1-50个字符")
    private String standards;
    /**
     * 计量单位
     */
    @ApiModelProperty(value = "计量单位")
    @NotEmpty(message = "计量单位不能为空")
    @Size(max = 20,message = "计量单位长度只能在1-20个字符")
    private String unit;
    /**
     * 单价
     */
    @ApiModelProperty(value = "单价")
    @NotNull(message = "单价不能为空")
    @Min(value = 0,message = "单价不符合范围")
    private Double unitPrice;
    /**
     * 库房
     */
    @ApiModelProperty(value = "库房")
//    @NotEmpty(message = "库房不能为空")
    @Size(max = 20,message = "库房长度只能在1-20个字符")
    private String storageRoom;
    /**
     * 位置（货架位置）
     */
    @ApiModelProperty(value = "位置（货架位置）")
//    @NotEmpty(message = "位置不能为空")
    @Size(max = 100, message = "位置名称只能是1-100个字符")
    private String position;
    /**
     * 购买日期
     */
    @ApiModelProperty(value = "购买日期")
    //    @NotNull(message = "购买日期不能为空")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date emptionDate;
    /**
     * 到期日期
     */
    @ApiModelProperty(value = "到期日期")
//    @NotNull(message = "到期日期不能为空")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expireDate;
    /**
     * 数量
     */
    @ApiModelProperty(value = "数量")
    @NotNull(message = "数量不能为空")
    @Min(value = 0,message = "数量不符合范围")
    private Integer quantity;
    /**
     * 供应商
     */
    @ApiModelProperty(value = "供应商")
//    @NotEmpty(message = "供应商不能为空")
    @Size(max = 100, message = "供应商只能是1-100个字符")
    private String supplier;
    /**
     * 设备厂家
     */
    @ApiModelProperty(value = "设备厂家")
//    @NotEmpty(message = "设备厂家不能为空")
    @Size(max = 100, message = "设备厂家只能是1-100个字符")
    private String factoryId;
    /**
     * 生产日期
     */
    @ApiModelProperty(value = "生产日期")
//    @NotNull(message = "生产日期不能为空")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date produceDate;
    /**
     * 出库日期
     */
    @ApiModelProperty(value = "出库日期")
//    @NotNull(message = "出库日期不能为空")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date outDate;
    /**
     * 物资管理部门
     */
    @ApiModelProperty(value = "物资管理部门")
    @NotEmpty(message = "物资管理部门不能为空")
    @Size(max = 20,message = "物资管理部门长度只能在1-20个字符")
    private String dept;
    /**
     * 是否删除1删除；0未删除
     */
    @ApiModelProperty(value = "是否删除1删除；0未删除",hidden = true)
    private Short isDel;
    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
//    @NotEmpty(message = "备注不能为空")
    @Size(max = 100,message = "备注长度只能在1-100个字符")
    private String remark;
    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人",hidden = true)
    @TableField(value = "CREATE_USER", fill = FieldFill.INSERT)
    private Long createUser;
    @ApiModelProperty(hidden = true)
    private String createUserName;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间",hidden = true)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;


    /**
     * 修改人
     */
    @ApiModelProperty(value = "修改人",hidden = true)
    @TableField(value = "UPDATE_USER", fill = FieldFill.INSERT_UPDATE)
    private Long updateUser;
    @ApiModelProperty(hidden = true)
    private String updateUserName;
    /**
     * 修改时间
     */
    @ApiModelProperty(value = "修改时间",hidden = true)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "UPDATE_TIME", fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    public @interface add{}
    public @interface update{}
}
