/*
 * Author: wangjian
 * 生成entity
 */
package com.tofly.flow.custom.dto;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;


/**
 * 仓储仪器设备
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@Data
@ApiModel
public class KeepGrainDeviceDTO implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * 是否有效1有效；0无效
     */
    @ApiModelProperty(value = "是否有效1有效；0无效",hidden = true)
    private Short isValid;
    /**
     * 是否删除1删除；0未删除
     */
    @ApiModelProperty(value = "是否删除1删除；0未删除",hidden = true)
    private Short isDel;
    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
//    @NotEmpty(message = "备注不能为空")
    @Size(max = 100,message = "备注长度只能在1-100个字符")
    private String remark;

    /**
     * 编号（出入库编码）
     */
    @ApiModelProperty(value = "编号（出入库编码）")
    private String stuffNumber;

    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人",hidden = true)
    @TableField(value = "CREATE_USER", fill = FieldFill.INSERT)
    private Long createUser;
    @ApiModelProperty(hidden = true)
    private String createUserName;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间",hidden = true)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;


    /**
     * 修改人
     */
    @ApiModelProperty(value = "修改人",hidden = true)
    @TableField(value = "UPDATE_USER", fill = FieldFill.INSERT_UPDATE)
    private Long updateUser;
    @ApiModelProperty(hidden = true)
    private String updateUserName;
    /**
     * 修改时间
     */
    @ApiModelProperty(value = "修改时间",hidden = true)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "UPDATE_TIME", fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    public @interface add{}
    public @interface update{}
    /**
     * ID
     */
    @TableId(value = "ID", type = IdType.INPUT)
    @ApiModelProperty(value = "ID")
    @NotNull(groups = MatterDTO.update.class,message = "ID不能为空")
    private Long id;
    /**
     * 物资ID
     */
    @ApiModelProperty(value = "物资ID")
    @NotNull(message = "物资ID不能为空")
    private Long keepGrainMatterId;
    /**
     * 设备唯一编号（系统生成）
     */
    @ApiModelProperty(value = "设备唯一编号（系统生成）",hidden = true)
    private String uniqueId;

    /**
     * 出厂唯一编号
     */
    @ApiModelProperty(value = "出厂唯一编号")
    @NotEmpty(message = "出厂唯一编号不能为空")
    @Size(max = 100, message = "出厂唯一编号长度只能在1-100之间")
    private String uniqueNumber;

    /**
     * 部件唯一编号1
     */
    @ApiModelProperty(value = "部件唯一编号1")
    @NotEmpty(message = "部件唯一编号1不能为空")
    @Size(max = 100, message = "部件唯一编号1长度只能在1-100之间")
    private String partsNumberOne;
    /**
     * 部唯一件编号2
     */
    @ApiModelProperty(value = "部唯一件编号2")
    @NotEmpty(message = "部件唯一编号2不能为空")
    @Size(max = 100, message = "部件唯一编号2长度只能在1-100之间")
    private String partsNumberTwo;
    /**
     * 部唯一件编号3
     */
    @ApiModelProperty(value = "部唯一件编号3")
    @NotEmpty(message = "部件唯一编号3不能为空")
    @Size(max = 100, message = "部件唯一编号3长度只能在1-100之间")
    private String partsNumberThree;
    /**
     * 铭牌信息
     */
    @ApiModelProperty(value = "铭牌信息")
    @NotEmpty(message = "铭牌信息不能为空")
    @Size(max = 100, message = "铭牌信息长度只能在1-100之间")
    private String nameplate;
    /**
     * 供应商
     */
    @ApiModelProperty(value = "供应商")
    @NotEmpty(message = "供应商不能为空")
    private String supplier;
    /**
     * 设备厂家
     */
    @ApiModelProperty(value = "设备厂家")
    @NotEmpty(message = "设备厂家不能为空")
    private String factoryId;
    /**
     * 价格
     */
    @ApiModelProperty(value = "价格")
    @NotNull(message = "价格不能为空")
    private Double price;
    /**
     * 生产日期
     */
    @ApiModelProperty(value = "生产日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @NotNull(message = "生产日期不能为空")
    private Date produceDate;
    /**
     * 购买日期
     */
    @ApiModelProperty(value = "购买日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @NotNull(message = "购买日期不能为空")
    private Date emptionDate;
    /**
     * 出库日期
     */
    @ApiModelProperty(value = "出库日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @NotNull(message = "出库日期不能为空")
    private Date outDate;
    /**
     * 到期日期
     */
    @ApiModelProperty(value = "到期日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @NotNull(message = "到期日期不能为空")
    private Date expireDate;
    /**
     * 采购工单ID
     */
    @ApiModelProperty(value = "采购工单ID")
    @NotNull(message = "采购工单ID不能为空")
    private Long purchaseId;
    /**
     * 出库工单ID
     */
    @ApiModelProperty(value = "出库工单ID")
    @NotNull(message = "出库工单ID不能为空")
    private Long streamId;
    /**
     * 折旧数值
     */
    @ApiModelProperty(value = "折旧数值")
    @NotNull(message = "折旧数值不能为空")
    private Double depreciation;
    /**
     * 折旧类型1:固定年折旧；2:固定月折旧；3:年折旧率；4:月折旧率
     */
    @ApiModelProperty(value = "折旧类型1:固定年折旧；2:固定月折旧；3:年折旧率；4:月折旧率")
    @NotNull(message = "折折旧类型不能为空")
    private Short depreciationType;
    /**
     * 数据来源:1导入；2入库
     */
    @ApiModelProperty(value = "数据来源:1导入；2入库")
    @NotNull(message = "数据来源不能为空")
    private Short dataSource;
    /**
     * 设备状态1:新；2:旧；3:损坏；4:出库；5:报废
     */
    @ApiModelProperty(value = "设备状态1:新；2:旧；3:损坏；4:出库；5:报废")
    @NotNull(message = "设备状态不能为空")
    private Short deviceState;
    /**
     * 物资类别类型1设备类;2车辆类;3耗材类
     */
    @ApiModelProperty(value = "物资类别类型1设备类;2车辆类;3耗材类")
    @NotNull(message = "物资类别类型不能为空")
    private Short categoryType;

}
