package com.tofly.flow.custom.dto;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.*;
import java.io.Serializable;
import java.util.Date;

/**
 * @author HaiQiu
 * @date 2022/4/8
 */
@Data
@ApiModel
public class KeepGrainMatterDTO implements Serializable {

    /**
     * ID
     */
    @TableId(value = "ID", type = IdType.INPUT)
    @ApiModelProperty(value = "ID")
    @NotNull(groups = MatterDTO.update.class,message = "ID不能为空")
    private Long id;
    /**
     * 仓储编码
     */
    @ApiModelProperty(value = "仓储编码")
    @NotEmpty(message = "仓储编码不能为空")
    @Size(max = 50,message = "仓储编码长度只能在1-20个字符")
    private String keepGrainNumber;
    /**
     * 物料ID
     */
    @ApiModelProperty(value = "物料ID")
    @NotNull(message = "物料ID不能为空")
    @Min(value = 0,message = "物料ID不符合范围")
    private Long matterId;
    /**
     * 入库数量
     */
    @ApiModelProperty(value = "入库数量")
    @NotNull(message = "入库数量不能为空")
    @Min(value = 0,message = "入库数量不符合范围")
    private Long inQuantity;
    /**
     * 出库数量
     */
    @ApiModelProperty(value = "出库数量")
    @NotNull(message = "出库数量不能为空")
    @Min(value = 0,message = "出库数量不符合范围")
    private Long outQuantity;
    /**
     * 现存数量
     */
    @ApiModelProperty(value = "现存数量")
    @NotNull(message = "现存数量不能为空")
    @Min(value = 0,message = "现存数量不符合范围")
    private Long quantity;
    /**
     * 采购总价
     */
    @ApiModelProperty(value = "采购总价")
    @NotNull(message = "采购总价不能为空")
    @Min(value = 0,message = "采购总价不符合范围")
    private Double purchaseTotal;
    /**
     * 最新单价
     */
    @ApiModelProperty(value = "最新单价")
    @NotNull(message = "最新单价不能为空")
    @Min(value = 0,message = "最新单价不符合范围")
    private Double unitPrice;
    /**
     * 库房
     */
    @ApiModelProperty(value = "库房")
    @NotEmpty(message = "库房不能为空")
    @Size(max = 50, message = "库房名称只能是1-50个字符")
    private String storageRoom;
    /**
     * 位置（货架位置）
     */
    @ApiModelProperty(value = "位置（货架位置）")
    @NotEmpty(message = "位置不能为空")
    @Size(max = 100, message = "位置名称只能是1-100个字符")
    private String position;
    /**
     * 物料状态1:新；2:旧
     */
    @ApiModelProperty(value = "物料状态1:新；2:旧")
    @NotNull(message = "物料状态不能为空")
    @Min(value = 1,message = "物料状态不符合范围，只能是物料状态1:新；2:旧")
    @Max(value = 2,message = "物料状态不符合范围，只能是物料状态1:新；2:旧")
    private Short matterState;
    /**
     * 是否应急物资1:是；0:否
     */
    @ApiModelProperty(value = "是否应急物资1:是；0:否")
    @NotNull(message = "是否应急物资不能为空")
    @Min(value = 0,message = "是否应急物资不符合范围，只能是1:是；0:否")
    @Max(value = 1,message = "是否应急物资不符合范围，只能是1:是；0:否")
    private Short isEmergency;
    /**
     * 最新入库时间
     */
    @ApiModelProperty(value = "最新入库时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date newTime;
    /**
     * 最近到期时间
     */
    @ApiModelProperty(value = "最近到期时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date recentlyExpired;
    /**
     * 是否有效1有效；0无效
     */
    @ApiModelProperty(value = "是否有效1有效；0无效",hidden = true)
    private Short isValid;
    /**
     * 是否删除1删除；0未删除
     */
    @ApiModelProperty(value = "是否删除1删除；0未删除",hidden = true)
    private Short isDel;
    /**
     * 备注
     */
    @ApiModelProperty(value = "备注",hidden = true)
    private String remark;
    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人")
    @TableField(value = "CREATE_USER", fill = FieldFill.INSERT)
    private Long createUser;
    @ApiModelProperty(hidden = true)
    private String createUserName;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;
    /**
     * 修改人
     */
    @ApiModelProperty(value = "修改人")
    @TableField(value = "UPDATE_USER", fill = FieldFill.INSERT_UPDATE)
    private Long updateUser;
    @ApiModelProperty(hidden = true)
    private String updateUserName;
    /**
     * 修改时间
     */
    @ApiModelProperty(value = "修改时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "UPDATE_TIME", fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;


    public @interface add{}
    public @interface update{}
}
