package com.tofly.flow.custom.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.io.Serializable;
import java.util.Date;

@Data
@EqualsAndHashCode
public class LoadProcessDto implements Serializable {

    /**
     * 关联编码
     */
    @ApiModelProperty(value = "关联编码")
    private String relationNumber;

    /**
     * 流程类别
     */
    @ApiModelProperty(value = "流程类别")
    private Long category;
    /**
     * 流程类型类型表ID
     */
    @ApiModelProperty(value = "流程类型类型表ID")
    private Long processType;
    /**
     * 流程配置ID
     */
    @ApiModelProperty(value = "流程配置ID")
    private Long configId;
    /**
     * 业务类型，基础类型表
     */
    @ApiModelProperty(value = "业务类型，基础类型表")
    private Long ywType;
    /**
     * 版本
     */
    @ApiModelProperty(value = "版本")
    private Integer edition;

    /**
     * 开始时间
     */
    @ApiModelProperty(value = "开始时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date startTime;
    /**
     * 结束时间
     */
    @ApiModelProperty(value = "结束时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date endTime;
    /**
     * 使用部门（申购部门）
     */
    @ApiModelProperty(value = "使用部门（申购部门）")
    private Long useDept;

    /**
     * 负责人(申请人)
     */
    @ApiModelProperty(value = "负责人(申请人)")
    private Long personLiable;
    /**
     * 供应商ID
     */
    @ApiModelProperty(value = "供应商ID")
    private Long supplierId;
    /**
     * 供应商名称（来文单位）
     */
    @ApiModelProperty(value = "供应商名称（来文单位）")
    private String supplier;
    /**
     * 联系(申请)人名称
     */
    @ApiModelProperty(value = "联系(申请)人名称")
    private String contacts;

    /**
     * 申请单位（人）
     */
    @ApiModelProperty(value = "申请单位（人）")
    private String applicant;

    /**
     * 工单状态：1草稿、2处理、3终止、4处理完成
     */
    @Min(1)
    @Max(4)
    @ApiModelProperty(value = "工单状态：1草稿、2处理、3终止、4处理完成")
    private Short processState;


    /**
     * 登记部门
     */
    @ApiModelProperty(value = "登记部门")
    private Long registerDept;
    /**
     * 登记人
     */
    @ApiModelProperty(value = "登记人")
    private Long registerStaff;

    /**
     * 当前流程节点 100表示终结归档
     */
    @ApiModelProperty(value = "当前流程节点 100表示终结归档")
    private Integer processNode;

    /**
     * 当前处理部门
     */
    @ApiModelProperty(value = "当前处理部门")
    private Long handleDept;
    /**
     * 当前处理人
     */
    @ApiModelProperty(value = "当前处理人")
    private Long handleStaff;
    /**
     * 搜索内容
     */
    @ApiModelProperty(value = "搜索内容")
    private String content;

    /**
     * 督办人部门ID
     */
    @ApiModelProperty(value = "督办人部门ID,与督办人id同时填充，单独无效")
    private Long supervisorDept;

    /**
     * 督办人ID
     */
    @ApiModelProperty(value = "督办人ID，与督办人部门同时填充，单独无效")
    private Long supervisor;

    /**
     * 我的工单:true 是查询我的工单
     */
    @ApiModelProperty(value = "我的工单:true 是查询我的工单")
    private Boolean iWorkOrder;

}
