package com.tofly.flow.custom.dto;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.flow.custom.entity.Matter;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.*;
import java.io.Serializable;
import java.util.Date;

/**
 * @author HaiQiu
 * @date 2022-04-06
 */
@Data
@ToString
@EqualsAndHashCode
@ApiModel
public class MatterDTO implements Serializable {

    /**
     * ID
     */
    @ApiModelProperty(value = "ID")
    @NotNull(groups = MatterDTO.update.class,message = "ID不能为空")
    private Long id;
    /**
     * 物资编码
     */
    @ApiModelProperty(value = "物资编码")
//    @NotEmpty(message = "物资编码不能为空")
    @Size(max = 50,message = "物资编码长度只能在1-20个字符")
    private String matterNumber;
//    /**
//     * 类别TF_YWPN_BASE_TYPE_W DATA_TYPE
//     */
//    @ApiModelProperty(value = "类别TF_YWPN_BASE_TYPE_W DATA_TYPE")
//    @NotNull(message = "物资类别不能为空")
//    private Long category;
    /**
     * 物资所属类型TF_YWPN_BASE_TYPE_W id
     */
    @ApiModelProperty(value = "物资所属类型TF_YWPN_BASE_TYPE_W id")
    @NotNull(message = "物资所属类型不能为空")
    private Long matterType;
    /**
     * 物资名称
     */
    @ApiModelProperty(value = "物资名称")
    @NotEmpty(message = "物资名称不能为空")
    @Size(max = 25,message = "物资名称长度只能在1-25个字符")
    private String matterName;
//    /**
//     * 别名（简称）
//     */
//    @ApiModelProperty(value = "别名（简称）")
//    @NotEmpty(message = "别名（简称）不能为空")
//    @Size(max = 15,message = "别名（简称）长度只能在1-15个字符")
//    private String alias;
//    /**
//     * 英文名称
//     */
//    @ApiModelProperty(value = "英文名称")
//    @NotEmpty(message = "英文名称不能为空")
//    @Size(max = 25,message = "英文名称长度只能在1-25个字符")
//    private String englishName;
    /**
     * 规格型号
     */
    @ApiModelProperty(value = "规格型号")
//    @NotEmpty(message = "规格型号不能为空")
    @Size(max = 25,message = "规格型号长度只能在1-25个字符")
    private String standards;
    /**
     * 计量单位
     */
    @ApiModelProperty(value = "计量单位")
//    @NotEmpty(message = "计量单位不能为空")
    @Size(max = 25,message = "计量单位长度只能在1-25个字符")
    private String unit;
//    /**
//     * 附件id用逗号分隔
//     */
//    @ApiModelProperty(value = "附件id用逗号分隔")
//    @Size(max = 2000,message = "附件id长度只能在1-2000个字符")
//    private String annex;

    /**
     * 物资类别类型1设备类;2车辆类;3耗材类
     */
    @ApiModelProperty(value = "物资类别类型1设备类;2车辆类;3耗材类")
    @NotNull(message = "物资类别类型不能为空")
    @Min(value = 1,message = "物资类型只能是1设备类;2车辆类;3耗材类")
    @Max(value = 3,message = "物资类型只能是1设备类;2车辆类;3耗材类")
    private Short categoryType;

    /**
     * 是否资产1:是;0:否
     */
    @ApiModelProperty(value = "是否资产1:是;0:否")
    @NotNull(message = "是否资产不能为空")
    @Min(value = 0,message = "是否资产1:是;0:否")
    @Max(value = 1,message = "是否资产1:是;0:否")
    private Short isAssets;

//    /**
//     * 是否有效1有效；0无效
//     */
//    @ApiModelProperty(value = "是否有效1有效；0无效")
//    @NotNull(message = "是否有效不能为空")
//    @Min(value = 0,message = "是否有效1有效；0无效")
//    @Max(value = 1,message = "是否有效1有效；0无效")
//    private Short isValid;

//    /**
//     * 是否删除1删除；0未删除
//     */
//    @ApiModelProperty(value = "是否删除1删除；0未删除")
//    @Min(value = 0,message = "是否删除1删除；0未删除")
//    @Max(value = 1,message = "是否删除1删除；0未删除")
//    private Short isDel;
//    /**
//     * 备注
//     */
//    @ApiModelProperty(value = "备注")
//    @Size(max = 100,message = "备注只能100个字符之内")
//    private String remark;

    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人",hidden = true)
    @TableField(value = "CREATE_USER", fill = FieldFill.INSERT)
    private Long createUser;
    @ApiModelProperty(hidden = true)
    private String createUserName;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间",hidden = true)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;
    /**
     * 修改人
     */
    @ApiModelProperty(value = "修改人",hidden = true)
    @TableField(value = "UPDATE_USER", fill = FieldFill.INSERT_UPDATE)
    private Long updateUser;
    @ApiModelProperty(hidden = true)
    private String updateUserName;
    /**
     * 修改时间
     */
    @ApiModelProperty(value = "修改时间",hidden = true)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "UPDATE_TIME", fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    public @interface add{}
    public @interface update{}

}
