package com.tofly.flow.custom.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.flow.common.FlowConstant;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

@Data
@EqualsAndHashCode
public class ProcessDeviceDto implements Serializable {

    /**
     * 历史数据ID
     */
    @ApiModelProperty(value = "历史数据ID")
    private Long id;

    /**
     * 仓库设备id，出库、退库必填
     */
    @ApiModelProperty(value = "设备唯一编号出库、退库必填")
    private Long stockId;
    /**
     * 出厂唯一编号
     */
    @ApiModelProperty(value = "出厂唯一编号")
    private String uniqueNumber;

    /**
     * 编号1
     */
    @ApiModelProperty(value = "编号1")
    private String partsNumberOne;
    /**
     * 编号2
     */
    @ApiModelProperty(value = "编号2")
    private String partsNumberTwo;
    /**
     * 编号3
     */
    @ApiModelProperty(value = "编号3")
    private String partsNumberThree;
    /**
     * 铭牌信息
     */
    @ApiModelProperty(value = "铭牌信息")
    private String nameplate;

    /**
     * 设备厂家ID
     */
    @ApiModelProperty(value = "设备厂家ID")
    private Long factoryId;
    /**
     * 价格
     */
    @ApiModelProperty(value = "价格")
    private Double price;
    /**
     * 附件id用逗号分隔
     */
    @ApiModelProperty(value = "附件id用逗号分隔")
    private String annex;
    /**
     * 生产日期
     */
    @ApiModelProperty(value = "生产日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date produceDate;
    /**
     * 购买日期
     */
    @ApiModelProperty(value = "购买日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date emptionDate;
    /**
     * 设备状态1:新；2:旧；3:损坏；4:出库；5:报废
     */
    @NotNull(message = FlowConstant.ISNOTNUL)
    @Min(1)
    @Max(5)
    @ApiModelProperty(value = "设备状态1:新；2:旧；3:损坏；4:出库；5:报废")
    private Short deviceState;

    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 文件名称；一次提交不允许重名
     */
    @ApiModelProperty(value = "文件名称；一次提交不允许重名")
    private List<String> fileNames;
}
