package com.tofly.flow.custom.dto;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.flow.common.FlowConstant;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;

@Data
@EqualsAndHashCode
public class ProcessDto implements Serializable {
    /**
     * ID
     */
    @ApiModelProperty(value = "ID")
    private Long id;
    /**
     * 关联编码ID入库需要关联采购申请等
     */
    @ApiModelProperty(value = "关联编码ID，")
    private Long relationId;
    /**
     * 流程配置ID
     */
    @NotNull(message = "configId"+FlowConstant.ISNOTNUL)
    @ApiModelProperty(value = "流程配置ID")
    private Long configId;
    /**
     * 业务类型，基础类型表
     */
    @ApiModelProperty(value = "业务类型，流程本身类型，非流程配置类型")
    private Long ywType;

    /**
     * 名称
     */
    @ApiModelProperty(value = "名称")
    private String name;
    /**
     * 开始时间
     */
    @ApiModelProperty(value = "开始时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date startTime;
    /**
     * 结束时间
     */
    @ApiModelProperty(value = "结束时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date endTime;
    /**
     * 使用部门（申购部门）
     */
    @ApiModelProperty(value = "使用部门（申购部门）")
    private Long useDept;
    /**
     * 所属部门（人员所属部门）
     */
    @ApiModelProperty(value = "所属部门（人员所属部门）")
    private Long belongDept;
    /**
     * 负责人(申请人)
     */
    @ApiModelProperty(value = "负责人(申请人)")
    private Long personLiable;
    /**
     * 供应商ID
     */
    @ApiModelProperty(value = "供应商ID")
    private Long supplierId;
    /**
     * 供应商名称（来文单位）
     */
    @ApiModelProperty(value = "供应商名称（来文单位）")
    private String supplier;
    /**
     * 联系(申请)人名称
     */
    @ApiModelProperty(value = "联系(申请)人名称")
    private String contacts;
    /**
     * 联系人电话
     */
    @ApiModelProperty(value = "联系人电话")
    private String telephone;
    /**
     * 申请单位（人）
     */
    @ApiModelProperty(value = "申请单位（人）")
    private String applicant;
    /**
     * 证件类型
     */
    @ApiModelProperty(value = "证件类型")
    private Long documentType;
    /**
     * 证件号
     */
    @ApiModelProperty(value = "证件号")
    private String idNumber;

    /**
     * 收货（出货）时间
     */
    @ApiModelProperty(value = "收货（出货）时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date handoverTime;
    /**
     * 收货（出货）人员
     */
    @ApiModelProperty(value = "收货（出货）人员")
    private Long handoverUser;
    /**
     * 总价
     */
    @ApiModelProperty(value = "总价")
    private Double totalPrice;

    /**
     * 表单类型：0、其它； 1、采购入库（关联采购工单）；2、余料退库（关联出库工单）；3、工程出库（关联工程工单）；4其它出库；5、维修出库（关联维修工单）
     */
    @NotNull(message = "operateState"+FlowConstant.ISNOTNUL)
    @ApiModelProperty(value = "表单类型：0、其它； 1、采购入库（关联采购工单）；2、余料退库（关联出库工单）；3、工程出库（关联工程工单）；4其它出库；5、维修出库（关联维修工单）")
    private Short operateState;
    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;
    /**
     * 登记时间
     */
    @ApiModelProperty(value = "登记时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date registerTime;

    /**
     * 第一步处理信息
     */
    @Valid
    @NotNull
    @ApiModelProperty(value = "第一步处理信息")
    private ProcessNodeDto processNode;

}
