package com.tofly.flow.custom.dto;

import com.tofly.flow.common.FlowConstant;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.List;

@Data
@EqualsAndHashCode
public class ProcessNodeDto implements Serializable {

    /**
     * 流程步骤
     */
    @NotNull(message = "processStep"+FlowConstant.ISNOTNUL)
    @Min(value = 1,message = "processStep" + FlowConstant.NOT_LESS_THAN+1)
    @ApiModelProperty(value = "流程步骤")
    private Integer processStep;
    /**
     * 步骤节点
     */
    @NotNull(message = "stepNode"+FlowConstant.ISNOTNUL)
    @Min(value = 1,message = "stepNode" + FlowConstant.NOT_LESS_THAN+1)
    @ApiModelProperty(value = "步骤节点")
    private Integer stepNode;
    /**
     * 部门
     */
    @ApiModelProperty(value = "部门")
    private Long dept;
    /**
     * 人员
     */
    @ApiModelProperty(value = "人员")
    private Long deptStaff;
    /**
     * 类型1（关联基础类型表ID）
     */
    @ApiModelProperty(value = "类型1（关联基础类型表ID）")
    private Long typeOne;
    /**
     * 类型2（关联基础类型表ID）
     */
    @ApiModelProperty(value = "类型2（关联基础类型表ID）")
    private Long typeTwo;
    /**
     * 条件（判断选择条件多个用逗号分隔）
     */
    @ApiModelProperty(value = "条件（判断选择条件多个用逗号分隔）")
    private String condition;
    /**
     * 处理意见
     */
    @ApiModelProperty(value = "处理意见")
    private String opinionOne;
    /**
     * 处理意见
     */
    @ApiModelProperty(value = "处理意见")
    private String opinionTwo;
    /**
     * 处理结果0:不处理；1:同意；2:不同意；
     */
    @NotNull(message = "handleResults"+FlowConstant.ISNOTNUL)
    @Min(value = 0,message = "handleResults"+FlowConstant.NOT_LESS_THAN+0)
    @Max(value = 2,message = "handleResults" +FlowConstant.NOT_GREATER_THAN+2)
    @ApiModelProperty(value = "处理结果0:不处理；1:同意；2:不同意；")
    private Short handleResults;
    /**
     * 经度
     */
    @ApiModelProperty(value = "经度")
    private Double longitude;
    /**
     * 纬度
     */
    @ApiModelProperty(value = "纬度")
    private Double latitude;
    /**
     * 地址
     */
    @ApiModelProperty(value = "地址")
    private String addre;
    /**
     * 附件id用逗号分隔
     */
    @ApiModelProperty(value = "附件id用逗号分隔")
    private String annex;

    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 上游处理ID(相关联重复提交如：工程日志、工程日志审核)必填
     */
    @ApiModelProperty(value = "上游处理ID(相关联重复提交如：工程日志、工程日志审核)必填")
    private Long upHandleId;


    /**
     * 处理状态 1提交;2:驳回;3:转派;4:终止;
     */
    @NotNull(message = "headleState"+FlowConstant.ISNOTNUL)
    @Min(value = 1,message = "headleState" + FlowConstant.NOT_LESS_THAN+1 )
    @Max(value = 4,message = "headleState" + FlowConstant.NOT_GREATER_THAN+4)
    @ApiModelProperty(value = "处理状态：1提交;2:驳回;3:转派;4:终止;")
    private Integer headleState;



    /**
     * 保存状态 0保存，1提交
     */
    @NotNull(message = "saveState"+FlowConstant.ISNOTNUL)
    @Min(value = 0,message = "saveState" + FlowConstant.NOT_LESS_THAN+0)
    @Max(value = 1,message = "saveState" + FlowConstant.NOT_GREATER_THAN+1)
    @ApiModelProperty(value = "0保存，1提交")
    private Integer saveState;


    /**
     * 下一步信息
     */
    @Valid
    @ApiModelProperty(value = "下一步信息,下一步，驳回，转派")
    private NextStepDto nextStep;

    /**
     * 合同信息信息
     */
    @Valid
    @ApiModelProperty(value = "合同信息信息")
    private List<ProjectContractAllDto> contractDto;


    /**
     * 出入库物资信息
     */
    @Valid
    @ApiModelProperty(value = "出入库物资信息")
    private List<ProcessStuffDto>  stuffDtos;

    /**
     * 费用信息
     */
    @Valid
    @ApiModelProperty(value = "费用信息")
    private List<ProjectDetailedDto> detailedDtos;

    /**
     * 设施关联信息
     */
    @Valid
    @ApiModelProperty(value = "设施关联信息")
    private List<ProcessRelationDto> prdtos;


    /**
     * 关联人员
     */
    @Valid
    @ApiModelProperty(value = "关联人员")
    private List<ProcessStaffDto> staffDtos;

    /**
     * 文档、公文
     */
    @Valid
    @ApiModelProperty(value = "文档、公文")
    private DocumentFileDto documentDto;

    /**
     * 工程勘测信息
     */
    @Valid
    @ApiModelProperty(value = "工程勘测信息")
    private ProcessSurveyDto processSurveyDto;

    /**
     * 工程信息
     */
    @Valid
    @ApiModelProperty(value = "工程信息")
    private ProcessJournalDto processJournalDto;



}
