package com.tofly.flow.custom.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.flow.common.FlowConstant;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 出入库物资信息
 */
@Data
@EqualsAndHashCode
public class ProcessStuffDto implements Serializable {

    /**
     * 历史数据ID
     */
    @ApiModelProperty(value = "历史数据ID")
    private Long id;

    /**
     * 物料ID
     */
    @NotNull(message = "matterId"+FlowConstant.ISNOTNUL)
    @ApiModelProperty(value = "物料ID")
    private Long matterId;
    /**
     * 设备厂家ID
     */
    @ApiModelProperty(value = "设备厂家ID")
    private Long factoryId;

    /**
     * 数量
     */
    @NotNull(message = "quantity"+FlowConstant.ISNOTNUL)
    @Min(0)
    @ApiModelProperty(value = "数量")
    private Long quantity;
    /**
     * 单价
     */
    @ApiModelProperty(value = "单价")
    private Double unitPrice;
    /**
     * 金额
     */
    @ApiModelProperty(value = "金额")
    private Double amountMoney;
    /**
     * 库房
     */
    @ApiModelProperty(value = "库房")
    private String storageRoom;
    /**
     * 到期日期
     */
    @ApiModelProperty(value = "到期日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expireDate;

    /**
     * 位置（货架位置）
     */
    @ApiModelProperty(value = "位置（货架位置）")
    private String position;

    /**
     * 安装位置、使用位置
     */
    @ApiModelProperty(value = "位置（安装位置、使用位置）")
    private String installLocation;
    /**
     * 附件id用逗号分隔
     */
    @ApiModelProperty(value = "附件id用逗号分隔")
    private String annex;
    /**
     * 物料状态1:新；2:旧
     */
    @NotNull(message = "matterState"+FlowConstant.ISNOTNUL)
    @Max(value = 2,message = "matterState"+FlowConstant.NOT_GREATER_THAN+2)
    @Min(value = 1,message = "matterState"+FlowConstant.NOT_LESS_THAN+1)
    @ApiModelProperty(value = "物料状态1:新；2:旧")
    private Short matterState;
    /**
     * 旧料是否退库1:退库，0不退库
     */
    @ApiModelProperty(value = "旧料是否退库1:退库，0不退库")
    private Short oldReturn;

    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 购买日期
     */
    @ApiModelProperty(value = "购买日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date emptionDate;

    /**
     * 入库日期
     */
    @ApiModelProperty(value = "入库日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date inDate;

    /**
     * 设备信息
     */
    @ApiModelProperty(value = "设备信息")
    private List<ProcessDeviceDto> devices;

    /**
     * 文件名称；一次提交不允许重名
     */
    @ApiModelProperty(value = "文件名称；一次提交不允许重名")
    private List<String> fileNames;
}


