/*
 * Author: wangjian
 * 生成entity
 */
package com.tofly.flow.custom.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.models.auth.In;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;


/**
 * 流程节点处理信息（操作步骤都有记录）
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "TF_YWPN_PROCESS_NODE_W", resultMap = "processNodeMap")
@KeySequence("TF_YWPN_PROCESS_NODE_W")
@ApiModel
public class ProcessNode extends Model<ProcessNode> {
    private static final long serialVersionUID = 1L;

    /**
     * ID
     */
    @TableId(value = "ID", type = IdType.INPUT)
    @ApiModelProperty(value = "ID")
    private Long id;
    /**
     * 流程ID
     */
    @ApiModelProperty(value = "流程ID")
    private Long processId;
    /**
     * 流程步骤
     */
    @ApiModelProperty(value = "流程步骤")
    private Integer processStep;
    /**
     * 步骤节点
     */
    @ApiModelProperty(value = "步骤节点")
    private Integer stepNode;
    /**
     * 部门
     */
    @ApiModelProperty(value = "部门")
    private Long dept;
    /**
     * 人员
     */
    @ApiModelProperty(value = "人员")
    private Long deptStaff;
    /**
     * 类型1（关联基础类型表ID）
     */
    @ApiModelProperty(value = "类型1（关联基础类型表ID）")
    private Long typeOne;
    /**
     * 类型2（关联基础类型表ID）
     */
    @ApiModelProperty(value = "类型2（关联基础类型表ID）")
    private Long typeTwo;
    /**
     * 条件（判断选择条件多个用逗号分隔）
     */
    @ApiModelProperty(value = "条件（判断选择条件多个用逗号分隔）")
    private String condition;
    /**
     * 处理意见
     */
    @ApiModelProperty(value = "处理意见")
    private String opinionOne;
    /**
     * 处理意见
     */
    @ApiModelProperty(value = "处理意见")
    private String opinionTwo;
    /**
     * 处理结果0:不处理；1:同意；2:不同意；
     */
    @ApiModelProperty(value = "处理结果0:不处理；1:同意；2:不同意；")
    private Short handleResults;
    /**
     * 经度
     */
    @ApiModelProperty(value = "经度")
    private Double longitude;
    /**
     * 纬度
     */
    @ApiModelProperty(value = "纬度")
    private Double latitude;
    /**
     * 地址
     */
    @ApiModelProperty(value = "地址")
    private String addre;
    /**
     * 附件id用逗号分隔
     */
    @ApiModelProperty(value = "附件id用逗号分隔")
    private String annex;
    /**
     * 上步骤ID
     */
    @ApiModelProperty(value = "上步骤ID")
    private Long upStep;
    /**
     * 上步骤节点ID
     */
    @ApiModelProperty(value = "上节点ID")
    private Long upStepNode;

    /**
     * 上游处理ID(前一步流程无论驳回、转派)
     */
    @ApiModelProperty(value = "上游处理ID(前一步流程无论驳回、转派)")
    private Long upHandleId;
    /**
     * 完成时限
     */
    @ApiModelProperty(value = "完成时限")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date timeLimit;

    /**
     * 当前处理部门
     */
    @ApiModelProperty(value = "当前处理部门")
    private Long handleDept;

    /**
     * 当前处理人
     */
    @ApiModelProperty(value = "当前处理人")
    private Long handleStaff;


    /**
     * 时间（处理时间等）
     */
    @ApiModelProperty(value = "时间（处理时间等）")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date handleTime;
    /**
     * 是否有效1有效；0无效(转派、驳回、被驳回为无效)
     */
    @ApiModelProperty(value = "是否有效1有效；0无效(转派、驳回、被驳回为无效)")
    private Short isValid;
    /**
     * 是否超时
     */
    @ApiModelProperty(value = "是否超时")
    private Short isOvertime;
    /**
     * 是否删除1删除；0未删除
     */
    @ApiModelProperty(value = "是否删除1删除；0未删除")
    private Short isDel;
    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 节点状态 0:待处理;1处理中;2提交;3:驳回;4:转派;5:终止;6:被驳回;
     */
    @ApiModelProperty(value = "节点状态0:待处理;1处理中;2提交;3:驳回;4:转派;5:终止;6:被驳回;")
    private Integer nodeState;

    /**
     * 处理状态 处理状态 1提交;2:驳回;3:转派;4:终止;
     */
    @ApiModelProperty(value = "处理状态 处理状态 1提交;2:驳回;3:转派;4:终止;")
    private Integer handleState;
    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人")
    @TableField(value = "CREATE_USER", fill = FieldFill.INSERT)
    private Long createUser;
    @ApiModelProperty(hidden = true)
    private String createUserName;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;
}
