/*
 * Author: wangjian
 * 生成entity
 */
package com.tofly.flow.custom.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;


/**
 * 流程中物料出入/退库信息
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "TF_YWPN_PROCESS_STUFF_W", resultMap = "processStuffMap")
@KeySequence("TF_YWPN_PROCESS_STUFF_W")
@ApiModel
public class ProcessStuff extends Model<ProcessStuff> {
    private static final long serialVersionUID = 1L;

    /**
     * ID
     */
    @TableId(value = "ID", type = IdType.INPUT)
    @ApiModelProperty(value = "ID")
    private Long id;
    /**
     * 流程ID
     */
    @ApiModelProperty(value = "流程ID")
    private Long processId;
    /**
     * 流程节点
     */
    @ApiModelProperty(value = "流程节点")
    private Integer processNode;
    /**
     * 流程节点分支
     */
    @ApiModelProperty(value = "流程节点分支")
    private Integer nodeBranch;
    /**
     * 物料ID
     */
    @ApiModelProperty(value = "物料ID")
    private Long matterId;
    /**
     * 设备厂家ID
     */
    @ApiModelProperty(value = "设备厂家ID")
    private Long factoryId;
    /**
     * 编号（出入库编码）
     */
    @ApiModelProperty(value = "编号（出入库编码）")
    private String stuffNumber;
    /**
     * 入库编码、出库编码
     */
    @ApiModelProperty(value = "入库编码、出库编码")
    private String inDepotNumber;

    /**
     * 采购编码、出库申请编码
     */
    @ApiModelProperty(value = "采购编码、出库申请编码")
    private String purchaseNumber;

    /**
     * 数量
     */
    @ApiModelProperty(value = "数量")
    private Long quantity;
    /**
     * 单价
     */
    @ApiModelProperty(value = "单价")
    private Double unitPrice;
    /**
     * 金额
     */
    @ApiModelProperty(value = "金额")
    private Double amountMoney;
    /**
     * 数据类型0:导入入库;1流程入库; 5出库
     */
    @ApiModelProperty(value = "数据类型0:导入入库;1流程入库;2退库; 5出库")
    private Long dataType;
    /**
     * 库房
     */
    @ApiModelProperty(value = "库房")
    private String storageRoom;
    /**
     * 到期日期
     */
    @ApiModelProperty(value = "到期日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expireDate;

    /**
     * 入库日期
     */
    @ApiModelProperty(value = "入库日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date inDate;

    /**
     * 购买日期
     */
    @ApiModelProperty(value = "购买日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date emptionDate;
    /**
     * 位置（货架位置）
     */
    @ApiModelProperty(value = "位置（货架位置）")
    private String position;

    /**
     * 安装位置、使用位置
     */
    @ApiModelProperty(value = "位置（安装位置、使用位置）")
    private String installLocation;

    /**
     * 附件id用逗号分隔
     */
    @ApiModelProperty(value = "附件id用逗号分隔")
    private String annex;

    /**
     * 物料状态1:新；2:旧
     */
    @ApiModelProperty(value = "物料状态1:新；2:旧")
    private Short matterState;
    /**
     * 旧料是否退库1:退库，0不退库
     */
    @ApiModelProperty(value = "旧料是否退库1:退库，0不退库")
    private Short oldReturn;
    /**
     * 是否删除1删除；0未删除
     */
    @ApiModelProperty(value = "是否删除1删除；0未删除")
    private Short isDel;
    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;
    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人")
    @TableField(value = "CREATE_USER", fill = FieldFill.INSERT)
    private Long createUser;
    @ApiModelProperty(hidden = true)
    private String createUserName;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;


}
