/*
 * Author: wangjian
 * 生成mapper
 */
package com.tofly.flow.custom.mapper;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.flow.custom.dto.AssetsQueryDTO;
import com.tofly.flow.custom.entity.Assets;
import com.tofly.flow.custom.entity.KeepGrainDevice;
import com.tofly.flow.custom.entity.Matter;
import com.tofly.flow.custom.vo.AssetsVO;
import com.tofly.flow.custom.vo.StatisticsVo;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 资产管理
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@Repository
public interface AssetsMapper extends BaseMapper<Assets> {

    /**
     * 车辆设备资产管理-分页查询
     * @param page 分页参数
     * @param queryWrapper 查询参数
     * @return
     */
    @Select("        SELECT assets.*,base.TYPE_NAME minTypeName,device.UNIQUE_NUMBER onlyCode," +
            "               matter.MATTER_NUMBER matterNumber,matter.MATTER_NAME matterName,matter.STANDARDS standards,"+
            "               matter.CATEGORY_TYPE categoryType,matter.IS_ASSETS isAssets"+
            "        FROM TF_YWPN_ASSETS_W assets " +
            "        LEFT JOIN TF_YWPN_BASE_TYPE_W base ON base.id = assets.ASSETS_TYPE " +
            "        LEFT JOIN TF_YWPN_MATTER_W matter ON matter.id = assets.MATTER_ID " +
            "        LEFT JOIN TF_YWPN_KEEP_GRAIN_DEVICE_W  device ON device.ID = assets.KEEP_GRAIN_DEVICE_ID " +
            "        ${ew.customSqlSegment}")
    Page<AssetsVO> pageList(Page page, @Param(Constants.WRAPPER) QueryWrapper<AssetsQueryDTO> queryWrapper);


    @Select("        SELECT assets.*,base.TYPE_NAME minTypeName,device.UNIQUE_NUMBER onlyCode," +
            "               matter.MATTER_NUMBER matterNumber,matter.MATTER_NAME matterName,matter.STANDARDS standards,"+
            "               matter.CATEGORY_TYPE categoryType,matter.IS_ASSETS isAssets"+
            "        FROM TF_YWPN_ASSETS_W assets " +
            "        LEFT JOIN TF_YWPN_BASE_TYPE_W base ON base.id = assets.ASSETS_TYPE " +
            "        LEFT JOIN TF_YWPN_MATTER_W matter ON matter.id = assets.MATTER_ID " +
            "        LEFT JOIN TF_YWPN_KEEP_GRAIN_DEVICE_W  device ON device.ID = assets.KEEP_GRAIN_DEVICE_ID " +
            "        ${ew.customSqlSegment}")
    List<AssetsVO> listAll(@Param(Constants.WRAPPER) QueryWrapper<AssetsQueryDTO> queryWrapper);


    /**
     *  固定资产台账-不分页查询
     * @param queryWrapper
     * @return
     */
    @Select("        SELECT assets.*,base.TYPE_NAME minTypeName,device.UNIQUE_NUMBER onlyCode," +
            "               matter.MATTER_NUMBER matterNumber,matter.MATTER_NAME matterName,matter.STANDARDS standards,"+
            "               matter.CATEGORY_TYPE categoryType,matter.IS_ASSETS isAssets, " +
            "               device.CATEGORY_TYPE categoryTypeId,device.DATA_SOURCE dataSourceNum "+
            "        FROM TF_YWPN_ASSETS_W assets " +
            "        LEFT JOIN TF_YWPN_BASE_TYPE_W base ON base.id = assets.ASSETS_TYPE " +
            "        LEFT JOIN TF_YWPN_MATTER_W matter ON matter.id = assets.MATTER_ID " +
            "        LEFT JOIN TF_YWPN_KEEP_GRAIN_DEVICE_W  device ON device.ID = assets.KEEP_GRAIN_DEVICE_ID " +
            "        ${ew.customSqlSegment}")
    List<AssetsVO> getList(@Param(Constants.WRAPPER) QueryWrapper<AssetsQueryDTO> queryWrapper);


    /**
     * 根据台帐ID查询物资模版数据
     * @param id 台帐ID
     * @return
     */
    @Select("SELECT * FROM TF_YWPN_MATTER_W " +
            "WHERE id IN ( " +
            "SELECT MATTER_ID FROM TF_YWPN_KEEP_GRAIN_MATTER_W  " +
            "WHERE id in( " +
            "SELECT KEEP_GRAIN_MATTER_ID FROM TF_YWPN_KEEP_GRAIN_DEVICE_W  " +
            "WHERE id in( " +
            "SELECT KEEP_GRAIN_DEVICE_ID FROM TF_YWPN_ASSETS_W WHERE id = #{id} " +
            ") " +
            ") " +
            ")")
    Matter selectStandardsByAssetsId(Long id);


    /**
     * 资产根据类型名称统计
     * @return
     */
    List<StatisticsVo> statisticsByMatterType();


    /**
     * 查询新增资产数量
     * @return
     */
    @Select("SELECT count(*) count " +
            "FROM TF_YWPN_ASSETS_W assets " +
            "WHERE IS_DEL != 1 AND IN_DATE IS NOT NULL AND ASSETS_STATE != 2")
    long statisticsByInsert();


    /**
     * 查询总数量
     * @return
     */
    @Select("SELECT count(*) count FROM TF_YWPN_ASSETS_W where  IS_DEL != 1")
    long statisticsByCount();

    /**
     * 查询报废数量
     * @return
     */
    @Select("SELECT count(*) count " +
            "FROM TF_YWPN_ASSETS_W assets " +
            "WHERE IS_DEL != 1 AND  ASSETS_STATE = 2")
    long statisticsByDeath();


    /**
     * 统计资产状态
     * @return
     */
    List<Map<String,Object>> statisticsByState();

    /**
     * 统计资产部门
     * @return
     */
    List<Map<String,Object>> statisticsByDeptName();

    /**
     * 批量修改
     * @param list 资产集合
     * @return
     */
    Boolean updateAssets(@Param("list") List<Assets> list);

    /**
     * 批量保存流程入库资产信息
     * @param lists
     * @return
     */
    Boolean saveAssets(@Param("lists") List<Assets> lists);
    /**
     * 批量修改资产状态信息
     * @param lists 设备对象信息
     * @return
     */
    Boolean updateBatchByDeviceId(@Param("lists") List<KeepGrainDevice> lists);
}
