/*
 * Author: wangjian
 * 生成Service
 */
package com.tofly.flow.custom.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.flow.custom.dto.MatterDTO;
import com.tofly.flow.custom.dto.MatterQuerySmall;
import com.tofly.flow.custom.entity.Matter;
import com.tofly.flow.custom.vo.MatterVO;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 物资信息表
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
public interface MatterService extends IService<Matter> {

    /**
     * 新增
     * @param matter
     */
    void insert(MatterDTO matter);

    /**
     * 修改
     * @param matter
     */
    void updateByMatter(MatterDTO matter);

    /**
     * 删除
     * @param ids 数据ID集合
     */
    void delete(Set<Long> ids);

    /**
     * 分页查询
     * @param page 分页参数
     * @param dto 查询参数
     * @return 分页列表
     */
    Page<MatterVO> pageList(Page page, MatterQuerySmall dto);

    /**
     * 模板下载
     */
    void template(HttpServletResponse response);

    /**
     * 数据导入
     * @param baseTypeId 类型ID
     * @param file 文件数据
     */
    void importMatter(Long baseTypeId, MultipartFile file);

    /**
     * 导出数据
     */
    void export(HttpServletRequest request, HttpServletResponse response,MatterQuerySmall dto);

    /**
     * 根据基础类型移除绑定关系
     * @param ids 基本类型ID
     */
    void deleteBaseTypeIds(List<Long> ids);

    /**
     * 根据物资编码查询仓储物料 ID TF_YWPN_KEEP_GRAIN_MATTER_W id
     * @param codeNum 物资编码
     * @return
     */
    Long selectGrainMatterByMatterNum(String codeNum);


    /**
     * 根据物资编码查询物资信息
     * @param matterNums 物资编码
     * @return 物资信息模版
     */
    List<Matter> getByMatterNums(List<String> matterNums);

    /**
     * 根据导入记录ID查询关联的物资模版
     * @param id 导入ID
     * @return 物资模版
     */
    Matter selectMatterByMatterImportId(Long id);

    /**
     * 根据ID查询
     * @param id
     * @return
     */
    MatterVO get(Long id);
}
