/*
 * Author: wangjian
 * 生成Service
 */
package com.tofly.flow.custom.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.flow.custom.dto.ProcessStuffDto;
import com.tofly.flow.custom.dto.ProcessStuffOutDto;
import com.tofly.flow.custom.entity.Process;
import com.tofly.flow.custom.entity.ProcessStuff;
import com.tofly.flow.custom.vo.OutMatterVO;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Map;

/**
 * 流程中物料出入/退库信息
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
public interface ProcessStuffService extends IService<ProcessStuff> {
    /**
     * 保存验证 入库、出库申请物资
     * @param dto
     * @return
     */
    String saveCheck(ProcessStuffDto dto);
    /**
     * 保存入库、出库申请物资
     * @param dto 物料入库数据
     * @param processId 流程id、导入id
     * @param processNode 流程节点
     * @param nodeBranch 节点分支
     * @param stuffNumber 编号
     * @return
     */
    Boolean saveProcessStuff(ProcessStuffDto dto,Long processId,Integer processNode,Integer nodeBranch,String stuffNumber);

    /**
     * 批量验证出入库物资信息
     *
     * @param process
     * @param stuffDtos 出入库信息集合
     * @return
     */
    String saveListCheck(Process process, List<ProcessStuffDto> stuffDtos);
    /**
     * 同步流程出入库物资
     * @param processId 流程ID
     * @param start 出入库状态0入库，1退库，2出库
     */
    void synchroStuff(Long processId,int start);
    /**
     * 批量保存出入库物资信息
     * @param stuffDtos 出入库物资信息
     * @param processStep 流程步骤
     * @param stepNode 流程步骤节点
     * @param process 编号
     * @param deviceFiles 上传的文件信息
     * @return
     */
    Boolean saveListProcessStuff(List<ProcessStuffDto> stuffDtos, Integer processStep, Integer stepNode, Process process, Map<String, List<MultipartFile>> deviceFiles);


    /**
     * 处理物资信息
     * @param page 分页
     * @param dto 查询条件
     * @return 出库信息
     */
    Page<OutMatterVO> loadOutMatter(Page page, ProcessStuffOutDto dto);
}
