/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.AssetsDTO;
import com.tofly.flow.custom.dto.AssetsQueryDTO;
import com.tofly.flow.custom.entity.Assets;
import com.tofly.flow.custom.entity.BaseType;
import com.tofly.flow.custom.entity.KeepGrainDevice;
import com.tofly.flow.custom.mapper.AssetsMapper;
import com.tofly.flow.custom.service.AssetsService;
import com.tofly.flow.custom.service.BaseTypeService;
import com.tofly.flow.custom.vo.*;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.Period;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 资产管理
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@Service("assetsService")
public class AssetsServiceImpl extends ServiceImpl<AssetsMapper, Assets> implements AssetsService {

    @Autowired
    private AssetsMapper assetsMapper;

    @Autowired
    private BaseTypeService baseTypeService;

    @Autowired
    private RedisTemplate redisTemplate;

    public static final String YDM = "yyyy年MM月dd日";

    @Override
    public Page<AssetsVO> pageList(Page page, AssetsQueryDTO dto) {
        QueryWrapper<AssetsQueryDTO> queryWrapper = getAssetsQueryDTOQueryWrapper(dto);
        Page<AssetsVO> pageList = assetsMapper.pageList(page, queryWrapper);
        if (pageList.getTotal() > 0) {
            List<AssetsVO> records = pageList.getRecords();
            getVolist(records);
        }
        return pageList;
    }

    private void getVolist(List<AssetsVO> records) {
        for (AssetsVO record : records) {
            List<BaseType> baseTypes = baseTypeService.selectTypeParentById(record.getAssetsType());
            if (!CollectionUtils.isEmpty(baseTypes)) {
                record.setBaseTypes(baseTypes);
                record.setMinTypeName(baseTypes.get(0).getTypeName());
                record.setBigTypeName(baseTypes.get(baseTypes.size() - 1).getTypeName());
            }
            if (record.getPrice() != null && record.getQuantity() != null) {
                record.setTotalPrice(record.getPrice() * record.getQuantity());
            }
            if (record.getPrice() != null && record.getPurchase() != null) {
                record.setPurchaseMoney(record.getPrice() * record.getPurchase());
            }
            //计算折旧价格价值
            calculation(record);
        }
    }


    /**
     * 计算折旧价格价值
     *
     * @param record
     */
    private void calculation(AssetsVO record) {
        if (record.getDepreciationType() != null && record.getDepreciation() != null
                && record.getInDate() != null && record.getPrice() != null) {
            int yearDateDiff = yearDateDiff(record.getInDate(), new Date());
            int monthDateDiff = (yearDateDiff * 12) + monthDateDiff(record.getInDate(), new Date());
            record.setInYears(yearDateDiff);
            record.setInMonths(monthDateDiff);
            if (record.getDepreciationType().equals(FlowConstant.SHORT_ONE)) {
                record.setInYears(Math.max(yearDateDiff, 0));
                record.setOldMoney(record.getInYears() * record.getDepreciation());
                double v = record.getPrice() - record.getOldMoney();
                record.setNowMoney(v);
            } else if (record.getDepreciationType().equals(FlowConstant.SHORT_TWO)) {
                record.setInMonths(Math.max(monthDateDiff, 0));
                record.setOldMoney(record.getInMonths() * record.getDepreciation());
                double v = record.getPrice() - record.getOldMoney();
                record.setNowMoney(v);
            } else if (record.getDepreciationType().equals(FlowConstant.SHORT_THREE)) {
                record.setInYears(Math.max(yearDateDiff, 0));
                record.setOldMoney(record.getInYears() * record.getDepreciation() * record.getPrice());
                double v = record.getPrice() - record.getOldMoney();
                record.setNowMoney(v);
            } else if (record.getDepreciationType().equals(FlowConstant.SHORT_FOUR)) {
                record.setInMonths(Math.max(monthDateDiff, 0));
                record.setOldMoney(record.getInMonths() * record.getDepreciation() * record.getPrice());
                double v = record.getPrice() - record.getOldMoney();
                record.setNowMoney(v);
            }
            if (record.getNowMoney() != null && record.getNowMoney() <= 0) {
                record.setNowMoney((double) 0);
            }
        }
        if (record.getInYears() == null || record.getInYears() <= 0) {
            record.setInYears(0);
        }
        if (record.getInMonths() == null || record.getInMonths() <= 0) {
            record.setInMonths(0);
        }
        if (record.getOldMoney() != null) {
            String format = String.format("%.2f", record.getOldMoney());
            record.setOldMoney(Double.valueOf(format));
        }
        if (record.getNowMoney() != null) {
            String format = String.format("%.2f", record.getNowMoney());
            record.setNowMoney(Double.valueOf(format));
        }
    }

    //计算两个日期相差年数
    public static int yearDateDiff(Date startDate, Date endDate) {
        LocalDate start =  startDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        LocalDate end = endDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        Period period = Period.between(start, end);
        return period.getYears();
    }

    /**
     * 计算2个日期之间相差的  相差多少年月日
     * 比如：2011-02-02 到  2017-03-02 相差 6年，1个月，0天
     * @param fromDate YYYY-MM-DD
     * @param toDate YYYY-MM-DD
     * @return 年,月,日 例如 1,1,1
     */
    public static String dayComparePrecise(String fromDate, String toDate){

        Period period = Period.between(LocalDate.parse(fromDate), LocalDate.parse(toDate));

        StringBuffer sb = new StringBuffer();
        sb.append(period.getYears()).append(",")
                .append(period.getMonths()).append(",")
                .append(period.getDays());
        System.out.println(sb.toString());
        return sb.toString();
    }

    //计算两个日期相差月数
    public static int monthDateDiff(Date startDate, Date endDate) {
        Calendar calStartTime = Calendar.getInstance(); //获取日历实例
        Calendar calEndTime = Calendar.getInstance();
        calStartTime.setTime(startDate); //字符串按照指定格式转化为日期
        calEndTime.setTime(endDate);
        return calEndTime.get(Calendar.MONTH) - calStartTime.get(Calendar.MONTH);
    }


    @Override
    public void fixedExport(HttpServletRequest request, HttpServletResponse response, AssetsQueryDTO dto) {
        QueryWrapper<AssetsQueryDTO> queryWrapper = getAssetsQueryDTOQueryWrapper(dto);
        List<AssetsVO> list = assetsMapper.getList(queryWrapper);
        // excel模板路径
        InputStream resourceAsStream = this.getClass().getClassLoader().getResourceAsStream("report/export/固定物资台账导出模版.xlsx");
        assert resourceAsStream != null;
        XSSFWorkbook wb = null;
        try {
            // 读取excel模板
            wb = new XSSFWorkbook(resourceAsStream);
            // 读取了模板内所有sheet内容
            XSSFSheet sheet = wb.getSheetAt(0);
            // 在相应的单元格进行赋值
            int rowIndex = 1;
            int j = 1;
            for (AssetsVO data : list) {
                XSSFRow row = sheet.getRow(rowIndex);
                if (null == row) {
                    row = sheet.createRow(rowIndex);
                }
                XSSFCell cell0 = row.getCell(0);
                if (null == cell0) {
                    cell0 = row.createCell(0);
                }
                cell0.setCellValue(data.getAssetsNumber() == null ? "" : data.getAssetsNumber());// 资产编码

                XSSFCell cell1 = row.getCell(1);
                if (null == cell1) {
                    cell1 = row.createCell(1);
                }
                cell1.setCellValue(data.getMatterName() == null ? "" : data.getMatterName());// 资产类型

                XSSFCell cell2 = row.getCell(2);
                if (null == cell2) {
                    cell2 = row.createCell(2);
                }
                cell2.setCellValue(data.getMinTypeName() == null ? "" : data.getMinTypeName());// 资产小类


                XSSFCell cell3 = row.getCell(3);
                if (null == cell3) {
                    cell3 = row.createCell(3);
                }
                cell3.setCellValue(data.getAssetsName() == null ? "" : data.getAssetsName());// 资产名称

                XSSFCell cell4 = row.getCell(4);
                if (null == cell4) {
                    cell4 = row.createCell(4);
                }
                cell4.setCellValue(data.getStandards() == null ? "" : data.getStandards());// 规格型号


                XSSFCell cell5 = row.getCell(5);
                if (null == cell5) {
                    cell5 = row.createCell(5);
                }
                cell5.setCellValue(data.getDeptName() == null ? "" : data.getDeptName());// 管理部门

                XSSFCell cell6 = row.getCell(6);
                if (null == cell6) {
                    cell6 = row.createCell(6);
                }
                //资产状态0:在库；1:在用；2:报废；3:出售；4:闲置
                if (data.getAssetsState() == null) {
                    cell6.setCellValue("");// 资产状态
                } else if (data.getAssetsState().equals(FlowConstant.SHORT_ZERO)) {
                    cell6.setCellValue("在库");// 资产状态
                } else if (data.getAssetsState().equals(FlowConstant.SHORT_ONE)) {
                    cell6.setCellValue("在用");// 资产状态
                } else if (data.getAssetsState().equals(FlowConstant.SHORT_TWO)) {
                    cell6.setCellValue("报废");// 资产状态
                } else if (data.getAssetsState().equals(FlowConstant.SHORT_THREE)) {
                    cell6.setCellValue("出售");// 资产状态
                } else if (data.getAssetsState().equals(FlowConstant.SHORT_FOUR)) {
                    cell6.setCellValue("闲置");// 资产状态
                } else {
                    cell6.setCellValue("未知");// 资产状态
                }


                XSSFCell cell7 = row.getCell(7);
                if (null == cell7) {
                    cell7 = row.createCell(7);
                }
                cell7.setCellValue(data.getPrice());// 采购价格（元）

                XSSFCell cell8 = row.getCell(8);
                if (null == cell8) {
                    cell8 = row.createCell(8);
                }
                cell8.setCellValue(data.getNowMoney() == null ? 0 : data.getNowMoney());// 当前价值（元）

                XSSFCell cell9 = row.getCell(9);
                if (null == cell9) {
                    cell9 = row.createCell(9);
                }
                //折旧类型1:固定年折旧；2:固定月折旧；3:年折旧率；4:月折旧率
                if (data.getDepreciationType() == null) {
                    cell9.setCellValue("");// 折旧方式
                } else if (data.getDepreciationType().equals(FlowConstant.SHORT_ONE)) {
                    cell9.setCellValue("固定年折旧");// 折旧方式
                } else if (data.getDepreciationType().equals(FlowConstant.SHORT_TWO)) {
                    cell9.setCellValue("固定月折旧");// 折旧方式
                } else if (data.getDepreciationType().equals(FlowConstant.SHORT_THREE)) {
                    cell9.setCellValue("年折旧率");// 折旧方式
                } else if (data.getDepreciationType().equals(FlowConstant.SHORT_FOUR)) {
                    cell9.setCellValue("月折旧率");// 折旧方式
                } else {
                    cell9.setCellValue("未知");// 资产状态
                }

                XSSFCell cell10 = row.getCell(10);
                if (null == cell10) {
                    cell10 = row.createCell(10);
                }
                cell10.setCellValue(DateUtil.format(data.getProduceTime(), YDM));// 生产日期

                XSSFCell cell11 = row.getCell(11);
                if (null == cell11) {
                    cell11 = row.createCell(11);
                }
                cell11.setCellValue(DateUtil.format(data.getInDate(), YDM));// 入库日期

                XSSFCell cell12 = row.getCell(12);
                if (null == cell12) {
                    cell12 = row.createCell(12);
                }
                cell12.setCellValue(DateUtil.format(data.getExpireDate(), YDM));// 到期时间

                XSSFCell cell13 = row.getCell(13);
                if (null == cell13) {
                    cell13 = row.createCell(13);
                }
                cell13.setCellValue(data.getInYears() == null ? 0 : data.getInYears());// 入库年限

                XSSFCell cell14 = row.getCell(14);
                if (null == cell14) {
                    cell14 = row.createCell(14);
                }
                cell14.setCellValue(data.getBigTypeName() == null ? "" : data.getBigTypeName());// 资产分类

                XSSFCell cell15 = row.getCell(15);
                if (null == cell15) {
                    cell15 = row.createCell(15);
                }
                cell15.setCellValue(data.getOnlyCode() == null ? "" : data.getOnlyCode());// 出厂唯一编码


                XSSFCell cell16 = row.getCell(16);
                if (null == cell16) {
                    cell16 = row.createCell(16);
                }
                cell16.setCellValue(data.getSupply() == null ? "" : data.getSupply());// 供货商


                XSSFCell cell17 = row.getCell(17);
                if (null == cell17) {
                    cell17 = row.createCell(17);
                }
                cell17.setCellValue(data.getProduce() == null ? "" : data.getProduce());// 生产厂商


                XSSFCell cell18 = row.getCell(18);
                if (null == cell18) {
                    cell18 = row.createCell(18);
                }
                cell18.setCellValue(data.getRemark() == null ? "" : data.getRemark());// 备注

                rowIndex++;
            }

            String fileName = URLEncoder.encode("物资台账数据导出列表");
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            wb.write(os);
            byte[] content = os.toByteArray();
            InputStream is = new ByteArrayInputStream(content);
            // 设置response参数，可以打开下载页面
            response.reset();
            response.setContentType("application/vnd.ms-excel;charset=utf-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + new String((fileName + ".xlsx").getBytes(), "iso-8859-1"));
            ServletOutputStream sout = response.getOutputStream();
            BufferedInputStream bis = null;
            BufferedOutputStream bos = null;

            try {
                bis = new BufferedInputStream(is);
                bos = new BufferedOutputStream(sout);
                byte[] buff = new byte[2048];
                int bytesRead;
                // Simple read/write loop.
                while (-1 != (bytesRead = bis.read(buff, 0, buff.length))) {
                    bos.write(buff, 0, bytesRead);
                }
            } catch (Exception e) {
//                logger.error("导出excel出现异常:", e);
            } finally {
                if (bis != null) {
                    bis.close();
                }
                if (bos != null) {
                    bos.close();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
//            logger.error("导出excel出现异常:", e);
        }
    }

    @Override
    public void export(HttpServletRequest request, HttpServletResponse response, AssetsQueryDTO dto) {
        QueryWrapper<AssetsQueryDTO> queryWrapper = getAssetsQueryDTOQueryWrapper(dto);
        List<AssetsVO> list = assetsMapper.getList(queryWrapper);
        XSSFWorkbook wb = null;
        try {
            // excel模板路径
            File fi = new File("mabianFlow/mabian-flow-boot/src/main/resources/report/export/物资台账导出模版.xlsx");
            // 读取excel模板
            wb = new XSSFWorkbook(new FileInputStream(fi));
            // 读取了模板内所有sheet内容
            XSSFSheet sheet = wb.getSheetAt(0);
            // 在相应的单元格进行赋值
            int rowIndex = 1;
            int j = 1;
            for (AssetsVO data : list) {
                XSSFRow row = sheet.getRow(rowIndex);
                if (null == row) {
                    row = sheet.createRow(rowIndex);
                }
                XSSFCell cell0 = row.getCell(0);
                if (null == cell0) {
                    cell0 = row.createCell(0);
                }
                cell0.setCellValue(data.getAssetsNumber() == null ? "" : data.getAssetsNumber());// 资产编码

                XSSFCell cell1 = row.getCell(1);
                if (null == cell1) {
                    cell1 = row.createCell(1);
                }
                cell1.setCellValue(data.getMatterName() == null ? "" : data.getMatterName());// 资产类型

                XSSFCell cell2 = row.getCell(2);
                if (null == cell2) {
                    cell2 = row.createCell(2);
                }
                cell2.setCellValue(data.getStandards() == null ? "" : data.getStandards());// 规格型号


                XSSFCell cell3 = row.getCell(3);
                if (null == cell3) {
                    cell3 = row.createCell(3);
                }
                cell3.setCellValue(data.getBigTypeName() == null ? "" : data.getBigTypeName());// 物资类别

                XSSFCell cell4 = row.getCell(4);
                if (null == cell4) {
                    cell4 = row.createCell(4);
                }
                //物资类别类型1设备类;2车辆类;3耗材类
                if (data.getCategoryTypeId().equals(FlowConstant.SHORT_ONE)) {
                    cell4.setCellValue("设备类");// 物资类别
                } else if (data.getAssetsState().equals(FlowConstant.SHORT_TWO)) {
                    cell4.setCellValue("车辆类");// 物资类别
                } else if (data.getAssetsState().equals(FlowConstant.SHORT_THREE)) {
                    cell4.setCellValue("耗材类");// 物资类别
                }


                XSSFCell cell5 = row.getCell(5);
                if (null == cell5) {
                    cell5 = row.createCell(5);
                }
                cell5.setCellValue(data.getIsAssets() == null ? "未知" :
                        data.getIsAssets().equals(FlowConstant.SHORT_ONE) ? "是" : "否");// 是否为资产

                XSSFCell cell6 = row.getCell(6);
                if (null == cell6) {
                    cell6 = row.createCell(6);
                }
                cell6.setCellValue(data.getUnit() == null ? "" : data.getUnit());// 规格型号


                XSSFCell cell7 = row.getCell(7);
                if (null == cell7) {
                    cell7 = row.createCell(7);
                }
                cell7.setCellValue(data.getPurchase() == null ? 0 : data.getPurchase());// 采购总数量

                XSSFCell cell8 = row.getCell(8);
                if (null == cell8) {
                    cell8 = row.createCell(8);
                }
                cell8.setCellValue(data.getQuantity() == null ? 0 : data.getQuantity());// 库存数量

                XSSFCell cell9 = row.getCell(9);
                if (null == cell9) {
                    cell9 = row.createCell(9);
                }
                cell9.setCellValue(data.getPurchaseMoney() == null ? 0 : data.getPurchaseMoney());// 采购价格合计（元）

                XSSFCell cell10 = row.getCell(10);
                if (null == cell10) {
                    cell10 = row.createCell(10);
                }
                cell10.setCellValue(data.getPrice() == null ? 0 : data.getPrice());// 最近采购单价（元）

                XSSFCell cell11 = row.getCell(11);
                if (null == cell11) {
                    cell11 = row.createCell(11);
                }
                cell11.setCellValue(data.getTotalPrice() == null ? 0 : data.getTotalPrice());// 库存市场总价（元）

                // 数据来源:1导入；2入库
                if (data.getDataSourceNum() == null) {
                    cell4.setCellValue("未知");//  数据来源:1导入；2入库
                } else if (data.getDataSourceNum().equals(FlowConstant.SHORT_ONE)) {
                    cell4.setCellValue("导入");//  数据来源:1导入；2入库
                } else if (data.getDataSourceNum().equals(FlowConstant.SHORT_TWO)) {
                    cell4.setCellValue("入库");//  数据来源:1导入；2入库
                } else {
                    cell4.setCellValue("未知");//  数据来源:1导入；2入库
                }


                rowIndex++;
            }

            String fileName = URLEncoder.encode("物资台账导出数据");
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            wb.write(os);
            byte[] content = os.toByteArray();
            InputStream is = new ByteArrayInputStream(content);
            // 设置response参数，可以打开下载页面
            response.reset();
            response.setContentType("application/vnd.ms-excel;charset=utf-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + new String((fileName + ".xlsx").getBytes(), "iso-8859-1"));
            ServletOutputStream sout = response.getOutputStream();
            BufferedInputStream bis = null;
            BufferedOutputStream bos = null;

            try {
                bis = new BufferedInputStream(is);
                bos = new BufferedOutputStream(sout);
                byte[] buff = new byte[2048];
                int bytesRead;
                // Simple read/write loop.
                while (-1 != (bytesRead = bis.read(buff, 0, buff.length))) {
                    bos.write(buff, 0, bytesRead);
                }
            } catch (Exception e) {
//                logger.error("导出excel出现异常:", e);
            } finally {
                if (bis != null) {
                    bis.close();
                }
                if (bos != null) {
                    bos.close();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
//            logger.error("导出excel出现异常:", e);
        }
    }

    @Override
    public void insert(AssetsDTO assetsDTO) {
        handleTime(assetsDTO);
        Assets assets = new Assets();
        BeanUtils.copyProperties(assetsDTO, assets);
        if (!this.save(assets)) {
            throw new RuntimeException("新增失败");
        }
    }

    @Override
    public void edit(AssetsDTO assetsDTO) {
        handleTime(assetsDTO);
        Assets assets = new Assets();
        BeanUtils.copyProperties(assetsDTO, assets);
        if (!this.updateById(assets)) {
            throw new RuntimeException("修改失败");
        }
    }

    @Override
    public Map<String, Object> statistics() {

        Map<String, Object> map = new HashMap<>(4);
        //资产根据入库方式和状态统计
        selectByDevice(map);
        //资产根据类型名称统计
        selectByType(map);
        //资产根据管理部门统计
        selectByDeptName(map);
        return map;
    }

    private void selectByDeptName(Map<String, Object> map) {
        map.put("selectByDeptName",assetsMapper.statisticsByDeptName());
        List<Assets> list = this.list(new QueryWrapper<Assets>().eq("IS_DEL",0));
        List<AssetsVO> assetsVOList = list.stream().map(assets -> {
            AssetsVO assetsVO = new AssetsVO();
            BeanUtils.copyProperties(assets, assetsVO);
            return assetsVO;
        }).collect(Collectors.toList());
        getVolist(assetsVOList);
        double sum = assetsVOList.stream().filter(assetsVO -> assetsVO.getNowMoney() != null).mapToDouble(AssetsVO::getNowMoney).sum();
        sum = sum == 0 ? 1 : sum;
//        List<StatisticsVo> statisticsVos = new ArrayList<>();
//        Map<String, List<AssetsVO>> listMap = assetsVOList.stream().filter(assetsVO -> assetsVO.getDeptName() != null).collect(Collectors.groupingBy(AssetsVO::getDeptName));
//        for (Map.Entry<String, List<AssetsVO>> listEntry : listMap.entrySet()) {
//            List<AssetsVO> lists = listEntry.getValue();
//            if (!CollectionUtils.isEmpty(lists)) {
//                StatisticsVo statisticsVo = new StatisticsVo();
//                statisticsVo.setCount((long) lists.size());
//                statisticsVo.setName(lists.get(0).getDeptName());
//                double nowMoney = lists.stream().filter(assetsVO -> assetsVO.getNowMoney() != null).mapToDouble(AssetsVO::getNowMoney).sum();
//                if (nowMoney == 0) {
//                    statisticsVo.setProportion("0%");
//                }
//                float v = (float) nowMoney / (float) sum * 100;
//                statisticsVo.setProportion(v + "%");
//                statisticsVos.add(statisticsVo);
//            }
//        }
//        if (!CollectionUtils.isEmpty(statisticsVos)) {
//            map.put("selectByDeptName", statisticsVos);
            map.put("totalMoney", sum);
//        } else {
//            statisticsVos.add(new StatisticsVo());
//            map.put("selectByDeptName", statisticsVos);
//        }
    }

    private void selectByType(Map<String, Object> map) {
        List<StatisticsVo> statisticsVos = assetsMapper.statisticsByMatterType();
        if (!CollectionUtils.isEmpty(statisticsVos)) {
            long sum = statisticsVos.stream().filter(statisticsVo -> statisticsVo.getCount() != null).mapToLong(StatisticsVo::getCount).sum();
            for (StatisticsVo statisticsVo : statisticsVos) {
                float v = (float) statisticsVo.getCount() / (float) sum * 100;
                statisticsVo.setProportion(v + "%");
            }
            map.put("selectByType", statisticsVos);
        } else {
            statisticsVos.add(new StatisticsVo());
            map.put("selectByType", statisticsVos);
        }
    }

    private void selectByDevice(Map<String, Object> map) {
        map.put("selectByDevice",assetsMapper.statisticsByState());
//        List<StatisticsVo> statisticsVos = new ArrayList<>();
//        //1.查询新增资产占比
//        long statisticsByInsert = assetsMapper.statisticsByInsert();
//        StatisticsVo statisticsVo1 = new StatisticsVo();
//        statisticsVo1.setCount(statisticsByInsert);
//        statisticsVo1.setName("insert");
//        statisticsVos.add(statisticsVo1);
//        //2.查询报废数量
//        long statisticsByDeath = assetsMapper.statisticsByDeath();
//        StatisticsVo statisticsVo2 = new StatisticsVo();
//        statisticsVo2.setCount(statisticsByDeath);
//        statisticsVo2.setName("death");
//        statisticsVos.add(statisticsVo2);
//        //3.查询总数量
//        long statisticsByCount = assetsMapper.statisticsByCount();
//        StatisticsVo statisticsVo3 = new StatisticsVo();
//        statisticsVo3.setCount(statisticsByCount - statisticsByDeath - statisticsByInsert);
//        statisticsVo3.setName("other");
//        statisticsVos.add(statisticsVo3);
//        if (!CollectionUtils.isEmpty(statisticsVos)) {
//            for (StatisticsVo statisticsVo : statisticsVos) {
//                float v = statisticsVo.getCount() / (float) statisticsByCount * 100;
//                statisticsVo.setProportion(v + "%");
//            }
//            map.put("selectByDevice", statisticsVos);
//        } else {
//            statisticsVos.add(new StatisticsVo());
//            map.put("selectByDevice", statisticsVos);
//        }
    }


    /**
     * 处理报废时间和出售时间
     *
     * @param assetsDTO
     */
    private void handleTime(AssetsDTO assetsDTO) {
        if (assetsDTO.getAssetsState() != null
                && (assetsDTO.getAssetsState().equals(FlowConstant.SHORT_TWO) || assetsDTO.getAssetsState().equals(FlowConstant.SHORT_THREE))
        ) {
            assetsDTO.setHandleTime(new Date());
        }

    }

    /**
     * 设置查询条件
     *
     * @param dto
     * @return
     */
    private QueryWrapper<AssetsQueryDTO> getAssetsQueryDTOQueryWrapper(AssetsQueryDTO dto) {
        QueryWrapper<AssetsQueryDTO> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("assets.IS_DEL", FlowConstant.SHORT_ZERO);
        if (dto != null) {
            if (dto.getTypeId() != null) {
                queryWrapper.eq("assets.ASSETS_TYPE", dto.getTypeId());
            }
            if (!StringUtils.isEmpty(dto.getSearch())) {
                queryWrapper.and(query -> {
                    query.like("assets.ASSETS_NUMBER", dto.getSearch()).or()
                            .like("assets.ASSETS_NAME", dto.getSearch());
                });
            }
            if (!StringUtils.isEmpty(dto.getStartTime()) && !StringUtils.isEmpty(dto.getEndTime())) {
                queryWrapper.between("assets.IN_DATE", dto.getStartTime(), dto.getEndTime());
            }
            if (!StringUtils.isEmpty(dto.getIsAssets()) && dto.getIsAssets()) {
                queryWrapper.eq("matter.IS_ASSETS", FlowConstant.SHORT_ONE);
            } else if (!StringUtils.isEmpty(dto.getIsAssets()) && !dto.getIsAssets()) {
                queryWrapper.eq("matter.IS_ASSETS", FlowConstant.SHORT_ZERO);
            }
            if (!StringUtils.isEmpty(dto.getIsDevice()) && dto.getIsDevice()) {
                queryWrapper.and(q -> {
                    q.eq("matter.CATEGORY_TYPE", FlowConstant.SHORT_ONE)
                            .or()
                            .eq("matter.CATEGORY_TYPE", FlowConstant.SHORT_TWO);
                });
            }
            if (dto.getCategoryType() != null) {
                queryWrapper.eq("matter.CATEGORY_TYPE", dto.getCategoryType());
            }
            if (dto.getMatterId() != null) {
                queryWrapper.eq("assets.MATTER_ID", dto.getMatterId());
            }
            if (!StringUtils.isEmpty(dto.getDeptName())) {
                queryWrapper.eq("assets.DEPT_NAME", dto.getDeptName());
            }
            queryWrapper.orderByDesc("assets.CREATE_TIME");
        }
        return queryWrapper;
    }


    /**
     * 出入库流程设备保存资产
     *
     * @param start 操作类型0入库，1出库
     * @param zcs   设备信息
     * @return
     */
    @Override
    public Boolean saveAssetes(int start, List<Assets> zcs) {
        if (FlowConstant.SHORT_ONE == start) {
            //出库
            this.baseMapper.updateAssets(zcs);
        } else if (FlowConstant.SHORT_ZERO == start) {
            Long index = this.getObj(new QueryWrapper<Assets>().select("MAX(ID) as ID")
                    , (o) -> {
                        return Long.parseLong(String.valueOf(o));
                    });
            if (index == null) {
                index = 0l;
            }
            for (Assets assets : zcs) {
                assets.setCreateUser(FlowConstant.LONG_ONE);
                assets.setCreateTime(new Date());
                assets.setId(++index);
            }
            //入库
            this.baseMapper.saveAssets(zcs);
        }
        return true;
    }

    @Override
    public Boolean updateBatchByDeviceId(List<KeepGrainDevice> lists) {
        return this.baseMapper.updateBatchByDeviceId(lists);
    }

    /**
     * 获取资产编码
     *
     * @param code 资产类型编码
     * @return 资产编码
     */
    public String getAssetsNumber(String code) {
        return getAssetsNumber(FlowConstant.ZC, code);
    }


    /**
     * 获取资产编码
     *
     * @param zc   资产前缀
     * @param code 资产类型编码
     * @return 资产编码
     */
    public String getAssetsNumber(String zc, String code) {
        if (StringUtils.isEmpty(zc)) {
            zc = FlowConstant.ZC;
        }
        code = zc + code;
        String key = FlowConstant.PROCESS_ZC_REDIS_KEY + code + DateUtil.format(new Date(), "yyyyMMdd");
        RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());
        Long value = counter.incrementAndGet();
        if (value == 1) {
            String number = this.getObj(new QueryWrapper<Assets>().select("MAX(ASSETS_NUMBER) as ID").lambda()
                            .ge(Assets::getCreateTime, DateUtil.beginOfDay(new Date()))
                    , (o) -> {
                        return String.valueOf(o);
                    });
            if (org.apache.commons.lang3.StringUtils.isNotEmpty(number)) {
                Long num = Long.parseLong(number.substring(number.length() - 5));
                counter.set(num);
                long betweenDay = DateUtil.between(new Date(), DateUtil.endOfDay(new Date()), DateUnit.MINUTE);
                counter.expire(betweenDay, TimeUnit.MINUTES);
                value = counter.incrementAndGet();
            }
        }
        return code + DateUtil.format(new Date(), "yyyyMMdd") + String.format("%05d", value);
    }

    @Override
    public Map<String,Object> statisticsDevice() {
        QueryWrapper<AssetsQueryDTO> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("assets.IS_DEL", FlowConstant.SHORT_ZERO);
        List<AssetsVO> assetsVOList = assetsMapper.listAll(queryWrapper);
        if (CollectionUtils.isEmpty(assetsVOList)) {
            return null;
        }
        Map<String,Object> map = new HashMap<>();
        List<AssetsStattisticsVo> assetsStattisticsVos = new ArrayList<>();
        getVolist(assetsVOList);
        Map<String, List<AssetsVO>> collect = assetsVOList.stream().collect(Collectors.groupingBy(AssetsVO::getBigTypeName));
        for (Map.Entry<String, List<AssetsVO>> entry : collect.entrySet()) {
            List<AssetsVO> maxTypeNames = entry.getValue();
            if (CollectionUtils.isEmpty(maxTypeNames)) {
                break;
            }
            AssetsStattisticsVo vo = new AssetsStattisticsVo();
            vo.setBigName(maxTypeNames.get(0).getBigTypeName());
            Map<String, List<AssetsVO>> listMap = maxTypeNames.stream().filter(assetsVO -> assetsVO.getMinTypeName() != null)
                    .collect(Collectors.groupingBy(AssetsVO::getMinTypeName));
            List<AssetsStattisticsMinVo> minVos = new ArrayList<>();
            for (Map.Entry<String, List<AssetsVO>> listEntry : listMap.entrySet()) {
                List<AssetsVO> minTypeNames = listEntry.getValue();
                if (CollectionUtils.isEmpty(minTypeNames)) {
                    break;
                }
                AssetsStattisticsMinVo minVo = new AssetsStattisticsMinVo();
                minVo.setTotal(minTypeNames.size());
                minVo.setMinName(minTypeNames.get(0).getMinTypeName());
                minVos.add(minVo);
            }
            vo.setMinVos(minVos);
            assetsStattisticsVos.add(vo);
        }
        map.put("devicesData",assetsStattisticsVos);
        map.put("data",assetsVOList);
        return map;
    }


}
