/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.BaseTypeDto;
import com.tofly.flow.custom.dto.UpdateBaseTypeDto;
import com.tofly.flow.custom.entity.BaseType;
import com.tofly.flow.custom.entity.Matter;
import com.tofly.flow.custom.entity.ProcessConfig;
import com.tofly.flow.custom.service.BaseTypeService;
import com.tofly.flow.custom.service.MatterService;
import com.tofly.flow.custom.service.ProcessConfigService;
import com.tofly.flow.custom.vo.BaseTypeDataTypeVo;
import com.tofly.flow.custom.vo.BaseTypeVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.BaseTypeMapper;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 基础类型表
 *
 * @author xqf
 * @date Mon Mar 21 00:00:00 CST 2022
 */
@Service("baseTypeService")
public class BaseTypeServiceImpl extends ServiceImpl<BaseTypeMapper, BaseType> implements BaseTypeService {


    /**
     * 流程配置信息
     */
    @Autowired
    private ProcessConfigService processConfigService;
    /**
     * 物资名称
     */
    @Autowired
    private MatterService matterService;

    /**
     * 保存基础类型
     *
     * @param dto
     * @return
     */
    @Override
    public Object saveBaseType(BaseTypeDto dto) {
        BaseType dt = null;
        if(dto.getDataType()!=null&&dto.getDataType()!=FlowConstant.LONG_ZERO) {
            dt = this.getById(dto.getDataType());
            if (dt == null && (dt.getIsValid() == FlowConstant.SHORT_ZERO || dt.getIsDel() == FlowConstant.SHORT_ONE)) {
                return FlowConstant.DATATYPE_ERROR;
            }
        }
        if (dto.getParentId() == null) {
            dto.setParentId(FlowConstant.LONG_ZERO);
        } else if (dto.getParentId() > 0) {
            int count = this.count(new QueryWrapper<BaseType>().lambda().eq(BaseType::getId, dto.getParentId())
                    .eq(BaseType::getDataType, dto.getDataType())
                    .eq(BaseType::getIsDel, FlowConstant.SHORT_ZERO));
            if (count == 0) {
                return FlowConstant.DATA_NOTHINGNESS;
            }
        }
        int count = this.count(new QueryWrapper<BaseType>().lambda()
                .eq(BaseType::getDataType, dto.getDataType()).eq(BaseType::getParentId, dto.getParentId())
                .eq(BaseType::getTypeName, dto.getTypeName()).eq(BaseType::getIsDel, FlowConstant.SHORT_ZERO));
        if (count > 0) {
            return FlowConstant.NAME_EXISTS;
        }
        if(dt!=null) {
            /**判断类型关联流程配置*/
            if (dt.getParentId() == FlowConstant.LONG_ONE) {
                count = processConfigService.count(new QueryWrapper<ProcessConfig>().lambda()
                        .eq(ProcessConfig::getBusinessType, dto.getParentId()).eq(ProcessConfig::getProcessType, dto.getDataType())
                        .eq(ProcessConfig::getIsDel, FlowConstant.SHORT_ZERO));
                /**判断类型关联物资*/
            } else if (dt.getId() == FlowConstant.LONG_TWENTY_TWO) {
                count = matterService.count(new QueryWrapper<Matter>().lambda()
                        .eq(Matter::getMatterType, dto.getParentId())
                        .eq(Matter::getIsDel, FlowConstant.SHORT_ZERO));
            }
            if (count > FlowConstant.INTEGER_ZERO) {
                return FlowConstant.BASE_TYPE_LIKE_PROCESS_CONFIG;
            }
        }
        BaseType baseType = new BaseType();
        BeanUtils.copyProperties(dto, baseType);
        if (baseType.getIsValid() == null) {
            baseType.setIsValid(FlowConstant.SHORT_ONE);
        }
        baseType.setIsDel(FlowConstant.SHORT_ZERO);
        return save(baseType);
    }

    /**
     * 修改基础类型信息
     *
     * @param dto
     * @return
     */
    @Override
    public Object updateBaseType(UpdateBaseTypeDto dto) {
        BaseType bt = this.getById(dto.getId());
        if (bt == null || bt.getIsDel() == FlowConstant.SHORT_ONE) {
            return FlowConstant.DATA_NOTHINGNESS;
        }
        int count = this.count(new QueryWrapper<BaseType>().lambda()
                .eq(BaseType::getDataType, bt.getDataType()).eq(BaseType::getParentId, bt.getParentId())
                .eq(BaseType::getTypeName, dto.getTypeName()).eq(BaseType::getIsDel, FlowConstant.SHORT_ZERO)
                .ne(BaseType::getId, dto.getId()));
        if (count > 0) {
            return FlowConstant.NAME_EXISTS;
        }
        BaseType baseType = new BaseType();
        if (dto.getIsValid() != null) {
            Object obj = updateValid(dto.getId(), dto.getIsValid());
            if (obj instanceof String) {
                return obj;
            }
        }
        BeanUtils.copyProperties(dto, baseType);
        return updateById(baseType);
    }


    /**
     * 修改类型是否有效
     *
     * @param id    id主键
     * @param valid 有效无效 0 无效，1有效
     * @return
     */
    @Override
    public Object updateValid(Long id, Short valid) {
        if (id == null) {
            return FlowConstant.ISNOTNUL;
        }
        if (FlowConstant.SHORT_ZERO.equals(valid)) {
            List<Long> ids = this.baseMapper.getTopToBottomId(id);
            return update(new UpdateWrapper<BaseType>().lambda().set(BaseType::getIsValid, FlowConstant.SHORT_ZERO)
                    .set(BaseType::getUpdateTime, new Date()).set(BaseType::getUpdateUser, SecurityUtils.getUserId())
                    .eq(BaseType::getIsValid, FlowConstant.SHORT_ONE)
                    .eq(BaseType::getIsDel, FlowConstant.SHORT_ZERO)
                    .in(BaseType::getId, ids));
        } else if (FlowConstant.SHORT_ONE.equals(valid)) {
            List<Long> ids = this.baseMapper.getBottomToTopId(id);
            return update(new UpdateWrapper<BaseType>().lambda().set(BaseType::getIsValid, FlowConstant.SHORT_ONE)
                    .set(BaseType::getUpdateTime, new Date()).set(BaseType::getUpdateUser, SecurityUtils.getUserId())
                    .eq(BaseType::getIsValid, FlowConstant.SHORT_ZERO)
                    .eq(BaseType::getIsDel, FlowConstant.SHORT_ZERO)
                    .in(BaseType::getId, ids));
        } else {
            return FlowConstant.PARAMETER_ERROR;
        }
    }

    @Override
    public List<Long> selectTypeChildById(Long id) {
        return this.baseMapper.selectTypeChildById(id);
    }

    @Override
    public List<BaseType> selectTypeParentById(Long id) {
        return this.baseMapper.selectTypeParentById(id);
    }


    /**
     * 逻辑删除急匆匆类型
     *
     * @param id 主键id
     * @return
     */
    @Override
    public boolean removeUpdateById(Long id) {
        BaseType bt = this.getById(id);
        if(bt==null){
            return false;
        }
        int count = 0;
        List<Long> ids = this.baseMapper.getTopToBottomId(id);
        if (CollUtil.isNotEmpty(ids)) {
            if (bt.getDataType() == FlowConstant.LONG_TWENTY_TWO) {
                count = matterService.count(new QueryWrapper<Matter>().lambda()
                        .in(Matter::getMatterType, ids)
                        .eq(Matter::getIsDel, FlowConstant.SHORT_ZERO));
                if (count != FlowConstant.INTEGER_ZERO) {
                    return false;
                }
            }else {
                BaseType dtbt = this.getById(bt.getDataType());
                if (dtbt.getParentId() == FlowConstant.LONG_ONE) {
                    count = processConfigService.count(new QueryWrapper<ProcessConfig>().lambda()
                            .in(ProcessConfig::getBusinessType, ids).eq(ProcessConfig::getProcessType, bt.getDataType())
                            .eq(ProcessConfig::getIsDel, FlowConstant.SHORT_ZERO));
                    if (count != FlowConstant.INTEGER_ZERO) {
                        return false;
                    }
                }
            }
            return update(new UpdateWrapper<BaseType>().lambda().set(BaseType::getIsDel, FlowConstant.SHORT_ONE)
                    .set(BaseType::getUpdateTime, new Date()).set(BaseType::getUpdateUser, SecurityUtils.getUserId())
                    .eq(BaseType::getIsDel, FlowConstant.SHORT_ZERO)
                    .in(BaseType::getId, ids));
        }
        return false;
    }

    @Override
    public Object removeUpdateByIds(String ids) {
        if (StringUtils.isEmpty(ids)) {
            return FlowConstant.ISNOTNUL;
        }
        try {
            List<Long> lids = Arrays.asList(ids.split(",")).stream().map(st -> Long.parseLong(st)).collect(Collectors.toList());
            lids = this.baseMapper.getTopToBottomIds(lids);
            if (CollUtil.isNotEmpty(lids)) {
                int i = this.baseMapper.update(new BaseType(), new UpdateWrapper<BaseType>().lambda()
                        .set(BaseType::getIsDel, FlowConstant.SHORT_ONE)
                        .eq(BaseType::getIsDel, FlowConstant.SHORT_ZERO)
                        .in(BaseType::getId, lids));
                if (i != 0) {
                    //移除对应物资信息绑定关系
                    matterService.deleteBaseTypeIds(lids);
                }
                return i;
            }
            return false;
        } catch (Exception e) {
            log.error("异常：", e);
            return e.getMessage();
        }
    }

    @Override
    public Object getTypeTree(String datatypes, Long id, Integer type) {
        if (type == FlowConstant.INTEGER_ONE && (StringUtils.isEmpty(datatypes) || !datatypes.contains(FlowConstant.FLAG) || datatypes.trim().length() > 2)) {
            return FlowConstant.DATATYPE_ERROR;
        }
        if (StringUtils.isEmpty(datatypes) && id == null) {
            return FlowConstant.ISNOTNUL;
        }
        try {
            List<Long> lids = null;
            if (StringUtils.isNotEmpty(datatypes)) {
                lids = Arrays.asList(datatypes.split(",")).stream().map(st -> Long.parseLong(st)).collect(Collectors.toList());
            }
            List<BaseTypeVo> lists = null;
            if (type == FlowConstant.INTEGER_ONE) {
                lists = this.baseMapper.getDataTypeNode(FlowConstant.LONG_ZERO, id);

            } else {
                if (id != null) {
                    lists = this.baseMapper.getBaseTypeId(lids, id);
                } else {
                    lists = this.baseMapper.getBaseTypeDataType(lids);
                }
            }
            Map<Long, BaseTypeDataTypeVo> mapdt = new HashMap<>();
            Map<Long, BaseTypeVo> map = new HashMap<>();
            if (CollUtil.isNotEmpty(lists)) {
                for (BaseTypeVo vo : lists) {
                    vo.setVos(new ArrayList<>());
                    if (map.containsKey(vo.getParentId())) {
                        BaseTypeVo tovo = map.get(vo.getParentId());
                        tovo.getVos().add(vo);
                    } else {
                        if (mapdt.containsKey(vo.getDataType())) {
                            BaseTypeDataTypeVo dtvo = mapdt.get(vo.getDataType());
                            dtvo.getVos().add(vo);
                        } else {
                            BaseTypeDataTypeVo dtvo = new BaseTypeDataTypeVo();
                            dtvo.setDataType(vo.getDataType());
                            dtvo.setDataTypeName(vo.getDataTypeName());
                            dtvo.setVos(new ArrayList<>());
                            dtvo.getVos().add(vo);
                            mapdt.put(dtvo.getDataType(), dtvo);
                        }
                    }
                    map.put(vo.getId(), vo);
                }
            }
            return new ArrayList<>(mapdt.values());
        } catch (Exception e) {
            log.error("异常：", e);
            return e.getMessage();
        }
    }


}
