/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ConferenceRoomDto;
import com.tofly.flow.custom.dto.UpdateConferenceRoomDto;
import com.tofly.flow.custom.entity.ConferenceRoom;
import com.tofly.flow.custom.entity.ProcessSupervise;
import com.tofly.flow.custom.service.ConferenceRoomService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ConferenceRoomMapper;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 会议室
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@Service("conferenceRoomService")
public class ConferenceRoomServiceImpl extends ServiceImpl<ConferenceRoomMapper, ConferenceRoom> implements ConferenceRoomService {

    /**
     * 新增会议室
     * @param dto
     * @return
     */
    @Override
    public Object saveConferenceRoom(ConferenceRoomDto dto) {
        int count = this.count(new QueryWrapper<ConferenceRoom>().lambda()
                .eq(ConferenceRoom::getAddress,dto.getAddress()).eq(ConferenceRoom::getRoomName,dto.getRoomName())
                .eq(ConferenceRoom::getRoomNumber,dto.getRoomNumber()).eq(ConferenceRoom::getIsDel, FlowConstant.SHORT_ZERO));
        if(count>0){
            return FlowConstant.DATA_EXISTS;
        }
        ConferenceRoom room = new ConferenceRoom();
        BeanUtils.copyProperties(dto,room);
        room.setIsDel(FlowConstant.SHORT_ZERO);
        return save(room);
    }

    /**
     * 修改会议室信息
     * @param dto
     * @return
     */
    @Override
    public Object updateConferenceRoom(UpdateConferenceRoomDto dto) {

        int count = this.count(new QueryWrapper<ConferenceRoom>().lambda()
                .eq(ConferenceRoom::getAddress,dto.getAddress()).eq(ConferenceRoom::getRoomName,dto.getRoomName())
                .eq(ConferenceRoom::getRoomNumber,dto.getRoomNumber()).eq(ConferenceRoom::getIsDel, FlowConstant.SHORT_ZERO)
                .ne(ConferenceRoom::getId,dto.getId()));
        if(count>0){
            return FlowConstant.DATA_EXISTS;
        }
        ConferenceRoom room = new ConferenceRoom();
        BeanUtils.copyProperties(dto,room);
        return updateById(room);
    }

    /**
     * 删除会议室信息
     * @param id id
     * @return
     */
    @Override
    public boolean removeUpdateById(Long id) {
        return update(new UpdateWrapper<ConferenceRoom>().lambda().set(ConferenceRoom::getIsDel, FlowConstant.SHORT_ONE)
                .eq(ConferenceRoom::getIsDel, FlowConstant.SHORT_ZERO)
                .eq(ConferenceRoom::getId, id));
    }

    @Override
    public Object removeUpdateByIds(String ids) {
        if(StringUtils.isEmpty(ids)){
            return FlowConstant.ISNOTNUL;
        }
        try {
            List<Long> lids= Arrays.asList(ids.split(",")).stream().map(st->Long.parseLong(st)).collect(Collectors.toList());
            if(CollUtil.isNotEmpty(lids)) {
                return this.baseMapper.update(new ConferenceRoom(), new UpdateWrapper<ConferenceRoom>().lambda()
                        .set(ConferenceRoom::getIsDel, FlowConstant.SHORT_ONE)
                        .eq(ConferenceRoom::getIsDel, FlowConstant.SHORT_ZERO)
                        .in(ConferenceRoom::getId, lids));
            }
            return false;
        }catch (Exception e){
            log.error("异常：",e);
            return e.getMessage();
        }
    }


}
