/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.LevelStaffDto;
import com.tofly.flow.custom.dto.PCStaffDto;
import com.tofly.flow.custom.dto.StepStaffDto;
import com.tofly.flow.custom.entity.ConfigStepStaff;
import com.tofly.flow.custom.entity.ProcessConfigStep;
import com.tofly.flow.custom.service.ConfigStepStaffService;
import com.tofly.flow.custom.service.ProcessConfigStepService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ConfigStepStaffMapper;

import java.util.*;

/**
 * 流程配置步骤配置的人员
 *
 * @author xqf
 * @date Mon Mar 21 00:00:00 CST 2022
 */
@Service("configStepStaffService")
public class ConfigStepStaffServiceImpl extends ServiceImpl<ConfigStepStaffMapper, ConfigStepStaff> implements ConfigStepStaffService {
    /**
     * 配置流程步骤
     */
    @Autowired
    private ProcessConfigStepService pStepService;
    /**
     * 验证人员
     * @param staffDtos
     * @param nodeNumber
     * @return
     */
    @Override
    public String checkingStaff(List<PCStaffDto> staffDtos, Short nodeNumber) {
        Set<Short> levels = new HashSet<>();
        Short max = staffDtos.get(0).getLevelNumber();
        List<Long> depts = new ArrayList<>();
        for (PCStaffDto dto:staffDtos) {
            Short num = dto.getLevelNumber();
            if(num==null){
                return FlowConstant.LEVEL_NUMBER_NULL;
            }
            levels.add(num);
            if(max<num){
                max = num;
            }
            if(depts.contains(dto.getDeptId())){
                return FlowConstant.STAFF_DEPT_ERROR;
            }
            if(dto.getStaffId()==null){
                depts.add(dto.getDeptId());
            }
        }
        if(levels.size()!=max||max!=nodeNumber){
            return FlowConstant.LEVE_STAFF_ERROR;
        }
        return null;
    }

    /**
     * 保存人员信息
     * @param dbrs 数据
     * @param id 关联主键
     * @param edition 版本
     * @param staffType 人员类型
     * @return
     */
    @Override
    public boolean saveStaff(List<PCStaffDto> dbrs, Long id, Integer edition,Short staffType) {
        Long index = this.getObj(new QueryWrapper<ConfigStepStaff>().select("MAX(ID) as ID")
                ,(o)->{return Long.parseLong(String.valueOf(o));} );
        if(index==null){
            index=0l;
        }
        List<ConfigStepStaff> lists = new ArrayList<>();
        for (PCStaffDto dto:dbrs) {
            ConfigStepStaff staff = new ConfigStepStaff();
            BeanUtils.copyProperties(dto,staff);
            lists.add(staff);
            staff.setStepId(id);
            staff.setEdition(edition);
            staff.setStaffType(staffType);
            staff.setIsDel(FlowConstant.SHORT_ZERO);
            staff.setId(++index);
        }
        this.baseMapper.saveConfigStepStaff(lists);

        return true;
    }

    /**
     * 逻辑删除人员信息
     * @param oldId 关联主键id
     * @param oldEdition 版本信息
     * @param shortTwo 人员类型
     * @return
     */
    @Override
    public boolean delUpdateStaff(Long oldId, Integer oldEdition, Short shortTwo) {
        return this.update(new UpdateWrapper<ConfigStepStaff>().lambda()
                .eq(ConfigStepStaff::getStepId,oldId).le(ConfigStepStaff::getEdition,oldEdition)
                .eq(ConfigStepStaff::getIsDel,FlowConstant.SHORT_ZERO)
                .set(ConfigStepStaff::getIsDel,FlowConstant.SHORT_ONE));
    }


    /**
     * 逻辑批量根据流程删除人员
     * @param oldId 配置流程id
     * @param oldEdition 版本信息
     * @param shortOne 人员类型
     * @return
     */
    @Override
    public boolean delUpdateLcStaff(Long oldId, Integer oldEdition, Short shortOne) {

        return this.baseMapper.updateLcAllStaff(oldId,oldEdition,shortOne);
    }

    /**
     * 修改步骤处理人员信息
     * @param dto
     * @return
     */
    @Override
    public Object updateConfigStepStaff(StepStaffDto dto) {
        ProcessConfigStep configStep = pStepService.getOne(new QueryWrapper<ProcessConfigStep>().lambda()
                .eq(ProcessConfigStep::getId,dto.getStepId()).eq(ProcessConfigStep::getIsDel,FlowConstant.SHORT_ZERO));
        if(configStep==null){
            return FlowConstant.DATA_NOTHINGNESS;
        }
        List<PCStaffDto> dbrs = new ArrayList<>();
        List<Short> levelnumbers = new ArrayList<>();
        for (LevelStaffDto levelDtos: dto.getLevelDtos() ) {
            if(levelDtos.getLevelNumber()<FlowConstant.SHORT_ONE||levelDtos.getLevelNumber()>configStep.getNodeNumber()){
                return FlowConstant.STAFF_LEVEL_ERROR;
            }
            if(!levelnumbers.contains(levelDtos.getLevelNumber())){
                levelnumbers.add(levelDtos.getLevelNumber());
            }
            for (PCStaffDto pcstaffdto: levelDtos.getStaffDtos()) {
                pcstaffdto.setLevelNumber(levelDtos.getLevelNumber());
                dbrs.add(pcstaffdto);
            }
        }

        this.remove(new QueryWrapper<ConfigStepStaff>().lambda().eq(ConfigStepStaff::getStepId,dto.getStepId()).like(ConfigStepStaff::getLevelNumber,levelnumbers));
        saveStaff(dbrs,dto.getStepId(),configStep.getEdition(),FlowConstant.SHORT_ONE);
        return null;
    }
}
