/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.*;
import com.tofly.flow.custom.entity.Document;
import com.tofly.flow.custom.entity.Process;
import com.tofly.flow.custom.entity.ProcessStaff;
import com.tofly.flow.custom.service.DocumentService;
import com.tofly.flow.custom.service.ProcessRelationService;
import com.tofly.flow.custom.service.ProcessService;
import com.tofly.flow.custom.service.ProcessStaffService;
import com.tofly.flow.custom.vo.RelationInfoVo;
import com.tofly.flow.service.FileService;
import com.tofly.flow.util.SendTidingsUtil;
import com.tofly.flow.util.UploadFiles;
import com.tofly.flow.custom.vo.DocumentVo;
import com.tofly.pojo.message.Messagepush;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.DocumentMapper;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 文档管理
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@Service("documentService")
public class DocumentServiceImpl extends ServiceImpl<DocumentMapper, Document> implements DocumentService {
    /**
     * redis
     */
    @Autowired
    private RedisTemplate redisTemplate;
    /**
     * 文件保存
     */
    @Autowired
    private FileService fileService;

    @Value("${ftp.flow.document}")
    private String documentfile;

    /**
     * 工单流程
     */
    @Autowired
    private ProcessService pService;

    /**
     * 关联人员（会议、报文）
     */
    @Autowired
    private ProcessStaffService pStaffService;

    /**
     * 发送消息
     */
    @Autowired
    private SendTidingsUtil sendTidingsUtil;

    @Autowired
    private ProcessRelationService prsService;

    /**
     * 保存文档
     * @param dto 数据
     * @return
     */
    @Override
    public Long saveDocument(DocumentDto dto, List<MultipartFile> fileList) {
        return saveDocument(dto,null,null,null,fileList);
    }

    /**
     * 校验文档、公告
     * @param dto 文档、公共信息
     * @param processId 流程ID
     * @return
     */
    public String check(DocumentDto dto,Long processId){
        int count = this.count(new QueryWrapper<Document>().lambda()
                .isNull(processId==null,Document::getProcessId)
                .eq(processId!=null,Document::getProcessId,processId)
                .eq(Document::getIsDel,FlowConstant.SHORT_ZERO).eq(Document::getDocumentType,dto.getDocumentType())
                .eq(Document::getTheme,dto.getTheme()));
        if(count>0){
            return FlowConstant.NAME_EXISTS;
        }
        return null;
    }

    /**
     * 修改数据验证
     * @param dto 文档数据
     * @return
     */
    public String updateCheck(UpdateDocumentDto dto){
        Document document = this.getById(dto.getId());
        if(document==null&&document.getIsDel()==FlowConstant.SHORT_ONE){
            return FlowConstant.DATA_NOTHINGNESS;
        }
        int count = this.count(new QueryWrapper<Document>().lambda()
                .isNull(document.getProcessId()==null,Document::getProcessId)
                .eq(document.getProcessId()!=null,Document::getProcessId,document.getProcessId())
                .eq(Document::getIsDel,FlowConstant.SHORT_ZERO).eq(Document::getDocumentType,document.getDocumentType())
                .eq(Document::getTheme,dto.getTheme()).ne(Document::getId,dto.getId()));
        if(count>0){
            return FlowConstant.NAME_EXISTS;
        }
        return null;
    }

    /**
     * 删除文档
     * @param id 主键id
     * @return
     */
    @Override
    public boolean removeUpdateById(Long id) {
        return this.update(new UpdateWrapper<Document>().lambda()
                .eq(Document::getId,id).eq(Document::getIsDel,FlowConstant.SHORT_ZERO)
                .set(Document::getIsDel,FlowConstant.SHORT_ONE));
    }

    /**
     * 批量删除文档
     * @param lids id主键
     * @return
     */
    @Override
    public int removeUpdateByIds(List<Long> lids) {
        return this.baseMapper.update(new Document(), new UpdateWrapper<Document>().lambda()
                .set(Document::getIsDel, FlowConstant.SHORT_ONE)
                .eq(Document::getIsDel, FlowConstant.SHORT_ZERO)
                .in(Document::getId, lids));
    }

    @Override
    public Page<DocumentVo> loadTypeDocumentPage(Page page, LoadFolderDocumentDto dto) {
        return this.baseMapper.loadTypeDocumentPage(page,dto);
    }

    /**
     * 保存校验文档、公告
     * @param dto 文档、公告信息
     * @param map 文件信息
     * @return
     */
    @Override
    public String saveCheck(DocumentFileDto dto, Map<String, MultipartFile> map) {
        List<String> fileNames = dto.getFileNames();
        if(CollUtil.isNotEmpty(fileNames)){
            if(CollUtil.isEmpty(map)){
                return FlowConstant.FILE_ERROR;
            }
            for (String name:fileNames) {
                if(map.containsKey(name)){
                    return FlowConstant.FILE_ERROR;
                }
            }
        }
        return null;
    }

    /**
     * 会议消息通知
     * @param id 流程ID
     */
    @Override
    public void meetingTidings(Long id) {
        Process process = pService.getProcessMeeting(id);
        if(process!=null){
            List<ProcessStaff> staffs= pStaffService.list(new QueryWrapper<ProcessStaff>().lambda()
                    .eq(ProcessStaff::getProcessId,id).eq(ProcessStaff::getIsDel,FlowConstant.SHORT_ZERO)
                    .isNotNull(ProcessStaff::getProcessNode).isNotNull(ProcessStaff::getNodeBranch));
            RelationInfoVo infoVo =prsService.getRelationInfoVo(id,FlowConstant.INTEGER_ONE,FlowConstant.INTEGER_ONE);
            if(CollUtil.isNotEmpty(staffs) && infoVo!=null){
                //组织发送消息
                String topic = process.getName().contains(FlowConstant.MEETING)?process.getName():process.getName()+FlowConstant.MEETING;
                StringBuilder sb = new StringBuilder(topic);
                sb.append(FlowConstant.ADDRESS).append(FlowConstant.COLON).append(infoVo.getFacilitiesVo().getAddress()).append(infoVo.getFacilitiesVo().getName());
                sb.append(FlowConstant.TIME).append(process.getStartTime()).append(FlowConstant.LINE).append(process.getEndTime());

                Messagepush message = sendTidingsUtil.getMessage(id,sb.toString(),topic);
                StringBuilder sbip = new StringBuilder(process.getProcessNumber());
                sbip.append("|").append(FlowConstant.INTEGER_ONE);
                sbip.append(";").append(process.getProcessType()).append("|").append(FlowConstant.INTEGER_ONE);

                message.setRouteIp(sbip.toString());
                message.setRouteIpApp(sbip.toString());
                for (ProcessStaff staff:staffs ) {
                    sendTidingsUtil.setMessageRecipient(message,staff.getDeptStaff());
                }
            }
        }

    }

    @Override
    public DocumentVo getDocumentById(Long id) {
        return this.baseMapper.getDocumentById(id);
    }

    /**
     * 修改文档信息
     * @param dto 文档对象
     * @param fileList 文件
     * @return
     */
    @Override
    public Object updateDocument(UpdateDocumentDto dto, List<MultipartFile> fileList) {
        Document document = new Document();
        BeanUtils.copyProperties(dto,document);
        Document dt = this.getById(dto.getId());
        if(CollUtil.isNotEmpty(fileList)) {
            String filepath = UploadFiles.saveFiles(fileService, fileList, dt.getAnnex(), dto.getAnnex(), documentfile, FlowConstant.TF_YWPN_DOCUMENT_W);
            if(StringUtils.isEmpty(filepath)){
                this.update(new UpdateWrapper<Document>().lambda()
                        .set(Document::getAnnex,filepath).eq(Document::getId,document.getId()));
            }
            document.setAnnex(filepath);
        }
        return updateById(document);
    }

    /**
     * 保存文档
     * @param dto 数据
     * @param processId 流程id
     * @param processNode 流程步骤
     * @param nodeBranch 步骤分支
     * @return
     */
    @Override
    public Long saveDocument(DocumentDto dto,Long processId,Integer processNode,Integer nodeBranch, List<MultipartFile> fileList) {
        if(processId!=null) {
            this.remove(new QueryWrapper<Document>().lambda().eq(Document::getProcessId, processId).eq(Document::getIsDel, FlowConstant.SHORT_ZERO)
                    .eq(Document::getProcessNode, processNode).eq(Document::getNodeBranch, nodeBranch));
        }
        Document document = new Document();
        BeanUtils.copyProperties(dto,document);
        if(CollUtil.isNotEmpty(fileList)) {
            StringBuilder sb = fileService.saveFileList(fileList, documentfile, FlowConstant.TF_YWPN_DOCUMENT_W);
            if(sb==null){
                return FlowConstant.LONG_NEGATIVE_ONE;
            }
            document.setAnnex(sb.toString());
        }
        document.setDocumentNumber(getProcessNumber());
        document.setProcessId(processId);
        document.setProcessNode(processNode);
        document.setNodeBranch(nodeBranch);
        document.setIsDel(FlowConstant.SHORT_ZERO);
        save(document);
        return document.getId();
    }



    /**
     * 获取编码
     * @return
     */
    private String getProcessNumber(){
        String key = FlowConstant.REDIS_KEY_WD+ DateUtil.format(new Date(), "yyyyMMdd");
        RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());
        Long value = counter.incrementAndGet();
        if(value==1){
            String number = this.baseMapper.getDocumentNumber(DateUtil.beginOfDay(new Date()));
            if(StringUtils.isNotEmpty(number)) {
                Long num = Long.parseLong(number.substring(number.length()-5));
                counter.set(num);
                long betweenDay = DateUtil.between(new Date(), DateUtil.endOfDay(new Date()), DateUnit.MINUTE);
                counter.expire(betweenDay, TimeUnit.MINUTES);
                value = counter.incrementAndGet();
            }
        }
        return FlowConstant.WDBM+DateUtil.format(new Date(), "yyyyMMdd")+String.format("%04d", value);
    }
}
