/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.io.ExcelUtil;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.*;
import com.tofly.flow.custom.entity.BaseType;
import com.tofly.flow.custom.entity.ImportInfo;
import com.tofly.flow.custom.entity.Matter;
import com.tofly.flow.custom.service.ImportInfoService;
import com.tofly.flow.custom.vo.MatterVO;
import lombok.SneakyThrows;
import org.springframework.beans.BeanUtils;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ImportInfoMapper;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static com.tofly.flow.custom.service.impl.AssetsServiceImpl.monthDateDiff;
import static com.tofly.flow.custom.service.impl.AssetsServiceImpl.yearDateDiff;

/**
 * 物资导入详情
 *
 * @author xqf
 * @date Fri Apr 08 00:00:00 CST 2022
 */
@Service("importInfoService")
public class ImportInfoServiceImpl extends ServiceImpl<ImportInfoMapper, ImportInfo> implements ImportInfoService {

    @Override
    public void insert(ImportInfoDTO dto) {
        dto.setIsDel((short) 0);
        ImportInfo importInfo = new ImportInfo();
        BeanUtils.copyProperties(dto,importInfo);
        if (!this.save(importInfo)) {
            throw new RuntimeException("新增失败");
        }
    }

    @Override
    public void edit(ImportInfoDTO dto) {
        ImportInfo importInfo = new ImportInfo();
        BeanUtils.copyProperties(dto, importInfo);
        if (!this.updateById(importInfo)) {
            throw new RuntimeException("修改失败");
        }
    }

    @Override
    public void delete(Set<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new RuntimeException("未选择数据ID");
        }
        int update = this.baseMapper.update(new ImportInfo(), new UpdateWrapper<ImportInfo>().lambda()
                .set(ImportInfo::getIsDel, FlowConstant.SHORT_ONE)
                .eq(ImportInfo::getIsDel, FlowConstant.SHORT_ZERO)
                .in(ImportInfo::getId, ids)
        );
        if (update == 0) {
            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public Page<ImportInfo> pageList(Page<ImportInfo> page, SearchImportInfoVo search) {
        QueryWrapper<ImportInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_DEL",FlowConstant.SHORT_ZERO);
        if (!ObjectUtils.isEmpty(search)){
            if (!StringUtils.isEmpty(search.getSearch())){
                queryWrapper.like("MATTE_RNAME",search);
            }
            if (!StringUtils.isEmpty(search.getMatterNum())){
                queryWrapper.eq("MATTER_NUMBER",search.getMatterNum());
            }
            if (search.getDevice()!=null && search.getDevice()){
                queryWrapper.isNotNull("UNIQUE_NUMBER");
            }else if (search.getDevice()!=null && !search.getDevice()){
                queryWrapper.isNull("UNIQUE_NUMBER");
            }
            if (!StringUtils.isEmpty(search.getImportId())){
                queryWrapper.eq("IMPORT_ID",search.getImportId());
            }
            queryWrapper.orderByDesc("CREATE_TIME");
        }
        Page<ImportInfo> infoPage = page(page, queryWrapper);
        if (infoPage.getTotal()>0){
            for (ImportInfo record : infoPage.getRecords()) {
                List<BaseType> baseTypes = this.baseMapper.selectBaseTypeNameByMatterNumberAndMatterName(record.getMatterNumber(), record.getMatterName());
                if (!CollectionUtils.isEmpty(baseTypes)){
                    record.setBaseTypeName(baseTypes.get(0).getTypeName());
                }
                if (record.getInTime()!=null){
                    int yearDateDiff = yearDateDiff(record.getInTime(), new Date());
                    int monthDateDiff = (yearDateDiff * 12) + monthDateDiff(record.getInTime(), new Date());
                    record.setInYear(Math.max(yearDateDiff, 0));
                    record.setInMouth(Math.max(monthDateDiff,0));
                }
            }
        }
        return infoPage;
    }

    @Override
    public void template(HttpServletResponse response) {
        // 获取resource 下的资源
        InputStream resourceAsStream = this.getClass().getClassLoader().getResourceAsStream("report/import/物资录入模板.xlsx");
        assert resourceAsStream != null;
        MatterServiceImpl.download((FileInputStream) resourceAsStream,"物资录入模板.xlsx",response);
    }

    @SneakyThrows
    public void download(File file,HttpServletResponse response) {
        ServletOutputStream out = null;
        FileInputStream in = null;
        try {
            in = new FileInputStream(file);
            String fileName = URLEncoder.encode(file.getName(), "UTF-8");
//            fileName = new String(fileName.getBytes("UTF-8"), "ISO8859-1");
            // 自定义的header
            response.setHeader("requestType", "file");
            // 自定义的header
            response.setHeader("requestFileName", fileName);
            //设置这个header 可见
            response.setHeader("Access-Control-Expose-Headers", "requestType,requestFileName");

//             设置响应类型为html，编码为utf-8，处理相应页面文本显示的乱码
            response.setContentType("application/octet-stream;charset=UTF-8");
//            // 设置文件头：最后一个参数是设置下载文件名
            response.setHeader("Content-disposition", "attachment;fileName=" + fileName);
            out = response.getOutputStream();
            // 读取文件流
            int len = 0;
            byte[] buffer = new byte[1024 * 10];
            while ((len = in.read(buffer)) != -1) {
                out.write(buffer, 0, len);
            }
            out.flush();
        } catch (FileNotFoundException e) {
            throw new RuntimeException("文件下载失败");
        }

    }

    @Override
    public void importMatter(Long importId, MultipartFile file) {
        if (file == null || file.isEmpty()) {
            throw new RuntimeException("文件不存在");
        }
        String filename = file.getOriginalFilename();
        assert filename != null;
        String suffix = filename.substring(filename.lastIndexOf(".")+1);
        if (!"xlsx".equals(suffix)){
            throw new RuntimeException("不支持的文件类型");
        }
        InputStream inputStream = null;
        try {
            inputStream = file.getInputStream();
        } catch (IOException e) {
            e.printStackTrace();
        }
        ExcelUtil<ImportInfoExcelDTO> excelUtil = new ExcelUtil<>(ImportInfoExcelDTO.class);
        List<ImportInfoExcelDTO> list =  new ArrayList<>();
        try {
            list = excelUtil.importExcel("", inputStream);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (CollectionUtils.isEmpty(list)) {
            throw new RuntimeException("导入数据为空");
        }
        for (ImportInfoExcelDTO data : list) {
//            if (data==null){
//                throw new RuntimeException("数据为空");
//            }
//            if (StringUtils.isEmpty(data.getMatterName())) {
//                throw new RuntimeException("物资类型名称不能为空");
//            }
//            if (StringUtils.isEmpty(data.getMatterNumber())) {
//                throw new RuntimeException("物资类型编码不能为空");
//            }
//            MatterDTO matterDTO = new MatterDTO();
//            if (StringUtils.isEmpty(data.getIsAssets()) || "否".equals(data.getIsAssets())){
//                matterDTO.setIsAssets((short)0);
//            }else if ("是".equals(data.getIsAssets())){
//                matterDTO.setIsAssets((short)1);
//            }else {
//                throw new RuntimeException("是否资产填写错误，只能填写是或者否");
//            }
//            if (!StringUtils.isEmpty(data.getCategoryType())){
//                if ("设备类".equals(data.getCategoryType())){
//                    matterDTO.setCategoryType((short)1);
//                }else if("车辆类".equals(data.getCategoryType())){
//                    matterDTO.setCategoryType((short)2);
//                }else if("耗材类".equals(data.getCategoryType())){
//                    matterDTO.setCategoryType((short)3);
//                }else {
//                    throw new RuntimeException("物资类别类型错误，只能是：设备类;车辆类;耗材类");
//                }
//            }
//            matterDTO.setMatterType(baseTypeId);
            ImportInfoDTO importInfo = new ImportInfoDTO();
            BeanUtils.copyProperties(data, importInfo);
            insert(importInfo);
        }
    }

    @Override
    public void export(HttpServletRequest request, HttpServletResponse response) {

    }
}
