/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.io.ExcelUtil;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.AssetsQueryDTO;
import com.tofly.flow.custom.dto.KeepGrainDeviceDTO;
import com.tofly.flow.custom.dto.KeepGrainDeviceExcelDTO;
import com.tofly.flow.custom.entity.*;
import com.tofly.flow.custom.mapper.KeepGrainDeviceMapper;
import com.tofly.flow.custom.service.*;
import com.tofly.flow.custom.vo.KeepGrainDeviceVO;
import com.tofly.flow.custom.vo.KeepGrainMatterVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 仓储仪器设备
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@Service("keepGrainDeviceService")
public class KeepGrainDeviceServiceImpl extends ServiceImpl<KeepGrainDeviceMapper, KeepGrainDevice> implements KeepGrainDeviceService {

    @Autowired
    private MaterialImportService materialImportService;

    @Autowired
    private ImportInfoService importInfoService;

    @Autowired
    MatterService matterService;

    @Autowired
    KeepGrainMatterService keepGrainMatterService;

    @Autowired
    private AssetsService assetsService;

    @Autowired
    private BaseTypeService baseTypeService;

    @Autowired
    ProcessStuffService processStuffService;

    @Autowired
    private RedisTemplate redisTemplate;

    @Override
    public void insert(KeepGrainDeviceDTO dto) {
        KeepGrainDevice keepGrainDevice = new KeepGrainDevice();
        if (dto != null) {
            dto.setIsDel((short) 0);
            dto.setIsValid((short) 1);
        }
        BeanUtils.copyProperties(dto, keepGrainDevice);
        if (!this.save(keepGrainDevice)) {
            throw new RuntimeException("新增失败");
        }
    }

    @Override
    public void edit(KeepGrainDeviceDTO dto) {
        KeepGrainDevice keepGrainDevice = new KeepGrainDevice();
        BeanUtils.copyProperties(dto, keepGrainDevice);
        if (!this.updateById(keepGrainDevice)) {
            throw new RuntimeException("修改失败");
        }
    }

    @Override
    public void delete(Set<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new RuntimeException("未选择数据ID");
        }
        int update = this.baseMapper.update(new KeepGrainDevice(), new UpdateWrapper<KeepGrainDevice>().lambda()
                .set(KeepGrainDevice::getIsDel, FlowConstant.SHORT_ONE)
                .eq(KeepGrainDevice::getIsDel, FlowConstant.SHORT_ZERO)
                .in(KeepGrainDevice::getId, ids)
        );
        if (update == 0) {
            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public Page<KeepGrainDeviceVO> pageList(Page<KeepGrainMatterVO> page, AssetsQueryDTO queryDTO) {
        QueryWrapper<AssetsQueryDTO> queryWrapper = getAssetsQueryDTOQueryWrapper(queryDTO);
        Page<KeepGrainDeviceVO> pageList = baseMapper.pageList(page, queryWrapper);
        getTypeList(pageList);
        return pageList;
    }

    /**
     * 设置分类和模版信息
     *
     * @param pageList
     */
    private void getTypeList(Page<KeepGrainDeviceVO> pageList) {
        if (pageList.getTotal() != 0) {
            for (KeepGrainDeviceVO record : pageList.getRecords()) {
                if (record.getBaseTypeId() != null) {
                    List<BaseType> baseTypes = baseTypeService.selectTypeParentById(record.getBaseTypeId());
                    if (!CollectionUtils.isEmpty(baseTypes)) {
                        record.setBaseTypes(baseTypes);
                        record.setMinTypeName(baseTypes.get(0).getTypeName());
                        record.setBigTypeName(baseTypes.get(baseTypes.size() - 1).getTypeName());
                    }
                }
            }

        }
    }

    /**
     * 设置查询条件
     *
     * @param dto
     * @return
     */
    private QueryWrapper<AssetsQueryDTO> getAssetsQueryDTOQueryWrapper(AssetsQueryDTO dto) {
        QueryWrapper<AssetsQueryDTO> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("device.IS_DEL", FlowConstant.SHORT_ZERO);
        if (dto != null) {
            if (dto.getId() != null) {
                queryWrapper.eq("device.ID", dto.getId());
            }
            if (dto.getTypeId() != null) {
                queryWrapper.eq("matter.MATTER_TYPE", dto.getTypeId());
            }
            if (!StringUtils.isEmpty(dto.getSearch())) {
                queryWrapper.and(query -> {
                    query.like("device.UNIQUE_ID", dto.getSearch()).or()
                            .like("matter.MATTER_NAME", dto.getSearch());
                });
            }
            if (!StringUtils.isEmpty(dto.getStartTime()) && !StringUtils.isEmpty(dto.getEndTime())) {
                queryWrapper.between("device.IN_DATE", dto.getStartTime(), dto.getEndTime());
            }
            if (dto.getMatterId() != null) {
                queryWrapper.eq("matter.ID", dto.getMatterId());
            }
            if (!StringUtils.isEmpty(dto.getIsAssets()) && dto.getIsAssets()) {
                queryWrapper.eq("matter.IS_ASSETS", FlowConstant.SHORT_ONE);
            } else if (!StringUtils.isEmpty(dto.getIsAssets()) && !dto.getIsAssets()) {
                queryWrapper.eq("matter.IS_ASSETS", FlowConstant.SHORT_ZERO);
            }
            queryWrapper.orderByDesc("device.CREATE_TIME");
        }
        return queryWrapper;
    }


    @Override
    public void template(HttpServletResponse response) {
        // 获取resource 下的资源
        InputStream resourceAsStream = this.getClass().getClassLoader().getResourceAsStream("report/import/仓储设备仪器信息模版.xlsx");
        assert resourceAsStream != null;
        MatterServiceImpl.download(resourceAsStream, "仓储设备仪器信息模版.xlsx", response);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void importMatter(MultipartFile file) {
        //excel文件解析判断
        List<KeepGrainDeviceExcelDTO> list = getKeepGrainDeviceExcelDTOS(file);
        //设置总记录数据和详情记录
        MaterialImport materialImport = setRecord(list);
        //分组存储 +  存储数据
        Map<String, List<KeepGrainDeviceExcelDTO>> collect = list.stream().collect(Collectors.
                groupingBy(KeepGrainDeviceExcelDTO::getCodeNum));
        //遍历分组
        for (Map.Entry<String, List<KeepGrainDeviceExcelDTO>> entry : collect.entrySet()) {
            List<KeepGrainDeviceExcelDTO> value = entry.getValue();
            if (!CollectionUtils.isEmpty(value)) {
                //查询物资ID
                Long grainMatterId = matterService.selectGrainMatterByMatterNum(value.get(0).getCodeNum());
                if (grainMatterId == null) {
                    throw new RuntimeException("物资类型编码" + value.get(0).getCodeNum() + "不存在，导入失败");
                }

                QueryWrapper<Matter> wrapper = new QueryWrapper<>();
                wrapper.eq("MATTER_NUMBER", value.get(0).getCodeNum());
                wrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO);
                Matter matter = matterService.getOne(wrapper);
                for (KeepGrainDeviceExcelDTO dto : value) {
                    //记录物资设备车辆记录
                    KeepGrainDevice keepGrainDevice = new KeepGrainDevice();
                    keepGrainDevice.setStuffNumber(materialImport.getImportNumber());
                    keepGrainDevice.setFactoryId(dto.getProductName());
                    keepGrainDevice.setSupplier(dto.getSupply());
                    keepGrainDevice.setPurchaseId(materialImport.getId());
                    keepGrainDevice.setIsDel(FlowConstant.SHORT_ZERO);
                    keepGrainDevice.setIsValid(FlowConstant.SHORT_ONE);
                    keepGrainDevice.setRemark(dto.getRemark());
                    keepGrainDevice.setCategoryType(FlowConstant.SHORT_ONE);
                    keepGrainDevice.setKeepGrainMatterId(grainMatterId);
                    keepGrainDevice.setUniqueId(RandomUtil.randomString(10));
                    keepGrainDevice.setUniqueNumber(dto.getProductCode());
                    keepGrainDevice.setPrice(dto.getUnitPrice());
                    keepGrainDevice.setProduceDate(dto.getProduceTime());
                    keepGrainDevice.setOutDate(dto.getEndTime());
                    keepGrainDevice.setExpireDate(dto.getEndTime());
                    keepGrainDevice.setDeviceState(FlowConstant.SHORT_ONE);
                    keepGrainDevice.setInRemark(dto.getInRemark());
                    keepGrainDevice.setInDate(dto.getInTime());
                    keepGrainDevice.setDataSource(FlowConstant.SHORT_ONE);
                    this.save(keepGrainDevice);
                    //同步到台帐记录
                    Assets assets = new Assets();
                    assets.setMatterId(matter.getId());
                    assets.setAddre(dto.getUseLocation());
                    assets.setAssetsName(matter.getMatterName());
                    assets.setAssetsType(matter.getMatterType());
                    assets.setAssetsNumber(keepGrainMatterService.getProcessNumber("ZC", matter.getMatterNumber()));
                    assets.setKeepGrainDeviceId(keepGrainDevice.getId());
                    assets.setModel(matter.getStandards());
                    assets.setUseType(FlowConstant.SHORT_ONE);
                    assets.setProduceTime(keepGrainDevice.getProduceDate());
                    assets.setBrand(dto.getProductCode());
                    assets.setExpireDate(dto.getEndTime());
                    assets.setPrice(dto.getUnitPrice());
                    assets.setStatus(FlowConstant.SHORT_ZERO);
                    assets.setAssetsState(FlowConstant.SHORT_ONE);
                    assets.setQuantity(1);
                    assets.setDepreciation(0d);
                    assets.setDepreciationType(FlowConstant.SHORT_ZERO);
                    assets.setInDate(dto.getInTime());
                    assets.setUnit(matter.getUnit());
                    assets.setIsDel(FlowConstant.SHORT_ZERO);
                    assets.setIsValid(FlowConstant.SHORT_ONE);
                    assets.setRemark(dto.getRemark());
                    assets.setInRemark(dto.getInRemark());
                    assets.setDeptName(dto.getDeptName());
                    assets.setProduce(dto.getProductName());
                    assets.setSupply(dto.getSupply());
                    assets.setNameplate(dto.getMessage());
                    if (StringUtils.isEmpty(dto.getEmergent()) || !"是".equals(dto.getEmergent())) {
                        assets.setIsEmergency(FlowConstant.SHORT_ZERO);
                    } else {
                        assets.setIsEmergency(FlowConstant.SHORT_ONE);
                    }
                    assetsService.save(assets);
                }
            }
        }


    }

    /**
     * excel文件解析判断
     *
     * @param file 文件
     * @return 返回数据集合
     */
    private List<KeepGrainDeviceExcelDTO> getKeepGrainDeviceExcelDTOS(MultipartFile file) {
        MatterServiceImpl.checkFile(file);
        InputStream inputStream = null;
        try {
            inputStream = file.getInputStream();
        } catch (IOException e) {
            e.printStackTrace();
            throw new RuntimeException("文件解析错误，请确认模版和数据是否正确");
        }
        ExcelUtil<KeepGrainDeviceExcelDTO> excelUtil = new ExcelUtil<>(KeepGrainDeviceExcelDTO.class);
        List<KeepGrainDeviceExcelDTO> list = new ArrayList<>();
        try {
            list = excelUtil.importExcel("", inputStream);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (CollectionUtils.isEmpty(list)) {
            throw new RuntimeException("导入数据为空");
        }
        return list;
    }

    /**
     * 设置总记录数据和详情记录
     *
     * @param list 记录数据
     */
    private MaterialImport setRecord(List<KeepGrainDeviceExcelDTO> list) {
        //检验数据
        list.forEach(k -> {
            if (StringUtils.isEmpty(k.getCodeNum())) {
                throw new RuntimeException("模版错误或物资类型编码");
            }
            if (StringUtils.isEmpty(k.getProductCode())) {
                throw new RuntimeException("模版错误或物资出厂唯一编码不能为空");
            }
            if (StringUtils.isEmpty(k.getMessage()) && StringUtils.isEmpty(k.getSupply()) &&
                    StringUtils.isEmpty(k.getProductName()) && StringUtils.isEmpty(k.getProduceTime())) {
                throw new RuntimeException("模板不正确，请下载正确模板");
            }
            if (StringUtils.isEmpty(k.getUnitPrice())) {
                throw new RuntimeException("采购单价不能为空");
            }
            //判断唯一编码是否重复
            QueryWrapper<KeepGrainDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO)
                    .eq("UNIQUE_NUMBER", k.getProductCode());
            int count = count(queryWrapper);
            if (count > 0) {
                throw new RuntimeException("已存在该资产，无法导入");
            }
        });

        //添加导入记录数
        MaterialImport materialImport = new MaterialImport();
        materialImport.setDataCount(list.size());
        materialImport.setImportType(FlowConstant.SHORT_ONE);
        materialImport.setIsDel(FlowConstant.SHORT_ZERO);
        materialImport.setImportNumber(KeepGrainMatterServiceImpl.createNum());
        materialImportService.save(materialImport);

        //分组存储
        Map<String, List<KeepGrainDeviceExcelDTO>> collect = list.stream().collect(Collectors.
                groupingBy(KeepGrainDeviceExcelDTO::getCodeNum));
        //遍历分组
        for (Map.Entry<String, List<KeepGrainDeviceExcelDTO>> entry : collect.entrySet()) {
            String key = entry.getKey();
            List<KeepGrainDeviceExcelDTO> value = entry.getValue();
            if (!CollectionUtils.isEmpty(value)) {

                //记录物料信息
                QueryWrapper<Matter> wrapper = new QueryWrapper<>();
                wrapper.eq("MATTER_NUMBER", value.get(0).getCodeNum());
                wrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO);
                Matter matter = matterService.getOne(wrapper);

                if (ObjectUtils.isEmpty(matter)) {
                    throw new RuntimeException("物资类型编码：" + value.get(0).getCodeNum() + "系统中不存在，导入失败");
                }
                //设置物料统计数据
                //查询物料ID是否存在,不存在则没有记录物料信息，新增
                QueryWrapper<KeepGrainMatter> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("MATTER_ID", matter.getId()).eq("IS_DEL", FlowConstant.SHORT_ZERO);
                List<KeepGrainMatter> grainMatters = keepGrainMatterService.list(queryWrapper);
                KeepGrainMatter keepGrainMatter = new KeepGrainMatter();
                if (CollectionUtils.isEmpty(grainMatters)) {
                    //新增物料统计信息
                    keepGrainMatter = new KeepGrainMatter();
                    keepGrainMatter.setMatterState(FlowConstant.SHORT_ONE);
                    keepGrainMatter.setKeepGrainNumber(keepGrainMatterService.getProcessNumber(matter.getMatterNumber()));
                    keepGrainMatter.setMatterId(matter.getId());
                    keepGrainMatter.setInQuantity((long) value.size());
                    keepGrainMatter.setOutQuantity(0L);
                    if (StringUtils.isEmpty(value.get(0).getEmergent()) || !"是".equals(value.get(0).getEmergent())) {
                        keepGrainMatter.setIsEmergency(FlowConstant.SHORT_ZERO);
                    } else {
                        keepGrainMatter.setIsEmergency(FlowConstant.SHORT_ONE);
                    }
                    keepGrainMatter.setQuantity((long) value.size());
                    keepGrainMatter.setPurchaseTotal(value.get(0).getUnitPrice() * value.size());
                    keepGrainMatter.setUnitPrice(value.get(0).getUnitPrice());
                    keepGrainMatter.setStorageRoom(value.get(0).getLocation());
                    keepGrainMatter.setPosition(value.get(0).getLocationDetail());
                    keepGrainMatter.setMatterState(FlowConstant.SHORT_ONE);
                    keepGrainMatter.setIsEmergency(FlowConstant.SHORT_ZERO);
                    keepGrainMatter.setNewTime(value.get(0).getInTime());
                    keepGrainMatter.setRecentlyExpired(value.get(0).getEndTime());
                    keepGrainMatter.setIsValid(FlowConstant.SHORT_ONE);
                    keepGrainMatter.setIsDel(FlowConstant.SHORT_ZERO);
                    keepGrainMatter.setRemark(value.get(0).getRemark());
                    keepGrainMatter.setDataSource(FlowConstant.SHORT_ONE);
                    keepGrainMatter.setEmptionDate(value.get(0).getInTime());
                    keepGrainMatterService.save(keepGrainMatter);
                } else {
                    BeanUtils.copyProperties(grainMatters.get(0), keepGrainMatter);

                    keepGrainMatter.setInQuantity(keepGrainMatter.getInQuantity() + (long) value.size());
                    keepGrainMatter.setOutQuantity(0L);
                    if (StringUtils.isEmpty(value.get(0).getEmergent()) || !"是".equals(value.get(0).getEmergent())) {
                        keepGrainMatter.setIsEmergency(FlowConstant.SHORT_ZERO);
                    } else {
                        keepGrainMatter.setIsEmergency(FlowConstant.SHORT_ONE);
                    }
                    keepGrainMatter.setQuantity(keepGrainMatter.getQuantity() + (long) value.size());
                    keepGrainMatter.setPurchaseTotal(keepGrainMatter.getPurchaseTotal() + value.get(0).getUnitPrice() * value.size());
                    keepGrainMatter.setUnitPrice(value.get(0).getUnitPrice());
                    keepGrainMatter.setCreateUserName(null);
                    keepGrainMatter.setCreateUser(null);
                    keepGrainMatter.setUpdateUserName(null);
                    keepGrainMatter.setDataSource(FlowConstant.SHORT_ONE);
                    keepGrainMatter.setEmptionDate(value.get(0).getInTime());
                    keepGrainMatterService.updateById(keepGrainMatter);
                }
                //记录详情

                for (KeepGrainDeviceExcelDTO data : value) {
                    if (data == null) {
                        throw new RuntimeException("数据为空");
                    }
                    if (StringUtils.isEmpty(data.getCodeNum())) {
                        throw new RuntimeException("物资类型编码不能为空");
                    }
                    if (StringUtils.isEmpty(data.getProductCode())) {
                        throw new RuntimeException("物资出厂唯一编码不能为空");
                    }
                    if (StringUtils.isEmpty(data.getMessage())) {
                        throw new RuntimeException("铭牌信息不能为空");
                    }
                    ImportInfo importInfo = new ImportInfo();
                    importInfo.setMatterId(matter.getId());
                    importInfo.setImportId(materialImport.getId());
                    importInfo.setIsDel(FlowConstant.SHORT_ZERO);
                    importInfo.setDept(data.getDeptName());
                    importInfo.setOutDate(data.getEndTime());
                    importInfo.setFactoryId(data.getProductName());
                    importInfo.setPosition(data.getLocationDetail());
                    importInfo.setUniqueNumber(data.getProductCode());
                    importInfo.setMatterNumber(matter.getMatterNumber());
                    importInfo.setRemark(data.getRemark());
                    importInfo.setInRemark(data.getInRemark());
                    importInfo.setProduceDate(data.getProduceTime());
                    importInfo.setExpireDate(data.getEndTime());
                    importInfo.setUnitPrice(data.getUnitPrice());
                    importInfo.setStandards(matter.getStandards());
                    importInfo.setUnit(matter.getUnit());
                    importInfo.setMatterName(matter.getMatterName());
                    importInfo.setStorageRoom(data.getLocation());
                    importInfo.setSupplier(data.getSupply());
                    importInfo.setQuantity(1);
                    importInfo.setInTime(data.getInTime());
                    importInfoService.save(importInfo);

                    //记录入库信息列表
                    ProcessStuff processStuff = new ProcessStuff();
                    processStuff.setStuffNumber(materialImport.getImportNumber());
                    processStuff.setRemark(data.getRemark());
                    processStuff.setInDate(data.getInTime());
                    processStuff.setExpireDate(data.getEndTime());
                    processStuff.setDataType(FlowConstant.LONG_ZERO);
                    processStuff.setUnitPrice(data.getUnitPrice());
                    processStuff.setProcessId(materialImport.getId());
                    processStuff.setQuantity(FlowConstant.LONG_ONE);
                    processStuff.setAmountMoney(data.getUnitPrice());
                    processStuff.setMatterState(FlowConstant.SHORT_ONE);
                    processStuff.setIsDel(FlowConstant.SHORT_ZERO);
                    processStuff.setMatterId(matter.getId());
                    processStuff.setPosition(data.getLocationDetail());
                    processStuff.setStorageRoom(data.getLocation());
                    //记录入库
                    processStuffService.save(processStuff);
                }
            }
        }
        return materialImport;
    }


    /**
     * 设置详情信息
     *
     * @param value
     * @param materialImport
     * @param importInfoService
     */
//    static void setDetailData(List<KeepGrainDeviceExcelDTO> value, MaterialImport materialImport,
//                              ImportInfoService importInfoService, Matter matter, ProcessStuff processStuff) {
//        for (KeepGrainDeviceExcelDTO data : value) {
//            if (data == null) {
//                throw new RuntimeException("数据为空");
//            }
//            if (StringUtils.isEmpty(data.getCodeNum())) {
//                throw new RuntimeException("物资类型编码不能为空");
//            }
//            if (StringUtils.isEmpty(data.getProductCode())) {
//                throw new RuntimeException("物资出厂唯一编码不能为空");
//            }
//            if (StringUtils.isEmpty(data.getMessage())) {
//                throw new RuntimeException("铭牌信息不能为空");
//            }
//            ImportInfo importInfo = new ImportInfo();
//            importInfo.setImportId(materialImport.getId());
//            importInfo.setIsDel(FlowConstant.SHORT_ZERO);
//            importInfo.setDept(data.getDeptName());
//            importInfo.setOutDate(data.getEndTime());
//            importInfo.setFactoryId(data.getProductName());
//            importInfo.setPosition(data.getLocationDetail());
//            importInfo.setUniqueNumber(data.getProductCode());
//            importInfo.setMatterNumber(matter.getMatterNumber());
//            importInfo.setRemark(data.getRemark());
//            importInfo.setInRemark(data.getInRemark());
//            importInfo.setProduceDate(data.getProduceTime());
//            importInfo.setExpireDate(data.getEndTime());
//            importInfo.setUnitPrice(data.getUnitPrice());
//            importInfo.setStandards(matter.getStandards());
//            importInfo.setUnit(matter.getUnit());
//            importInfo.setMatterName(matter.getMatterName());
//            importInfo.setStorageRoom(data.getLocation());
//            importInfo.setSupplier(data.getSupply());
//            importInfo.setQuantity(1);
//            importInfoService.save(importInfo);
//            importInfo.setInTime(data.getInTime());
//            //记录入库信息列表
//            processStuff.setStuffNumber(materialImport.getImportNumber());
//            processStuff.setRemark(data.getRemark());
//            processStuff.setInDate(data.getInTime());
//            processStuff.setExpireDate(data.getEndTime());
//            processStuff.setDataType(Long.valueOf(FlowConstant.SHORT_ONE));
//            processStuff.setUnitPrice(data.getUnitPrice());
//            processStuff.setProcessId(materialImport.getId());
//            processStuff.setQuantity(1L);
//            processStuff.setAmountMoney(data.getUnitPrice());
//            processStuff.setMatterState(FlowConstant.SHORT_ONE);
//            processStuff.setIsDel(FlowConstant.SHORT_ZERO);
//            processStuff.setMatterId(matter.getId());
//            processStuff.setPosition(data.getLocationDetail());
//            processStuff.setStorageRoom(data.getLocation());
//        }
//    }

    /**
     * 设置物料统计数据
     *
     * @param list
     * @param value
     * @param matter
     */
    private void setKeepGrainMatterData(List<KeepGrainDeviceExcelDTO> list, List<KeepGrainDeviceExcelDTO> value, Matter matter) {
        //查询物料ID是否存在,不存在则没有记录物料信息，新增
        Long grainMatterId = matterService.selectGrainMatterByMatterNum(value.get(0).getCodeNum());
        KeepGrainMatter keepGrainMatter;
        if (grainMatterId == null) {
            //新增物料统计信息
            keepGrainMatter = new KeepGrainMatter();
            keepGrainMatter.setMatterState(FlowConstant.SHORT_ONE);
            keepGrainMatter.setKeepGrainNumber(keepGrainMatterService.getProcessNumber(matter.getMatterNumber()));
            keepGrainMatter.setMatterId(matter.getId());
            keepGrainMatter.setInQuantity((long) list.size());
            keepGrainMatter.setOutQuantity(0L);
            if (StringUtils.isEmpty(value.get(0).getEmergent()) || !"是".equals(value.get(0).getEmergent())) {
                keepGrainMatter.setIsEmergency(FlowConstant.SHORT_ZERO);
            } else {
                keepGrainMatter.setIsEmergency(FlowConstant.SHORT_ONE);
            }
            keepGrainMatter.setQuantity((long) list.size());
            keepGrainMatter.setPurchaseTotal(value.get(0).getUnitPrice() * list.size());
            keepGrainMatter.setUnitPrice(value.get(0).getUnitPrice());
            keepGrainMatter.setStorageRoom(value.get(0).getLocation());
            keepGrainMatter.setPosition(value.get(0).getLocationDetail());
            keepGrainMatter.setMatterState(FlowConstant.SHORT_ONE);
            keepGrainMatter.setIsEmergency(FlowConstant.SHORT_ZERO);
            keepGrainMatter.setNewTime(value.get(0).getInTime());
            keepGrainMatter.setRecentlyExpired(value.get(0).getEndTime());
            keepGrainMatter.setIsValid(FlowConstant.SHORT_ONE);
            keepGrainMatter.setIsDel(FlowConstant.SHORT_ZERO);
            keepGrainMatter.setRemark(value.get(0).getRemark());
            keepGrainMatter.setDataSource(FlowConstant.SHORT_ONE);
            keepGrainMatter.setEmptionDate(value.get(0).getInTime());
            keepGrainMatterService.save(keepGrainMatter);
        } else {
            keepGrainMatter = keepGrainMatterService.getById(grainMatterId);
            keepGrainMatter.setInQuantity((long) list.size());
            keepGrainMatter.setOutQuantity(0L);
            if (StringUtils.isEmpty(value.get(0).getEmergent()) || !"是".equals(value.get(0).getEmergent())) {
                keepGrainMatter.setIsEmergency(FlowConstant.SHORT_ZERO);
            } else {
                keepGrainMatter.setIsEmergency(FlowConstant.SHORT_ONE);
            }
            keepGrainMatter.setQuantity(keepGrainMatter.getQuantity() + (long) list.size());
            keepGrainMatter.setPurchaseTotal(keepGrainMatter.getPurchaseTotal() + value.get(0).getUnitPrice() * list.size());
            keepGrainMatter.setUnitPrice(value.get(0).getUnitPrice());
            keepGrainMatter.setCreateUserName(null);
            keepGrainMatter.setCreateUser(null);
            keepGrainMatter.setUpdateUserName(null);
            keepGrainMatter.setDataSource(FlowConstant.SHORT_ONE);
            keepGrainMatter.setEmptionDate(value.get(0).getInTime());
            keepGrainMatterService.updateById(keepGrainMatter);
        }

    }

    @Override
    public void export(HttpServletRequest request, HttpServletResponse response, AssetsQueryDTO queryDTO) {
//        //查询数据
//        QueryWrapper<AssetsQueryDTO> queryWrapper = getAssetsQueryDTOQueryWrapper(queryDTO);
//        List<KeepGrainDeviceVO> keepGrainDeviceVOS = this.baseMapper.voList(queryWrapper);
//        if (CollectionUtils.isEmpty(keepGrainDeviceVOS)){
//            throw new RuntimeException("数据为空");
//        }
//        for (KeepGrainDeviceVO vo : keepGrainDeviceVOS) {
//            if (vo.getIsAssets()!=null && vo.getIsAssets().equals(FlowConstant.SHORT_ZERO)){
//                vo.setIsAssetsStr("否");
//            }else if (vo.getIsAssets().equals(FlowConstant.SHORT_ONE)){
//                vo.setIsAssetsStr("是");
//            }
//            if (vo.getCategoryType()!=null && vo.getCategoryType().equals(FlowConstant.SHORT_ZERO)){
//                vo.setCategoryTypeStr("设备类");
//            }else if (vo.getCategoryType().equals(FlowConstant.SHORT_TWO)){
//                vo.setCategoryTypeStr("车辆类");
//            }else if (vo.getCategoryType().equals(FlowConstant.SHORT_THREE)){
//                vo.setCategoryTypeStr("耗材类");
//            }
//        }
//        // 根据templateServiceType,templateFlag获取对应模板数据
//        ExcelExportFileTypeEnum excelExportFileTypeEnum = ExcelExportFileTypeEnum.
//                getExcelExportFileTypeEnum("exportService", "2");
//        if (null == excelExportFileTypeEnum) {
//            throw new ToflyDeniedException("导出失败");
//        }
//        //将查询的结果转json字符串的格式，因为导出的时候只能传一个字符串
//        String resultListToString = JSONArray.toJSONStringWithDateFormat(keepGrainMatterVOS, "yyyy-MM-dd HH:mm:ss");
//        // 从模型类型枚举根据templateServiceType获取策略对应模板数据
//        List<Map<String, Object>> dataList = excelStrategyContext
//                .getExportData("exportService", "1", resultListToString);
//        com.tofly.flow.util.ExcelUtil.export(dataList.get(0), excelExportFileTypeEnum, Constant.FILE_PATH_TEST_EXPORT_EXCEL, response);
    }

    /**
     * 批量存储或修改仓库设备数据
     *
     * @param start 出入库状态0入库，1出库
     * @param kgds  设备信息数据
     * @return
     */
    @Override
    public Boolean insertOrUpdateDevice(int start, List<KeepGrainDevice> kgds) {
        if (start == FlowConstant.INTEGER_ZERO) {

            for (KeepGrainDevice kgd : kgds) {
                kgd.setUniqueId(getProcessNumber());
                kgd.setCreateUser(FlowConstant.LONG_ONE);
                kgd.setCreateTime(new Date());
            }
            this.baseMapper.saveBatchKeepGrainDevice(kgds);
        } else if (start == FlowConstant.INTEGER_ONE) {
            this.updateBatchById(kgds);
        }
        return true;
    }

    @Override
    public Boolean updateDevice(List<KeepGrainDevice> ukgds) {
        assetsService.updateBatchByDeviceId(ukgds);
        return this.updateBatchById(ukgds);
    }

    @Override
    public List<KeepGrainDevice> selectMatterIdByMaterialImportId(Long importId) {
        return this.baseMapper.selectMatterIdByMaterialImportId(importId);
    }


    /**
     * 获取编码
     *
     * @return
     */
    private String getProcessNumber() {

        String key = FlowConstant.REDIS_KEY_DEVICE + DateUtil.format(new Date(), "yyyyMMdd");
        RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());
        Long value = counter.incrementAndGet();
        if (value == 1) {
            String number = this.getObj(new QueryWrapper<KeepGrainDevice>().select("MAX(UNIQUE_ID) as ID").lambda()
                            .ge(KeepGrainDevice::getCreateTime, DateUtil.beginOfDay(new Date()))
                    , (o) -> {
                        return String.valueOf(o);
                    });
            if (org.apache.commons.lang3.StringUtils.isNotEmpty(number)) {
                Long num = Long.parseLong(number.substring(number.length() - 5));
                counter.set(num);
                long betweenDay = DateUtil.between(new Date(), DateUtil.endOfDay(new Date()), DateUnit.MINUTE);
                counter.expire(betweenDay, TimeUnit.MINUTES);
                value = counter.incrementAndGet();
            }
        }
        return DateUtil.format(new Date(), "yyyyMMdd") + String.format("%05d", value);
    }
}
