/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.haiqiu.tools.excel.utils.HqExcelUtils;
import com.tofly.common.core.util.io.ExcelUtil;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.AssetsQueryDTO;
import com.tofly.flow.custom.dto.KeepGrainDeviceDTO;
import com.tofly.flow.custom.dto.KeepGrainMatterImportExcelDTO;
import com.tofly.flow.custom.entity.Process;
import com.tofly.flow.custom.entity.*;
import com.tofly.flow.custom.mapper.KeepGrainMatterMapper;
import com.tofly.flow.custom.service.*;
import com.tofly.flow.custom.vo.KeepGrainMatterVO;
import com.tofly.flow.custom.vo.ProcessDeviceVo;
import com.tofly.flow.custom.vo.ProcessStuffVo;
import com.tofly.flow.strategy.ExcelStrategyContext;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 仓储物料信息
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@Service("keepGrainMatterService")
public class KeepGrainMatterServiceImpl extends ServiceImpl<KeepGrainMatterMapper, KeepGrainMatter> implements KeepGrainMatterService {

    @Autowired
    private MaterialImportService materialImportService;

    @Autowired
    private ImportInfoService importInfoService;

    @Autowired
    private MatterService matterService;

    @Autowired
    private BaseTypeService baseTypeService;

    @Autowired
    private KeepGrainMatterService keepGrainMatterService;

    @Autowired
    private ExcelStrategyContext excelStrategyContext;

    @Autowired
    private ProcessStuffService processStuffService;

    @Autowired
    private KeepGrainDeviceService kGDService;

    @Autowired
    private AssetsService assetsService;

    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private ProcessService processService;


    @Override
    public void insert(KeepGrainDeviceDTO dto) {

    }

    @Override
    public void edit(KeepGrainDeviceDTO dto) {

    }

    @Override
    public void delete(Set<Long> ids) {

    }

    @Override
    public Page<KeepGrainMatterVO> pageList(Page<KeepGrainMatterVO> page, AssetsQueryDTO queryDTO) {
        QueryWrapper<AssetsQueryDTO> queryWrapper = getAssetsQueryDTOQueryWrapper(queryDTO);
        Page<KeepGrainMatterVO> pageList = baseMapper.pageList(page, queryWrapper);
        return pageList;
    }

    /**
     * 设置查询条件
     *
     * @param dto
     * @return
     */
    private QueryWrapper<AssetsQueryDTO> getAssetsQueryDTOQueryWrapper(AssetsQueryDTO dto) {
        QueryWrapper<AssetsQueryDTO> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("grain.IS_DEL", FlowConstant.SHORT_ZERO);
        if (dto != null) {
            if (dto.getId() != null) {
                queryWrapper.eq("grain.ID", dto.getId());
            }
            if (dto.getTypeId() != null) {
                queryWrapper.eq("matter.MATTER_TYPE", dto.getTypeId());
            }
            if (dto.getCategoryType() != null) {
                queryWrapper.eq("matter.CATEGORY_TYPE", dto.getCategoryType());
            }
            if (!StringUtils.isEmpty(dto.getSearch())) {
                queryWrapper.and(query -> {
                    query.like("grain.KEEP_GRAIN_NUMBER", dto.getSearch()).or()
                            .like("matter.MATTER_NAME", dto.getSearch());
                });
            }
            if (!StringUtils.isEmpty(dto.getStartBuyTime()) && !StringUtils.isEmpty(dto.getEndBuyTime())) {
                queryWrapper.between("grain.EMPTION_DATE", dto.getStartBuyTime(), dto.getEndBuyTime());
            }
            if (!StringUtils.isEmpty(dto.getIsAssets()) && dto.getIsAssets()) {
                queryWrapper.eq("matter.IS_ASSETS", FlowConstant.SHORT_ONE);
            } else if (!StringUtils.isEmpty(dto.getIsAssets()) && !dto.getIsAssets()) {
                queryWrapper.eq("matter.IS_ASSETS", FlowConstant.SHORT_ZERO);
            }
            queryWrapper.orderByDesc("grain.CREATE_TIME");
        }
        return queryWrapper;
    }


    @Override
    public void template(HttpServletResponse response) {
        // 获取resource 下的资源
        InputStream resourceAsStream = this.getClass().getClassLoader().getResourceAsStream("report/import/仓储物料信息模版.xlsx");
        assert resourceAsStream != null;
        MatterServiceImpl.download(resourceAsStream, "仓储物料信息模版.xlsx", response);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void importMatter(MultipartFile file) {
        //excel文件解析判断
        List<KeepGrainMatterImportExcelDTO> list = getKeepGrainExcelDTOS(file);
        //设置总记录数据和详情记录
        setRecord(list);
    }

    @Override
    public KeepGrainMatterVO get(Long keepGrainMatterId) {
        AssetsQueryDTO queryDTO = new AssetsQueryDTO();
        queryDTO.setId(keepGrainMatterId);
        Page<KeepGrainMatterVO> keepGrainDeviceVOPage = pageList(new Page<>(1, 1), queryDTO);
        if (keepGrainDeviceVOPage.getTotal() > 0) {
            return keepGrainDeviceVOPage.getRecords().get(0);
        }
        return null;
    }

    /**
     * excel文件解析判断
     *
     * @param file 文件
     * @return 返回数据集合
     */
    private List<KeepGrainMatterImportExcelDTO> getKeepGrainExcelDTOS(MultipartFile file) {
        MatterServiceImpl.checkFile(file);
        InputStream inputStream = null;
        try {
            inputStream = file.getInputStream();
        } catch (IOException e) {
            e.printStackTrace();
            throw new RuntimeException("文件解析错误，请确认模版和数据是否正确");
        }
        ExcelUtil<KeepGrainMatterImportExcelDTO> excelUtil = new ExcelUtil<>(KeepGrainMatterImportExcelDTO.class);
        List<KeepGrainMatterImportExcelDTO> list = new ArrayList<>();
        try {
            list = excelUtil.importExcel("", inputStream);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (CollectionUtils.isEmpty(list)) {
            throw new RuntimeException("导入数据为空");
        }
        return list;
    }

    /**
     * 设置总记录数据和详情记录
     *
     * @param list 记录数据
     */
    private void setRecord(List<KeepGrainMatterImportExcelDTO> list) {
        //检验数据
        list.forEach(k -> {
            if (StringUtils.isEmpty(k.getCodeNum())) {
                throw new RuntimeException("物资类型编码不能为空或模板错误");
            }
            if (StringUtils.isEmpty(k.getUnitPrice())) {
                throw new RuntimeException("采购单价不能为空");
            }
            if (StringUtils.isEmpty(k.getCount())) {
                throw new RuntimeException("入库数量不能为空或模版错误");
            }
            if (k.getCount() <= FlowConstant.INTEGER_ZERO) {
                throw new RuntimeException("入库数量不能等于0或模板错误");
            }
            if (k.getCount() > 10000000) {
                throw new RuntimeException("入库数量不能等于或超过10000000");
            }
        });
        //分组存储
        Map<String, List<KeepGrainMatterImportExcelDTO>> collect = list.stream().collect(Collectors.
                groupingBy(KeepGrainMatterImportExcelDTO::getCodeNum));

        //添加导入记录数
        //计算总数
        Iterator<KeepGrainMatterImportExcelDTO> iterator1 = list.iterator();
        int importSum = 0;
        while (iterator1.hasNext()) {
            KeepGrainMatterImportExcelDTO next = iterator1.next();
            if (next.getCount() != null) {
                importSum = importSum + next.getCount();
            }
        }
        MaterialImport materialImport = new MaterialImport();
        materialImport.setDataCount(importSum);
        materialImport.setImportType(FlowConstant.SHORT_TWO);
        materialImport.setIsDel(FlowConstant.SHORT_ZERO);
        materialImport.setImportNumber(createNum());
        materialImportService.save(materialImport);

        //遍历分组
        for (Map.Entry<String, List<KeepGrainMatterImportExcelDTO>> entry : collect.entrySet()) {
            String key = entry.getKey();
            List<KeepGrainMatterImportExcelDTO> value = entry.getValue();
            if (!CollectionUtils.isEmpty(value)) {
                //计算总数
                Iterator<KeepGrainMatterImportExcelDTO> iterator = value.iterator();
                int sum = 0;
                while (iterator.hasNext()) {
                    KeepGrainMatterImportExcelDTO next = iterator.next();
                    if (next.getCount() != null) {
                        sum = sum + next.getCount();
                    }
                }


                //记录物料信息
                QueryWrapper<Matter> wrapper = new QueryWrapper<>();
                wrapper.eq("MATTER_NUMBER", value.get(0).getCodeNum());
                wrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO);
                Matter matter = matterService.getOne(wrapper);

                if (ObjectUtils.isEmpty(matter)) {
                    throw new RuntimeException("物资类型编码：" + value.get(0).getCodeNum() + "系统中不存在，导入失败");
                }
                //设置物料统计数据
                //查询物料ID是否存在,不存在则没有记录物料信息，新增
                QueryWrapper<KeepGrainMatter> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("MATTER_ID", matter.getId()).eq("IS_DEL", FlowConstant.SHORT_ZERO);
                List<KeepGrainMatter> grainMatters = keepGrainMatterService.list(queryWrapper);
                KeepGrainMatter keepGrainMatter = new KeepGrainMatter();
                if (CollectionUtils.isEmpty(grainMatters)) {
                    //新增物料统计信息
                    keepGrainMatter.setMatterState(FlowConstant.SHORT_ONE);
                    keepGrainMatter.setKeepGrainNumber(getProcessNumber(matter.getMatterNumber()));
                    keepGrainMatter.setMatterId(matter.getId());
                    keepGrainMatter.setInQuantity(sum == 0 ? (long) value.size() : (long) sum);
                    keepGrainMatter.setOutQuantity(0L);
                    keepGrainMatter.setQuantity(sum == 0 ? (long) value.size() : (long) sum);
                    keepGrainMatter.setPurchaseTotal(value.get(0).getUnitPrice() * sum);
                    keepGrainMatter.setUnitPrice(value.get(0).getUnitPrice());
                    keepGrainMatter.setStorageRoom(value.get(0).getLocation());
                    keepGrainMatter.setPosition(value.get(0).getLocationDetail());
                    keepGrainMatter.setMatterState(FlowConstant.SHORT_ONE);
                    keepGrainMatter.setIsEmergency(FlowConstant.SHORT_ZERO);
                    keepGrainMatter.setNewTime(value.get(0).getInTime());
                    keepGrainMatter.setRecentlyExpired(value.get(0).getEndTime());
                    keepGrainMatter.setIsValid(FlowConstant.SHORT_ONE);
                    keepGrainMatter.setIsDel(FlowConstant.SHORT_ZERO);
                    keepGrainMatter.setRemark(value.get(0).getInRemark());
                    keepGrainMatter.setDataSource(FlowConstant.SHORT_ONE);
                    keepGrainMatter.setEmptionDate(value.get(0).getInTime());
                    keepGrainMatterService.save(keepGrainMatter);
                } else {
                    BeanUtils.copyProperties(grainMatters.get(0), keepGrainMatter);

                    keepGrainMatter.setInQuantity(keepGrainMatter.getInQuantity() + (sum == 0 ? (long) value.size() : (long) sum));
                    keepGrainMatter.setOutQuantity(0L);
                    keepGrainMatter.setQuantity(keepGrainMatter.getQuantity() + (sum == 0 ? (long) value.size() : (long) sum));
                    keepGrainMatter.setPurchaseTotal(keepGrainMatter.getPurchaseTotal() + (value.get(0).getUnitPrice() * sum));
                    keepGrainMatter.setUnitPrice(value.get(0).getUnitPrice());
                    keepGrainMatter.setCreateUserName(null);
                    keepGrainMatter.setCreateUser(null);
                    keepGrainMatter.setUpdateUserName(null);
                    keepGrainMatter.setDataSource(FlowConstant.SHORT_ONE);
                    keepGrainMatter.setEmptionDate(value.get(0).getInTime());
                    keepGrainMatterService.updateById(keepGrainMatter);
                }
                //记录详情
                for (KeepGrainMatterImportExcelDTO data : value) {
                    if (data == null) {
                        throw new RuntimeException("数据为空");
                    }
                    if (StringUtils.isEmpty(data.getCodeNum())) {
                        throw new RuntimeException("物资类型编码不能为空");
                    }
                    ImportInfo importInfo = new ImportInfo();
                    importInfo.setMatterId(matter.getId());
                    importInfo.setImportId(materialImport.getId());
                    importInfo.setIsDel(FlowConstant.SHORT_ZERO);
                    importInfo.setPosition(data.getLocationDetail());
                    importInfo.setMatterNumber(matter.getMatterNumber());
                    importInfo.setInRemark(data.getInRemark());
                    importInfo.setExpireDate(data.getEndTime());
                    importInfo.setUnitPrice(data.getUnitPrice());
                    importInfo.setStandards(matter.getStandards());
                    importInfo.setUnit(matter.getUnit());
                    importInfo.setMatterName(matter.getMatterName());
                    importInfo.setStorageRoom(data.getLocation());
                    importInfo.setQuantity(data.getCount());
                    importInfo.setInTime(data.getInTime());
                    importInfoService.save(importInfo);
                    //记录入库信息列表
                    ProcessStuff processStuff = new ProcessStuff();
                    processStuff.setStuffNumber(materialImport.getImportNumber());
                    processStuff.setRemark(data.getInRemark());
                    processStuff.setInDate(data.getInTime());
                    processStuff.setExpireDate(data.getEndTime());
                    processStuff.setDataType(FlowConstant.LONG_ZERO);
                    processStuff.setUnitPrice(data.getUnitPrice());
                    processStuff.setProcessId(materialImport.getId());
                    processStuff.setQuantity((long) data.getCount());
                    processStuff.setAmountMoney(data.getUnitPrice());
                    processStuff.setMatterState(FlowConstant.SHORT_ONE);
                    processStuff.setIsDel(FlowConstant.SHORT_ZERO);
                    processStuff.setMatterId(matter.getId());
                    processStuff.setPosition(data.getLocationDetail());
                    processStuff.setStorageRoom(data.getLocation());
                    //记录入库
                    processStuffService.save(processStuff);
                }
            }

        }
    }

    /**
     * 创建编码
     * @return
     */
    public static String createNum(){
       return  "RKPC" +
                DateUtil.format(new Date(), "yyyyMMdd") +
                RandomUtil.randomNumbers(4);
    }


    /**
     * 生成编码
     * @param prefix 前缀
     * @return
     */
    public static String createNum(String prefix){
        return  prefix +
                DateUtil.format(new Date(), "yyyyMMdd") +
                RandomUtil.randomNumbers(4);
    }

    @Override
    public void export(HttpServletRequest request, HttpServletResponse response, AssetsQueryDTO queryDTO) {
        //查询数据
        QueryWrapper<AssetsQueryDTO> queryWrapper = getAssetsQueryDTOQueryWrapper(queryDTO);
        List<KeepGrainMatterVO> keepGrainMatterVOS = this.baseMapper.voList(queryWrapper);
        if (CollectionUtils.isEmpty(keepGrainMatterVOS)) {
            throw new RuntimeException("数据为空");
        }
        HqExcelUtils.getListToExcel(KeepGrainMatterVO.class,keepGrainMatterVOS,"物资台账");
//        for (KeepGrainMatterVO vo : keepGrainMatterVOS) {
//            if (vo.getIsAssets() != null && vo.getIsAssets().equals(FlowConstant.SHORT_ZERO)) {
//                vo.setIsAssetsStr("否");
//            } else if (vo.getIsAssets().equals(FlowConstant.SHORT_ONE)) {
//                vo.setIsAssetsStr("是");
//            }
//            if (vo.getCategoryType() != null && vo.getCategoryType().equals(FlowConstant.SHORT_ZERO)) {
//                vo.setCategoryTypeStr("设备类");
//            } else if (vo.getCategoryType().equals(FlowConstant.SHORT_TWO)) {
//                vo.setCategoryTypeStr("车辆类");
//            } else if (vo.getCategoryType().equals(FlowConstant.SHORT_THREE)) {
//                vo.setCategoryTypeStr("耗材类");
//            }
//            if (vo.getDataSource() != null) {
//                if (vo.getDataSource().equals(FlowConstant.SHORT_ONE)) {
//                    vo.setDataSourceStr("导入");
//                } else if (vo.getDataSource().equals(FlowConstant.SHORT_TWO)) {
//                    vo.setDataSourceStr("入库");
//                }
//            }
//        }
//        // 根据templateServiceType,templateFlag获取对应模板数据
//        ExcelExportFileTypeEnum excelExportFileTypeEnum = ExcelExportFileTypeEnum.
//                getExcelExportFileTypeEnum("exportService", "2");
//        if (null == excelExportFileTypeEnum) {
//            throw new ToflyDeniedException("导出失败");
//        }
//        //将查询的结果转json字符串的格式，因为导出的时候只能传一个字符串
//        String resultListToString = JSONArray.toJSONStringWithDateFormat(keepGrainMatterVOS, "yyyy-MM-dd HH:mm:ss");
//        // 从模型类型枚举根据templateServiceType获取策略对应模板数据
//        List<Map<String, Object>> dataList = excelStrategyContext
//                .getExportData("exportService", "1", resultListToString);
//        com.tofly.flow.util.ExcelUtil.export(dataList.get(0), excelExportFileTypeEnum, Constant.FILE_PATH_TEST_EXPORT_EXCEL, response);
    }

    @Override
    public KeepGrainMatter getBymatterId(Long matterId) {
        QueryWrapper<KeepGrainMatter> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO).eq("MATTER_ID", matterId);
        Page<KeepGrainMatter> page = keepGrainMatterService.page(new Page<>(1, 1), queryWrapper);
        if (page.getTotal() == 0) {
            return null;
        } else {
            return page.getRecords().get(0);
        }
    }

    /**
     * 同步流程出入库物资数据
     *
     * @param stuffVos  物资信息
     * @param start     出入库状态0入库，1出库
     * @param processId 工单ID
     * @return
     */
    @Override
    public Boolean synchroKeepGrainMatter(List<ProcessStuffVo> stuffVos, int start, Long processId) {
        List<Long> matterids = stuffVos.stream().map(r -> r.getMatterId()).collect(Collectors.toList());

        Set<Long> typeid = stuffVos.stream().map(r -> r.getMatterNodeVo().getMatterType()).collect(Collectors.toSet());

        Map<Long, BaseType> btmap = baseTypeService.list(new QueryWrapper<BaseType>().lambda().in(BaseType::getId, typeid)
                .eq(BaseType::getIsDel, FlowConstant.SHORT_ZERO)).stream().collect(Collectors.toMap(BaseType::getId, v -> v));

        Map<Long, KeepGrainMatter> kgmmap = this.list(new QueryWrapper<KeepGrainMatter>().lambda().eq(KeepGrainMatter::getIsDel, FlowConstant.SHORT_ZERO)
                .in(KeepGrainMatter::getMatterId, matterids)).stream().collect(Collectors.toMap(KeepGrainMatter::getMatterId, r -> r));
        Long index = this.getObj(new QueryWrapper<KeepGrainMatter>().select("MAX(ID) as ID")
                , (o) -> {
                    return Long.parseLong(String.valueOf(o));
                });
        if (index == null) {
            index = 0l;
        }
        Long dindex = kGDService.getObj(new QueryWrapper<KeepGrainDevice>().select("MAX(ID) as ID")
                , (o) -> {
                    return Long.parseLong(String.valueOf(o));
                });

        if (dindex == null) {
            dindex = 0l;
        }
        List<KeepGrainMatter> ins = new ArrayList<>();
        List<KeepGrainMatter> ups = new ArrayList<>();
        List<KeepGrainDevice> kgds = new ArrayList<>();
        List<KeepGrainDevice> ukgds = new ArrayList<>();
        List<Assets> zcs = new ArrayList<>();

        Process process = processService.getById(processId);

        for (ProcessStuffVo vo : stuffVos) {
            String typeCode = btmap.get(vo.getMatterNodeVo().getMatterType()).getTypeCode();
            KeepGrainMatter kgm = new KeepGrainMatter();
            Short categoryType = vo.getMatterNodeVo().getCategoryType();
            kgm.setNewTime(vo.getInDate());
            kgm.setMatterId(vo.getMatterId());
            kgm.setCreateUser(FlowConstant.LONG_ONE);
            kgm.setCreateTime(new Date());
            kgm.setUpdateTime(new Date());
            kgm.setUpdateUser(FlowConstant.LONG_ONE);
            if (kgmmap.containsKey(vo.getMatterId())) {
                KeepGrainMatter okgm = kgmmap.get(vo.getMatterId());
                kgm.setId(okgm.getId());
                //入库
                if (start == FlowConstant.INTEGER_ZERO) {
                    kgm.setInQuantity(okgm.getInQuantity() + vo.getQuantity());
                    kgm.setUnitPrice(vo.getUnitPrice());
                    kgm.setPurchaseTotal(okgm.getPurchaseTotal() + vo.getAmountMoney());
                    kgm.setQuantity(okgm.getQuantity() + vo.getQuantity());
                    kgm.setEmptionDate(vo.getEmptionDate());
                    kgm.setRecentlyExpired(vo.getExpireDate());
                    kgm.setDataSource(FlowConstant.SHORT_TWO);
                    if (!StringUtils.isEmpty(vo.getStorageRoom()) && !okgm.getStorageRoom().contains(vo.getStorageRoom())) {
                        kgm.setStorageRoom(okgm.getStorageRoom() + FlowConstant.CSV + vo.getStorageRoom());
                    }
                    if (!StringUtils.isEmpty(vo.getPosition()) && !okgm.getPosition().contains(vo.getPosition())) {
                        kgm.setPosition(okgm.getPosition() + FlowConstant.CSV + vo.getPosition());
                    }
                    if (categoryType == FlowConstant.SHORT_ONE || categoryType == FlowConstant.SHORT_TWO) {
                        if (CollUtil.isNotEmpty(vo.getDeviceVos())) {
                            for (ProcessDeviceVo dvo : vo.getDeviceVos()) {
                                KeepGrainDevice kg = new KeepGrainDevice();
                                BeanUtils.copyProperties(dvo, kg);
                                kg.setId(++dindex);
                                fillKeepGrainDevice(processId, vo, kgm.getId(), zcs, kg, typeCode);
                                kgds.add(kg);
                            }
                        } else {
                            dindex = fillKeepGrainDeviceList(processId, dindex, kgds, zcs, vo, typeCode, kgm);
                        }
                    }
                    //退库
                } else if (start == FlowConstant.INTEGER_ONE) {
                    kgm.setQuantity(kgm.getQuantity() + vo.getQuantity());
                    kgm.setOutQuantity(kgm.getOutQuantity() - vo.getQuantity());
                    if (categoryType == FlowConstant.SHORT_ONE || categoryType == FlowConstant.SHORT_TWO) {
                        if (CollUtil.isNotEmpty(vo.getDeviceVos())) {
                            List<Long> dids = new ArrayList<>();
                            for (ProcessDeviceVo dvo : vo.getDeviceVos()) {
                                if (dvo.getDeviceState() == FlowConstant.SHORT_THREE || dvo.getDeviceState() == FlowConstant.SHORT_FIVE) {
                                    kgm.setQuantity(kgm.getQuantity() - 1);
                                    kgm.setOutQuantity(kgm.getOutQuantity() + 1);
                                }
                                KeepGrainDevice kg = new KeepGrainDevice();
                                if (dvo.getKeepGrainDeviceId() == null) {
                                    BeanUtils.copyProperties(dvo, kg);
                                    kg.setId(++dindex);
                                    fillKeepGrainDevice(processId, vo, kgm.getId(), zcs, kg, typeCode);
                                    kgds.add(kg);
                                } else {
                                    kg.setKeepGrainMatterId(kgm.getId());
                                    kg.setId(dvo.getKeepGrainDeviceId());
                                    kg.setDeviceState(dvo.getDeviceState());
                                    dids.add(dvo.getKeepGrainDeviceId());
                                    ukgds.add(kg);
                                }
                            }
                            if (CollUtil.isNotEmpty(dids)) {
                                int count = kGDService.count(new QueryWrapper<KeepGrainDevice>().lambda().eq(KeepGrainDevice::getKeepGrainMatterId, kgm.getId())
                                        .in(KeepGrainDevice::getId, dids).eq(KeepGrainDevice::getIsDel, FlowConstant.SHORT_ZERO));
                                if (count != dids.size()) {
                                    return false;
                                }
                            }
                        }
                    }
                    //出库
                } else if (start == FlowConstant.INTEGER_TWO) {
                    kgm.setQuantity(kgm.getQuantity() - vo.getQuantity());
                    kgm.setOutQuantity(kgm.getOutQuantity() + vo.getQuantity());
                    if (categoryType == FlowConstant.SHORT_ONE || categoryType == FlowConstant.SHORT_TWO) {
                        if (CollUtil.isNotEmpty(vo.getDeviceVos())) {
                            for (ProcessDeviceVo dvo : vo.getDeviceVos()) {
                                KeepGrainDevice kg = new KeepGrainDevice();
                                //没有设备信息不允许保存
                                if (dvo.getKeepGrainDeviceId() == null) {
                                    return false;
                                }
                                kg.setId(dvo.getKeepGrainDeviceId());
                                kg.setOutDate(vo.getInDate());
                                kg.setDeviceState(dvo.getDeviceState());
                                kg.setUniqueNumber(dvo.getUniqueNumber());
                                kg.setStreamId(processId);
                                kgds.add(kg);
                                if (FlowConstant.SHORT_ONE == vo.getMatterNodeVo().getIsAssets()) {
                                    Assets assets = new Assets();
                                    assets.setKeepGrainDeviceId(dvo.getKeepGrainDeviceId());
                                    assets.setAssetsState(FlowConstant.SHORT_ONE);
                                    assets.setUseDept(process.getUseDept());
                                    assets.setPersonLiable(process.getPersonLiable());
                                    zcs.add(assets);
                                }
                            }
                        }
                    }
                }
                ups.add(kgm);
            } else {
                //入库
                if (start == FlowConstant.INTEGER_ZERO || start == FlowConstant.INTEGER_ONE) {
                    KeepGrainMatter okgm = new KeepGrainMatter();
                    BeanUtils.copyProperties(vo, okgm);
                    kgm.setId(++index);
                    dindex = fillKeepGrainMatter(processId, dindex, kgds, zcs, vo, kgm, typeCode);
                    ins.add(kgm);
                } else if (start == FlowConstant.INTEGER_TWO) {
                    return false;
                }
            }
        }

        if (CollUtil.isNotEmpty(ups)) {
            this.updateBatchById(ups);

        }
        if (CollUtil.isNotEmpty(ins)) {
            this.baseMapper.insertBatch(ins);
        }

        if (CollUtil.isNotEmpty(ukgds)) {
            kGDService.updateDevice(ukgds);
        }
        if (CollUtil.isNotEmpty(kgds)) {
            //入库新增、出库修改
            kGDService.insertOrUpdateDevice(start, kgds);
        }
        if (CollUtil.isNotEmpty(zcs)) {
            assetsService.saveAssetes(start, zcs);
        }
        return true;
    }

    /**
     * 填充设备集合
     *
     * @param processId 流程ID
     * @param dindex    主键id增列
     * @param kgds      设备集合
     * @param zcs       资产集合
     * @param vo        上传资产信息
     * @param typeCode  类型CODE
     * @param kgm       入库物资
     * @return 增长id
     */
    private Long fillKeepGrainDeviceList(Long processId, Long dindex, List<KeepGrainDevice> kgds, List<Assets> zcs, ProcessStuffVo vo, String typeCode, KeepGrainMatter kgm) {
        Long quantity = vo.getQuantity();
        while (quantity > 0) {
            KeepGrainDevice kg = new KeepGrainDevice();
            kg.setDeviceState(FlowConstant.SHORT_ONE);
            kg.setId(++dindex);
            fillKeepGrainDevice(processId, vo, kgm.getId(), zcs, kg, typeCode);
            kgds.add(kg);
            quantity--;
        }
        return dindex;
    }

    /**
     * 填充仓库物资信息
     *
     * @param processId 流程ID
     * @param dindex    设备ID
     * @param kgds      保存仓库设备对象集合
     * @param zcs       资产集合
     * @param vo        流程物资
     * @param kgm       仓库物资
     * @param typeCode  物资类型code
     * @return 设备ID
     */
    private Long fillKeepGrainMatter(Long processId, Long dindex, List<KeepGrainDevice> kgds, List<Assets> zcs, ProcessStuffVo vo, KeepGrainMatter kgm, String typeCode) {
        kgm.setInQuantity(vo.getQuantity());
        kgm.setKeepGrainNumber(getProcessNumber(typeCode));
        kgm.setNewTime(vo.getInDate());
        kgm.setPurchaseTotal(vo.getAmountMoney());
        kgm.setUnitPrice(vo.getUnitPrice());
        kgm.setRecentlyExpired(vo.getExpireDate());
        kgm.setOutQuantity(FlowConstant.LONG_ZERO);
        kgm.setQuantity(vo.getQuantity());
        kgm.setDataSource(FlowConstant.SHORT_TWO);
        kgm.setIsValid(FlowConstant.SHORT_ONE);
        kgm.setIsDel(FlowConstant.SHORT_ZERO);
        kgm.setStorageRoom(vo.getStorageRoom());
        kgm.setPosition(vo.getPosition());
        kgm.setMatterState(FlowConstant.SHORT_ONE);
        kgm.setCreateUser(FlowConstant.LONG_ONE);
        kgm.setIsEmergency(FlowConstant.SHORT_ZERO);
        kgm.setCreateTime(new Date());
        Short categoryType = vo.getMatterNodeVo().getCategoryType();
        if (categoryType == FlowConstant.SHORT_ONE || categoryType == FlowConstant.SHORT_TWO) {
            if (CollUtil.isNotEmpty(vo.getDeviceVos())) {
                for (ProcessDeviceVo dvo : vo.getDeviceVos()) {
                    KeepGrainDevice kg = new KeepGrainDevice();
                    BeanUtils.copyProperties(dvo, kg);
                    kg.setId(++dindex);
                    if (dvo.getDeviceState() == FlowConstant.SHORT_THREE || dvo.getDeviceState() == FlowConstant.SHORT_FIVE) {
                        kgm.setQuantity(kgm.getQuantity() - 1);
                    }
                    fillKeepGrainDevice(processId, vo, kgm.getId(), zcs, kg, typeCode);
                    kgds.add(kg);
                }
            } else {
                dindex = fillKeepGrainDeviceList(processId, dindex, kgds, zcs, vo, typeCode, kgm);
            }
        }
        return dindex;
    }


    /**
     * 填充仓库设备信息
     *
     * @param processId         流程ID
     * @param vo                流程出入库物资
     * @param keepGrainMatterId 物资ID
     * @param zcs               资产集合
     * @param typecode          物资类型code
     * @param kg
     */
    private void fillKeepGrainDevice(Long processId, ProcessStuffVo vo, Long keepGrainMatterId, List<Assets> zcs, KeepGrainDevice kg, String typecode) {
        kg.setPurchaseId(processId);
        kg.setEmptionDate(vo.getEmptionDate());
        kg.setExpireDate(vo.getExpireDate());
        kg.setInDate(vo.getInDate());
        kg.setKeepGrainMatterId(keepGrainMatterId);
        kg.setIsValid(FlowConstant.SHORT_ONE);
        kg.setIsDel(FlowConstant.SHORT_ZERO);
        kg.setCategoryType(vo.getMatterNodeVo().getCategoryType());
        kg.setDataSource(FlowConstant.SHORT_TWO);
        if (FlowConstant.SHORT_ONE == vo.getMatterNodeVo().getIsAssets()) {
            Assets assets = new Assets();
            BeanUtils.copyProperties(kg, assets);
            assets.setAssetsNumber(assetsService.getAssetsNumber(typecode));
            assets.setMatterId(vo.getMatterNodeVo().getId());
            assets.setKeepGrainDeviceId(kg.getId());
            assets.setAssetsType(vo.getMatterNodeVo().getMatterType());
            assets.setAssetsName(vo.getMatterNodeVo().getMatterName());
            assets.setModel(vo.getMatterNodeVo().getStandards());
            assets.setBrand(kg.getUniqueNumber());
            assets.setNumberOne(kg.getPartsNumberOne());
            assets.setNumberTwo(kg.getPartsNumberTwo());
            assets.setNumberThree(kg.getPartsNumberThree());
            assets.setBuyTime(kg.getEmptionDate());
            if (FlowConstant.SHORT_ONE == kg.getDeviceState()) {
                assets.setStatus(FlowConstant.SHORT_ONE);
                assets.setAssetsState(FlowConstant.SHORT_ZERO);
            } else if (FlowConstant.SHORT_TWO == kg.getDeviceState()) {
                assets.setStatus(FlowConstant.SHORT_TWO);
                assets.setAssetsState(FlowConstant.SHORT_ZERO);
            } else {
                assets.setStatus(FlowConstant.SHORT_THREE);
                assets.setAssetsState(FlowConstant.SHORT_TWO);
            }
            assets.setUnit(vo.getMatterNodeVo().getUnit());
            assets.setQuantity(FlowConstant.INTEGER_ONE);
            assets.setProduce(vo.getFactoryName());
            zcs.add(assets);
        }
    }


    /**
     * 获取物资编码
     *
     * @param wztype 物资类型
     * @return 物资编码
     */
    public String getProcessNumber(String wztype) {
        return getProcessNumber(FlowConstant.WZ, wztype);
    }


    /**
     * 获取物资编码
     *
     * @param wz     物资前缀
     * @param wztype 物资类型
     * @return 物资编码
     */
    public String getProcessNumber(String wz, String wztype) {
        if (StringUtils.isEmpty(wz)) {
            wz = FlowConstant.WZ;
        }
        String code = wz + wztype;

        String key = FlowConstant.PROCESS_WZ_REDIS_KEY + code + DateUtil.format(new Date(), "yyyyMMdd");
        RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());
        Long value = counter.incrementAndGet();
        if (value == 1) {
            String number = this.getObj(new QueryWrapper<KeepGrainMatter>().select("MAX(KEEP_GRAIN_NUMBER) as ID").lambda()
                            .ge(KeepGrainMatter::getCreateTime, DateUtil.beginOfDay(new Date()))
                    , (o) -> {
                        return String.valueOf(o);
                    });
            if (org.apache.commons.lang3.StringUtils.isNotEmpty(number)) {
                Long num = Long.parseLong(number.substring(number.length() - 5));
                counter.set(num);
                long betweenDay = DateUtil.between(new Date(), DateUtil.endOfDay(new Date()), DateUnit.MINUTE);
                counter.expire(betweenDay, TimeUnit.MINUTES);
                value = counter.incrementAndGet();
            }
        }
        return code + DateUtil.format(new Date(), "yyyyMMdd") + String.format("%05d", value);
    }
}
