/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ImportInfoDTO;
import com.tofly.flow.custom.entity.*;
import com.tofly.flow.custom.mapper.MaterialImportMapper;
import com.tofly.flow.custom.service.*;
import com.tofly.flow.custom.vo.MaterialImportVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 物资导入
 *
 * @author xqf
 * @date Fri Apr 08 00:00:00 CST 2022
 */
@Service("materialImportService")
public class MaterialImportServiceImpl extends ServiceImpl<MaterialImportMapper, MaterialImport> implements MaterialImportService {


    @Autowired
    private ImportInfoService importInfoService;

    @Autowired
    private KeepGrainDeviceService keepGrainDeviceService;

    @Autowired
    private KeepGrainMatterService keepGrainMatterService;
    ;

    @Autowired
    private MatterService matterService;

    @Autowired
    private AssetsService assetsService;

    @Autowired
    private ProcessStuffService processStuffService;

    @Override
    public void insert(ImportInfoDTO dto) {
        MaterialImport importInfo = new MaterialImport();
        BeanUtils.copyProperties(dto, importInfo);
        importInfo.setIsDel(FlowConstant.SHORT_ZERO);
        if (!this.save(importInfo)) {
            throw new RuntimeException("新增失败");
        }
    }

    @Override
    public void edit(ImportInfoDTO dto) {
        MaterialImport importInfo = new MaterialImport();
        BeanUtils.copyProperties(dto, importInfo);
        if (!this.updateById(importInfo)) {
            throw new RuntimeException("修改失败");
        }
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(Set<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new RuntimeException("未选择数据ID");
        }
        QueryWrapper<MaterialImport> query = new QueryWrapper<>();
        query.eq("IS_DEL", FlowConstant.SHORT_ZERO).in("ID", ids);
        List<MaterialImport> imports = list(query);

        int update = this.baseMapper.update(new MaterialImport(), new UpdateWrapper<MaterialImport>().lambda()
                .set(MaterialImport::getIsDel, FlowConstant.SHORT_ONE)
                .eq(MaterialImport::getIsDel, FlowConstant.SHORT_ZERO)
                .in(MaterialImport::getId, ids)
        );
        if (update == 0) {
            throw new RuntimeException("删除失败");
        }
        //查询详情
        QueryWrapper<ImportInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO).in("IMPORT_ID", ids);
        List<ImportInfo> importInfos = importInfoService.list(queryWrapper);
        Set<Long> matterIds = importInfos.stream().filter(importInfo -> importInfo.getMatterId() != null).map(ImportInfo::getMatterId)
                .collect(Collectors.toSet());
        //查询物料统计信息
        QueryWrapper<KeepGrainMatter> grainMatterQueryWrapper = new QueryWrapper<>();
        grainMatterQueryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO)
                //.eq("DATA_SOURCE", FlowConstant.SHORT_ONE)
                .in("MATTER_ID", matterIds);
        List<KeepGrainMatter> keepGrainMatters = keepGrainMatterService.list(grainMatterQueryWrapper);
        //动态修改库存量和价格
        for (ImportInfo importInfo : importInfos) {
            for (KeepGrainMatter keepGrainMatter : keepGrainMatters) {
                if (importInfo.getMatterId().equals(keepGrainMatter.getMatterId())) {
                    keepGrainMatter.setInQuantity(Math.max(keepGrainMatter.getInQuantity() - (long) importInfo.getQuantity(), 0));
                    keepGrainMatter.setQuantity(Math.max(keepGrainMatter.getQuantity() - (long) importInfo.getQuantity(), 0));
                    keepGrainMatter.setPurchaseTotal(Math.max(keepGrainMatter.getPurchaseTotal() -
                            importInfo.getQuantity() * importInfo.getUnitPrice(), 0));
                    keepGrainMatter.setCreateUser(null);
                    keepGrainMatter.setCreateUserName(null);
                    keepGrainMatter.setUpdateUserName(null);
                    keepGrainMatter.setUpdateUser(null);
                    keepGrainMatterService.updateById(keepGrainMatter);
                }
            }
        }
        //删除绑定的数据
        deleteOther(imports);
    }

    /**
     * 获取物资模版信息，删除其他绑定信息
     *
     * @param imports 导入记录
     */
    private void deleteOther(List<MaterialImport> imports) {
        Set<Long> importIds = imports.stream().map(MaterialImport::getId).collect(Collectors.toSet());
        //物资明细数据
        QueryWrapper<KeepGrainDevice> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.in("PURCHASE_ID", importIds).eq("IS_DEL", FlowConstant.SHORT_ZERO);
                //.eq("DATA_SOURCE", FlowConstant.SHORT_ONE);
        List<KeepGrainDevice> keepGrainDevices = keepGrainDeviceService.list(queryWrapper1);
        Set<Long> deviceIds = keepGrainDevices.stream().filter(keepGrainDevice -> keepGrainDevice.getId() != null)
                .map(KeepGrainDevice::getId).collect(Collectors.toSet());
        keepGrainDeviceService.update(new KeepGrainDevice(), new UpdateWrapper<KeepGrainDevice>().lambda()
                .eq(KeepGrainDevice::getIsDel, FlowConstant.SHORT_ZERO)
                .eq(KeepGrainDevice::getDataSource, FlowConstant.SHORT_ONE)
                .set(KeepGrainDevice::getIsDel, FlowConstant.SHORT_ONE)
                .in(KeepGrainDevice::getPurchaseId, importIds));


        //物资入库数据
        processStuffService.update(new ProcessStuff(), new UpdateWrapper<ProcessStuff>().lambda()
                .eq(ProcessStuff::getIsDel, FlowConstant.SHORT_ZERO)
                .eq(ProcessStuff::getDataType, FlowConstant.SHORT_ZERO)
                .set(ProcessStuff::getIsDel, FlowConstant.SHORT_ONE)
                .in(ProcessStuff::getMatterId, importIds));


        //导入数据数据
        importInfoService.update(new ImportInfo(), new UpdateWrapper<ImportInfo>().lambda()
                .eq(ImportInfo::getIsDel, FlowConstant.SHORT_ZERO)
                .set(ImportInfo::getIsDel, FlowConstant.SHORT_ONE)
                .in(ImportInfo::getImportId, importIds));

        //固定资产数据
        if (!CollectionUtils.isEmpty(deviceIds)) {
            assetsService.update(new Assets(), new UpdateWrapper<Assets>().lambda()
                    .eq(Assets::getIsDel, FlowConstant.SHORT_ZERO)
                    .set(Assets::getIsDel, FlowConstant.SHORT_ONE)
                    .in(Assets::getKeepGrainDeviceId, deviceIds));
        }

    }


    @Override
    public Page pageList(Page page, String search) {
        QueryWrapper<MaterialImport> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO);
        if (!StringUtils.isEmpty(search)) {
            queryWrapper.and(query -> {
                query.like("IMPORT_NUMBER", search)
                        .or()
                        .like("CREATE_USER", search);
            });
        }
        queryWrapper.orderByDesc("CREATE_TIME");
        return page(page, queryWrapper);
    }

    @Override
    public void template(HttpServletResponse response) {

    }

    @Override
    public void importMatter(Long baseTypeId, MultipartFile file) {

    }

    @Override
    public void export(HttpServletRequest request, HttpServletResponse response) {

    }

    @Override
    public MaterialImport selectByImportNumber(String importNumber) {
        QueryWrapper<MaterialImport> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IMPORT_NUMBER", importNumber);
        MaterialImport materialImport = this.getOne(queryWrapper);
        return materialImport;
    }


    @Override
    public Object checkDelete(Long importId) {
        HashMap<String, Object> map = new HashMap<>(6);

        //物资明细数据：删除1个。
        QueryWrapper<KeepGrainDevice> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("PURCHASE_ID", importId).eq("IS_DEL", FlowConstant.SHORT_ZERO)
                .in("DATA_SOURCE", FlowConstant.SHORT_ONE,FlowConstant.SHORT_ZERO);
        int detailCount1 = keepGrainDeviceService.count(queryWrapper1);
        List<KeepGrainDevice> keepGrainDevices = keepGrainDeviceService.list(queryWrapper1);
        Set<Long> deviceIds = keepGrainDevices.stream().filter(keepGrainDevice -> keepGrainDevice.getId() != null)
                .map(KeepGrainDevice::getId).collect(Collectors.toSet());
        map.put("detailData", "物资明细数据：删除" + detailCount1 + "个");

        //物资入库数据：删除2个。
        QueryWrapper<ProcessStuff> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("PROCESS_ID", importId).eq("IS_DEL", FlowConstant.SHORT_ZERO)
                .in("DATA_TYPE", FlowConstant.SHORT_ZERO, FlowConstant.SHORT_ONE);
        int detailCount2 = processStuffService.count(queryWrapper2);
        map.put("into", "物资入库数据：删除" + detailCount2 + "个");

        //导入数据数据：删除10个。
        QueryWrapper<ImportInfo> queryWrapper3 = new QueryWrapper<ImportInfo>();
        queryWrapper3.in("IMPORT_ID", importId).eq("IS_DEL", FlowConstant.SHORT_ZERO);
        int detailCount3 = importInfoService.count(queryWrapper3);
        map.put("import", "导入数据数据：删除" + detailCount3 + "个");

        //固定资产数据：删除0个。
        if (CollectionUtils.isEmpty(deviceIds)){
            map.put("fixed", "固定资产数据：删除0个");
        }else {
            QueryWrapper<Assets> queryWrapper4 = new QueryWrapper<>();
            queryWrapper4.in("KEEP_GRAIN_DEVICE_ID", deviceIds).eq("IS_DEL", FlowConstant.SHORT_ZERO);
            int detailCount4 = assetsService.count(queryWrapper4);
            map.put("fixed", "固定资产数据：删除" + detailCount4 + "个");
        }
        return map;
    }
}
