/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.io.ExcelUtil;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.MatterDTO;
import com.tofly.flow.custom.dto.MatterImportDTO;
import com.tofly.flow.custom.dto.MatterQuerySmall;
import com.tofly.flow.custom.entity.Assets;
import com.tofly.flow.custom.entity.BaseType;
import com.tofly.flow.custom.entity.Matter;
import com.tofly.flow.custom.mapper.MatterMapper;
import com.tofly.flow.custom.service.AssetsService;
import com.tofly.flow.custom.service.BaseTypeService;
import com.tofly.flow.custom.service.KeepGrainMatterService;
import com.tofly.flow.custom.service.MatterService;
import com.tofly.flow.custom.vo.BaseTypeSmallVO;
import com.tofly.flow.custom.vo.MatterVO;
import lombok.SneakyThrows;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 物资信息表
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@Service("matterService")
public class MatterServiceImpl extends ServiceImpl<MatterMapper, Matter> implements MatterService {

    @Autowired
    private BaseTypeService baseTypeService;

    @Autowired
    private AssetsService assetsService;

    @Autowired
    private KeepGrainMatterService keepGrainMatterService;

    @SneakyThrows
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insert(MatterDTO matterDTO) {
        checkData(matterDTO);

        Matter matter = new Matter();
        matter.setIsDel(FlowConstant.SHORT_ZERO);
        matter.setIsValid(FlowConstant.SHORT_ONE);
        BeanUtils.copyProperties(matterDTO, matter);
        if (!this.save(matter)) {
            throw new RuntimeException("新增失败");
        }
        createTypeNum(matter.getMatterType(), matter);
    }

    private void checkData(MatterDTO matterDTO) {
        QueryWrapper<Matter> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(matterDTO.getMatterType()!=null,"MATTER_TYPE", matterDTO.getMatterType())
                .eq(!StringUtils.isEmpty(matterDTO.getMatterName()),"MATTER_NAME", matterDTO.getMatterName())
                .eq("IS_DEL", FlowConstant.SHORT_ZERO)
                .eq(!StringUtils.isEmpty(matterDTO.getStandards()),"STANDARDS",matterDTO.getStandards());
        List<Matter> matter = this.list(queryWrapper);
        if (!CollectionUtils.isEmpty(matter) &&
                (matterDTO.getId() == null || !matterDTO.getId().equals(matter.get(0).getId()))) {
            throw new RuntimeException("相同分类下（物资类型名称.规格型号）存在数据重复");
        }
    }


    private void check(Long matterId) {
        QueryWrapper<Assets> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("MATTER_ID",matterId).eq("IS_DEL",FlowConstant.SHORT_ZERO);
        int count = assetsService.count(queryWrapper);
        if (count > 0) {
            throw new RuntimeException("无法删除，挂接了物资的物资类型不允许删除");
        }
    }


    private void createTypeNum(Long typeId, Matter matter) {
        String Num = null;
        if (matter.getId() < 10) {
            Num = "000" + matter.getId().toString();
        } else if (matter.getId() < 100) {
            Num = "00" + matter.getId().toString();
        } else if (matter.getId() < 1000) {
            Num = "0" + matter.getId().toString();
        } else {
            Num = matter.getId().toString();
        }
        BaseType baseType = baseTypeService.getById(typeId);
        if (ObjectUtils.isEmpty(baseType)) {
            throw new RuntimeException("类型不存在");
        }
        //生成编码类型
        String typeNum = baseType.getTypeCode() + Num;
        //修改类型编码
        matter.setMatterNumber(typeNum);
        this.updateById(matter);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateByMatter(MatterDTO matterDTO) {
        checkData(matterDTO);
        Matter matter = new Matter();
        BeanUtils.copyProperties(matterDTO, matter);
        if (!this.updateById(matter)) {
            throw new RuntimeException("修改失败");
        }
//        createTypeNum(matter.getMatterType(), matter);
    }

    @Override
    public void delete(Set<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new RuntimeException("未选择数据ID");
        }
        for (Long id : ids) {
            check(id);
        }
        int update = this.baseMapper.update(new Matter(), new UpdateWrapper<Matter>().lambda()
                .set(Matter::getIsDel, FlowConstant.SHORT_ONE)
                .eq(Matter::getIsDel, FlowConstant.SHORT_ZERO)
                .in(Matter::getId, ids)
        );
        if (update == 0) {
            throw new RuntimeException("删除失败");
        }

    }

    @Override
    public Page<MatterVO> pageList(Page page, MatterQuerySmall dto) {
        //设置查询参数
        QueryWrapper<MatterQuerySmall> queryWrapper = setQueryCondition(dto);
        Page<MatterVO> list = this.baseMapper.pageList(page, queryWrapper);
        if (list.getTotal() == 0) {
            return page;
        }
        //转换数据类型
        List<MatterVO> matterVoList = getMatterVoList(list.getRecords());
        page.setRecords(matterVoList);
        page.setTotal(list.getTotal());
        page.setPages(list.getPages());
        return page;

    }

    @Override
    public void template(HttpServletResponse response) {
        // 获取resource 下的资源
        InputStream resourceAsStream = this.getClass().getClassLoader().getResourceAsStream("report/import/物资类型导入模板.xlsx");
        assert resourceAsStream != null;
        MatterServiceImpl.download(resourceAsStream, "物资类型导入模板.xlsx", response);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void importMatter(Long baseTypeId, MultipartFile file) {
        BaseType baseType = baseTypeService.getById(baseTypeId);
        if (ObjectUtils.isEmpty(baseType)) {
            throw new RuntimeException("该类型分类不存在");
        }
        checkFile(file);
        InputStream inputStream = null;
        try {
            inputStream = file.getInputStream();
        } catch (IOException e) {
            e.printStackTrace();
        }
        ExcelUtil<MatterImportDTO> excelUtil = new ExcelUtil<>(MatterImportDTO.class);
        List<MatterImportDTO> matters = new ArrayList<>();
        try {
            matters = excelUtil.importExcel("", inputStream);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (CollectionUtils.isEmpty(matters)) {
            throw new RuntimeException("导入数据为空");
        }
        for (MatterImportDTO matter : matters) {
            if (matter == null) {
                throw new RuntimeException("数据为空");
            }
            if (StringUtils.isEmpty(matter.getMatterName())) {
                throw new RuntimeException("物资类型名称不能为空");
            }
            MatterDTO matterDTO = new MatterDTO();
            if (StringUtils.isEmpty(matter.getIsAssets().trim()) || "否".equals(matter.getIsAssets().trim())) {
                matterDTO.setIsAssets((short) 0);
            } else if ("是".equals(matter.getIsAssets())) {
                matterDTO.setIsAssets((short) 1);
            } else {
                throw new RuntimeException("是否资产填写错误，只能填写是或者否");
            }
            if (!StringUtils.isEmpty(matter.getCategoryType().trim())) {
                if ("设备类".equals(matter.getCategoryType().trim())) {
                    matterDTO.setCategoryType((short) 1);
                } else if ("车辆类".equals(matter.getCategoryType().trim())) {
                    matterDTO.setCategoryType((short) 2);
                    matterDTO.setIsAssets(FlowConstant.SHORT_ONE);
                } else if ("耗材类".equals(matter.getCategoryType().trim())) {
                    matterDTO.setCategoryType((short) 3);
                } else {
                    throw new RuntimeException("物资类别错误，只能是：设备类;车辆类;耗材类");
                }
            }
            matterDTO.setMatterType(baseTypeId);
            BeanUtils.copyProperties(matter, matterDTO);
            checkData(matterDTO);
            Matter matterData = new Matter();
            matterData.setIsDel(FlowConstant.SHORT_ZERO);
            matterData.setIsValid(FlowConstant.SHORT_ONE);
            BeanUtils.copyProperties(matterDTO, matterData);
            if (!this.save(matterData)) {
                throw new RuntimeException("新增失败");
            }
            createTypeNum(matterData.getMatterType(), matterData);
        }
    }

    /**
     * 检测文件类型
     *
     * @param file 文件类型
     */
    public static void checkFile(MultipartFile file) {
        if (file == null || file.isEmpty()) {
            throw new RuntimeException("文件不存在");
        }
        String filename = file.getOriginalFilename();
        assert filename != null;
        String suffix = filename.substring(filename.lastIndexOf(".") + 1);
        if (!"xlsx".equals(suffix) && !"xls".equals(suffix)) {
            throw new RuntimeException("不支持的文件类型");
        }
    }

    @Override
    public void export(HttpServletRequest request, HttpServletResponse response,MatterQuerySmall dto) {
        QueryWrapper<MatterQuerySmall> queryWrapper = setQueryCondition(dto);
        List<MatterVO> matterVOS = baseMapper.list(queryWrapper);
        if (CollectionUtils.isEmpty(matterVOS)) {
            return;
        }
        List<MatterVO> matterVoList = getMatterVoList(matterVOS);
        // excel模板路径
        InputStream resourceAsStream = this.getClass().getClassLoader().getResourceAsStream("report/export/物资类型信息数据.xlsx");
        XSSFWorkbook wb = null;
        try {
            // 读取excel模板
            assert resourceAsStream != null;
            wb = new XSSFWorkbook(resourceAsStream);
            // 读取了模板内所有sheet内容
            XSSFSheet sheet = wb.getSheetAt(0);
            // 在相应的单元格进行赋值
            int rowIndex = 1;
            int j = 1;
            for (MatterVO matterExportVo : matterVoList) {
                XSSFRow row = sheet.getRow(rowIndex);
                if (null == row) {
                    row = sheet.createRow(rowIndex);
                }
//                XSSFCell cell0 = row.getCell(0);
//                if (null == cell0) {
//                    cell0 = row.createCell(0);
//                }
//                cell0.setCellValue(matterExportVo.getMatterNumber() == null ? "" : matterExportVo.getMatterNumber());// 物资类型编号

                XSSFCell cell1 = row.getCell(0);
                if (null == cell1) {
                    cell1 = row.createCell(0);
                }
                cell1.setCellValue(matterExportVo.getMatterName() == null ? "" : matterExportVo.getMatterName());// 物资类型名称

                XSSFCell cell2 = row.getCell(1);
                if (null == cell2) {
                    cell2 = row.createCell(1);
                }
                cell2.setCellValue(matterExportVo.getStandards() == null ? "" : matterExportVo.getStandards());// 规格型号

                XSSFCell cell3 = row.getCell(2);
                if (null == cell3) {
                    cell3 = row.createCell(2);
                }
                cell3.setCellValue(matterExportVo.getUnit() == null ? "" : matterExportVo.getUnit());// 单位

                XSSFCell cell4 = row.getCell(3);
                if (null == cell4) {
                    cell4 = row.createCell(3);
                }

                if (matterExportVo.getCategoryType() == null) {
                    cell4.setCellValue("未知类");// 物资类别
                } else if (matterExportVo.getCategoryType() == (short) 1) {
                    cell4.setCellValue("设备类");// 物资类别
                } else if (matterExportVo.getCategoryType() == (short) 2) {
                    cell4.setCellValue("车辆类");// 物资类别
                } else if (matterExportVo.getCategoryType() == (short) 3) {
                    cell4.setCellValue("耗材类");// 物资类别
                }


                XSSFCell cell5 = row.getCell(4);
                if (null == cell5) {
                    cell5 = row.createCell(4);
                }
                cell5.setCellValue(matterExportVo.getIsAssets() == null ? "" : matterExportVo.getIsAssets() == (short) 1 ? "是" : "否");// 是否为资产

                XSSFCell cell6 = row.getCell(5);
                if (null == cell6) {
                    cell6 = row.createCell(5);
                }
                cell6.setCellValue(matterExportVo.getLevel1() == null ? "" : matterExportVo.getLevel1());// 物资1级分类


                XSSFCell cell7 = row.getCell(6);
                if (null == cell7) {
                    cell7 = row.createCell(6);
                }
                cell7.setCellValue(matterExportVo.getLevel2() == null ? "" : matterExportVo.getLevel2());// 物资2级分类

                XSSFCell cell8 = row.getCell(7);
                if (null == cell8) {
                    cell8 = row.createCell(7);
                }
                cell8.setCellValue(matterExportVo.getLevel3() == null ? "" : matterExportVo.getLevel3());// 物资3级分类
                rowIndex++;
            }

            String fileName = URLEncoder.encode("物资类型信息数据");
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            wb.write(os);
            byte[] content = os.toByteArray();
            InputStream is = new ByteArrayInputStream(content);
            // 设置response参数，可以打开下载页面
            response.reset();
            response.setContentType("application/vnd.ms-excel;charset=utf-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + new String((fileName + ".xlsx").getBytes(), "iso-8859-1"));
            ServletOutputStream sout = response.getOutputStream();
            BufferedInputStream bis = null;
            BufferedOutputStream bos = null;

            try {
                bis = new BufferedInputStream(is);
                bos = new BufferedOutputStream(sout);
                byte[] buff = new byte[2048];
                int bytesRead;
                // Simple read/write loop.
                while (-1 != (bytesRead = bis.read(buff, 0, buff.length))) {
                    bos.write(buff, 0, bytesRead);
                }
            } catch (Exception e) {
//                logger.error("导出excel出现异常:", e);
            } finally {
                if (bis != null) {
                    bis.close();
                }
                if (bos != null) {
                    bos.close();
                }
            }

        } catch (Exception e) {
//            logger.error("导出excel出现异常:", e);
        }
    }

    @Override
    public void deleteBaseTypeIds(List<Long> ids) {
        this.baseMapper.update(new Matter(), new UpdateWrapper<Matter>().lambda()
                .set(Matter::getMatterType, FlowConstant.SHORT_ZERO)
                .in(Matter::getMatterType, ids));
    }

    @Override
    public Long selectGrainMatterByMatterNum(String codeNum) {
        return this.baseMapper.selectGrainMatterByMatterNum(codeNum);
    }

    @Override
    public List<Matter> getByMatterNums(List<String> matterNums) {
        QueryWrapper<Matter> wrapper = new QueryWrapper<>();
        wrapper.in("MATTER_NUMBER", matterNums);
        wrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO);
        List<Matter> matters = this.list(wrapper);
        return matters;
    }

    @Override
    public Matter selectMatterByMatterImportId(Long id) {
        List<Matter> matters = this.baseMapper.selectMatterByMatterImportId(id);
        if (CollectionUtils.isEmpty(matters)){
            return null;
        }
        return matters.get(0);
    }

    @Override
    public MatterVO get(Long id) {
        Matter byId = this.getById(id);
        if (ObjectUtils.isEmpty(byId)){
            return new MatterVO();
        }
        MatterVO matter = new MatterVO();
        BeanUtils.copyProperties(byId,matter);
        if (matter.getMatterType() != null) {
            List<BaseType> baseTypes = baseTypeService.selectTypeParentById(matter.getMatterType());
            if (!CollectionUtils.isEmpty(baseTypes)) {
                //设置等级数组
                List<BaseTypeSmallVO> smallVOS = baseTypes.stream().map(baseType -> {
                    BaseTypeSmallVO baseTypeSmallVO = new BaseTypeSmallVO();
                    baseTypeSmallVO.setTypeName(baseType.getTypeName());
                    baseTypeSmallVO.setId(baseType.getId());
                    return baseTypeSmallVO;
                }).collect(Collectors.toList());
                matter.setBaseTypeSmallVOList(smallVOS);
                //设置等级
                if (baseTypes.size() >= 3) {
                    matter.setLevel1(baseTypes.get(baseTypes.size() - 1).getTypeName());
                    matter.setLevel2(baseTypes.get(baseTypes.size() - 2).getTypeName());
                    matter.setLevel3(baseTypes.get(baseTypes.size() - 3).getTypeName());
                } else if (baseTypes.size() == 2) {
                    matter.setLevel1(baseTypes.get(baseTypes.size() - 1).getTypeName());
                    matter.setLevel2(baseTypes.get(baseTypes.size() - 2).getTypeName());
                } else if (baseTypes.size() == 1) {
                    matter.setLevel1(baseTypes.get(0).getTypeName());
                }
            }
        }
        return matter;
    }


    /**
     * 设置参数属性值
     *
     * @param list 列表数据
     * @return VO列表数据
     */
    private List<MatterVO> getMatterVoList(List<MatterVO> list) {
        List<MatterVO> matterVOS = list.stream().map(matter -> {
            if (matter.getMatterType() != null) {
                List<BaseType> baseTypes = baseTypeService.selectTypeParentById(matter.getMatterType());
                if (!CollectionUtils.isEmpty(baseTypes)) {
                    //设置等级数组
                    List<BaseTypeSmallVO> smallVOS = baseTypes.stream().map(baseType -> {
                        BaseTypeSmallVO baseTypeSmallVO = new BaseTypeSmallVO();
                        baseTypeSmallVO.setTypeName(baseType.getTypeName());
                        baseTypeSmallVO.setId(baseType.getId());
                        return baseTypeSmallVO;
                    }).collect(Collectors.toList());
                    matter.setBaseTypeSmallVOList(smallVOS);
                    //设置等级
                    if (baseTypes.size() >= 3) {
                        matter.setLevel1(baseTypes.get(baseTypes.size() - 1).getTypeName());
                        matter.setLevel2(baseTypes.get(baseTypes.size() - 2).getTypeName());
                        matter.setLevel3(baseTypes.get(baseTypes.size() - 3).getTypeName());
                    } else if (baseTypes.size() == 2) {
                        matter.setLevel1(baseTypes.get(baseTypes.size() - 1).getTypeName());
                        matter.setLevel2(baseTypes.get(baseTypes.size() - 2).getTypeName());
                    } else if (baseTypes.size() == 1) {
                        matter.setLevel1(baseTypes.get(0).getTypeName());
                    }
                }
            }
            return matter;
        }).collect(Collectors.toList());
        return matterVOS;
    }

    /**
     * 设置查询参数
     *
     * @param dto 查询参数
     */
    private QueryWrapper<MatterQuerySmall> setQueryCondition(MatterQuerySmall dto) {
        QueryWrapper<MatterQuerySmall> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("m.IS_DEL", FlowConstant.SHORT_ZERO);
        queryWrapper.orderByAsc("m.MATTER_NUMBER");
        if (dto != null) {
            if (dto.getMatterType() != null) {
                List<Long> typeIds = baseTypeService.selectTypeChildById(dto.getMatterType());
                if (!CollectionUtils.isEmpty(typeIds)) {
                    queryWrapper.in("m.MATTER_TYPE", typeIds);
                }
            }
            if (!StringUtils.isEmpty(dto.getMatterName())) {
                queryWrapper.and(matterQueryWrapper -> {
                    matterQueryWrapper.like("m.MATTER_NAME", dto.getMatterName())
                            .or().like("m.MATTER_NUMBER", dto.getMatterName())
                            .or().like("m.REMARK", dto.getMatterName());
                });
            }
            if (!StringUtils.isEmpty(dto.getSearch())) {
                queryWrapper.and(matterQueryWrapper -> {
                    matterQueryWrapper.like("t.TYPE_NAME", dto.getSearch())
                            .or().like("m.MATTER_NAME", dto.getSearch())
                            .or().like("m.STANDARDS", dto.getSearch());
                });
            }
            if (!CollectionUtils.isEmpty(dto.getType())) {
                queryWrapper.in("m.CATEGORY_TYPE", dto.getType());
            }
        }
        return queryWrapper;
    }


    /**
     * 文件流下载
     *
     * @param file     文件
     * @param response 返回数据
     */
    @SneakyThrows
    public static void download(File file, HttpServletResponse response) {
        ServletOutputStream out = null;
        FileInputStream in = null;
        try {
            in = new FileInputStream(file);
            String fileName = URLEncoder.encode(file.getName(), "UTF-8");
//            fileName = new String(fileName.getBytes("UTF-8"), "ISO8859-1");
            // 自定义的header
            response.setHeader("requestType", "file");
            // 自定义的header
            response.setHeader("requestFileName", fileName);
            //设置这个header 可见
            response.setHeader("Access-Control-Expose-Headers", "requestType,requestFileName");

//             设置响应类型为html，编码为utf-8，处理相应页面文本显示的乱码
            response.setContentType("application/octet-stream;charset=UTF-8");
//            // 设置文件头：最后一个参数是设置下载文件名
            response.setHeader("Content-disposition", "attachment;fileName=" + fileName);
            out = response.getOutputStream();
            // 读取文件流
            int len = 0;
            byte[] buffer = new byte[1024 * 10];
            while ((len = in.read(buffer)) != -1) {
                out.write(buffer, 0, len);
            }
            out.flush();
        } catch (FileNotFoundException e) {
            throw new RuntimeException("文件下载失败");
        }

    }

    /**
     * 文件流下载
     *
     * @param response 返回数据
     */
    @SneakyThrows
    public static void download(InputStream in, String name, HttpServletResponse response) {
        ServletOutputStream out = null;
        try {
            String fileName = URLEncoder.encode(name, "UTF-8");
//            fileName = new String(fileName.getBytes("UTF-8"), "ISO8859-1");
            // 自定义的header
            response.setHeader("requestType", "file");
            // 自定义的header
            response.setHeader("requestFileName", fileName);
            //设置这个header 可见
            response.setHeader("Access-Control-Expose-Headers", "requestType,requestFileName");

//             设置响应类型为html，编码为utf-8，处理相应页面文本显示的乱码
            response.setContentType("application/octet-stream;charset=UTF-8");
//            // 设置文件头：最后一个参数是设置下载文件名
            response.setHeader("Content-disposition", "attachment;fileName=" + fileName);
            out = response.getOutputStream();
            // 读取文件流
            int len = 0;
            byte[] buffer = new byte[1024 * 10];
            while ((len = in.read(buffer)) != -1) {
                out.write(buffer, 0, len);
            }
            out.flush();
            out.close();
        } catch (FileNotFoundException e) {
            throw new RuntimeException("文件下载失败");
        }

    }
}
