/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.PConfigInfoDto;
import com.tofly.flow.custom.entity.ProcessConfigInfo;
import com.tofly.flow.custom.service.ProcessConfigInfoService;
import com.tofly.flow.custom.service.ProcessConfigService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ProcessConfigInfoMapper;

/**
 * 流程配置信息
 *
 * @author xqf
 * @date Mon Mar 21 00:00:00 CST 2022
 */
@Service("processConfigInfoService")
public class ProcessConfigInfoServiceImpl extends ServiceImpl<ProcessConfigInfoMapper, ProcessConfigInfo> implements ProcessConfigInfoService {

    @Autowired
    private ProcessConfigService processConfigService;
    /**
     * 校验流程基础信息
     *
     * @param id
     * @param infoDto 流程信息
     * @param size 流程步骤数
     * @param db 是否有督办人员
     * @return
     */
    @Override
    public String checkingInfo(Long id, PConfigInfoDto infoDto, int size, boolean db) {
        int count = this.count(new QueryWrapper<ProcessConfigInfo>().lambda().ne(id!=null,ProcessConfigInfo::getConfigId,id)
                .eq(ProcessConfigInfo::getProcessCode,infoDto.getProcessCode()).eq(ProcessConfigInfo::getIsDel,FlowConstant.SHORT_ZERO)
                .eq(ProcessConfigInfo::getIsValid,FlowConstant.SHORT_ONE));
        if(count>FlowConstant.INTEGER_ZERO){
            return FlowConstant.CODE_EXISTS;
        }


        if(infoDto.getStepNumber()==null||infoDto.getStepNumber()!=size){
            infoDto.setStepNumber(size);
        }
        if (infoDto.getIsendTidings()==null||
                (infoDto.getIsendTidings()!= FlowConstant.SHORT_ZERO&&
                        infoDto.getIsendTidings()!=FlowConstant.SHORT_ONE)){
            infoDto.setIsendTidings(FlowConstant.SHORT_ZERO);
        }
        if(infoDto.getIsurge()==FlowConstant.SHORT_ONE&&!db){
            return FlowConstant.DBRY_ISNULL;
        }
        if(db){
            infoDto.setIsurge(FlowConstant.SHORT_ONE);
        }
        if(infoDto.getIsValid()==null){
            infoDto.setIsValid(FlowConstant.SHORT_ONE);
        }
        return null;
    }

    /**
     * 保存流程信息
     * @param infoDto 流程信息
     * @param id 流程id
     * @param edition 版本信息
     * @return
     */
    @Override
    public boolean saveInfo(PConfigInfoDto infoDto, Long id,Integer edition) {
        ProcessConfigInfo info = new ProcessConfigInfo();
        BeanUtils.copyProperties(infoDto,info);
        info.setIsDel(FlowConstant.SHORT_ZERO);
        info.setConfigId(id);
        info.setEdition(edition);
        return save(info);
    }

    /**
     * 逻辑删除相关流程信息
     * @param oldId 流程id
     * @param oldEdition 流程版本
     * @return
     */
    @Override
    public boolean delUpdateInfo(Long oldId, Integer oldEdition) {
        return this.update(new UpdateWrapper<ProcessConfigInfo>().lambda()
                .set(ProcessConfigInfo::getIsValid,FlowConstant.SHORT_ZERO).set(ProcessConfigInfo::getIsDel,FlowConstant.SHORT_ONE)
                .eq(ProcessConfigInfo::getConfigId,oldId).le(ProcessConfigInfo::getEdition,oldEdition)
                .eq(ProcessConfigInfo::getIsDel,FlowConstant.SHORT_ZERO));
    }

}
