/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.entity.ProcessConfigInfo;
import com.tofly.flow.custom.service.*;
import com.tofly.flow.custom.dto.LoadProcessConfigDto;
import com.tofly.flow.custom.dto.ProcessConfigAllDto;
import com.tofly.flow.custom.dto.ProcessConfigDto;
import com.tofly.flow.custom.dto.UpdateProcessConfigAllDto;
import com.tofly.flow.custom.entity.BaseType;
import com.tofly.flow.custom.entity.ProcessConfig;
import com.tofly.flow.custom.vo.ConfigStepStaffVo;
import com.tofly.flow.custom.vo.LevleStepStaffVo;
import com.tofly.flow.custom.vo.ProcessConfigStepVo;
import com.tofly.flow.custom.vo.ProcessConfigVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ProcessConfigMapper;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 审批流程配置
 *
 * @author xqf
 * @date Mon Mar 21 00:00:00 CST 2022
 */
@Service("processConfigService")
public class ProcessConfigServiceImpl extends ServiceImpl<ProcessConfigMapper, ProcessConfig> implements ProcessConfigService {

    /**
     * 基础类型
     */
    @Autowired
    private BaseTypeService baseTypeService;

    /**
     * redis
     */
    @Autowired
    private RedisTemplate redisTemplate;
    /**
     * 配置基础信息
     */
    @Autowired
    private ProcessConfigInfoService pInfoService;
    /**
     * 配置流程步骤
     */
    @Autowired
    private ProcessConfigStepService pStepService;

    /**
     * 人员
     */
    @Autowired
    private ConfigStepStaffService staffService;

    /**
     * 新增流程配置名称类型
     * @param dto
     * @return
     */
    @Override
    public Object saveProcessConfigName(ProcessConfigDto dto) {
        String typeError = checkingPf(dto);
        if (typeError != null) return typeError;
        ProcessConfig processConfig = new ProcessConfig();
        BeanUtils.copyProperties(dto,processConfig);
        return saveProrcessConfig(processConfig);
    }

    /**
     * 保存
     * @param processConfig
     * @return
     */
    private Boolean saveProrcessConfig(ProcessConfig processConfig) {
        BaseType baseType =baseTypeService.getById(processConfig.getBusinessType());
        processConfig.setProcessNumber(baseType.getTypeCode());
        processConfig.setEdition(FlowConstant.INTEGER_ONE);
        processConfig.setIsDel(FlowConstant.SHORT_ZERO);
        if(processConfig.getIsValid()==null){
            processConfig.setIsValid(FlowConstant.SHORT_ONE);
        }
        return save(processConfig);
    }

    /**
     * 验证信息准确性
     * @param dto
     * @return
     */
    private String checkingPf(ProcessConfigDto dto) {
        if(StringUtils.isEmpty(dto.getProcessName())){
            return FlowConstant.NAME_ISNULL;
        }
        if(dto.getProcessType()==null){
            return FlowConstant.PROCESS_TYPE_NULL;
        }
        if(dto.getBusinessType()==null){
            return FlowConstant.BUSINESS_TYPE_NULL;
        }
        int count = baseTypeService.count(new QueryWrapper<BaseType>().lambda()
                .eq(BaseType::getId, dto.getBusinessType()).eq(BaseType::getDataType,dto.getProcessType()).eq(BaseType::getIsDel, FlowConstant.SHORT_ZERO));
        if(count==0){
            return FlowConstant.TYPE_ERROR;
        }
        count = this.count(new QueryWrapper<ProcessConfig>().lambda().eq(ProcessConfig::getBusinessType, dto.getBusinessType())
                .eq(ProcessConfig::getProcessName, dto.getProcessName())
                .eq(ProcessConfig::getIsDel,FlowConstant.SHORT_ZERO));
        if(count>0){
            return FlowConstant.NAME_EXISTS;
        }
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean removeUpdateById(Long id) {
        pInfoService.update(new UpdateWrapper<ProcessConfigInfo>().lambda().set(ProcessConfigInfo::getIsDel,FlowConstant.SHORT_ONE)
                .set(ProcessConfigInfo::getIsValid,FlowConstant.SHORT_ZERO).eq(ProcessConfigInfo::getIsDel,FlowConstant.SHORT_ZERO)
                .eq(ProcessConfigInfo::getConfigId,id));
        return update(new UpdateWrapper<ProcessConfig>().lambda().set(ProcessConfig::getIsDel, FlowConstant.SHORT_ONE)
                .set(ProcessConfig::getUpdateTime, new Date()).set(ProcessConfig::getUpdateUser, SecurityUtils.getUserId())
                .eq(ProcessConfig::getIsDel, FlowConstant.SHORT_ZERO)
                .eq(ProcessConfig::getId, id));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object removeUpdateByIds(String ids) {
        if(StringUtils.isEmpty(ids)){
            return FlowConstant.ISNOTNUL;
        }
        try {
            List<Long> lids= Arrays.asList(ids.split(",")).stream().map(st->Long.parseLong(st)).collect(Collectors.toList());
            if(CollUtil.isNotEmpty(lids)) {
                pInfoService.update(new UpdateWrapper<ProcessConfigInfo>().lambda().set(ProcessConfigInfo::getIsDel,FlowConstant.SHORT_ONE)
                        .set(ProcessConfigInfo::getIsValid,FlowConstant.SHORT_ZERO).eq(ProcessConfigInfo::getIsDel,FlowConstant.SHORT_ZERO)
                        .in(ProcessConfigInfo::getConfigId,lids));
                return this.baseMapper.update(new ProcessConfig(), new UpdateWrapper<ProcessConfig>().lambda()
                        .set(ProcessConfig::getIsDel, FlowConstant.SHORT_ONE)
                        .eq(ProcessConfig::getIsDel, FlowConstant.SHORT_ZERO)
                        .in(ProcessConfig::getId, lids));
            }
            return false;
        }catch (Exception e){
            log.error("异常：",e);
            return e.getMessage();
        }
    }

    /**
     * 保存流程配置信息
     * @param dto
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object saveProcessConfig(ProcessConfigAllDto dto) {
        String str = checkingPf(dto);
        if(StringUtils.isNotEmpty(str)){
            return str;
        }
        str = checkingRelevancyData(dto,null);
        if (str != null) return str;
        ProcessConfig processConfig = new ProcessConfig();
        BeanUtils.copyProperties(dto,processConfig);
        if(!saveProrcessConfig(processConfig)){
            return false;
        }
        if(CollUtil.isNotEmpty(dto.getDbrs())) {
            staffService.saveStaff(dto.getDbrs(), processConfig.getId(), processConfig.getEdition(),FlowConstant.SHORT_TWO);
        }
        if(!pInfoService.saveInfo(dto.getInfoDto(),processConfig.getId(),processConfig.getEdition())){
            throw new RuntimeException("流程基础信息存储异常");
        }
        pStepService.saveStepService(dto.getStepDtos(),processConfig.getId(), processConfig.getEdition());

        return true;
    }

    /**
     * 校验关联数据
     * @param dto
     * @param id
     * @return
     */
    private String checkingRelevancyData(ProcessConfigAllDto dto, Long id) {
        String str;
        str = pInfoService.checkingInfo(id,dto.getInfoDto(), dto.getStepDtos().size(),CollUtil.isNotEmpty(dto.getDbrs()));
        if(StringUtils.isNotEmpty(str)){
            return str;
        }
        str = pStepService.checkingStep(dto.getStepDtos());
        if(StringUtils.isNotEmpty(str)){
            return str;
        }
        return null;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object updateProcessConfig(UpdateProcessConfigAllDto dto) {
        ProcessConfig config = this.getById(dto.getId());
        String str = updateChecking(dto,config);
        if(StringUtils.isNotEmpty(str)){
            return str;
        }
        str = checkingRelevancyData(dto,dto.getId());
        if (str != null) return str;
        Long oldId = dto.getId();
        Integer oldEdition = config.getEdition();
        Long id = dto.getId();
        Integer edition = config.getEdition() + 1;
        ProcessConfig processConfig = new ProcessConfig();
        BeanUtils.copyProperties(dto,processConfig);
        if(typeupdate(dto,config)){
            this.update(new UpdateWrapper<ProcessConfig>().lambda()
                    .set(ProcessConfig::getIsValid,FlowConstant.SHORT_ZERO).set(ProcessConfig::getIsDel,FlowConstant.SHORT_ONE)
                    .set(ProcessConfig::getUpdateTime, new Date()).set(ProcessConfig::getUpdateUser, SecurityUtils.getUserId())
                    .eq(ProcessConfig::getId,oldId));
            processConfig.setId(null);
            if(!saveProrcessConfig(processConfig)){
                return false;
            }
            id = processConfig.getId();
            edition = processConfig.getEdition();
        }else {
            processConfig.setEdition(edition);
            if(!updateById(processConfig)){
                return false;
            }
        }
        staffService.delUpdateStaff(oldId,oldEdition,FlowConstant.SHORT_TWO);
        if(CollUtil.isNotEmpty(dto.getDbrs())) {
            staffService.saveStaff(dto.getDbrs(), id, edition,FlowConstant.SHORT_TWO);
        }
        pInfoService.delUpdateInfo(oldId,oldEdition);
        if(!pInfoService.saveInfo(dto.getInfoDto(),processConfig.getId(),processConfig.getEdition())){
            throw new RuntimeException("流程基础信息存储异常");
        }
        pStepService.delUpdateStep(oldId,oldEdition);
        pStepService.saveStepService(dto.getStepDtos(),processConfig.getId(), processConfig.getEdition());

        return true;
    }

    @Override
    public Page<ProcessConfigVo> pageProcessConfig(Page page, LoadProcessConfigDto dto) {
        Page<ProcessConfigVo> pg = this.baseMapper.loadProcessConfig(page,dto);
        for (ProcessConfigVo vo:pg.getRecords()) {
            vo.setTypeName(gettype(vo.getAllTypeName(),true));
            vo.setChildLevelName(gettype(vo.getAllTypeName(),false));
        }
        return pg;
    }

    /**
     * 根据ID获取流程配置详情
     * @param id
     * @return
     */
    @Override
    public ProcessConfigVo getProcessConfigById(Long id) {
        ProcessConfigVo vo = this.baseMapper.getProcessConfig(id);
        //vo.setProcessTypeName(ProcessTypeEnum.loadValue(vo.getProcessType()));
        vo.setTypeName(gettype(vo.getAllTypeName(),true));
        vo.setChildLevelName(gettype(vo.getAllTypeName(),false));
        if(vo.getSteps()!=null) {
            for (ProcessConfigStepVo step:vo.getSteps()) {
                if(step.getStaffs()!=null){
                    Map<Short,LevleStepStaffVo> levelstaffs = new HashMap<>();
                    for (ConfigStepStaffVo staffvo : step.getStaffs()) {
                        LevleStepStaffVo  levelvo = null;
                        Short levelnumber= staffvo.getLevelNumber();
                        if(levelstaffs.containsKey(levelnumber)){
                            levelvo=levelstaffs.get(levelnumber);
                        }else {
                            levelvo = new LevleStepStaffVo();
                            levelvo.setStaffs(new ArrayList<>());
                            levelvo.setLevelNumber(levelnumber);
                            if(levelnumber<step.getCoerceLevel()){
                                levelvo.setMust(FlowConstant.SHORT_ONE);
                            }else {
                                levelvo.setMust(FlowConstant.SHORT_ZERO);
                            }
                            levelstaffs.put(levelnumber,levelvo);
                        }
                        levelvo.getStaffs().add(staffvo);

                    }
                    step.setLevelstaffs(new ArrayList<>(levelstaffs.values()));
                    step.setStaffs(null);
                }
            }
        }
        return vo;
    }

    /**
     * 根据业务类型查询配置流程数
     * @param businessType 业务类型
     * @return 配置流程数
     */
    @Override
    public Integer getConfigByType(Long businessType) {
        return this.count(new QueryWrapper<ProcessConfig>().lambda().eq(ProcessConfig::getBusinessType,businessType)
                .eq(ProcessConfig::getIsDel,FlowConstant.SHORT_ZERO));
    }

    /**
     * 修改流程配置是否有效
     * @param id 流程主键
     * @param isValid 是否有效
     * @return
     */
    @Override
    public boolean updateValidById(Long id, Short isValid) {
        if(isValid==null||(isValid!=FlowConstant.SHORT_ZERO&&isValid!=FlowConstant.SHORT_ONE)){
            return false;
        }
        return update(new UpdateWrapper<ProcessConfig>().lambda().set(ProcessConfig::getIsValid,isValid)
                .set(ProcessConfig::getUpdateTime, new Date()).set(ProcessConfig::getUpdateUser, SecurityUtils.getUserId())
                .eq(ProcessConfig::getIsDel, FlowConstant.SHORT_ZERO)
                .eq(ProcessConfig::getId, id));
    }

    private String gettype(String typename,boolean begOrEnd){
        if(typename.contains("-")){
            if(begOrEnd){
                return typename.substring(0,typename.indexOf("-"));
            }else {
                return typename.substring(typename.lastIndexOf("-")+1);
            }
        }
        return typename;
    }


    private boolean typeupdate(UpdateProcessConfigAllDto dto,ProcessConfig config){
        if((dto.getBusinessType()!=null&&dto.getBusinessType()!=config.getBusinessType())
                ||(dto.getProcessType()!=null&&dto.getProcessType()!=config.getProcessType())){
            return true;
        }
        return false;
    }

    /**
     * 校验修改数据
     * @param dto
     * @param config
     * @return
     */
    private String updateChecking(UpdateProcessConfigAllDto dto,ProcessConfig config) {
        if(config==null||config.getIsDel()==FlowConstant.SHORT_ONE){
            return FlowConstant.DATA_NOTHINGNESS;
        }
        if((dto.getBusinessType()!=null&&dto.getBusinessType()!=config.getBusinessType())
                ||(dto.getProcessType()!=null&&dto.getProcessType()!=config.getProcessType())){
            String str = checkingPf(dto);
            if(StringUtils.isNotEmpty(str)){
                return str;
            }
        }else if(StringUtils.isNotEmpty(dto.getProcessName()) ){
            int count = this.count(new QueryWrapper<ProcessConfig>().lambda().eq(ProcessConfig::getBusinessType, dto.getBusinessType())
                    .eq(ProcessConfig::getProcessName, dto.getProcessName())
                    .ne(ProcessConfig::getId,dto.getId())
                    .eq(ProcessConfig::getIsDel,FlowConstant.SHORT_ZERO));
            if(count>0){
                return FlowConstant.NAME_EXISTS;
            }
        }
        return null;
    }


    /**
     * 获取编码
     * @return
     */
    private String getProcessNumber(){
        String key = FlowConstant.REDIS_KEY_LC+ DateUtil.format(new Date(), "yyyyMMdd");
        RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());
        Long value = counter.incrementAndGet();
        if(value==1){
            String number = this.baseMapper.getProcessNumber(DateUtil.beginOfDay(new Date()));
            if(StringUtils.isNotEmpty(number)) {
                Long num = Long.parseLong(number.substring(number.length()-5));
                counter.set(num);
                long betweenDay = DateUtil.between(new Date(), DateUtil.endOfDay(new Date()), DateUnit.MINUTE);
                counter.expire(betweenDay, TimeUnit.MINUTES);
                value = counter.incrementAndGet();
            }
        }
        return FlowConstant.JCLX+DateUtil.format(new Date(), "yyyyMMdd")+String.format("%05d", value);
    }

}
