/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.LevelStaffDto;
import com.tofly.flow.custom.dto.PCStaffDto;
import com.tofly.flow.custom.dto.PConfigStepDto;
import com.tofly.flow.custom.entity.ProcessConfigStep;
import com.tofly.flow.custom.service.ConfigStepStaffService;
import com.tofly.flow.custom.service.ProcessConfigStepService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ProcessConfigStepMapper;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 流程配置步骤
 *
 * @author xqf
 * @date Mon Mar 21 00:00:00 CST 2022
 */
@Service("processConfigStepService")
public class ProcessConfigStepServiceImpl extends ServiceImpl<ProcessConfigStepMapper, ProcessConfigStep> implements ProcessConfigStepService {

    @Autowired
    private ConfigStepStaffService staffService;

    /**
     * 流程步骤校验
     * @param stepDtos 流程步骤数据
     * @return
     */
    @Override
    public String checkingStep(List<PConfigStepDto> stepDtos) {
        Set<Integer> stepOrders = new HashSet<>();
        Integer max = stepDtos.get(0).getStepOrder();
        Map<Integer,PConfigStepDto> map = stepDtos.stream().collect(  Collectors.toMap(PConfigStepDto::getStepOrder,p->p));

        for (PConfigStepDto dto:stepDtos) {
            stepOrders.add(dto.getStepOrder());
            //最大步骤序号
            if(dto.getStepOrder()>max){
                max=dto.getStepOrder();
            }
            if(StringUtils.isNotEmpty(dto.getNextStep())) {
                List<Integer> ids = Arrays.stream(dto.getNextStep().split(",")).map(r -> Integer.parseInt(r)).collect(Collectors.toList());
                for (Integer i : ids) {
                    if (i < dto.getStepOrder()){
                        return FlowConstant.NEXT_STEP_ERROR;
                    }
                }
            }

            if(dto.getIsEnable()==null){
                dto.setIsEnable(FlowConstant.SHORT_ONE);
            }

            if(dto.getUnanimousOrder()==FlowConstant.SHORT_ZERO){
                dto.setUnanimousOrder(null);
            }
            List<PCStaffDto> staffs = new ArrayList<>();
            if(dto.getUnanimousOrder()!=null) {
                if (dto.getUnanimousOrder() < dto.getStepOrder()) {
                    return FlowConstant.UNANIMOUS_ERROR;
                }
                PConfigStepDto stepDto = map.get(dto.getUnanimousOrder());
                if(stepDto==null){
                    return FlowConstant.UNANIMOUS_ORDER_NOT_EXISTS;
                }
                if(dto.getNodeNumber()!=stepDto.getNodeNumber()||dto.getCoerceLevel()!=stepDto.getCoerceLevel()){
                    return FlowConstant.LEVEL_NOT_UNANIMOUS;
                }
            }else {
                if((CollUtil.isEmpty(dto.getStaffDtos()))){
                    return FlowConstant.STAFF_ISNULL;
                }
                for (LevelStaffDto level :dto.getStaffDtos()) {
                    if((dto.getCoerceLevel()==0||dto.getCoerceLevel()<level.getLevelNumber())&&FlowConstant.SHORT_ONE==level.getMust()){
                        dto.setCoerceLevel(level.getLevelNumber());
                    }
                    for (PCStaffDto staff:level.getStaffDtos()) {
                        staff.setLevelNumber(level.getLevelNumber());
                        staffs.add(staff);
                    }
                }
                if(dto.getNodeNumber()==FlowConstant.SHORT_ZERO||dto.getCoerceLevel()==FlowConstant.SHORT_ZERO
                        ||dto.getCoerceLevel()>dto.getNodeNumber()){
                    return FlowConstant.NODE_LEVEL_ERROR;
                }

            }
            if(dto.getIsRepeat()==null){
                dto.setIsRepeat(FlowConstant.SHORT_ZERO);
            }else {
                if (dto.getIsRepeat() == FlowConstant.SHORT_ONE && dto.getNodeNumber() != FlowConstant.SHORT_ONE) {
                    return FlowConstant.REPEAT_ERROR;
                }
            }
            if(dto.getIsParallel()==FlowConstant.SHORT_ONE&&dto.getNodeNumber()!=FlowConstant.SHORT_ONE){
                return FlowConstant.PARALLEL_ERROR;
            }

            if(dto.getIsParallel()==null){
                dto.setIsParallel(FlowConstant.SHORT_ZERO);
            }

            if(dto.getUnanimousOrder()!=null){
                dto.setStaffDtos(null);
            }
            if(CollUtil.isNotEmpty(staffs)){
                String str= staffService.checkingStaff(staffs,dto.getNodeNumber());
                if(StringUtils.isNotEmpty(str)){
                    return str;
                }
            }
        }
        if(max!=stepOrders.size()){
            return FlowConstant.STEP_ORDER_ERROR;
        }
        return null;
    }

    /**
     * 保存流程步骤信息
     * @param stepDtos 流程步骤数据
     * @param id 流程配置ID
     * @param edition 流程配置版本
     * @return
     */
    @Override
    public Boolean saveStepService(List<PConfigStepDto> stepDtos, Long id, Integer edition) {
        for (PConfigStepDto dto:stepDtos) {
            ProcessConfigStep step = new ProcessConfigStep();
            BeanUtils.copyProperties(dto,step);
            step.setConfigId(id);
            step.setEdition(edition);
            step.setIsValid(FlowConstant.SHORT_ONE);
            step.setIsDel(FlowConstant.SHORT_ZERO);
            if(save(step)) {
                List<PCStaffDto> staffs = new ArrayList<>();
                if (CollUtil.isNotEmpty(dto.getStaffDtos())) {
                    for (LevelStaffDto level :dto.getStaffDtos()) {
                        if((dto.getCoerceLevel()==0||dto.getCoerceLevel()<level.getLevelNumber())&&FlowConstant.SHORT_ONE==level.getMust()){
                            dto.setCoerceLevel(level.getLevelNumber());
                        }
                        for (PCStaffDto staff:level.getStaffDtos()) {
                            staff.setLevelNumber(level.getLevelNumber());
                            staffs.add(staff);
                        }
                    }
                    staffService.saveStaff(staffs, step.getId(), edition, FlowConstant.SHORT_ONE);
                }
            }else {
                throw new RuntimeException("流程配置步骤保存失败！");
            }
        }

        return true;
    }

    /**
     * 删除流程配置
     * @param oldId 配置ID
     * @param oldEdition 配置版本
     * @return
     */
    @Override
    public Boolean delUpdateStep(Long oldId, Integer oldEdition) {
        staffService.delUpdateLcStaff(oldId,oldEdition, FlowConstant.SHORT_ONE);
        return this.update(new UpdateWrapper<ProcessConfigStep>().lambda()
                .set(ProcessConfigStep::getIsValid,FlowConstant.SHORT_ZERO)
                .set(ProcessConfigStep::getIsDel,FlowConstant.SHORT_ONE)
                .eq(ProcessConfigStep::getConfigId,oldId).le(ProcessConfigStep::getEdition,oldEdition)
                .eq(ProcessConfigStep::getIsDel,FlowConstant.SHORT_ZERO));
    }
}
