/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.BatchDeviceExWarehouseDto;
import com.tofly.flow.custom.dto.DeviceExWarehouseDto;
import com.tofly.flow.custom.dto.ProcessDeviceDto;
import com.tofly.flow.custom.entity.KeepGrainDevice;
import com.tofly.flow.custom.entity.ProcessDevice;
import com.tofly.flow.custom.entity.ProcessStuff;
import com.tofly.flow.custom.service.KeepGrainDeviceService;
import com.tofly.flow.custom.service.ProcessDeviceService;
import com.tofly.flow.custom.service.ProcessStuffService;
import com.tofly.flow.service.FileService;
import com.tofly.flow.util.UploadFiles;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ProcessDeviceMapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 流程出入/退库设备
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@Service("processDeviceService")
public class ProcessDeviceServiceImpl extends ServiceImpl<ProcessDeviceMapper, ProcessDevice> implements ProcessDeviceService {

    /**
     * 出入库物资
     */
    @Autowired
    private ProcessStuffService processStuffService;

    /**
     * 仓库物资设备
     */
    private KeepGrainDeviceService kGDService;

    /**
     * 文件保存
     */
    @Autowired
    private FileService fileService;


    @Value("${ftp.flow.processdevice}")
    private String processDevicefile;

    /**
     * 保存流程出入库设备信息
     * @param dtos 流程出入库设备
     * @param processStuffId  流程输入框物资id
     * @param factoryId 厂家ID
     * @param price 价格
     * @param oldId
     * @param deviceFiles
     * @return
     */
    @Override
    public boolean saveProcessDevice(List<ProcessDeviceDto> dtos, Long processStuffId, Long factoryId, Double price, Long oldId, Map<String, List<MultipartFile>> deviceFiles) {
        Map<Long, ProcessDevice> oldmap = null;
        if(oldId!=null) {
            oldmap = this.list(new QueryWrapper<ProcessDevice>().lambda()
                    .eq(ProcessDevice::getProcessStuffId, oldId)
                    .eq(ProcessDevice::getIsDel, FlowConstant.SHORT_ZERO)).stream().collect(Collectors.toMap(ProcessDevice::getId, r -> r));
        }
        if(CollUtil.isNotEmpty(dtos)){
            List<ProcessDevice> lists = new ArrayList<>();
            Long index = this.getObj(new QueryWrapper<ProcessDevice>().select("MAX(ID) as ID")
                    ,(o)->{return Long.parseLong(String.valueOf(o));} );
            if(index==null){
                index=1l;
            }
            Set<Long> ids = dtos.stream().filter(r->r.getStockId()!=null).map(r->r.getStockId()).collect(Collectors.toSet());
            Map<Long,KeepGrainDevice> map = null;
            if(CollUtil.isNotEmpty(ids)) {
                map = kGDService.list(new QueryWrapper<KeepGrainDevice>().lambda().eq(KeepGrainDevice::getIsDel, FlowConstant.SHORT_ZERO)
                        .eq(KeepGrainDevice::getIsValid, FlowConstant.SHORT_ONE).in(KeepGrainDevice::getId, ids)).stream()
                        .collect(Collectors.toMap(KeepGrainDevice::getId, v -> v));
            }
            for (ProcessDeviceDto dto:dtos) {
                ProcessDevice pd = new ProcessDevice();
                if(map!=null&&dto.getStockId()!=null&&map.containsKey(dto.getStockId())){
                    KeepGrainDevice kgd = map.get(dto.getStockId());
                    BeanUtils.copyProperties(kgd, pd);
                    pd.setKeepGrainDeviceId(kgd.getId());
                } else {
                    BeanUtils.copyProperties(dto, pd);
                }
                pd.setProcessStuffId(processStuffId);
                pd.setIsDel(FlowConstant.SHORT_ZERO);
                pd.setId(++index);
                if(pd.getFactoryId()==null&&factoryId!=null){
                    pd.setFactoryId(factoryId);
                }
                if(pd.getPrice()==null&&price!=null){
                    pd.setPrice(price);
                }
                ProcessDevice pn = null;
                if(dto.getId()!=null&&oldmap!=null){
                    pn = oldmap.get(dto.getId());
                }
                if(CollUtil.isNotEmpty(dto.getFileNames())) {
                    List<MultipartFile> fileList = deviceFiles.get(String.join("_",dto.getFileNames()));
                    if (CollUtil.isNotEmpty(fileList)) {
                        String filepath = UploadFiles.saveFiles(fileService, fileList, pn == null ? "" : pn.getAnnex(), dto.getAnnex(), processDevicefile, FlowConstant.TF_YWPN_PROCESS_DEVICE_W);
                        pd.setAnnex(filepath);
                    }
                }
                lists.add(pd);
            }
            if(CollUtil.isNotEmpty(lists)) {
                this.baseMapper.saveBatchProcessDevice(lists);
            }
        }
        //清除历史数据
        if(oldId!=null) {
            this.update(new UpdateWrapper<ProcessDevice>().lambda()
                    .eq(ProcessDevice::getProcessStuffId, oldId)
                    .eq(ProcessDevice::getIsDel, FlowConstant.SHORT_ZERO)
                    .set(ProcessDevice::getIsDel, FlowConstant.SHORT_ONE));
        }
        return true;
    }

    /**
     * 流程设备出库
     * @param dto 出库设备信息
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object deviceExWarehouse(BatchDeviceExWarehouseDto dto) {

        for (DeviceExWarehouseDto pd: dto.getDatas()) {
            int count = processStuffService.count(new QueryWrapper<ProcessStuff>()
                    .lambda().eq(ProcessStuff::getId,pd.getProcessStuffId())
                    .eq(ProcessStuff::getDataType,dto.getDataType())
                    .eq(ProcessStuff::getIsDel,FlowConstant.SHORT_ZERO));
            if(count == FlowConstant.SHORT_ZERO){
                return FlowConstant.DATA_NOTHINGNESS;
            }

            List<KeepGrainDevice> list = kGDService.list(new QueryWrapper<KeepGrainDevice>().lambda()
                    .eq(KeepGrainDevice::getId,pd.getKeepGrainDeviceIds())
                    .eq(KeepGrainDevice::getIsValid,FlowConstant.SHORT_ONE).eq(KeepGrainDevice::getIsDel,FlowConstant.SHORT_ZERO)
                    .le(KeepGrainDevice::getDeviceState,FlowConstant.SHORT_THREE));
            if(list.size() != pd.getKeepGrainDeviceIds().size()){
                return FlowConstant.DEVICE_ERROR;
            }
            Long index = this.getObj(new QueryWrapper<ProcessDevice>().select("MAX(ID) as ID")
                    ,(o)->{return Long.parseLong(String.valueOf(o));} );
            if(index==null){
                index = 1l;
            }
            List<ProcessDevice> datas = list.stream().map(r->{
                ProcessDevice processDevice = new ProcessDevice();
                BeanUtils.copyProperties(r,processDevice);
                processDevice.setKeepGrainDeviceId(r.getId());
                processDevice.setProcessStuffId(pd.getProcessStuffId());
                return processDevice;
            }).collect(Collectors.toList());
            this.baseMapper.saveBatchProcessDevice(datas);

        }

        return true;
    }

    /**
     * 逻辑删除关联出入库设备
     * @param keySet 物资id集合
     */
    @Override
    public void delProcessDevice(Set<Long> keySet) {
        this.update(new UpdateWrapper<ProcessDevice>().lambda()
                .in(ProcessDevice::getProcessStuffId, keySet)
                .eq(ProcessDevice::getIsDel, FlowConstant.SHORT_ZERO)
                .set(ProcessDevice::getIsDel, FlowConstant.SHORT_ONE));
    }
}
