/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProcessJournalDto;
import com.tofly.flow.custom.dto.ProcessJournalSaveDto;
import com.tofly.flow.custom.entity.ProcessJournal;
import com.tofly.flow.custom.entity.ProcessSurvey;
import com.tofly.flow.custom.service.ProcessJournalService;
import com.tofly.flow.custom.vo.ProcessJournalVO;
import com.tofly.flow.service.FileService;
import com.tofly.flow.util.UploadFiles;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ProcessJournalMapper;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 工程施工日志
 *
 * @author xqf
 * @date Wed May 25 00:00:00 CST 2022
 */
@Service("processJournalService")
public class ProcessJournalServiceImpl extends ServiceImpl<ProcessJournalMapper, ProcessJournal> implements ProcessJournalService {
    /**
     * 文件保存
     */
    @Autowired
    private FileService fileService;



    @Value("${ftp.flow.processjournal}")
    private String processJournalFile;
    /**
     * 工程施工信息
     * @param dto 工程信息
     * @param processId 流程ID
     * @param processStep 步骤
     * @param stepNode 节点
     * @param pjfiles 文件
     * @return
     */
    @Override
    public Boolean saveProcessJournal(ProcessJournalDto dto, Long processId, Integer processStep, Integer stepNode, List<MultipartFile> pjfiles) {
        ProcessJournal pj = new ProcessJournal();
        BeanUtils.copyProperties(dto,pj);
        ProcessJournal pj_db= this.getOne(new QueryWrapper<ProcessJournal>().lambda().eq(ProcessJournal::getProcessId,processId)
                .eq(ProcessJournal::getProcessStep,processStep).eq(ProcessJournal::getStepNode,stepNode).eq(ProcessJournal::getIsDel, FlowConstant.SHORT_ZERO));
        pj.setProcessId(processId);
        pj.setProcessStep(processStep);
        pj.setStepNode(stepNode);
        pj.setIsDel(FlowConstant.SHORT_ZERO);
        if(CollUtil.isNotEmpty(pjfiles)) {
            String filepath = UploadFiles.saveFiles(fileService, pjfiles, pj_db ==null?"": pj_db.getAnnex(), dto.getAnnex(), processJournalFile, FlowConstant.TF_YWPN_PROCESS_JOURNAL_W);

            pj.setAnnex(filepath);
        }
        this.remove(new QueryWrapper<ProcessJournal>().lambda().eq(ProcessJournal::getProcessId,processId)
                .eq(ProcessJournal::getProcessStep,processStep).eq(ProcessJournal::getStepNode,stepNode).eq(ProcessJournal::getIsDel, FlowConstant.SHORT_ZERO));

        return this.save(pj);
    }

    @Override
    public Object saveProcessJournal(ProcessJournalSaveDto dto, List<MultipartFile> fileList) {
        ProcessJournal pj = new ProcessJournal();
        BeanUtils.copyProperties(dto,pj);
        ProcessJournal pj_db= null;
        if(dto.getId()!=null){
            pj_db= this.getById(dto.getId());
            if(pj_db==null||pj_db.getIsDel()==FlowConstant.SHORT_ONE){
                return FlowConstant.DATA_NOTHINGNESS;
            }
        }
        if(CollUtil.isNotEmpty(fileList)) {
            String filepath = UploadFiles.saveFiles(fileService, fileList, pj_db ==null?"": pj_db.getAnnex(), dto.getAnnex(), processJournalFile, FlowConstant.TF_YWPN_PROCESS_JOURNAL_W);
            pj.setAnnex(filepath);
            if(dto.getId()!=null&& StringUtils.isEmpty(filepath)){
                this.update(new UpdateWrapper<ProcessJournal>().lambda().eq(ProcessJournal::getId,dto.getId()).set(ProcessJournal::getAnnex,""));
            }
        }
        if(pj.getIsDel()==null){
            pj.setIsDel(FlowConstant.SHORT_ZERO);
        }
        return this.saveOrUpdate(pj);
    }

    @Override
    public ProcessJournalVO getProcessJournalById(Long id) {
        return this.baseMapper.getProcessJournalById(id);
    }
}
