/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.config.ForeignConfig;
import com.tofly.flow.custom.dto.*;
import com.tofly.flow.custom.entity.*;
import com.tofly.flow.custom.entity.Process;
import com.tofly.flow.custom.service.*;
import com.tofly.flow.custom.vo.ProcessNodeVo;
import com.tofly.flow.service.FileService;
import com.tofly.flow.util.SendTidingsUtil;
import com.tofly.flow.util.UploadFiles;
import com.tofly.pojo.message.Messagepush;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ProcessNodeMapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 流程节点处理信息（操作步骤都有记录）
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@Slf4j
@Service("processNodeService")
public class ProcessNodeServiceImpl extends ServiceImpl<ProcessNodeMapper, ProcessNode> implements ProcessNodeService {

    /**
     * 流程基础信息
     */
    @Autowired
    private ProcessService processService;

    /**
     * 配置流程
     */
    @Autowired
    private ProcessConfigService pcService;

    /**
     * 配置流程步骤
     */
    @Autowired
    private ProcessConfigStepService pStepService;

    /**
     * 配置基础信息
     */
    @Autowired
    private ProcessConfigInfoService pInfoService;

    /**
     * 合同
     */
    @Autowired
    private ProjectContractService pCService;

    /**
     * 出入库物资信息
     */
    @Autowired
    private ProcessStuffService pSService;

    /**
     * 费用信息
     */
    @Autowired
    private ProjectDetailedService pDService;

    /**
     * 关联设施信息
     */
    @Autowired
    private ProcessRelationService pRService;

    /**
     * 关联人员（会议、报文）
     */
    @Autowired
    private ProcessStaffService pStaffService;

    /**
     * 文档、公告
     */
    @Autowired
    private DocumentService documentService;
    /**
     * 工程勘测
     */
    @Autowired
    private ProcessSurveyService pSurveyService;

    /**
     * 配置文件
     */
    @Autowired
    private ForeignConfig foreignConfig;
    /**
     * redis
     */
    @Autowired
    private RedisTemplate redisTemplate;
    /**
     * 发送消息
     */
    @Autowired
    private SendTidingsUtil sendTidingsUtil;
    /**
     * 工程信息
     */
    @Autowired
    private ProcessJournalService pJService;

    /**
     * 文件保存
     */
    @Autowired
    private FileService fileService;


    @Autowired
    private ProcessStuffService processStuffService;

    @Autowired
    private KeepGrainMatterService keepGrainMatterService;

    @Value("${ftp.flow.process}")
    private String processfile;

    /**
     * 根据流程ID、步骤、步骤节点 查询处理信息
     * @param processId 流程ID
     * @param processNode 步骤号
     * @param nodeBranch 步骤节点号
     * @return
     */
    @Override
    public ProcessNode getProcessNode(Long processId, Integer processNode, Integer nodeBranch) {

        return this.baseMapper.getProcessNode(processId,processNode,nodeBranch,FlowConstant.INTEGER_TWO);
    }

    /**
     * 校验流程步骤信息
     *
     * @param process
     * @param processNode 步骤信息数据
     * @param processId 流程ID
     * @param configId 流程配置ID
     * @param edition
     * @return
     */
    @Override
    public String saveCheck(Process process, ProcessNodeDto processNode, Long processId, Long configId, Integer edition, Map<String, MultipartFile> map) {
        /** 验证步骤信息 */
        String str = checkProcessNode(processNode,processId,configId,edition);
        if(StringUtils.isNotEmpty(str)){
            return str;
        }
        /**校验合同信息*/
        if(CollUtil.isNotEmpty(processNode.getContractDto())) {
            str = pCService.saveCheck(processNode.getContractDto(), map);
            if (StringUtils.isNotEmpty(str)) {
                return str;
            }
        }
        /**检验出入库物资信息*/
        if(CollUtil.isNotEmpty(processNode.getStuffDtos())){
            str = pSService.saveListCheck(process,processNode.getStuffDtos());
            if (StringUtils.isNotEmpty(str)) {
                return str;
            }
        }
        /**检验关联信息*/
        if(CollUtil.isNotEmpty(processNode.getPrdtos())){
            str = pRService.saveListCheck(processNode.getPrdtos());
            if (StringUtils.isNotEmpty(str)) {
                return str;
            }
        }
        /**检验文档、公告*/
        if(processNode.getDocumentDto()!=null){
            str = documentService.saveCheck(processNode.getDocumentDto(),map);
            if (StringUtils.isNotEmpty(str)) {
                return str;
            }
        }

        return null;
    }

    /**
     * 保存流程处理步骤
     * @param dto 步骤信息数据
     * @param processId 流程ID
     * @param configId 流程配置ID
     * @param edition 版本
     * @return
     */
    @Override
    public Boolean saveProcessNode(ProcessNodeDto dto, Long processId, Long configId, Integer edition, Map<String,MultipartFile> map) {
        List<ProjectContractFileDto> contracts = new ArrayList<>();
        if(CollUtil.isNotEmpty(dto.getContractDto())){
            for (ProjectContractAllDto pcdto:dto.getContractDto()) {
                List<MultipartFile> files = null;
                if(CollUtil.isNotEmpty(pcdto.getFileNames())){
                    files = new ArrayList<>();
                    for (String name :pcdto.getFileNames()) {
                        if(!map.containsKey(name)){
                            throw new RuntimeException("上传文件未找到！");
                        }
                        files.add(map.get(name)) ;
                        map.remove(name);
                    }
                }
                contracts.add(new ProjectContractFileDto(pcdto,files));
            }
        }
        /**公告文件*/
        List<MultipartFile> dfiles = new ArrayList<>();
        if(dto.getDocumentDto()!=null&&CollUtil.isNotEmpty(dto.getDocumentDto().getFileNames())){
            for (String name:dto.getDocumentDto().getFileNames()) {
                if(!map.containsKey(name)){
                    throw new RuntimeException("上传文件未找到！");
                }
                dfiles.add(map.get(name));
                map.remove(name);
            }
        }
        /**工程勘测*/
        List<MultipartFile> psfiles = new ArrayList<>();
        if(dto.getProcessSurveyDto()!=null&&CollUtil.isNotEmpty(dto.getProcessSurveyDto().getFileNames())){
            for (String name:dto.getProcessSurveyDto().getFileNames()) {
                if(!map.containsKey(name)){
                    throw new RuntimeException("上传文件未找到！");
                }
                psfiles.add(map.get(name));
                map.remove(name);
            }
        }

        /**工程勘测*/
        List<MultipartFile> pjfiles = new ArrayList<>();
        if(dto.getProcessJournalDto()!=null&&CollUtil.isNotEmpty(dto.getProcessJournalDto().getFileNames())){
            for (String name:dto.getProcessJournalDto().getFileNames()) {
                if(!map.containsKey(name)){
                    throw new RuntimeException("上传文件未找到！");
                }
                pjfiles.add(map.get(name));
                map.remove(name);
            }
        }
        /** 物资设备*/
        Map<String,List<MultipartFile>> deviceFiles = new HashMap<>();
        if(CollUtil.isNotEmpty(dto.getStuffDtos())){
            for (ProcessStuffDto stuff:dto.getStuffDtos()) {
                if(CollUtil.isNotEmpty(stuff.getFileNames())){
                    String key = String.join("_",stuff.getFileNames());
                    List<MultipartFile> stufffile = new ArrayList<>();
                    for (String name:stuff.getFileNames()) {
                        if(!map.containsKey(name)){
                            throw new RuntimeException("上传文件未找到！");
                        }
                        stufffile.add(map.get(name));
                        map.remove(name);
                    }
                    deviceFiles.put(key,stufffile);
                }
                if(CollUtil.isNotEmpty(stuff.getDevices())){
                    for (ProcessDeviceDto deviceDto:stuff.getDevices()) {
                        if(CollUtil.isNotEmpty(deviceDto.getFileNames())){
                            String key = String.join("_",deviceDto.getFileNames());
                            List<MultipartFile> devicefile = new ArrayList<>();
                            for (String name:deviceDto.getFileNames()) {
                                if(!map.containsKey(name)){
                                    throw new RuntimeException("上传文件未找到！");
                                }
                                devicefile.add(map.get(name));
                                map.remove(name);
                            }
                            deviceFiles.put(key,devicefile);
                        }
                    }
                }
            }
        }



        //合同剩余的文件为附件
        List<MultipartFile> fileList = new ArrayList<>(map.values());
        /** 保存流程步骤节点*/
        ProcessNode processNode = saveNodeInfo(dto, processId, configId, edition, fileList);
        if (processNode == null) return false;

        /** 保存流程中合同信息*/
        if(CollUtil.isNotEmpty(contracts)) {
            pCService.saveBatchProjectContract(contracts, processNode);
        }

        Process process = processService.getById(processId);
        /**保存流程中出入库信息*/
        if(CollUtil.isNotEmpty(dto.getStuffDtos())){
            pSService.saveListProcessStuff(dto.getStuffDtos(),processNode.getProcessStep(),processNode.getStepNode(),process,deviceFiles);
        }
        /**保存费用信息*/
        if(CollUtil.isNotEmpty(dto.getDetailedDtos())){
            pDService.saveListProjectDetailed(dto.getDetailedDtos(),processId,processNode.getProcessStep(),processNode.getStepNode());
        }
        /**保存关联设施信息*/
        if(CollUtil.isNotEmpty(dto.getPrdtos())){
            pRService.saveListProcessRelation(dto.getPrdtos(),processId,processNode.getProcessStep(),processNode.getStepNode());
        }
        /**保存通知、监督*/
        if(CollUtil.isNotEmpty(dto.getStaffDtos())){
            pStaffService.saveListpStaffService(dto.getStaffDtos(),processId,processNode.getProcessStep(),processNode.getStepNode());
        }
        /**保存文档、公告*/
        if(dto.getDocumentDto()!=null){
            documentService.saveDocument(dto.getDocumentDto(),processId,processNode.getProcessStep(),processNode.getStepNode(),dfiles);
        }
        /**保存工程勘测信息*/
        if(dto.getProcessSurveyDto()!=null){
            pSurveyService.saveProcessSurvey(dto.getProcessSurveyDto(),processId,processNode.getProcessStep(),processNode.getStepNode(),psfiles);
        }

        if(dto.getProcessJournalDto()!=null){
            pJService.saveProcessJournal(dto.getProcessJournalDto(),processId,processNode.getProcessStep(),processNode.getStepNode(),pjfiles);
        }

        if(process.getYwType()==4 && dto.getHeadleState()==1 && dto.getSaveState()==1 && dto.getNextStep()==null){
            //提交审核,流程审核完成了
            List<ProcessStuff> listStuff=processStuffService.list(Wrappers.<ProcessStuff>lambdaQuery()
                    .eq(ProcessStuff::getProcessId,processId));
             //
            listStuff.forEach(a->{
                KeepGrainMatter keepGrainMatter=keepGrainMatterService.list(Wrappers.<KeepGrainMatter>lambdaQuery()
                        .eq(KeepGrainMatter::getMatterId,a.getMatterId())).stream().findFirst().orElse(null);

             if(keepGrainMatter!=null){

                 Long nowQuantity=keepGrainMatter.getQuantity()-a.getQuantity();
                 if(nowQuantity<0){
                     throw new RuntimeException("审核失败，库存数量不足！");
                 }
                 KeepGrainMatter updateGrainMatter=new KeepGrainMatter();
                 updateGrainMatter.setId(keepGrainMatter.getId());
                 updateGrainMatter.setOutQuantity(keepGrainMatter.getOutQuantity()+a.getQuantity());
                 updateGrainMatter.setQuantity(nowQuantity);
                 keepGrainMatterService.updateById(updateGrainMatter);
             }
            });
        }

        return true;
    }

    /**
     * 保存流程步骤节点数据
     * @param dto
     * @param processId
     * @param configId
     * @param edition
     * @param fileList
     * @return
     */
    private ProcessNode saveNodeInfo(ProcessNodeDto dto, Long processId, Long configId, Integer edition, List<MultipartFile> fileList) {
        ProcessNode pn = null;
        ProcessConfigStep pcs = pStepService.getOne(new QueryWrapper<ProcessConfigStep>().lambda().eq(ProcessConfigStep::getConfigId,configId)
                .eq(ProcessConfigStep::getEdition,edition).eq(ProcessConfigStep::getStepOrder,dto.getProcessStep()));
        ProcessNode processNode = new ProcessNode();
        BeanUtils.copyProperties(dto,processNode);
        if (!fillProcessNode(dto, processId, fileList,  pcs, processNode)) {
            throw new RuntimeException("步骤错误");
        }
        //删除下一步、转派、驳回操作待执行信息
        this.remove(new QueryWrapper<ProcessNode>().lambda().eq(ProcessNode::getUpHandleId,processNode.getId()).
                eq(ProcessNode::getProcessId, processId).eq(ProcessNode::getNodeState,FlowConstant.SHORT_ZERO)
                .eq(ProcessNode::getIsDel,FlowConstant.SHORT_ZERO));

        if(dto.getSaveState()==FlowConstant.INTEGER_ZERO&& dto.getHeadleState()==FlowConstant.INTEGER_TWO){
            return processNode;
        }
        ProcessNode pNext = saveNextProcessNode(dto, processId, configId, edition,processNode.getId());
        /**更改流程基础信息*/
        if(dto.getSaveState()==FlowConstant.INTEGER_ONE) {
            ProcessConfig pc = pcService.getById(configId);
            //取消待办
            if(dto.getProcessStep()!=FlowConstant.INTEGER_ONE||dto.getStepNode()!=FlowConstant.INTEGER_ONE) {
                sendTidingsUtil.removeDB(processId, pc.getProcessName() + "_" + pcs.getStepName());
            }
            Process process = updateProcessInfo(dto, processId, pNext);
            if(pNext!=null) {
                // 发送消息、待办
                sendTidings(pc,processId, configId, edition,pNext);
            }
            if(process.getProcessState()==FlowConstant.SHORT_FOUR){
                ProcessConfigInfo configInfo = pInfoService.getOne(new QueryWrapper<ProcessConfigInfo>().lambda()
                        .eq(ProcessConfigInfo::getConfigId,configId).eq(ProcessConfigInfo::getEdition,edition));
                //发送流程ID到redis缓存队列
                redisTemplate.opsForList().leftPush(FlowConstant.REDIS_QUERY_KEY+pc.getProcessNumber().toLowerCase(), process.getId().toString());
            }
        }
        return processNode;
    }

    /**
     * 填充保存步骤信息
     * @param dto 步骤节点信息
     * @param processId 流程ID
     * @param fileList 文件信息
     * @param pcs 本步骤配置信息
     * @param processNode 步骤节点对象
     * @return 操作结果
     */
    private Boolean fillProcessNode(ProcessNodeDto dto, Long processId, List<MultipartFile> fileList,  ProcessConfigStep pcs, ProcessNode processNode) {
        processNode.setProcessId(processId);
        processNode.setHandleDept(SecurityUtils.getUserDeptId());
        processNode.setIsValid(processNode.getIsValid()==null?FlowConstant.SHORT_ONE:processNode.getIsValid());
        processNode.setIsDel(FlowConstant.SHORT_ZERO);
        processNode.setHandleStaff(SecurityUtils.getUserId());
        processNode.setHandleTime(new Date());
        processNode.setIsOvertime(FlowConstant.SHORT_ZERO);
        ProcessNode pn = null;
        if(processId !=null) {
            //未提交
            pn = this.getOne(new QueryWrapper<ProcessNode>().lambda()
                    .eq(ProcessNode::getProcessId, processId).eq(ProcessNode::getProcessStep, dto.getProcessStep())
                    .eq(ProcessNode::getStepNode, dto.getStepNode()).eq(ProcessNode::getIsDel, FlowConstant.SHORT_ZERO)
                    .eq(ProcessNode::getHandleStaff,SecurityUtils.getUserId())
                    .lt(ProcessNode::getNodeState, FlowConstant.SHORT_TWO));
            if(pn ==null){
                pn = this.baseMapper.getProcessNode(processId, dto.getProcessStep(), dto.getStepNode(),FlowConstant.INTEGER_THREE);
                //重复提交
                if(pn !=null&& pn.getNodeState()==FlowConstant.INTEGER_TWO) {
                    //是否允许重复
                    if(pcs.getIsRepeat()==FlowConstant.SHORT_ZERO){
                        return false;
                    }
                    processNode.setUpStep(pn.getUpStep());
                    processNode.setUpStepNode(pn.getUpStepNode());
                    if(processNode.getUpHandleId()==null){
                        processNode.setUpHandleId(pn.getUpHandleId());
                    }
                    processNode.setIsDel(FlowConstant.SHORT_ZERO);
                }
            }
        }
        if(pn !=null&& pn.getNodeState()!=FlowConstant.INTEGER_TWO){
            processNode.setId(pn.getId());
            if(pn.getTimeLimit()!=null&& pn.getTimeLimit().getTime()<new Date().getTime()){
                processNode.setIsOvertime(FlowConstant.SHORT_ONE);
            }
        }else if(dto.getProcessStep()!=FlowConstant.INTEGER_ONE|| dto.getStepNode()!=FlowConstant.INTEGER_ONE){
            return false;
        }
        if(CollUtil.isNotEmpty(fileList)) {
            String filepath = UploadFiles.saveFiles(fileService, fileList, pn ==null?"": pn.getAnnex(), dto.getAnnex(), processfile, FlowConstant.TF_YWPN_PROCESS_NODE_W);
            if(StringUtils.isEmpty(filepath)&& processNode.getId()!=null){
                this.update(new UpdateWrapper<ProcessNode>().lambda()
                        .set(ProcessNode::getAnnex,filepath).eq(ProcessNode::getId, processNode.getId()));
            }
            processNode.setAnnex(filepath);
        }
        //保存，标识处理中
        if(dto.getSaveState()==FlowConstant.INTEGER_ZERO){
            processNode.setNodeState(FlowConstant.INTEGER_ONE);
        //提交
        }else {
            processNode.setNodeState(dto.getHeadleState()+FlowConstant.INTEGER_ONE);
            if(dto.getHeadleState()==FlowConstant.INTEGER_TWO|| dto.getHeadleState()==FlowConstant.INTEGER_THREE){
                processNode.setIsValid(FlowConstant.SHORT_ZERO);
            }
        }
        //保存本步，流程操作
        if(!this.saveOrUpdate(processNode)){
            throw new RuntimeException("保存失败");
        }
        return true;
    }

    /**
     * 发送消息、待办
     * @param pc 流程配置信息
     * @param processId 流程ID
     * @param configId 配置ID
     * @param edition 配置版本
     * @param pNext 流程下一步信息
     */
    private void sendTidings(ProcessConfig pc,Long processId, Long configId, Integer edition, ProcessNode pNext) {
        String  stepName = pStepService.getObj(new QueryWrapper<ProcessConfigStep>().select(" STEP_NAME ").lambda().eq(ProcessConfigStep::getConfigId,configId)
                .eq(ProcessConfigStep::getEdition,edition).eq(ProcessConfigStep::getStepOrder,pNext.getProcessStep()),(o)->{return String.valueOf(o);});
        StringBuilder sb = new StringBuilder("[");
        sb.append(pc.getProcessName()).append("]");
        sb.append(FlowConstant.ADD_HANDLE).append("[").append(stepName).append("]");

        Messagepush message = sendTidingsUtil.getMessage(processId,sb.toString(),pc.getProcessName()+"_"+stepName);
        StringBuilder sbip = new StringBuilder(pc.getProcessNumber());
        sbip.append("|").append(pNext.getProcessStep());
        sbip.append(";").append(pc.getBusinessType()).append("|").append(pNext.getProcessStep());

        message.setRouteIp(sbip.toString());
        message.setRouteIpApp(sbip.toString());
        sendTidingsUtil.setMessageRecipient(message,pNext.getHandleStaff(),pNext.getTimeLimit());
    }

    /**
     * 保存步骤节点处理信息
     * @param dto
     * @param fileList
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object saveProcessNodeAll(ProcessNodeAllDto dto, List<MultipartFile> fileList) {
        Process process = processService.getById(dto.getProcessId());
        if(process==null||process.getIsDel()==FlowConstant.SHORT_ONE||
                process.getProcessState()==FlowConstant.SHORT_THREE||process.getProcessState()==FlowConstant.SHORT_FOUR){
            return FlowConstant.PROCESS_NULL;
        }
        Map<String,MultipartFile> map = fileList.stream().collect(Collectors.toMap(MultipartFile::getOriginalFilename, v -> v));
        /**校验*/
        String str = saveCheck(process, dto,process.getId(),process.getConfigId(),process.getEdition(),map);
        if(StringUtils.isNotEmpty(str)){
            return str;
        }
        /**保存*/
        return saveProcessNode(dto,process.getId(),process.getConfigId(),process.getEdition(),map);
    }

    /**
     * 根据流程步骤ID查询流程步骤信息
     * @param id 流程步骤ID
     * @return 流程步骤信息
     */
    @Override
    public ProcessNodeVo getProcessNodeById(Long id) {
        return this.baseMapper.getProcessNodeById(id);
    }

    /**
     * 保存下一步流程步骤
     * @param dto 请求传入步骤处理信息
     * @param processId 流程ID
     * @param configId 配置ID
     * @param edition 配置版本
     * @param processNodeId 库中存储的本步骤信息主键ID
     * @return
     */
    private ProcessNode saveNextProcessNode(ProcessNodeDto dto, Long processId, Long configId, Integer edition, Long processNodeId) {
        ProcessNode processNode = this.getById(processNodeId);
        ProcessNode pNext = null;
        if(dto.getHeadleState()==FlowConstant.INTEGER_TWO||(dto.getNextStep()!=null&& dto.getHeadleState()!=FlowConstant.INTEGER_FOUR)){
            pNext = new ProcessNode();
            NextStepDto nextdto = dto.getNextStep();
            ProcessConfigStep npc = null;
            //下一步
            if(dto.getHeadleState()==FlowConstant.INTEGER_ONE){
                //当前步骤
                pNext.setProcessStep(nextdto.getProcessStep());
                pNext.setStepNode(nextdto.getStepNode());
                pNext.setUpStepNode(processNode.getId());
                //上一步骤
                if(nextdto.getStepNode()==FlowConstant.INTEGER_ONE){
                    if(processNode.getStepNode()==FlowConstant.INTEGER_ONE) {
                        pNext.setUpStep(processNode.getId());
                    }else {
                        Long upstepId = this.getObj(new QueryWrapper<ProcessNode>().select("MAX(ID) as ID").lambda()
                                        .eq(ProcessNode::getProcessId, processId).eq(ProcessNode::getProcessStep, processNode.getProcessStep())
                                        .eq(ProcessNode::getStepNode,FlowConstant.INTEGER_ONE).eq(ProcessNode::getNodeState,FlowConstant.SHORT_TWO)
                                        .eq(ProcessNode::getIsValid,FlowConstant.SHORT_ONE).eq(ProcessNode::getIsDel,FlowConstant.SHORT_ZERO)
                                ,(o)->{return Long.parseLong(String.valueOf(o));} );
                        pNext.setUpStep(upstepId);
                    }
                }else {
                    pNext.setUpStep(processNode.getUpStep());
                }
                npc = pStepService.getOne(new QueryWrapper<ProcessConfigStep>().lambda()
                        .eq(ProcessConfigStep::getConfigId, configId).eq(ProcessConfigStep::getStepOrder,pNext.getProcessStep())
                        .eq(ProcessConfigStep::getEdition, edition));
                //当前处理人
                if(npc.getUnanimousOrder()==null){
                    pNext.setHandleDept(nextdto.getDept());
                    pNext.setHandleStaff(nextdto.getDeptStaff());
                }else {
                    ProcessNode upn = this.getOne(new QueryWrapper<ProcessNode>().lambda().eq(ProcessNode::getProcessId, processId)
                            .eq(ProcessNode::getProcessStep,npc.getUnanimousOrder()).eq(ProcessNode::getStepNode,nextdto.getStepNode())
                            .eq(ProcessNode::getNodeState,FlowConstant.SHORT_TWO).eq(ProcessNode::getIsValid,FlowConstant.SHORT_ONE)
                            .eq(ProcessNode::getIsDel,FlowConstant.SHORT_ZERO));
                    pNext.setHandleDept(upn.getHandleDept());
                    pNext.setHandleStaff(upn.getHandleStaff());
                }
                //驳回
            }else if(dto.getHeadleState()==FlowConstant.INTEGER_TWO){

                //根据驳回方式修改对应处理为状态为被驳回
                ProcessConfigInfo pcf = pInfoService.getOne(new QueryWrapper<ProcessConfigInfo>().lambda()
                        .eq(ProcessConfigInfo::getConfigId, configId).eq(ProcessConfigInfo::getEdition, edition));
                //逐步驳回
                ProcessNode upn = null;
                if(pcf.getReiectMode()==FlowConstant.SHORT_ONE){
                    upn = this.getById(processNode.getUpStepNode());
                    if(upn == null ){
                        throw new RuntimeException(FlowConstant.DATA_NOTHINGNESS);
                    }
                    this.update(new UpdateWrapper<ProcessNode>().lambda()
                            .eq(ProcessNode::getId, processNode.getUpStepNode())
                            .set(ProcessNode::getNodeState,FlowConstant.SHORT_SIX).set(ProcessNode::getIsValid,FlowConstant.SHORT_ZERO));
                    //主流程驳回
                }else if(pcf.getReiectMode()==FlowConstant.SHORT_TWO){
                    if(processNode.getUpStep()!=null){
                        upn = this.getById(processNode.getUpStep());
                        Integer upnProcessStep = upn.getProcessStep();
                        this.update(new UpdateWrapper<ProcessNode>().lambda()
                                .eq(ProcessNode::getProcessId, processId).eq(ProcessNode::getNodeState,FlowConstant.SHORT_TWO)
                                .eq(ProcessNode::getIsValid,FlowConstant.SHORT_ONE).eq(ProcessNode::getIsDel,FlowConstant.SHORT_ZERO)
                                .and(v->v.eq(ProcessNode::getProcessStep,upnProcessStep).or().eq(ProcessNode::getProcessStep, processNode.getProcessStep()))
                                .set(ProcessNode::getNodeState,FlowConstant.SHORT_SIX).set(ProcessNode::getIsValid,FlowConstant.SHORT_ZERO));
                    }else {
                        upn = this.getOne(new QueryWrapper<ProcessNode>().lambda().eq(ProcessNode::getProcessId, processId)
                                .eq(ProcessNode::getProcessStep,FlowConstant.SHORT_ONE).eq(ProcessNode::getStepNode,FlowConstant.SHORT_ONE)
                                .eq(ProcessNode::getNodeState,FlowConstant.SHORT_TWO).eq(ProcessNode::getIsValid,FlowConstant.SHORT_ONE)
                                .eq(ProcessNode::getIsDel,FlowConstant.SHORT_ZERO));
                        this.update(new UpdateWrapper<ProcessNode>().lambda()
                                .eq(ProcessNode::getProcessId, processId).eq(ProcessNode::getNodeState,FlowConstant.SHORT_TWO)
                                .eq(ProcessNode::getIsValid,FlowConstant.SHORT_ONE).eq(ProcessNode::getIsDel,FlowConstant.SHORT_ZERO)
                                .eq(ProcessNode::getProcessStep, processNode.getProcessStep())
                                .set(ProcessNode::getNodeState,FlowConstant.SHORT_SIX).set(ProcessNode::getIsValid,FlowConstant.SHORT_ZERO));
                    }
                }
                //设置下一步处理
                copyAttribute(pNext, upn);
                //转派
            }else if(dto.getHeadleState()==FlowConstant.INTEGER_THREE){

                pNext.setProcessStep(nextdto.getProcessStep());
                pNext.setStepNode(nextdto.getStepNode());
                pNext.setUpStepNode(processNode.getUpStepNode());
                pNext.setUpStep(processNode.getUpStep());
                pNext.setHandleDept(nextdto.getDept());
                pNext.setHandleStaff(nextdto.getDeptStaff());
            }
            pNext.setProcessId(processId);
            pNext.setIsValid(FlowConstant.SHORT_ONE);
            pNext.setIsDel(FlowConstant.SHORT_ZERO);
            pNext.setNodeState(FlowConstant.INTEGER_ZERO);
            pNext.setUpHandleId(processNode.getId());
            if(dto.getSaveState()==FlowConstant.INTEGER_ONE) {
                if (npc == null) {
                    npc = pStepService.getOne(new QueryWrapper<ProcessConfigStep>().lambda()
                            .eq(ProcessConfigStep::getConfigId, configId).eq(ProcessConfigStep::getStepOrder, pNext.getProcessStep())
                            .eq(ProcessConfigStep::getEdition, edition));
                }
                //处理时限
                if (npc.getTimeLimit() != null) {
                    pNext.setTimeLimit(DateUtil.offsetMinute(new Date(), npc.getTimeLimit().intValue()));
                }
            }
            this.save(pNext);
        }
        return pNext;
    }

    /**
     * 更改流程基础信息
     * @param dto 传入保存流程步骤信息
     * @param processId 流程ID
     * @param pNext 保存下一步流程步骤信息
     */
    private Process updateProcessInfo(ProcessNodeDto dto, Long processId, ProcessNode pNext) {
        String passingNode = processService.getObj(new QueryWrapper<Process>().select(" PASSING_NODE ").lambda()
                        .eq(Process::getId, processId).eq(Process::getIsDel,FlowConstant.SHORT_ZERO)
                ,(o)-> String.valueOf(o) );
        Process process = new Process();
        process.setId(processId);
        //流程中步骤终止或流程结束，修改流程为归档
        if(dto.getHeadleState()==FlowConstant.INTEGER_FOUR||(dto.getHeadleState()==FlowConstant.INTEGER_ONE&& dto.getNextStep()==null)){
            if(dto.getHeadleState()==FlowConstant.INTEGER_FOUR){
                process.setProcessState(FlowConstant.SHORT_THREE);
            }else {
                process.setProcessState(FlowConstant.SHORT_FOUR);
            }
            process.setProcessNode(FlowConstant.INTEGER_ONE_HUNDRED);
            if(!passingNode.contains(String.valueOf(dto.getProcessStep()))){
                process.setPassingNode(passingNode+FlowConstant.CSV+ dto.getProcessStep());
            }
        }else {
            process.setProcessState(FlowConstant.SHORT_TWO);
            process.setHandleDept(pNext.getHandleDept());
            process.setHandleStaff(pNext.getHandleStaff());
            process.setProcessNode(pNext.getProcessStep());
            process.setNodeBranch(pNext.getStepNode());
            if(pNext.getUpStepNode()==null){
                process.setPassingNode(null);
                processService.update(new UpdateWrapper<Process>().setSql(" PASSING_NODE = NULL ").lambda().eq(Process::getId,processId));
            }else {
                String processStep = this.getObj(new QueryWrapper<ProcessNode>().select(" PROCESS_STEP ").lambda()
                        .eq(ProcessNode::getId, pNext.getUpStepNode()).eq(ProcessNode::getIsValid,FlowConstant.SHORT_ONE)
                        .eq(ProcessNode::getIsDel,FlowConstant.SHORT_ZERO),(o)->{return String.valueOf(o);} );
                int index = passingNode.indexOf(processStep);
                if(index>0){
                    process.setPassingNode(passingNode.substring(0,index+1));
                }else {
                    process.setPassingNode(passingNode+FlowConstant.CSV+ dto.getProcessStep());
                }
            }
        }
        processService.updateById(process);

        return process;

    }

    /**
     * 赋值拷贝信息
     * @param pNext 下步处理信息
     * @param upn 转派、驳回
     */
    private void copyAttribute(ProcessNode pNext, ProcessNode upn) {
        pNext.setProcessStep(upn.getProcessStep());
        pNext.setStepNode(upn.getStepNode());
        pNext.setUpStepNode(upn.getUpStepNode());
        pNext.setUpStep(upn.getUpStep());
        pNext.setHandleDept(upn.getHandleDept());
        pNext.setHandleStaff(upn.getHandleStaff());
    }

    /**
     * 校验流程步骤基础信息信息
     * @param dto 步骤信息数据
     * @param processId 流程ID
     * @param configId 流程配置ID
     * @param edition 版本
     * @return
     */
    private String checkProcessNode(ProcessNodeDto dto, Long processId, Long configId, Integer edition){

        if(processId==null&&(dto.getProcessStep()!=FlowConstant.INTEGER_ONE||dto.getStepNode()!=FlowConstant.INTEGER_ONE)){
            return FlowConstant.PROCESS_ID_NULL;
        }
        //processId 不为null 流程步骤一定存在
        if(dto.getProcessStep()!=FlowConstant.INTEGER_ONE||dto.getStepNode()!=FlowConstant.INTEGER_ONE){
            int count = this.count(new QueryWrapper<ProcessNode>().lambda()
                    .eq(ProcessNode::getProcessId,processId).eq(ProcessNode::getProcessStep,dto.getProcessStep())
                    .eq(ProcessNode::getStepNode,dto.getStepNode()).eq(ProcessNode::getIsDel,FlowConstant.SHORT_ZERO)
                    .lt(ProcessNode::getNodeState,FlowConstant.SHORT_TWO));
            if(count==FlowConstant.INTEGER_ZERO){
                return FlowConstant.HANDLE_STEP_WRONG;
            }
        }

        /**流程步骤信息*/
        Map<Integer,ProcessConfigStep> map = pStepService.list(new QueryWrapper<ProcessConfigStep>().lambda()
                .eq(ProcessConfigStep::getConfigId,configId).eq(ProcessConfigStep::getEdition,edition)
                .orderByAsc(ProcessConfigStep::getStepOrder)).stream().collect(Collectors.toMap(ProcessConfigStep::getStepOrder,pc->pc));
        ProcessConfigStep pcs = map.get(dto.getProcessStep());


        /**验证本步骤是否有此处理方式*/
        if(!pcs.getHandleModel().contains(String.valueOf(dto.getHeadleState())) ){
            return FlowConstant.HEADLE_AUTHORITY_NO;
        }

        NextStepDto ns = dto.getNextStep();
        //if(dto.getSaveState()==FlowConstant.INTEGER_ONE) {
        if (ns != null && (ns.getProcessStep() == null || ns.getStepNode() == null)) {
            dto.setNextStep(null);
            ns = null;
        }
       // }


        /** 提交验证转派 */
        if(dto.getSaveState()==FlowConstant.INTEGER_ONE&&dto.getHeadleState()==FlowConstant.INTEGER_THREE){
            if(dto.getNextStep()==null) {
                return FlowConstant.REASSIGNMENT_NULL;
            }
            if(dto.getNextStep().getDept()==null||dto.getNextStep().getDeptStaff()==null){
                return FlowConstant.NEXT_STAFF_NULL;
            }
        }

        /**不等于提交下一步直接返回，驳回、转派、终止，下一步信息为空*/
        if(dto.getHeadleState()>FlowConstant.INTEGER_ONE){
            return null;
        }
        if(dto.getSaveState()==FlowConstant.INTEGER_ZERO&&dto.getNextStep()==null){
            return null;
        }
        if(processId!=null){
            Process process = processService.getById(processId);
            if (process==null||process.getIsDel()==FlowConstant.SHORT_ONE){
                return FlowConstant.PROCESS_NULL;
            }

//            if(process.getHandleStaff()!=SecurityUtils.getUserId()){
//                return FlowConstant.HEADLE_AUTHORITY_NO;
//            }
            log.info("操作用户"+SecurityUtils.getUserId()+";指定操作用户"+process.getHandleStaff());




            /**流程错误*/
            if(process.getProcessNode()!=dto.getProcessStep()||process.getNodeBranch()!=dto.getStepNode()) {
                if (process.getProcessNode() < dto.getProcessStep() || pcs.getIsRepeat() != FlowConstant.SHORT_ONE) {
                    return FlowConstant.HANDLE_STEP_WRONG;
                }
                //重复提交
                int count = this.count(new QueryWrapper<ProcessNode>().lambda().eq(ProcessNode::getProcessId, processId)
                        .eq(ProcessNode::getProcessStep, dto.getProcessStep()).eq(ProcessNode::getStepNode, dto.getStepNode())
                        .eq(ProcessNode::getNodeState, FlowConstant.SHORT_TWO));
                if(count==FlowConstant.INTEGER_ZERO){
                    return FlowConstant.HANDLE_STEP_WRONG;
                }
                return null;
            }
        }

        /**流程配置基础信息*/
        ProcessConfigInfo pci = pInfoService.getOne(new QueryWrapper<ProcessConfigInfo>().lambda()
                .eq(ProcessConfigInfo::getConfigId,configId).eq(ProcessConfigInfo::getEdition,edition));

        /**流程最后一步骤，步骤节点大于强制执行步骤；下一步为空则默认归档*/
        if(dto.getSaveState() == FlowConstant.INTEGER_ONE&&dto.getProcessStep()==pci.getStepNumber()
                &&dto.getNextStep()==null&&dto.getStepNode()>=pcs.getCoerceLevel()){
            return null;

        }

        //下一步信息
        if(ns==null){
            return FlowConstant.NEXT_DATA_NULL;
        }

        if(dto.getSaveState() == FlowConstant.INTEGER_ONE) {
            if (dto.getProcessStep().equals(ns.getProcessStep())) {
                if (dto.getStepNode() + 1 != ns.getStepNode() || ns.getStepNode() > pcs.getNodeNumber()) {
                    return FlowConstant.NEXT_STEP_NODE_ERROR;
                }
            } else {
                /**强制执行层级*/
                if (dto.getStepNode() < pcs.getCoerceLevel()) {
                    return FlowConstant.STEP_NODE_NOT_FINISHED;
                }
                /**步骤第一次必须为1*/
                if (ns.getStepNode() != FlowConstant.INTEGER_ONE) {
                    return FlowConstant.NEXT_STEP_NODE_ERROR;
                }
                if (!map.containsKey(ns.getProcessStep())) {
                    return FlowConstant.CONFIG_STEP_NULL;
                }
                ProcessConfigStep npc = map.get(ns.getProcessStep());
                if (npc.getUnanimousOrder() == null && (ns.getDept() == null && ns.getDeptStaff() == null)) {
                    return FlowConstant.NEXT_STAFF_NULL;
                }
                if (npc.getUnanimousOrder() != null) {
                    int count = this.count(new QueryWrapper<ProcessNode>().lambda().eq(ProcessNode::getProcessId, processId)
                            .eq(ProcessNode::getProcessStep, npc.getUnanimousOrder()).eq(ProcessNode::getStepNode, ns.getStepNode())
                            .eq(ProcessNode::getNodeState, FlowConstant.SHORT_TWO).eq(ProcessNode::getIsValid, FlowConstant.SHORT_ONE)
                            .eq(ProcessNode::getIsDel, FlowConstant.SHORT_ZERO));
                    if (count == FlowConstant.INTEGER_ZERO) {
                        return FlowConstant.UNANIMOUS_STEP_NULL;
                    }
                }

                if (!pcs.getNextStep().contains(String.valueOf(ns.getProcessStep()))) {
                    return FlowConstant.NEXT_SELECT_ERROR;
                }
            }
        }
        return null;
    }

}
