/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProcessRelationDto;
import com.tofly.flow.custom.entity.ProcessRelation;
import com.tofly.flow.custom.entity.ProjectDetailed;
import com.tofly.flow.custom.service.ProcessRelationService;
import com.tofly.flow.custom.vo.RelationInfoVo;
import com.tofly.flow.em.ProcessTypeEnum;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ProcessRelationMapper;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 流程关联信息表（会议室、车辆、文档等）
 *
 * @author xqf
 * @date Thu Apr 07 00:00:00 CST 2022
 */
@Service("processRelationService")
public class ProcessRelationServiceImpl extends ServiceImpl<ProcessRelationMapper, ProcessRelation> implements ProcessRelationService {

    /**
     * 验证关联信息
     * @param prdtos
     * @return
     */
    @Override
    public String saveListCheck(List<ProcessRelationDto> prdtos) {
        if(CollUtil.isNotEmpty(prdtos)) {
            List<ProcessRelation> pr = prdtos.stream().map(r->{
                ProcessRelation processRelation = new ProcessRelation();
                processRelation.setRelationTable(ProcessTypeEnum.loadTable(r.getTableId()));
                processRelation.setRelationId(r.getRelationId());
                return processRelation;
            }).collect(Collectors.toList());
            int count = this.baseMapper.loadRelationCount(pr);
            if(count!=prdtos.size()){
                return FlowConstant.DATA_NOTHINGNESS;
            }
        }
        return null;
    }

    /**
     * 保存关联信息
     * @param prdtos 关联信息数据
     * @param processId 流程ID
     * @param processStep 步骤
     * @param stepNode 步骤节点
     * @return
     */
    @Override
    public Boolean saveListProcessRelation(List<ProcessRelationDto> prdtos, Long processId, Integer processStep, Integer stepNode) {
        this.update(new UpdateWrapper<ProcessRelation>().lambda().set(ProcessRelation::getIsDel,FlowConstant.SHORT_ONE)
                .eq(ProcessRelation::getProcessId,processId).eq(ProcessRelation::getProcessNode,processStep)
                .eq(ProcessRelation::getNodeBranch,stepNode).eq(ProcessRelation::getIsDel,FlowConstant.SHORT_ZERO));
        if(CollUtil.isNotEmpty(prdtos)) {
            Long index = this.getObj(new QueryWrapper<ProcessRelation>().select("MAX(ID) as ID")
                    ,(o)->{return Long.parseLong(String.valueOf(o));} );
            if(index==null){
                index = 0l;
            }
            List<ProcessRelation> pr = new ArrayList<>();
            for (ProcessRelationDto r:prdtos) {
                ProcessRelation processRelation = new ProcessRelation();
                processRelation.setRelationTable(ProcessTypeEnum.loadTable(r.getTableId()));
                processRelation.setTableId(r.getTableId());
                processRelation.setRelationId(r.getRelationId());
                processRelation.setProcessId(processId);
                processRelation.setProcessNode(processStep);
                processRelation.setNodeBranch(stepNode);
                processRelation.setIsDel(FlowConstant.SHORT_ZERO);
                processRelation.setId(++index);
                pr.add(processRelation);
            }
            this.baseMapper.saveProcessRelations(pr);
        }
        return true;
    }

    @Override
    public RelationInfoVo getRelationInfoVo(Long processId, Integer processNode, Integer nodeBranch) {
        return this.baseMapper.getRelationInfoVo(processId,processNode,nodeBranch);
    }

}
