/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.api.order.OrderClient;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.*;
import com.tofly.flow.custom.entity.*;
import com.tofly.flow.custom.entity.Process;
import com.tofly.flow.custom.service.*;
import com.tofly.flow.custom.vo.ProcessListVO;
import com.tofly.flow.custom.vo.ProcessNodeVo;
import com.tofly.flow.custom.vo.ProcessVo;
import com.tofly.flow.entity.FolderDocument;
import com.tofly.flow.util.SendTidingsUtil;
import com.tofly.pojo.maintain.Repairorder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ProcessMapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 流程信息
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@Service("processService")
public class ProcessServiceImpl extends ServiceImpl<ProcessMapper, Process> implements ProcessService {

    /**
     * 流程配置信息
     */
    @Autowired
    private ProcessConfigService processConfigService;

    /**
     * redis
     */
    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 流程步骤信息
     */
    @Autowired
    private ProcessNodeService processNodeService;

    /**
     * 配置流程步骤
     */
    @Autowired
    private ProcessConfigStepService pStepService;

    /**
     * 发送消息
     */
    @Autowired
    private SendTidingsUtil sendTidingsUtil;
    @Autowired
    private ProcessSqlService processSqlService;


    @Autowired
    private OrderClient orderClient;
    /**
     * 保存流程
     * @param dto 流程数据
     * @param fileList 文件
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object saveProcess(ProcessDto dto, List<MultipartFile> fileList) {
        Process process = new Process();
        BeanUtils.copyProperties(dto,process);
        ProcessNodeDto processNode = dto.getProcessNode();
        /**校验流程数据*/
        String str = saveCheck(process);
        if(StringUtils.isNotEmpty(str)){
            return str;
        }
        Map<String,MultipartFile> map = fileList.stream().collect(Collectors.toMap(MultipartFile::getOriginalFilename, v -> v));
        /**校验流程步骤数据*/
        str = processNodeService.saveCheck(process,processNode,process.getId(),process.getConfigId(),process.getEdition(),map);
        if(StringUtils.isNotEmpty(str)){
            return str;
        }
        /**保存信息*/
        if(saveProcess(process,processNode)){
            return processNodeService.saveProcessNode(processNode,process.getId(),process.getConfigId(),process.getEdition(),map);
        }else {
            return false;
        }

    }

    /**
     * 删除工单
     * @param id 工单ID
     * @return
     */
    @Override
    public boolean removeUpdateById(Long id) {
        return this.update(new UpdateWrapper<Process>().lambda()
                .eq(Process::getId,id).eq(Process::getIsDel,FlowConstant.SHORT_ZERO)
                .eq(Process::getRegisterStaff,SecurityUtils.getUserId())
                .and(r->r.eq(Process::getProcessState,FlowConstant.SHORT_THREE).or().eq(Process::getProcessState,FlowConstant.SHORT_FOUR).or()
                        .eq(Process::getProcessState,FlowConstant.SHORT_ONE))
                .set(Process::getIsDel,FlowConstant.SHORT_ONE));
    }

    /**
     * 根据ID获取流程信息
     * @param id 流程ID
     * @return 流程信息
     */
    @Override
    public ProcessVo getProcessById(Long id) {
        ProcessVo vo = this.baseMapper.getProcessById(id);
        if((vo.getProcessState()==FlowConstant.SHORT_TWO||vo.getProcessState()==FlowConstant.SHORT_ONE)&& CollUtil.isNotEmpty(vo.getProcessNodeVos())){
            Integer pn = vo.getProcessNode();
            Integer nb = vo.getNodeBranch();
            for (ProcessNodeVo pnvo:vo.getProcessNodeVos()) {
                if(pnvo.getNodeState()>FlowConstant.INTEGER_ONE){
                    continue;
                }
                if(pnvo.getProcessStep()==pn && pnvo.getStepNode()==nb &&
                        (pnvo.getNodeState()==FlowConstant.INTEGER_ONE||pnvo.getNodeState()==FlowConstant.INTEGER_ZERO)){
                    vo.setCurrentNode(pnvo);
                }else if((pnvo.getProcessStep()>pn || pnvo.getStepNode()>nb )&& pnvo.getNodeState()==FlowConstant.INTEGER_ZERO){
                    vo.setNextNode(pnvo);
                }
            }

        }
        return vo;
    }

    /**
     * 停止流程
     * @param id 流程id
     * @param processStep 流程步骤
     * @param stepNode 步骤节点
     * @return 停止结果
     */
    @Override
    public Object stopProcess(Long id, Integer processStep, Integer stepNode) {
        if(id==null){
            return FlowConstant.DATA_NOTHINGNESS;
        }
        if(processStep==null||stepNode==null){
            return FlowConstant.STEP_OR_NODE_NULL;
        }
        Process process = this.getById(id);
        if(process==null){
            return true;
        }
        Long configid = process.getConfigId();
        ProcessConfigStep pcs = pStepService.getOne(new QueryWrapper<ProcessConfigStep>().lambda().eq(ProcessConfigStep::getConfigId,configid)
                .eq(ProcessConfigStep::getStepOrder,processStep));
        if(pcs==null){
            return FlowConstant.CONFIG_STEP_NULL;
        }

        int count = processNodeService.count(new QueryWrapper<ProcessNode>().lambda().eq(ProcessNode::getProcessId,id)
                .eq(ProcessNode::getProcessStep,processStep).eq(ProcessNode::getStepNode,stepNode).eq(ProcessNode::getHandleStaff,SecurityUtils.getUserId())
                .eq(ProcessNode::getIsValid,FlowConstant.SHORT_ONE).eq(ProcessNode::getIsDel,FlowConstant.SHORT_ZERO));

        if(count==FlowConstant.INTEGER_ZERO){
            return FlowConstant.NO_RIGHT_TO_MODIFY;
        }
        if(pcs.getHandleModel().contains(FlowConstant.STRING_FOUR)){
            return FlowConstant.NO_RIGHT_TO_MODIFY;
        }
        this.update(new UpdateWrapper<Process>().lambda().eq(Process::getId,id).set(Process::getProcessState,FlowConstant.SHORT_THREE));

        //取消本工单所有待办
        sendTidingsUtil.removeDB(id);

        return true;
    }

    /**
     * 查询流程
     * @param page 分页
     * @param dto 条件
     * @return 流程信息
     */
    @Override
    public Page<ProcessListVO> getProcessList(Page page, LoadProcessDto dto) {
        return this.baseMapper.getProcessList(page,dto,SecurityUtils.getUserId());
    }

    @Override
    public Page<Map<String, Object>> getProcesss(Page page,ProcessLDto dto) {
        ProcessSql psql = processSqlService.getById(dto.getId());
        if(psql==null || psql.getIsDel()==FlowConstant.SHORT_ONE){
            return null;
        }
        if(StringUtils.isEmpty(psql.getContent())){
            return null;
        }
        SqlSelectDto sqldto = JSONObject.parseObject(psql.getContent(),SqlSelectDto.class);
        Map<String,String> map = new HashMap<>();
        Map<String,Object> mapv = dto.getMap()==null?new HashMap<>():dto.getMap();
        if(CollUtil.isNotEmpty(sqldto.getWhere())) {
            for (SqlWhereDto wdto : sqldto.getWhere()) {
                String key = "#{" + wdto.getNumber() + "}";
                String sql = null;
                if (CollUtil.isNotEmpty(mapv)) {
                    StringBuilder sb = new StringBuilder();
                    if (CollUtil.isNotEmpty(wdto.getCondition())) {
                        for (SqlConditionDto cdto : wdto.getCondition()) {
                            if (CollUtil.isNotEmpty(cdto.getNames())) {
                                boolean isif = true;
                                for (String name : cdto.getNames()) {
                                    if (!mapv.containsKey(name)) {
                                        isif = false;
                                        break;
                                    }
                                }
                                if (isif && StringUtils.isNotEmpty(cdto.getSql())) {
                                    sb.append(cdto.getSql());
                                } else if (!isif && StringUtils.isNotEmpty(cdto.getElsesql())) {
                                    sb.append(cdto.getElsesql());
                                }
                            }
                        }
                    }
                    sql = sb.toString();
//                    if (wdto.getNumber() == FlowConstant.INTEGER_ONE) {
//                        sql = sql.replaceFirst("AND", "");
//                    }
                }
                map.put(key, sql);
            }
        }
        String sql = sqldto.getSelect();
        for (Map.Entry<String,String> entry:map.entrySet()) {
            String value = entry.getValue();
            if(StringUtils.isNotEmpty(value)){
                int kindex = sql.indexOf(entry.getKey());
                int windex = sql.lastIndexOf("WHERE",kindex);
                if(sql.substring(windex,kindex).trim().length()==0){
                    if(value.trim().indexOf("AND") == 0) {
                        value = value.replaceFirst("AND", "");
                    }
                }
            }
            sql = sql.replace(entry.getKey(),value==null?"":value);
        }
        String countsql = "SELECT COUNT(1) CT FROM ("+sql+") COUNT_TABLE";
        mapv.put("sql",countsql);
        int count = baseMapper.getCount(mapv);

        if (page.getSize() * (page.getCurrent() - 1) <= count) {
            // 当前页第一条数据在List中的位置
            int start = (int) ((page.getCurrent() - 1) * page.getSize());
            // 当前页最后一条数据在List中的位置
            int end = (int) ((start + page.getSize()) > count ? count : (page.getSize() * page.getCurrent()));
            page.setRecords(new ArrayList<>());
            String pagesql = "SELECT * FROM ( SELECT ROWNUM r, COUNT_TABLE.* FROM ("+sql+") COUNT_TABLE) FY_TABLE WHERE r BETWEEN "+(start==0?start:start+1)+ " AND "+end;
            mapv.put("sql",pagesql);
            List<Map<String, Object>> datas = baseMapper.getProcesss(mapv);

            // 分隔列表 当前页存在数据时 设置
            page.setRecords(datas);
        }
        page.setTotal(count);

        return page;
    }

    /**
     * 根据ID查询会议流程通知
     * @param id 流程ID
     * @return 会议名称
     */
    @Override
    public Process getProcessMeeting(Long id) {
        return this.baseMapper.getProcessMeeting( id);
    }

    /**
     * 保存流程信息
     * @param process 流程基础信息
     * @param processNode 流程步骤信息
     */
    private Boolean saveProcess(Process process,ProcessNodeDto processNode){
        /**保存*/
        if(processNode.getSaveState()==FlowConstant.INTEGER_ZERO){
            process.setProcessState(FlowConstant.SHORT_ONE);
            process.setProcessNode(FlowConstant.INTEGER_ONE);
            process.setNodeBranch(FlowConstant.INTEGER_ONE);
            process.setHandleDept(SecurityUtils.getUserDeptId());
            process.setHandleStaff(SecurityUtils.getUserId());
            /**提交 终止或完成*/
        }else if(processNode.getSaveState()==FlowConstant.INTEGER_ONE&&processNode.getNextStep()==null){
            if(processNode.getHeadleState()==FlowConstant.INTEGER_FOUR) {
                process.setProcessState(FlowConstant.SHORT_THREE);
            }else {
                process.setProcessState(FlowConstant.SHORT_FOUR);
            }
            process.setProcessNode(FlowConstant.INTEGER_ONE_HUNDRED);
            process.setNodeBranch(FlowConstant.INTEGER_ONE);
            process.setPassingNode(FlowConstant.STRING_ONE);
            if(process.getId()!=null) {
                this.update(new UpdateWrapper<Process>().setSql(" HANDLE_DEPT = NULL,HANDLE_STAFF=NULL ").lambda().eq(Process::getId,process.getId()));
            }
            /**提交 下一步*/
        }else if(processNode.getSaveState()==FlowConstant.INTEGER_ONE){
            process.setProcessState(FlowConstant.SHORT_TWO);
            process.setProcessNode(processNode.getNextStep().getProcessStep());
            process.setNodeBranch(processNode.getNextStep().getStepNode());
            process.setHandleDept(processNode.getNextStep().getDept());
            process.setHandleStaff(processNode.getNextStep().getDeptStaff());
            process.setPassingNode(FlowConstant.STRING_ONE);
        }
        if(process.getId()==null) {
            process.setProcessNumber(getProcessNumber(process.getProcessType(),process.getRelationNumber(), process.getProcessNumber()));
        }
        return this.saveOrUpdate(process);
    }

    /**
     * 保存校验流程信息
     * @param process
     * @return
     */
    private String saveCheck(Process process){
        if(process.getId()!=null){
            Process pr = this.getOne(new QueryWrapper<Process>().lambda()
                    .eq(Process::getId,process.getId()).eq(Process::getIsDel,FlowConstant.SHORT_ZERO));
            if(pr==null){
                return FlowConstant.DATA_NOTHINGNESS;
            }
            if(!pr.getRegisterStaff().equals(SecurityUtils.getUserId())){
                return FlowConstant.NO_RIGHT_TO_MODIFY;
            }
            if(!pr.getConfigId().equals(process.getConfigId())){
                return FlowConstant.TYPE_NOT_UPEATE;
            }
            process.setEdition(pr.getEdition());
        }else {
            ProcessConfig pc = processConfigService.getById(process.getConfigId());
            if(pc==null||pc.getIsDel()== FlowConstant.SHORT_ONE||pc.getIsValid()==FlowConstant.SHORT_ZERO){
                return FlowConstant.PROCESS_CONFIG_NULL;
            }
            process.setEdition(pc.getEdition());
            process.setProcessType(pc.getBusinessType());
            process.setCategory(pc.getProcessType());
            process.setProcessNumber(pc.getProcessNumber());

            process.setRegisterDept(SecurityUtils.getUserDeptId());
            process.setRegisterStaff(SecurityUtils.getUserId());
            if(process.getRegisterTime()==null) {
                process.setRegisterTime(new Date());
            }
            process.setIsDel(FlowConstant.SHORT_ZERO);
        }
        if(process.getRelationId()!=null){
            if(process.getOperateState()==null){
                return FlowConstant.RELATION_PROCESS_TYPE_NULL;
            }
            //调用抢维修查询抢维修工单是否存在
            if(process.getOperateState()==FlowConstant.SHORT_FIVE){
                Repairorder repairorder =orderClient.getByIdRepairorder(process.getRelationId()).getResult();
                if(repairorder==null){
                    return FlowConstant.REPAIRORDER_NULL;
                }
                process.setRelationNumber(repairorder.getOrderCode());
                //查询流程工单
            }else if(process.getOperateState()>=FlowConstant.SHORT_ONE||process.getOperateState()<=FlowConstant.SHORT_FOUR){
                Process prs = this.getById(process.getRelationId());
                if (prs == null) {
                    return FlowConstant.PROCESS_NULL;
                }
                process.setRelationNumber(prs.getProcessNumber());
            }
        }

        return null;
    }


    /**
     * 获取编码
     * @return
     */
    private String getProcessNumber(Long processType,String rnumber,String code){
        if (StringUtils.isEmpty(code)) {
            code = "GDLC";
        }
        if(StringUtils.isNotEmpty(rnumber)){
            String key =FlowConstant.PROCESS_CONFIG_REDIS_KEY +  rnumber+code;
            RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());
            Long value = counter.incrementAndGet();
            if(value == 1) {
                String number = this.getObj(new QueryWrapper<Process>().select("MAX(PROCESS_NUMBER)").lambda().eq(Process::getRelationNumber, rnumber)
                        .eq(Process::getIsDel, FlowConstant.SHORT_ZERO), (o) -> {
                    return String.valueOf(o);
                });
                if(StringUtils.isNotEmpty(number)){
                    Long num = Long.parseLong(number.substring(number.length() - 2));
                    counter.set(num);
                    counter.expire(FlowConstant.INTEGER_TWENTY_FOUR, TimeUnit.HOURS);
                    value = counter.incrementAndGet();
                }
            }
            return rnumber+code + String.format("%02d", value);
        }else {
            String key = FlowConstant.PROCESS_CONFIG_REDIS_KEY + code + DateUtil.format(new Date(), "yyyyMMdd");
            RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());
            Long value = counter.incrementAndGet();
            if (value == 1) {
                String number = this.baseMapper.getProcessNumber(DateUtil.beginOfDay(new Date()), processType);
                if (StringUtils.isNotEmpty(number)) {
                    Long num = Long.parseLong(number.substring(number.length() - 5));
                    counter.set(num);
                    long betweenDay = DateUtil.between(new Date(), DateUtil.endOfDay(new Date()), DateUnit.MINUTE);
                    counter.expire(betweenDay, TimeUnit.MINUTES);
                    value = counter.incrementAndGet();

                }
            }
            return code + DateUtil.format(new Date(), "yyyyMMdd") + String.format("%05d", value);
        }
    }

}
