/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProcessLDto;
import com.tofly.flow.custom.dto.SqlConditionDto;
import com.tofly.flow.custom.dto.SqlSelectDto;
import com.tofly.flow.custom.dto.SqlWhereDto;
import com.tofly.flow.custom.entity.ProcessConfig;
import com.tofly.flow.custom.entity.ProcessSql;
import com.tofly.flow.custom.service.ProcessSqlService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ProcessSqlMapper;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 流程查询sql
 *
 * @author xqf
 * @date Tue May 24 00:00:00 CST 2022
 */
@Service("processSqlService")
public class ProcessSqlServiceImpl extends ServiceImpl<ProcessSqlMapper, ProcessSql> implements ProcessSqlService {

    /**
     * 逻辑删除sql查询
     * @param id 主键ID
     * @return 修改结果
     */
    @Override
    public boolean removeUpdateById(Long id) {
        return update(new UpdateWrapper<ProcessSql>().lambda().set(ProcessSql::getIsDel, FlowConstant.SHORT_ONE)
                .eq(ProcessSql::getIsDel, FlowConstant.SHORT_ZERO)
                .eq(ProcessSql::getId, id));
    }

    /**
     * 根据id查询数据列表
     * @param page 分页
     * @param dto sql的ID及查询条件
     * @return
     */
    @Override
    public Object getProcesssData(Page page, ProcessLDto dto) {
        ProcessSql psql = this.getById(dto.getId());
        if(psql==null || psql.getIsDel()==FlowConstant.SHORT_ONE){
            return FlowConstant.DATA_NOTHINGNESS;
        }
        if(StringUtils.isEmpty(psql.getContent())){
            return FlowConstant.DATA_SQL_NULL;
        }
        SqlSelectDto sqldto = JSONObject.parseObject(psql.getContent(),SqlSelectDto.class);
        Map<String,String> map = new HashMap<>();
        Map<String,Object> mapv = dto.getMap();
        if(CollUtil.isNotEmpty(sqldto.getWhere())&&CollUtil.isNotEmpty(mapv)){
            for (SqlWhereDto wdto:sqldto.getWhere()) {
                String key = "#{"+wdto.getNumber()+"}";
                StringBuilder sb = new StringBuilder();
                if(CollUtil.isNotEmpty(wdto.getCondition())){
                    for (SqlConditionDto cdto:wdto.getCondition()) {
                        if(CollUtil.isNotEmpty(cdto.getNames())){
                            boolean isif = true;
                            for (String name:cdto.getNames() ) {
                                if(!mapv.containsKey(name)){
                                    isif = false;
                                    break;
                                }
                            }
                            if(isif&&StringUtils.isNotEmpty(cdto.getSql())){
                                sb.append(cdto.getSql());
                            }else if(!isif&&StringUtils.isNotEmpty(cdto.getElsesql())){
                                sb.append(cdto.getElsesql());
                            }
                        }
                    }
                }
                String sql = sb.toString();
                if(wdto.getNumber()==FlowConstant.INTEGER_ONE){
                    sql= sql.replaceFirst("AND","");
                }
                map.put(key,sql);
            }
        }
        String sql = sqldto.getSelect();
        for (Map.Entry<String,String> entry:map.entrySet()) {
            sql = sql.replace(entry.getKey(),entry.getValue());
        }
        mapv.put("sql",sql);
        return this.baseMapper.getProcessList(page,mapv);
    }

}
