/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProcessDeviceDto;
import com.tofly.flow.custom.dto.ProcessStuffDto;
import com.tofly.flow.custom.dto.ProcessStuffOutDto;
import com.tofly.flow.custom.entity.*;
import com.tofly.flow.custom.entity.Process;
import com.tofly.flow.custom.service.*;
import com.tofly.flow.custom.vo.OutMatterVO;
import com.tofly.flow.custom.vo.ProcessStuffVo;
import com.tofly.flow.service.FileService;
import com.tofly.flow.util.UploadFiles;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ProcessStuffMapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 流程中物料出入/退库信息
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@Slf4j
@Service("processStuffService")
public class ProcessStuffServiceImpl extends ServiceImpl<ProcessStuffMapper, ProcessStuff> implements ProcessStuffService {

    /**
     * 物料
     */
    @Autowired
    private MatterService matterService;

    /**
     * 类型
     */
    @Autowired
    private BaseTypeService baseTypeService;

    /**
     * 设备厂家
     */
    @Autowired
    private DeviceFactoryService dFService;

    /**
     * 出入库设备信息
     */
    @Autowired
    private ProcessDeviceService pDService;

    /**
     * 仓库物资
     */
    @Autowired
    private KeepGrainMatterService kGMService;
    /**
     * 仓库设备
     */
    @Autowired
    private KeepGrainDeviceService kGDService;

    /**
     * 文件保存
     */
    @Autowired
    private FileService fileService;
    /**
     * redis
     */
    @Autowired
    private RedisTemplate redisTemplate;

    @Value("${ftp.flow.processstuff}")
    private String processStufffile;

    /**
     * 保存验证 入库、出库申请物资
     * @param dto
     * @return
     */
    @Override
    public String saveCheck(ProcessStuffDto dto) {
        int count = matterService.count(new QueryWrapper<Matter>().lambda()
                .eq(Matter::getId,dto.getMatterId()).eq(Matter::getIsDel, FlowConstant.SHORT_ZERO));
        if(count==0){
            return FlowConstant.MATTER_NOT_EXISTS;
        }
//        if(dto.getDataType()>FlowConstant.LONG_ZERO){
//            count = baseTypeService.count(new QueryWrapper<BaseType>().lambda()
//                    .eq(BaseType::getId,dto.getMatterId()).eq(BaseType::getIsDel, FlowConstant.SHORT_ZERO));
//            if(count==0){
//                return FlowConstant.BASETYPE_NOT_EXISTS;
//            }
//        }
        return null;
    }

    /**
     * 保存入库、出库申请物资
     * @param dto 物料入库数据
     * @param processId 流程id、导入id
     * @param processNode 流程节点
     * @param nodeBranch 节点分支
     * @param stuffNumber 编号
     * @return
     */
    public Boolean saveProcessStuff(ProcessStuffDto dto,Long processId,Integer processNode,Integer nodeBranch,String stuffNumber) {
        ProcessStuff ps = new ProcessStuff();
        BeanUtils.copyProperties(dto,ps);
        ps.setProcessId(processId);
        ps.setProcessNode(processNode);
        ps.setNodeBranch(nodeBranch);
        ps.setStuffNumber(stuffNumber);
        ps.setIsDel(FlowConstant.SHORT_ZERO);
        return this.save(ps);
    }


    /**
     * 批量验证出入库物资信息
     *
     * @param process
     * @param stuffDtos 出入库信息集合
     * @return
     */
    @Override
    public String saveListCheck(Process process, List<ProcessStuffDto> stuffDtos) {
        Set<Long> ids= stuffDtos.stream().map(r->r.getMatterId()).collect(Collectors.toSet());
        if(stuffDtos.size()!=ids.size()){
            return FlowConstant.MATTER_REPEAT;
        }
        Map<Long,Matter> map = matterService.list(new QueryWrapper<Matter>().lambda().in(Matter::getId,ids)
                .eq(Matter::getIsDel,FlowConstant.SHORT_ZERO).eq(Matter::getIsValid,FlowConstant.SHORT_ONE))
                .stream().collect(Collectors.toMap(Matter::getId,v->v));
        if(ids.size()!=map.size()){
            return FlowConstant.MATTER_NOT_EXISTS;
        }
        int count = 0;

        Set<Long> fids= stuffDtos.stream().filter(r->r.getFactoryId()!=null).map(r->r.getFactoryId()).collect(Collectors.toSet());
        if(CollUtil.isNotEmpty(fids)) {
            count = dFService.count(new QueryWrapper<DeviceFactory>().lambda().in(DeviceFactory::getId, fids));
            if (count != fids.size()) {
                return FlowConstant.FACTORY_NULL;
            }
        }

        List<Long> deviceIds = new ArrayList<>();
        for (ProcessStuffDto dto:stuffDtos) {
            Matter matter=map.get(dto.getMatterId());
            if(matter.getCategoryType()==FlowConstant.SHORT_ONE||matter.getCategoryType()==FlowConstant.SHORT_TWO){
                if(process.getOperateState()>=FlowConstant.SHORT_TWO&&CollUtil.isEmpty(dto.getDevices())){
                    return FlowConstant.DEVICE_INFO_NULL;
                }
                if(dto.getDevices()!=null) {
                    if (dto.getQuantity() != dto.getDevices().size()) {
                        return FlowConstant.DEVICE_NUMBER_ERROR;
                    }
                    for (ProcessDeviceDto ddto : dto.getDevices()) {
                        if (deviceIds.contains(ddto.getStockId())) {
                            return FlowConstant.DEVICE_EXISTS;
                        }
                        if(ddto.getStockId()!=null) {
                            deviceIds.add(ddto.getStockId());
                        }
                    }
                }
            }
        }
        if(CollUtil.isNotEmpty(deviceIds)){
            count = kGDService.count(new QueryWrapper<KeepGrainDevice>().lambda().eq(KeepGrainDevice::getIsDel,FlowConstant.SHORT_ZERO)
                    .in(KeepGrainDevice::getId,deviceIds).eq(KeepGrainDevice::getIsValid,FlowConstant.SHORT_ONE));
            if(count!=deviceIds.size()){
                return FlowConstant.DEVICE_NULL;
            }
        }
        return null;
    }

    /**
     * 批量保存出入库物资信息
     * @param stuffDtos 出入库物资信息
     * @param processStep 流程步骤
     * @param stepNode 流程步骤节点
     * @param process 编号
     * @param deviceFiles
     * @return
     */
    @Override
    public Boolean saveListProcessStuff(List<ProcessStuffDto> stuffDtos, Integer processStep, Integer stepNode, Process process, Map<String, List<MultipartFile>> deviceFiles) {

        Map<Long,ProcessStuff> map = this.list(new QueryWrapper<ProcessStuff>().lambda().eq(ProcessStuff::getProcessId,process.getId())
                .eq(ProcessStuff::getProcessNode,processStep).eq(ProcessStuff::getNodeBranch,stepNode)
                .eq(ProcessStuff::getIsDel,FlowConstant.SHORT_ZERO)).stream().collect(Collectors.toMap(ProcessStuff::getId,r->r));

//        this.update(new UpdateWrapper<ProcessStuff>().lambda().eq(ProcessStuff::getProcessId,process.getId())
//                .eq(ProcessStuff::getProcessNode,processStep).eq(ProcessStuff::getNodeBranch,stepNode)
//                .eq(ProcessStuff::getIsDel,FlowConstant.SHORT_ZERO).set(ProcessStuff::getIsDel,FlowConstant.SHORT_ONE));

        String number = getProcessNumber();
        for (ProcessStuffDto dto:stuffDtos) {
            ProcessStuff ps = new ProcessStuff();
            BeanUtils.copyProperties(dto,ps);
            ps.setProcessId(process.getId());
            ps.setProcessNode(processStep);
            ps.setNodeBranch(stepNode);
            //TODO 统一生成编码
            ps.setStuffNumber(KeepGrainMatterServiceImpl.createNum());
            ps.setInDepotNumber(process.getProcessNumber());
            ps.setPurchaseNumber(process.getRelationNumber());
            ps.setIsDel(FlowConstant.SHORT_ZERO);
            ProcessStuff pn = null;
            if(dto.getId()!=null){
                pn = map.get(dto.getId());
            }
            if(CollUtil.isNotEmpty(dto.getFileNames())) {
                List<MultipartFile> fileList = deviceFiles.get(String.join("_",dto.getFileNames()));
                if (CollUtil.isNotEmpty(fileList)) {
                    String filepath = UploadFiles.saveFiles(fileService, fileList, pn == null ? "" : pn.getAnnex(), dto.getAnnex(), processStufffile, FlowConstant.TF_YWPN_PROCESS_STUFF_W);

                    ps.setAnnex(filepath);
                }
            }
            this.save(ps);
            //保存物资设备信息
            pDService.saveProcessDevice(dto.getDevices(), ps.getId(), ps.getFactoryId(), ps.getUnitPrice(),pn == null ? null:pn.getId(),deviceFiles);

        }
        if(CollUtil.isNotEmpty(map.keySet())) {
            this.update(new UpdateWrapper<ProcessStuff>().lambda().in(ProcessStuff::getId, map.keySet())
                    .eq(ProcessStuff::getIsDel, FlowConstant.SHORT_ZERO).set(ProcessStuff::getIsDel, FlowConstant.SHORT_ONE));
            pDService.delProcessDevice(map.keySet());
        }
        return true;
    }

    /**
     * 处理物资信息
     * @param page 分页
     * @param dto 查询条件
     * @return 出库信息
     */
    @Override
    public Page<OutMatterVO> loadOutMatter(Page page, ProcessStuffOutDto dto) {
        Page<OutMatterVO> pg = this.baseMapper.getOutMatter(page,dto);



        return pg;
    }


    /**
     * 同步流程出入库物资
     * @param processId 流程ID
     * @param start 出入库状态0入库，1退库，2出库
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void synchroStuff(Long processId,int start) {
        List<ProcessStuffVo> stuffVos = this.baseMapper.loadProcessStuffByProcessId(processId);
        if(CollUtil.isNotEmpty(stuffVos)){
            if(kGMService.synchroKeepGrainMatter(stuffVos,start,processId)) {
                updateStuffStart(start, stuffVos);
            }
        }
        log.info("start::"+start+" 流程ID："+processId);
    }

    /**
     * 修改出入库物资状态
     * @param start
     * @param stuffVos
     */
    private void updateStuffStart(int start, List<ProcessStuffVo> stuffVos) {
        List<Long> ids = stuffVos.stream().map(r -> r.getId()).collect(Collectors.toList());
        //修改为出入库
        this.update(new UpdateWrapper<ProcessStuff>().lambda().eq(ProcessStuff::getIsDel, FlowConstant.SHORT_ZERO)
                .in(ProcessStuff::getId, ids).set(start == FlowConstant.INTEGER_ZERO, ProcessStuff::getDataType, FlowConstant.SHORT_ONE)
                .set(start == FlowConstant.INTEGER_ONE, ProcessStuff::getDataType, FlowConstant.SHORT_TWO)
                .eq(start == FlowConstant.INTEGER_TWO, ProcessStuff::getDataType, FlowConstant.SHORT_FIVE));
    }


    /**
     * 获取编码
     * @return
     */
    private String getProcessNumber(){

        String key = FlowConstant.REDIS_KEY_STUFF_NUMBER+ DateUtil.format(new Date(), "yyyyMMdd");
        RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());
        Long value = counter.incrementAndGet();
        if(value==1){
            String number = this.getObj(new QueryWrapper<ProcessStuff>().select("MAX(STUFF_NUMBER) as ID").lambda()
                            .ge(ProcessStuff::getCreateTime,DateUtil.beginOfDay(new Date())),(o)->{return String.valueOf(o);} );

            if(StringUtils.isNotEmpty(number)) {
                Long num = Long.parseLong(number.substring(number.length()-5));
                counter.set(num);
                long betweenDay = DateUtil.between(new Date(), DateUtil.endOfDay(new Date()), DateUnit.MINUTE);
                counter.expire(betweenDay, TimeUnit.MINUTES);
                value = counter.incrementAndGet();
            }
        }
        return DateUtil.format(new Date(), "yyyyMMdd")+String.format("%05d", value);
    }

}




