/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProcessSuperviseDto;
import com.tofly.flow.custom.entity.Process;
import com.tofly.flow.custom.entity.ProcessNode;
import com.tofly.flow.custom.entity.ProcessSupervise;
import com.tofly.flow.custom.mapper.ProcessSuperviseMapper;
import com.tofly.flow.custom.service.ProcessNodeService;
import com.tofly.flow.custom.service.ProcessService;
import com.tofly.flow.custom.service.ProcessSuperviseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;


/**
 * 流程督办记录信息
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@Service("processSuperviseService")
public class ProcessSuperviseServiceImpl extends ServiceImpl<ProcessSuperviseMapper, ProcessSupervise> implements ProcessSuperviseService {

    @Autowired
    private ProcessService processService;

    @Autowired
    private ProcessNodeService processNodeService;

    /**
     * 保存流程督办信息
     * @param dto
     * @return
     */
    @Override
    public Object saveProcessSupervise(ProcessSuperviseDto dto) {
        Process process = processService.getById(dto.getProcessId());
        if(process==null|| FlowConstant.SHORT_ONE == process.getIsDel()){
            return FlowConstant.DATA_NOTHINGNESS;
        }
        ProcessNode processNode = processNodeService.getProcessNode(dto.getProcessId(),dto.getProcessNode(),dto.getNodeBranch());
        if(processNode==null){
            return FlowConstant.PROCESS_STEP_ERROR;
        }
        ProcessSupervise ps = new ProcessSupervise();
        BeanUtils.copyProperties(dto,ps);
        ps.setHandleDept(processNode.getHandleDept());
        ps.setHandleStaff(processNode.getHandleStaff());
        ps.setIsDel(FlowConstant.SHORT_ZERO);
        return save(ps);
    }

    /**
     * 删除督办信息
     * @param id id
     * @return
     */
    @Override
    public boolean removeUpdateById(Long id) {
        return update(new UpdateWrapper<ProcessSupervise>().lambda().set(ProcessSupervise::getIsDel, FlowConstant.SHORT_ONE)
                .eq(ProcessSupervise::getIsDel, FlowConstant.SHORT_ZERO)
                .eq(ProcessSupervise::getId, id));
    }

    /**
     * 批量删除督办信息
     * @param ids id集合，逗号分隔
     * @return
     */
    @Override
    public Object removeUpdateByIds(String ids) {
        if(StringUtils.isEmpty(ids)){
            return FlowConstant.ISNOTNUL;
        }
        try {
            List<Long> lids= Arrays.asList(ids.split(",")).stream().map(st->Long.parseLong(st)).collect(Collectors.toList());
            if(CollUtil.isNotEmpty(lids)) {
                return this.baseMapper.update(new ProcessSupervise(), new UpdateWrapper<ProcessSupervise>().lambda()
                        .set(ProcessSupervise::getIsDel, FlowConstant.SHORT_ONE)
                        .eq(ProcessSupervise::getIsDel, FlowConstant.SHORT_ZERO)
                        .in(ProcessSupervise::getId, lids));
            }
            return false;
        }catch (Exception e){
            log.error("异常：",e);
            return e.getMessage();
        }
    }

}
