/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.ProjectContractAllDto;
import com.tofly.flow.custom.dto.ProjectContractDto;
import com.tofly.flow.custom.dto.ProjectContractFileDto;
import com.tofly.flow.custom.entity.BaseType;
import com.tofly.flow.custom.entity.Document;
import com.tofly.flow.custom.entity.ProcessNode;
import com.tofly.flow.custom.entity.ProjectContract;
import com.tofly.flow.custom.service.ProjectContractService;
import com.tofly.flow.service.FileService;
import com.tofly.flow.util.UploadFiles;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.tofly.flow.custom.mapper.ProjectContractMapper;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 流程合同信息
 *
 * @author xqf
 * @date Sat Apr 02 00:00:00 CST 2022
 */
@Service("projectContractService")
public class ProjectContractServiceImpl extends ServiceImpl<ProjectContractMapper, ProjectContract> implements ProjectContractService {

    /**
     * 文件保存
     */
    @Autowired
    private FileService fileService;

    @Value("${ftp.flow.contract}")
    private String contractfile;

    /**
     * 保存或修改合同信息
     * @param dto 合同信息
     * @param fileList 合同文件信息
     * @return
     */
    @Override
    public Object saveProjectContract(ProjectContractDto dto, List<MultipartFile> fileList) {
        if(dto.getId()!=null){
            int count = this.count(new QueryWrapper<ProjectContract>().lambda()
                    .eq(ProjectContract::getId,dto.getId()).eq(ProjectContract::getIsDel, FlowConstant.SHORT_ZERO));
            if(count==FlowConstant.INTEGER_ZERO){
                return FlowConstant.DATA_NOTHINGNESS;
            }
        }
        ProjectContract pc = new ProjectContract();
        BeanUtils.copyProperties(dto,pc);
        if(CollUtil.isNotEmpty(fileList)) {
            String filepath = UploadFiles.saveFiles(fileService, fileList, dto.getAnnex(), null, contractfile, FlowConstant.TF_YWPN_PROJECT_CONTRACT_W);
            if(StringUtils.isEmpty(filepath)&&dto.getId()!=null){
                this.update(new UpdateWrapper<ProjectContract>().lambda()
                        .set(ProjectContract::getAnnex,filepath).eq(ProjectContract::getId,dto.getId()));
            }
            pc.setAnnex(filepath);
        }
        pc.setContractState(FlowConstant.SHORT_ONE);
        pc.setIsDel(FlowConstant.SHORT_ZERO);
        return this.saveOrUpdate(pc);
    }

    /**
     * 删除作废合同
     * @param id 合同id
     * @return
     */
    @Override
    public boolean removeUpdateById(Long id) {
        return update(new UpdateWrapper<ProjectContract>().lambda().set(ProjectContract::getIsDel, FlowConstant.SHORT_ONE)
                .set(ProjectContract::getUpdateTime, new Date()).set(ProjectContract::getUpdateUser, SecurityUtils.getUserId())
                .set(ProjectContract::getContractState,FlowConstant.SHORT_ZERO)
                .eq(ProjectContract::getIsDel, FlowConstant.SHORT_ZERO)
                .eq(ProjectContract::getId, id));
    }

    /**
     * 校验合同信息保存
     * @param contractDto
     * @param map
     * @return
     */
    @Override
    public String saveCheck(List<ProjectContractAllDto> contractDto, Map<String,MultipartFile> map) {

        List<String> names = new ArrayList<>();
        List<Long> ids = new ArrayList<>();
        for (ProjectContractAllDto dto:contractDto) {
            if(CollUtil.isNotEmpty(dto.getFileNames())) {
                for (String name : dto.getFileNames()) {
                    if (!map.containsKey(name)||names.contains(name)) {
                        return FlowConstant.CONTRACT_FILE_ERROR;
                    }
                    names.add(name);
                }
            }
            if(dto.getId()!=null) {
                ids.add(dto.getId());
            }
        }
        if(CollUtil.isNotEmpty(ids)){
            int count = this.count(new QueryWrapper<ProjectContract>().lambda().eq(ProjectContract::getIsDel,FlowConstant.SHORT_ZERO).in(ProjectContract::getId,ids));
            if(count!=ids.size()){
                return FlowConstant.CONTRACT_DATA_NULL;
            }
        }
        return null;
    }

    /**
     * 批量保存合同信息
     * @param contracts 合同信息
     * @param processNode 步骤节点信息
     * @return
     */
    @Override
    public boolean saveBatchProjectContract(List<ProjectContractFileDto> contracts, ProcessNode processNode) {
        List<Long> ids = contracts.stream().filter(r->r.getDto().getId()!=null).map(r->r.getDto().getId()).collect(Collectors.toList());
        //根据id获取库中数据
        Map<Long, ProjectContract> map = null;
        if(CollUtil.isNotEmpty(ids)) {
            map = this.list(new QueryWrapper<ProjectContract>().lambda()
                    .eq(ProjectContract::getIsDel, FlowConstant.SHORT_ZERO).in(ProjectContract::getId, ids))
                    .stream().collect(Collectors.toMap(ProjectContract::getId, r -> r));
        }
        //清除库中的数据
        this.update(new UpdateWrapper<ProjectContract>().lambda().eq(ProjectContract::getProjectId,processNode.getProcessId())
                .eq(ProjectContract::getProjectNodeId,processNode.getId()).eq(ProjectContract::getProcessNode,processNode.getProcessStep())
                .eq(ProjectContract::getNodeBranch,processNode.getStepNode()).set(ProjectContract::getContractState,FlowConstant.SHORT_ZERO)
                .set(ProjectContract::getIsDel,FlowConstant.SHORT_ONE));

        //插入库中数据
        for (ProjectContractFileDto dto:contracts) {
            ProjectContract pc = new ProjectContract();
            BeanUtils.copyProperties(dto.getDto(),pc);
            ProjectContract projectContract=null;
            if(map!=null&&pc.getId()!=null&&!map.containsKey(pc.getId())){
                projectContract = map.get(pc.getId());
            }
            if(CollUtil.isNotEmpty(dto.getFileList())) {
                String filepath = UploadFiles.saveFiles(fileService, dto.getFileList(), dto.getDto().getAnnex()
                        ,projectContract!=null?projectContract.getAnnex():null, contractfile, FlowConstant.TF_YWPN_PROJECT_CONTRACT_W);
                pc.setAnnex(filepath);
            }
            pc.setProjectId(processNode.getProcessId());
            pc.setProjectNodeId(processNode.getId());
            pc.setProcessNode(processNode.getProcessStep());
            pc.setNodeBranch(processNode.getStepNode());
            pc.setContractState(FlowConstant.SHORT_ONE);
            pc.setIsDel(FlowConstant.SHORT_ZERO);
            this.save(pc);
        }
        return true;
    }


}
