/*
 * Author: wangjian
 * 生成entity
 */
package com.tofly.flow.custom.vo;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.haiqiu.tools.excel.annotation.HqExcel;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;


/**
 * 仓储物料信息
 *
 * @author xqf
 * @date Fri Apr 01 00:00:00 CST 2022
 */
@Data
@ApiModel
public class KeepGrainMatterVO extends Model<KeepGrainMatterVO> {
    private static final long serialVersionUID = 1L;

    /**
     * ID
     */
    @TableId(value = "ID", type = IdType.INPUT)
    @ApiModelProperty(value = "ID")
    private Long id;
    /**
     * 仓储编码
     */
    @ApiModelProperty(value = "仓储编码")
    @HqExcel(name = "入库编号",column = "A")
    private String keepGrainNumber;
    /**
     * 物料ID
     */
    @ApiModelProperty(value = "物料ID")
    private Long matterId;
    /**
     * 入库数量
     */
    @ApiModelProperty(value = "入库数量")
    @HqExcel(name = "采购总数量",column = "H")
    private Long inQuantity;
    /**
     * 出库数量
     */
    @ApiModelProperty(value = "出库数量")
    private Long outQuantity;
    /**
     * 现存数量
     */
    @ApiModelProperty(value = "现存数量")
    @HqExcel(name = "库存数量",column = "I")
    private Long quantity;
    /**
     * 采购总价
     */
    @ApiModelProperty(value = "采购总价")
    @HqExcel(name = "采购价格合计（元）",column = "J")
    private Double purchaseTotal;
    /**
     * 最新单价
     */
    @ApiModelProperty(value = "最新单价")
    @HqExcel(name = "最近采购单价（元）",column = "K")
    private Double unitPrice;
    /**
     * 库房
     */
    @ApiModelProperty(value = "库房")
    private String storageRoom;
    /**
     * 位置（货架位置）
     */
    @ApiModelProperty(value = "位置（货架位置）")
    private String position;
    /**
     * 物料状态1:新；2:旧
     */
    @ApiModelProperty(value = "物料状态1:新；2:旧")
    private Short matterState;
    /**
     * 是否应急物资1:是；0:否
     */
    @ApiModelProperty(value = "是否应急物资1:是；0:否")
    private Short isEmergency;
    /**
     * 最新入库时间
     */
    @ApiModelProperty(value = "最新入库时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date newTime;
    /**
     * 最近到期时间
     */
    @ApiModelProperty(value = "最近到期时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date recentlyExpired;
    /**
     * 是否有效1有效；0无效
     */
    @ApiModelProperty(value = "是否有效1有效；0无效")
    private Short isValid;
    /**
     * 是否删除1删除；0未删除
     */
    @ApiModelProperty(value = "是否删除1删除；0未删除")
    private Short isDel;

    /**
     * 数据来源:1导入；2入库
     */
    @ApiModelProperty(value = "数据来源:1导入；2入库")
    @HqExcel(name = "数据来源",column = "M",replace = {"1_导入","2_入库"})
    private Short dataSource;

    /**
     * 数据来源:1导入；2入库（字符串）
     */
    @ApiModelProperty(value = "数据来源:1导入；2入库(字符串)")
    private String dataSourceStr;

    /**
     * 购买日期（采购时间）
     */
    @ApiModelProperty(value = "购买日期（采购时间）")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date emptionDate;
    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;
    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人")
    @TableField(value = "CREATE_USER", fill = FieldFill.INSERT)
    private Long createUser;
    @ApiModelProperty(hidden = true)
    private String createUserName;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;
    /**
     * 修改人
     */
    @ApiModelProperty(value = "修改人")
    @TableField(value = "UPDATE_USER", fill = FieldFill.INSERT_UPDATE)
    private Long updateUser;
    @ApiModelProperty(hidden = true)
    private String updateUserName;
    /**
     * 修改时间
     */
    @ApiModelProperty(value = "修改时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "UPDATE_TIME", fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;



    /**
     * 物资编码
     */
    @ApiModelProperty(value = "物资编码")
    @HqExcel(name = "物资类型编码",column = "B")
    private String matterNumber;

    /**
     * 物资名称
     */
    @ApiModelProperty(value = "物资名称")
    private String matterName;


    /**
     * 物资分类
     */
    @ApiModelProperty(value = "物资分类")
    @HqExcel(name = "物资分类",column = "D")
    private String baseTypeName;

    /**
     * 规格型号
     */
    @ApiModelProperty(value = "规格型号")
    @HqExcel(name = "规格型号",column = "C")
    private String standards;

    /**
     * 物资类别类型1设备类;2车辆类;3耗材类
     */
    @ApiModelProperty(value = "物资类别类型1设备类;2车辆类;3耗材类")
    @HqExcel(name = "物资分类",column = "E",replace = {"1_设备类","2_车辆类","3_耗材类"})
    private Short categoryType;

    /**
     * 物资类别类型1设备类;2车辆类;3耗材类(字符串格式)
     */
    @ApiModelProperty(value = "物资类别类型1设备类;2车辆类;3耗材类（字符串格式）")
    private String categoryTypeStr;

    /**
     * 是否资产1:是;0:否(字符串格式)
     */
    @ApiModelProperty(value = "是否资产1:是;0:否(字符串格式)")
    private String isAssetsStr;

    /**
     * 是否资产1:是;0:否
     */
    @ApiModelProperty(value = "是否资产1:是;0:否")
    @HqExcel(name = "是否资产",column = "F",replace = {"0_否","1_是"})
    private Short isAssets;

    /**
     * 单位
     */
    @ApiModelProperty(value = "单位")
    @HqExcel(name = "单位",column = "G")
    private String unit;


    /**
     * 市场总价
     */
    @ApiModelProperty(value = "市场总价")
    @HqExcel(name = "库存市场总价（元）",column = "L")
    private Double marketTotal;


}
