package com.tofly.flow.custom.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 流程信息
 */
@Data
@EqualsAndHashCode
public class ProcessVo implements Serializable {

    /**
     * ID
     */
    @ApiModelProperty(value = "ID")
    private Long id;
    /**
     * 流程编号(工程、办公、采购、入库、出库等)
     */
    @ApiModelProperty(value = "流程编号(工程、办公、采购、入库、出库等)")
    private String processNumber;

    /**
     * 关联编码ID
     */
    @ApiModelProperty(value = "关联编码ID")
    private Long relationId;

    /**
     * 关联编码ID
     */
    @ApiModelProperty(value = "关联编码ID")
    private String relationNumber;
    /**
     * 流程类别
     */
    @ApiModelProperty(value = "流程类别")
    private Long category;
    /**
     * 流程类型类型表ID
     */
    @ApiModelProperty(value = "流程类型类型表ID")
    private Long processType;
    /**
     * 流程类型类型
     */
    @ApiModelProperty(value = "流程类型类型")
    private String processTypeName;
    /**
     * 流程配置ID
     */
    @ApiModelProperty(value = "流程配置ID")
    private Long configId;
    /**
     * 业务类型，基础类型表
     */
    @ApiModelProperty(value = "业务类型，基础类型表")
    private Long ywType;
    /**
     * 版本
     */
    @ApiModelProperty(value = "版本")
    private Integer edition;
    /**
     * 名称
     */
    @ApiModelProperty(value = "名称")
    private String name;
    /**
     * 开始时间
     */
    @ApiModelProperty(value = "开始时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date startTime;
    /**
     * 结束时间
     */
    @ApiModelProperty(value = "结束时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date endTime;
    /**
     * 使用部门（申购部门）
     */
    @ApiModelProperty(value = "使用部门（申购部门）")
    private Long useDept;
    /**
     * 使用部门（申购部门）
     */
    @ApiModelProperty(value = "使用部门（申购部门）")
    private String useDeptName;
    /**
     * 所属部门（人员所属部门）
     */
    @ApiModelProperty(value = "所属部门（人员所属部门）")
    private Long belongDept;
    /**
     * 所属部门（人员所属部门）
     */
    @ApiModelProperty(value = "所属部门（人员所属部门）")
    private String belongDeptName;
    /**
     * 负责人(申请人)
     */
    @ApiModelProperty(value = "负责人(申请人)")
    private Long personLiable;
    /**
     * 负责人(申请人)
     */
    @ApiModelProperty(value = "负责人(申请人)")
    private String personLiableName;
    /**
     * 供应商名称（来文单位）
     */
    @ApiModelProperty(value = "供应商名称（来文单位）")
    private String supplier;
    /**
     * 联系(申请)人名称
     */
    @ApiModelProperty(value = "联系(申请)人名称")
    private String contacts;
    /**
     * 联系人电话
     */
    @ApiModelProperty(value = "联系人电话")
    private String telephone;
    /**
     * 申请单位（人）
     */
    @ApiModelProperty(value = "申请单位（人）")
    private String applicant;
    /**
     * 证件类型
     */
    @ApiModelProperty(value = "证件类型")
    private Long documentType;
    /**
     * 证件号
     */
    @ApiModelProperty(value = "证件号")
    private String idNumber;
    /**
     * 工单状态：1草稿、2处理、3终止、4处理完成
     */
    @ApiModelProperty(value = "工单状态：1草稿、2处理、3终止、4处理完成")
    private Short processState;
    /**
     * 当前流程节点 100表示终结归档
     */
    @ApiModelProperty(value = "当前流程节点 100表示终结归档")
    private Integer processNode;
    /**
     * 流程节点分支
     */
    @ApiModelProperty(value = "流程节点分支")
    private Integer nodeBranch;
    /**
     * 收货（出货）时间
     */
    @ApiModelProperty(value = "收货（出货）时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date handoverTime;
    /**
     * 收货（出货）人员
     */
    @ApiModelProperty(value = "收货（出货）人员")
    private Long handoverUser;
    /**
     * 收货（出货）人员
     */
    @ApiModelProperty(value = "收货（出货）人员")
    private String handoverUserName;
    /**
     * 总价
     */
    @ApiModelProperty(value = "总价")
    private Double totalPrice;
    /**
     * 登记部门
     */
    @ApiModelProperty(value = "登记部门")
    private Long registerDept;

    /**
     * 登记部门
     */
    @ApiModelProperty(value = "登记部门")
    private String registerDeptName;
    /**
     * 登记人
     */
    @ApiModelProperty(value = "登记人")
    private Long registerStaff;
    /**
     * 登记人
     */
    @ApiModelProperty(value = "登记人")
    private String registerStaffName;
    /**
     * 登记时间
     */
    @ApiModelProperty(value = "登记时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date registerTime;
    /**
     * 已经通过的节点
     */
    @ApiModelProperty(value = "已经通过的节点")
    private String passingNode;

    /**
     * 当前处理部门
     */
    @ApiModelProperty(value = "当前处理部门")
    private Long handleDept;
    /**
     * 当前处理部门
     */
    @ApiModelProperty(value = "当前处理部门")
    private String handleDeptName;
    /**
     * 当前处理人
     */
    @ApiModelProperty(value = "当前处理人")
    private Long handleStaff;
    /**
     * 当前处理人
     */
    @ApiModelProperty(value = "当前处理人")
    private String handleStaffName;
    /**
     * 操作状态1:出库；2:余退库；4:旧退库
     */
    @ApiModelProperty(value = "操作状态1:出库；2:余退库；4:旧退库")
    private Short operateState;

    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 步骤信息
     */
    @ApiModelProperty(value = "步骤信息")
    private List<ProcessNodeVo> processNodeVos;

    /**
     * 当前步骤
     */
    @ApiModelProperty(value = "当前步骤")
    private ProcessNodeVo currentNode;


    /**
     * 下一步
     */
    @ApiModelProperty(value = "下一步")
    private ProcessNodeVo nextNode;


}
