/*
 * Author: wangjian
 * 生成entity
 */
package com.tofly.flow.dto;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.*;
import java.util.Date;


/**
 * 通讯录
 *
 * @author qf
 * @date Thu Apr 21 00:00:00 CST 2022
 */
@Data
@ApiModel
public class AddressBookDTO {
    private static final long serialVersionUID = 1L;

    /**
     * ID
     */
    @ApiModelProperty(value = "ID")
    @NotNull(groups = update.class,message = "ID不能为空")
    @Min(value = 1, message = "ID错误")
    private Long id;
    /**
     * 单位
     */
    @ApiModelProperty(value = "单位")
    private String company;

    /**
     * 职务
     */
    @ApiModelProperty(value = "职务")
    private String position;

    /**
     * 部门
     */
    @ApiModelProperty(value = "部门")
    @Size(max = 100,message = "部门只能在1-100个字符之间")
    private String dept;
    /**
     * 姓名
     */
    @ApiModelProperty(value = "姓名")
    @NotEmpty(message = "姓名不能为空")
    @Size(max = 20,message = "姓名只能在1-20个字符之间")
    private String name;
    /**
     * 手机
     */
    @ApiModelProperty(value = "手机")
    @NotEmpty(message = "手机不能为空")
    @Pattern(regexp = "^[1][3,4,5,6,7,8,9][0-9]{9}$",message = "手机号格式错误")
    @Size(max = 20,message = "手机只能在1-20个字符之间")
    private String phone;
    /**
     * 地址
     */
    @ApiModelProperty(value = "地址")
    @Size(max = 200,message = "地址只能在1-200个字符之间")
    private String addre;
    /**
     * 紧急联系电话
     */
    @ApiModelProperty(value = "紧急联系电话")
    private String urgentPhone;
    /**
     * 是否应急通讯1:是；0:否
     */
    @ApiModelProperty(value = "是否应急通讯1:是；0:否")
    private Short isEmergency;
    /**
     * 是否有效1有效；0无效
     */
    @ApiModelProperty(value = "是否有效1有效；0无效",hidden = true)
    private Short isValid;
    /**
     * 是否删除1删除；0未删除
     */
    @ApiModelProperty(value = "是否删除1删除；0未删除",hidden = true)
    private Short isDel;
    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    @Size(max = 500,message = "备注只能在1-500个字符之间")
    private String remark;

    /**
     * 电子邮箱
     */
    @ApiModelProperty(value = "电子邮箱")
    @Email(message = "邮箱格式错误")
    private String email;

    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人")
    @TableField(value = "CREATE_USER", fill = FieldFill.INSERT)
    private Long createUser;
    @ApiModelProperty(hidden = true)
    private String createUserName;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;
    /**
     * 修改人
     */
    @ApiModelProperty(value = "修改人")
    @TableField(value = "UPDATE_USER", fill = FieldFill.INSERT_UPDATE)
    private Long updateUser;
    @ApiModelProperty(hidden = true)
    private String updateUserName;
    /**
     * 修改时间
     */
    @ApiModelProperty(value = "修改时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "UPDATE_TIME", fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    public interface update{}

}
