/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.dto.AddressBookQuery;
import com.tofly.flow.dto.AddressBookDTO;
import com.tofly.flow.entity.AddressBook;
import com.tofly.flow.service.AddressBookService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import com.tofly.flow.mapper.AddressBookMapper;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
 * 通讯录
 *
 * @author qf
 * @date Thu Apr 21 00:00:00 CST 2022
 */
@Service("addressBookService")
public class AddressBookServiceImpl extends ServiceImpl<AddressBookMapper, AddressBook> implements AddressBookService {

    @Override
    public void insert(AddressBookDTO addressBookDTO) {
        checkData(addressBookDTO);
        setDefaultValue(addressBookDTO);
        AddressBook addressBook = new AddressBook();
        BeanUtils.copyProperties(addressBookDTO, addressBook);
        if(addressBook.getIsEmergency()==null){
            addressBook.setIsEmergency(FlowConstant.SHORT_ZERO);
        }
        if (!this.save(addressBook)) {
            throw new RuntimeException("新增失败");
        }
    }


    /**
     * 检测重复数据
     *
     * @param addressBookDTO 操作数据实体
     */
    private void checkData(AddressBookDTO addressBookDTO) {
        QueryWrapper<AddressBook> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO);
        queryWrapper.eq("NAME", addressBookDTO.getName());
        queryWrapper.eq("PHONE", addressBookDTO.getPhone());
        AddressBook one = this.getOne(queryWrapper);
        if (!ObjectUtils.isEmpty(one)) {
            throw new RuntimeException(addressBookDTO.getName() + "的手机已经存在，无需重复添加");
        }
    }

    @Override
    public void edit(AddressBookDTO addressBookDTO) {
        AddressBook addressBook = new AddressBook();
        BeanUtils.copyProperties(addressBookDTO, addressBook);
        if (!this.updateById(addressBook)) {
            throw new RuntimeException("修改失败");
        }
    }

    @Override
    public void delete(Set<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new AddressBook(), new UpdateWrapper<AddressBook>().lambda()
                .set(AddressBook::getIsDel, FlowConstant.SHORT_ONE)
                .eq(AddressBook::getIsDel, FlowConstant.SHORT_ZERO)
                .in(AddressBook::getId, ids));
        if (!update) {
            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public Page<AddressBook> pageList(Page page, AddressBookQuery query) {
        QueryWrapper<AddressBook> queryWrapper = getAddressBookQueryWrapper(query);
        Page data = this.page(page, queryWrapper);
        if (data.getTotal() > 0) {
            getDeptName(data.getRecords());
        }
        return data;
    }

    /**
     * 设置查询条件
     *
     * @param query 查询参数
     * @return QueryWrapper
     */
    private QueryWrapper<AddressBook> getAddressBookQueryWrapper(AddressBookQuery query) {
        QueryWrapper<AddressBook> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO);
        if (query != null) {
            if (!StringUtils.isEmpty(query.getName())) {
                queryWrapper.like("NAME", query.getName());
            }
            if (!StringUtils.isEmpty(query.getDeptName())) {
                queryWrapper.like("DEPT", query.getDeptName());
            }
            if (!StringUtils.isEmpty(query.getPosition())) {
                queryWrapper.like("POSITION", query.getPosition());
            }
            if(query.getIsEmergency()!=null){
                queryWrapper.eq("IS_EMERGENCY",query.getIsEmergency());
            }
        }
        return queryWrapper;
    }

    @Override
    public List<AddressBook> all(AddressBookQuery query) {
        QueryWrapper<AddressBook> addressBookQueryWrapper = getAddressBookQueryWrapper(query);
        return this.list(addressBookQueryWrapper);
    }

    @Override
    public List<AddressBook> getListByEmergencyId(Long emergencyId) {
        List<AddressBook> list = baseMapper.getListByEmergencyId(emergencyId);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        getDeptName(list);
        return list;
    }

    private void getDeptName(List<AddressBook> list) {
        for (AddressBook addressBook : list) {
            if (StringUtils.hasText(addressBook.getDept())) {
                addressBook.setDeptName(baseMapper.getDeptName(Long.parseLong(addressBook.getDept())));
            }
        }
    }


    /**
     * 设置默认值
     *
     * @param addressBookDTO 实体
     */
    private void setDefaultValue(AddressBookDTO addressBookDTO) {
        addressBookDTO.setIsDel(FlowConstant.SHORT_ZERO);
        addressBookDTO.setIsValid(FlowConstant.SHORT_ONE);
    }
}
