/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.entity.Assets;
import com.tofly.flow.custom.service.AssetsService;
import com.tofly.flow.custom.vo.AssetsVO;
import com.tofly.flow.dto.EmergencyFacilitiesDTO;
import com.tofly.flow.dto.EmergencyFacilitiesQuery;
import com.tofly.flow.entity.EmergencyFacilities;
import com.tofly.flow.service.EmergencyFacilitiesService;
import com.tofly.flow.vo.EmergencyFacilitiesVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.flow.mapper.EmergencyFacilitiesMapper;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.List;
import java.util.Set;

/**
 * 事故设施
 *
 * @author xqf
 * @date Mon Apr 25 00:00:00 CST 2022
 */
@Service("emergencyFacilitiesService")
public class EmergencyFacilitiesServiceImpl extends ServiceImpl<EmergencyFacilitiesMapper, EmergencyFacilities> implements EmergencyFacilitiesService {


    private final AssetsService assetsService;

    public EmergencyFacilitiesServiceImpl(AssetsService assetsService) {
        this.assetsService = assetsService;
    }


    @Override
    public void insert(EmergencyFacilitiesDTO dto) {
        setDefaultValue(dto);
        EmergencyFacilities data = new EmergencyFacilities();
        BeanUtils.copyProperties(dto, data);
        if (!this.save(data)) {
            throw new RuntimeException("新增失败");
        }
    }

    @Override
    public void batchInsert(List<EmergencyFacilitiesDTO> dtos) {
        for (EmergencyFacilitiesDTO dto : dtos) {
            insert(dto);
        }
    }


    /**
     * 检测重复数据
     *
     * @param dto 操作数据实体
     */
//    private void checkData(EmergencyFacilitiesDTO dto) {
//        QueryWrapper<Emergency> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO);
//        queryWrapper.eq("NAME", dto.getName());
//        queryWrapper.eq("PHONE", dto.getPhone());
//        Emergency one = this.getOne(queryWrapper);
//        if (!ObjectUtils.isEmpty(one)){
//            throw new RuntimeException(dto.getName() + "的手机已经存在，无需重复添加");
//        }
//    }
    @Override
    public void edit(EmergencyFacilitiesDTO dto) {
        EmergencyFacilities data = new EmergencyFacilities();
        BeanUtils.copyProperties(dto, data);
        if (!this.updateById(data)) {
            throw new RuntimeException("修改失败");
        }
    }

    @Override
    public void delete(Set<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new EmergencyFacilities(), new UpdateWrapper<EmergencyFacilities>().lambda()
                .set(EmergencyFacilities::getIsDel, FlowConstant.SHORT_ONE)
                .eq(EmergencyFacilities::getIsDel, FlowConstant.SHORT_ZERO)
                .in(EmergencyFacilities::getId, ids));
        if (!update) {
            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public void deleteEmergencyIds(Set<Long> emergencyIds) {
        if (CollectionUtils.isEmpty(emergencyIds)) {
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new EmergencyFacilities(), new UpdateWrapper<EmergencyFacilities>().lambda()
                .set(EmergencyFacilities::getIsDel, FlowConstant.SHORT_ONE)
                .eq(EmergencyFacilities::getIsDel, FlowConstant.SHORT_ZERO)
                .in(EmergencyFacilities::getEmergencyId, emergencyIds));
        if (!update) {
//            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public Page<EmergencyFacilities> pageList(Page page, EmergencyFacilitiesQuery query) {
        QueryWrapper<EmergencyFacilities> queryWrapper = getEmergencyQueryWrapper(query);
        Page<EmergencyFacilities> pageData = this.page(page, queryWrapper);
        if (pageData.getTotal() > 0) {
            for (EmergencyFacilities record : pageData.getRecords()) {
                if (record.getAssetsId() != null) {
                    record.setAssets(assetsService.getById(record.getAssetsId()));
                }
            }

        }
        return pageData;
    }

    /**
     * 设置查询条件
     *
     * @param query 查询参数
     * @return QueryWrapper
     */
    private QueryWrapper<EmergencyFacilities> getEmergencyQueryWrapper(EmergencyFacilitiesQuery query) {
        EmergencyFacilities emergencyFacilities = new EmergencyFacilities();
        BeanUtils.copyProperties(query, emergencyFacilities);
        QueryWrapper<EmergencyFacilities> queryWrapper = Wrappers.query(emergencyFacilities);
        return queryWrapper;
    }

    @Override
    public List<EmergencyFacilities> all(EmergencyFacilitiesQuery query) {
        QueryWrapper<EmergencyFacilities> addressBookQueryWrapper = getEmergencyQueryWrapper(query);
        return this.list(addressBookQueryWrapper);
    }



    @Override
    public List<EmergencyFacilitiesVO> getListByEmergencyId(Long emergencyId) {
        List<EmergencyFacilitiesVO> listByEmergencyId = baseMapper.getListByEmergencyId(emergencyId);
        if (!CollectionUtils.isEmpty(listByEmergencyId)){
            for (EmergencyFacilitiesVO facilitiesVO : listByEmergencyId) {
                //赋值绑定物资
                if (facilitiesVO.getAssetsId()!=null){
                    Assets assets = assetsService.getById(facilitiesVO.getAssetsId());
                    if (!ObjectUtils.isEmpty(assets)){
                        AssetsVO assetsVO = new AssetsVO();
                        BeanUtils.copyProperties(assets,assetsVO);
                        facilitiesVO.setAssetsVO(assetsVO);
                    }
                }
            }
        }
        return listByEmergencyId;
    }


    /**
     * 设置默认值
     *
     * @param dto 实体
     */
    private void setDefaultValue(EmergencyFacilitiesDTO dto) {
        dto.setIsDel(FlowConstant.SHORT_ZERO);
//        dto.setIsValid(FlowConstant.SHORT_ONE);
    }
}
