/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.entity.EmergencyHazard;
import com.tofly.flow.dto.AddressBookQuery;
import com.tofly.flow.entity.AddressBook;
import com.tofly.flow.entity.EmergencyHazard;
import com.tofly.flow.service.EmergencyHazardService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import com.tofly.flow.mapper.EmergencyHazardMapper;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Set;

/**
 * 事故危险源
 *
 * @author xqf
 * @date Mon Apr 25 00:00:00 CST 2022
 */
@Service("emergencyHazardService")
public class EmergencyHazardServiceImpl extends ServiceImpl<EmergencyHazardMapper, EmergencyHazard> implements EmergencyHazardService {


    @Override
    public void insert(EmergencyHazard dto) {
        setDefaultValue(dto);
        if (!this.save(dto)){
            throw new RuntimeException("新增失败");
        }
    }

    @Override
    public void batchInsert(List<EmergencyHazard> dtos) {
        for (EmergencyHazard dto : dtos) {
            insert(dto);
        }
    }


    /**
     * 检测重复数据
     * @param dto 操作数据实体
     */
//    private void checkData(EmergencyHazard dto) {
//        QueryWrapper<AddressBook> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO);
//        queryWrapper.eq("NAME", dto.getName());
//        queryWrapper.eq("PHONE", dto.getPhone());
//        AddressBook one = this.getOne(queryWrapper);
//        if (!ObjectUtils.isEmpty(one)){
//            throw new RuntimeException(dto.getName() + "的手机已经存在，无需重复添加");
//        }
//    }

    @Override
    public void edit(EmergencyHazard dto) {
        if (!this.updateById(dto)){
            throw new RuntimeException("修改失败");
        }
    }

    @Override
    public void delete(Set<Long> ids) {
        if (CollectionUtils.isEmpty(ids)){
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new EmergencyHazard(), new UpdateWrapper<EmergencyHazard>().lambda()
                .set(EmergencyHazard::getIsDel, FlowConstant.SHORT_ONE)
                .eq(EmergencyHazard::getIsDel, FlowConstant.SHORT_ZERO)
                .in(EmergencyHazard::getId, ids));
        if (!update){
            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public void deleteHazardId(Set<Long> hazardId) {
        if (CollectionUtils.isEmpty(hazardId)){
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new EmergencyHazard(), new UpdateWrapper<EmergencyHazard>().lambda()
                .set(EmergencyHazard::getIsDel, FlowConstant.SHORT_ONE)
                .eq(EmergencyHazard::getIsDel, FlowConstant.SHORT_ZERO)
                .in(EmergencyHazard::getHazardId, hazardId));
        if (!update){
            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public void deleteEmergencyId(Set<Long> emergencyId) {
        if (CollectionUtils.isEmpty(emergencyId)){
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new EmergencyHazard(), new UpdateWrapper<EmergencyHazard>().lambda()
                .set(EmergencyHazard::getIsDel, FlowConstant.SHORT_ONE)
                .eq(EmergencyHazard::getIsDel, FlowConstant.SHORT_ZERO)
                .in(EmergencyHazard::getEmergencyId, emergencyId));
        if (!update){
//            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public Page<EmergencyHazard> pageList(Page page, EmergencyHazard query) {
        QueryWrapper<EmergencyHazard> queryWrapper = getQueryWrapper(query);
        return this.page(page,queryWrapper);
    }

    /**
     * 设置查询条件
     * @param query 查询参数
     * @return QueryWrapper
     */
    private QueryWrapper<EmergencyHazard> getQueryWrapper(EmergencyHazard query) {
        QueryWrapper<EmergencyHazard> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_DEL",FlowConstant.SHORT_ZERO);
        if (query !=null){
//            if (!StringUtils.isEmpty(query.getName())){
//                queryWrapper.like("NAME", query.getName());
//            }
//            if (!StringUtils.isEmpty(query.getDeptName())){
//                queryWrapper.like("DEPT", query.getDeptName());
//            }
        }
        return queryWrapper;
    }

    @Override
    public List<EmergencyHazard> all(EmergencyHazard query) {
        QueryWrapper<EmergencyHazard> addressBookQueryWrapper = getQueryWrapper(query);
        return this.list(addressBookQueryWrapper);
    }

    @Override
    public List<EmergencyHazard> selectByEmergencyId(Long emergencyId) {
        QueryWrapper<EmergencyHazard> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_DEL",FlowConstant.SHORT_ZERO);
        queryWrapper.eq("EMERGENCY_ID",emergencyId);
        return this.list(queryWrapper);
    }


    /**
     * 设置默认值
     * @param dto 实体
     */
    private void setDefaultValue(EmergencyHazard dto) {
        dto.setIsDel(FlowConstant.SHORT_ZERO);
        dto.setIsValid(FlowConstant.SHORT_ONE);
        dto.setHazardState(FlowConstant.SHORT_TWO);
    }
}
