/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.service.KeepGrainDeviceService;
import com.tofly.flow.custom.service.KeepGrainMatterService;
import com.tofly.flow.custom.service.MatterService;
import com.tofly.flow.custom.vo.KeepGrainMatterVO;
import com.tofly.flow.custom.vo.MatterVO;
import com.tofly.flow.entity.EmergencyHazard;
import com.tofly.flow.entity.EmergencyMaterial;
import com.tofly.flow.service.EmergencyMaterialService;
import com.tofly.flow.vo.EmergencyMaterialVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.flow.mapper.EmergencyMaterialMapper;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * 事故应用的物资
 *
 * @author xqf
 * @date Mon Apr 25 00:00:00 CST 2022
 */
@Service("emergencyMaterialService")
public class EmergencyMaterialServiceImpl extends ServiceImpl<EmergencyMaterialMapper, EmergencyMaterial> implements EmergencyMaterialService {


    @Autowired
    private KeepGrainMatterService keepGrainMatterService;

    @Autowired
    private MatterService matterService;

    @Override
    public void insert(EmergencyMaterial dto) {
        setDefaultValue(dto);
        if (!this.save(dto)){
            throw new RuntimeException("新增失败");
        }
    }

    @Override
    public void batchInsert(List<EmergencyMaterial> dtos) {
        for (EmergencyMaterial dto : dtos) {
            insert(dto);
        }
    }


    /**
     * 检测重复数据
     * @param dto 操作数据实体
     */
//    private void checkData(EmergencyHazard dto) {
//        QueryWrapper<AddressBook> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO);
//        queryWrapper.eq("NAME", dto.getName());
//        queryWrapper.eq("PHONE", dto.getPhone());
//        AddressBook one = this.getOne(queryWrapper);
//        if (!ObjectUtils.isEmpty(one)){
//            throw new RuntimeException(dto.getName() + "的手机已经存在，无需重复添加");
//        }
//    }

    @Override
    public void edit(EmergencyMaterial dto) {
        if (!this.updateById(dto)){
            throw new RuntimeException("修改失败");
        }
    }

    @Override
    public void delete(Set<Long> ids) {
        if (CollectionUtils.isEmpty(ids)){
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new EmergencyMaterial(), new UpdateWrapper<EmergencyMaterial>().lambda()
                .set(EmergencyMaterial::getIsDel, FlowConstant.SHORT_ONE)
                .eq(EmergencyMaterial::getIsDel, FlowConstant.SHORT_ZERO)
                .in(EmergencyMaterial::getId, ids));
        if (!update){
            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public void deletekeepGrainMatterId(Set<Long> keepGrainMatterId) {
        if (CollectionUtils.isEmpty(keepGrainMatterId)){
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new EmergencyMaterial(), new UpdateWrapper<EmergencyMaterial>().lambda()
                .set(EmergencyMaterial::getIsDel, FlowConstant.SHORT_ONE)
                .eq(EmergencyMaterial::getIsDel, FlowConstant.SHORT_ZERO)
                .in(EmergencyMaterial::getKeepGrainMatterId, keepGrainMatterId));
        if (!update){
            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public void deleteEmergencyId(Set<Long> emergencyId) {
        if (CollectionUtils.isEmpty(emergencyId)){
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new EmergencyMaterial(), new UpdateWrapper<EmergencyMaterial>().lambda()
                .set(EmergencyMaterial::getIsDel, FlowConstant.SHORT_ONE)
                .eq(EmergencyMaterial::getIsDel, FlowConstant.SHORT_ZERO)
                .in(EmergencyMaterial::getEmergencyId, emergencyId));
        if (!update){
//            throw new RuntimeException("删除失败");
        }
    }


    @Override
    public Page<EmergencyMaterial> pageList(Page page, EmergencyMaterial query) {
        QueryWrapper<EmergencyMaterial> queryWrapper = getQueryWrapper(query);
        Page<EmergencyMaterial> pageData = this.page(page, queryWrapper);
        if (pageData.getTotal()>0){
            for (EmergencyMaterial record : pageData.getRecords()) {
                if (record.getKeepGrainMatterId()!=null){
                    KeepGrainMatterVO keepGrainMatterVO = keepGrainMatterService.get(record.getKeepGrainMatterId());
                    record.setKeepGrainMatterVO(keepGrainMatterVO);
                }
            }
        }
        return pageData;
    }

    /**
     * 设置查询条件
     * @param query 查询参数
     * @return QueryWrapper
     */
    private QueryWrapper<EmergencyMaterial> getQueryWrapper(EmergencyMaterial query) {
        QueryWrapper<EmergencyMaterial> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_DEL",FlowConstant.SHORT_ZERO);
        if (query !=null){
//            if (!StringUtils.isEmpty(query.getKeepGrainMatterId())){
//                queryWrapper.like("NAME", query.getName());
//            }
            if (query.getEmergencyId()!=null){
                queryWrapper.eq("EMERGENCY_ID", query.getEmergencyId());
            }
        }
        return queryWrapper;
    }

    @Override
    public List<EmergencyMaterial> all(EmergencyMaterial query) {
        QueryWrapper<EmergencyMaterial> queryWrapper = getQueryWrapper(query);
        return this.list(queryWrapper);
    }

    @Override
    public List<EmergencyMaterialVO> getListByEmergencyId(Long emergencyId) {
        EmergencyMaterial emergencyMaterial = new EmergencyMaterial();
        emergencyMaterial.setEmergencyId(emergencyId);
        QueryWrapper<EmergencyMaterial> queryWrapper = getQueryWrapper(emergencyMaterial);
        List<EmergencyMaterial> emergencyMaterials = list(queryWrapper);
        if (!CollectionUtils.isEmpty(emergencyMaterials)){
            List<EmergencyMaterialVO> emergencyMaterialVOS = new ArrayList<>();
            for (EmergencyMaterial material : emergencyMaterials) {
                EmergencyMaterialVO materialVO = new EmergencyMaterialVO();
                BeanUtils.copyProperties(material,materialVO);
                if (materialVO.getKeepGrainMatterId()!=null){
                    MatterVO matterVO = matterService.get(materialVO.getKeepGrainMatterId());
                    materialVO.setMatterVO(matterVO);
                }
                emergencyMaterialVOS.add(materialVO);
            }
           return emergencyMaterialVOS;
        }
        return null;
    }


    /**
     * 设置默认值
     * @param dto 实体
     */
    private void setDefaultValue(EmergencyMaterial dto) {
        dto.setIsDel(FlowConstant.SHORT_ZERO);
//        dto.setIsValid(FlowConstant.SHORT_ONE);
    }
}
