/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.entity.EmergencyHazard;
import com.tofly.flow.entity.EmergencyStaff;
import com.tofly.flow.service.EmergencyStaffService;
import org.springframework.stereotype.Service;
import com.tofly.flow.mapper.EmergencyStaffMapper;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Set;

/**
 * 事故人员
 *
 * @author xqf
 * @date Mon Apr 25 00:00:00 CST 2022
 */
@Service("emergencyStaffService")
public class EmergencyStaffServiceImpl extends ServiceImpl<EmergencyStaffMapper, EmergencyStaff> implements EmergencyStaffService {

    @Override
    public void insert(EmergencyStaff dto) {
        setDefaultValue(dto);
        if (!this.save(dto)){
            throw new RuntimeException("新增失败");
        }
    }

    @Override
    public void batchInsert(List<EmergencyStaff> dtos) {
        for (EmergencyStaff dto : dtos) {
            insert(dto);
        }
    }


    /**
     * 检测重复数据
     * @param dto 操作数据实体
     */
//    private void checkData(EmergencyHazard dto) {
//        QueryWrapper<AddressBook> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO);
//        queryWrapper.eq("NAME", dto.getName());
//        queryWrapper.eq("PHONE", dto.getPhone());
//        AddressBook one = this.getOne(queryWrapper);
//        if (!ObjectUtils.isEmpty(one)){
//            throw new RuntimeException(dto.getName() + "的手机已经存在，无需重复添加");
//        }
//    }

    @Override
    public void edit(EmergencyStaff dto) {
        if (!this.updateById(dto)){
            throw new RuntimeException("修改失败");
        }
    }

    @Override
    public void delete(Set<Long> ids) {
        if (CollectionUtils.isEmpty(ids)){
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new EmergencyStaff(), new UpdateWrapper<EmergencyStaff>().lambda()
                .set(EmergencyStaff::getIsDel, FlowConstant.SHORT_ONE)
                .eq(EmergencyStaff::getIsDel, FlowConstant.SHORT_ZERO)
                .in(EmergencyStaff::getId, ids));
        if (!update){
            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public void deleteAddressBookId(Set<Long> AddressBookId) {
        if (CollectionUtils.isEmpty(AddressBookId)){
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new EmergencyStaff(), new UpdateWrapper<EmergencyStaff>().lambda()
                .set(EmergencyStaff::getIsDel, FlowConstant.SHORT_ONE)
                .eq(EmergencyStaff::getIsDel, FlowConstant.SHORT_ZERO)
                .in(EmergencyStaff::getAddressBookId, AddressBookId));
        if (!update){
            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public void deleteEmergencyId(Set<Long> emergencyId) {
        if (CollectionUtils.isEmpty(emergencyId)){
            throw new RuntimeException("传入数据为空");
        }
        boolean update = this.update(new EmergencyStaff(), new UpdateWrapper<EmergencyStaff>().lambda()
                .set(EmergencyStaff::getIsDel, FlowConstant.SHORT_ONE)
                .eq(EmergencyStaff::getIsDel, FlowConstant.SHORT_ZERO)
                .in(EmergencyStaff::getEmergencyId, emergencyId));
        if (!update){
//            throw new RuntimeException("删除失败");
        }
    }

    @Override
    public Page<EmergencyStaff> pageList(Page page, EmergencyStaff query) {
        QueryWrapper<EmergencyStaff> queryWrapper = getQueryWrapper(query);
        return this.page(page,queryWrapper);
    }

    /**
     * 设置查询条件
     * @param query 查询参数
     * @return QueryWrapper
     */
    private QueryWrapper<EmergencyStaff> getQueryWrapper(EmergencyStaff query) {
        QueryWrapper<EmergencyStaff> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_DEL",FlowConstant.SHORT_ZERO);
        if (query !=null){
            if (!StringUtils.isEmpty(query.getName())){
                queryWrapper.like("NAME", query.getName());
            }
            if (query.getEmergencyId()!=null){
                queryWrapper.eq("EMERGENCY_ID", query.getEmergencyId());
            }
        }
        return queryWrapper;
    }

    @Override
    public List<EmergencyStaff> all(EmergencyStaff query) {
        QueryWrapper<EmergencyStaff> queryWrapper = getQueryWrapper(query);
        return this.list(queryWrapper);
    }

    @Override
    public List<EmergencyStaff> getListByEmergencyId(Long emergencyId) {
        EmergencyStaff emergencyStaff = new EmergencyStaff();
        emergencyStaff.setEmergencyId(emergencyId);
        QueryWrapper<EmergencyStaff> queryWrapper = getQueryWrapper(emergencyStaff);
        return this.list(queryWrapper);
    }


    /**
     * 设置默认值
     * @param dto 实体
     */
    private void setDefaultValue(EmergencyStaff dto) {
        dto.setIsDel(FlowConstant.SHORT_ZERO);
//        dto.setIsValid(FlowConstant.SHORT_ONE);
    }
}
