/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.custom.dto.DocumentDto;
import com.tofly.flow.custom.dto.LoadFolderDocumentDto;
import com.tofly.flow.custom.dto.UpdateDocumentDto;
import com.tofly.flow.custom.entity.Document;
import com.tofly.flow.custom.service.DocumentService;
import com.tofly.flow.entity.FolderDocument;
import com.tofly.flow.entity.FolderType;
import com.tofly.flow.service.FolderDocumentService;
import com.tofly.flow.service.FolderTypeService;
import com.tofly.flow.custom.vo.DocumentVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.flow.mapper.FolderDocumentMapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 文件夹与文档关联表
 *
 * @author xqf
 * @date Thu Apr 07 00:00:00 CST 2022
 */
@Service("folderDocumentService")
public class FolderDocumentServiceImpl extends ServiceImpl<FolderDocumentMapper, FolderDocument> implements FolderDocumentService {

    /**
     * 文档
     */
    @Autowired
    private DocumentService documentService;
    /**
     * 文件夹
     */
    @Autowired
    private FolderTypeService folderTypeService;


    /**
     * 保存文档及文件夹关系
     * @param dto
     * @param fileList
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object saveFolderDocument(DocumentDto dto, List<MultipartFile> fileList) {
        int count = folderTypeService.count(new QueryWrapper<FolderType>().lambda()
                .eq(FolderType::getId,dto.getDocumentType())
                .eq(FolderType::getIsDel, FlowConstant.SHORT_ZERO));
        if(count==0){
            return FlowConstant.DATA_NOTHINGNESS;
        }

        String str = documentService.check(dto,null);
        if(StringUtils.isNotEmpty(str)){
            return str;
        }
        Long docId = documentService.saveDocument(dto,fileList);
        if(FlowConstant.LONG_NEGATIVE_ONE ==docId){
            return FlowConstant.FAILED;
        }
        FolderDocument fd = new FolderDocument();
        fd.setDocumentId(docId);
        fd.setFolderId(dto.getDocumentType());
        fd.setIsDel(FlowConstant.SHORT_ZERO);
        return save(fd);
    }

    /**
     * 修改文档
     * @param dto 文档对象
     * @param fileList 文件
     * @return
     */
    @Override
    public Object updateFolderDocument(UpdateDocumentDto dto, List<MultipartFile> fileList) {
        String str = documentService.updateCheck(dto);
        if(StringUtils.isNotEmpty(str)){
            return str;
        }
        return documentService.updateDocument(dto,fileList);
    }

    /**
     * 根据文档id删除文档及关联信息
     * @param id 文档ID
     * @return
     */
    @Override
    public boolean removeUpdateById(Long id) {
        int count = this.count(new QueryWrapper<FolderDocument>().lambda()
                .eq(FolderDocument::getDocumentId,id)
                .eq(FolderDocument::getIsDel,FlowConstant.SHORT_ZERO));
        if(count == FlowConstant.INTEGER_ONE){
            documentService.removeUpdateById(id);
        }
        return this.update(new UpdateWrapper<FolderDocument>().lambda()
                .eq(FolderDocument::getDocumentId,id)
                .eq(FolderDocument::getIsDel,FlowConstant.SHORT_ZERO)
                .set(FolderDocument::getIsDel,FlowConstant.SHORT_ONE));
    }

    /**
     * 批量删除文档
     * @param ids
     * @return
     */
    @Override
    public Object removeUpdateByIds(String ids) {
        try {
            List<Long> lids= Arrays.asList(ids.split(",")).stream().map(st->Long.parseLong(st)).collect(Collectors.toList());
            if(CollUtil.isNotEmpty(lids)) {
                List<Long> dId = this.baseMapper.getFolderDocumentOne(lids);
                if(CollUtil.isNotEmpty(dId)) {
                    documentService.removeUpdateByIds(dId);
                }
                return this.baseMapper.update(new FolderDocument(), new UpdateWrapper<FolderDocument>().lambda()
                        .set(FolderDocument::getIsDel, FlowConstant.SHORT_ONE)
                        .eq(FolderDocument::getIsDel, FlowConstant.SHORT_ZERO)
                        .in(FolderDocument::getDocumentId, lids));
            }
            return FlowConstant.INTEGER_ZERO;
        }catch (Exception e){
            log.error("异常：",e);
            return e.getMessage();
        }
    }

    /**
     * 查询文档列表
     * @param page
     * @param dto
     * @return
     */
    @Override
    public Page<DocumentVo> LoadFolderDocumentpage(Page page, LoadFolderDocumentDto dto) {

        return documentService.loadTypeDocumentPage(page,dto);
    }

    /**
     * 转成文档
     * @param documentType 文档类型
     * @param documentId 文档ID
     * @return
     */
    @Override
    public Object transferred(Long documentType, Long documentId) {
        int count = documentService.count(new QueryWrapper<Document>().lambda()
                .eq(Document::getId,documentId).eq(Document::getIsDel,FlowConstant.SHORT_ZERO));
        if(count==FlowConstant.INTEGER_ZERO){
            return FlowConstant.DATA_NOTHINGNESS;
        }
        count = folderTypeService.count(new QueryWrapper<FolderType>().lambda()
                .eq(FolderType::getId,documentType)
                .eq(FolderType::getIsDel,FlowConstant.SHORT_ZERO).eq(FolderType::getCreateUser, SecurityUtils.getUserId()));
        if(count==FlowConstant.INTEGER_ZERO){
            return FlowConstant.DATA_NOTHINGNESS;
        }
        FolderDocument fd = new FolderDocument();
        fd.setDocumentId(documentId);
        fd.setFolderId(documentType);
        fd.setIsDel(FlowConstant.SHORT_ZERO);
        return save(fd);

    }
}
