/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.flow.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.dto.FolderTypeDto;
import com.tofly.flow.dto.UpdateFolderTypeDto;
import com.tofly.flow.entity.FolderDocument;
import com.tofly.flow.entity.FolderType;
import com.tofly.flow.service.FolderDocumentService;
import com.tofly.flow.service.FolderTypeService;
import com.tofly.flow.vo.FolderTypeTreeVo;
import com.tofly.flow.vo.FolderTypeVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.flow.mapper.FolderTypeMapper;

import java.util.*;

/**
 * 文件夹表
 *
 * @author xqf
 * @date Thu Apr 07 00:00:00 CST 2022
 */
@Service("folderTypeService")
public class FolderTypeServiceImpl extends ServiceImpl<FolderTypeMapper, FolderType> implements FolderTypeService {

    @Autowired
    private FolderDocumentService folderDocumentService;

    /**
     * 保存文件夹名称
     * @param dto 文件夹信息
     * @return
     */
    @Override
    public Object saveFolderTypeDto(FolderTypeDto dto) {
        if(dto.getParentId()==null){
            dto.setParentId(FlowConstant.LONG_ZERO);
        }
        int count = 0;
        if(dto.getParentId()!=FlowConstant.LONG_ZERO){
            count = this.count(new QueryWrapper<FolderType>().lambda()
                    .eq(FolderType::getId,dto.getParentId())
                    .eq(FolderType::getIsDel,FlowConstant.SHORT_ZERO));
            if(count==0){
                return FlowConstant.DATA_NOTHINGNESS;
            }
        }
        if(dto.getModuleType()==null){
            dto.setModuleType(FlowConstant.SHORT_ZERO);
        }

        if(FlowConstant.LONG_ZERO<=dto.getDataType() && FlowConstant.LONG_TWO>=dto.getDataType()) {
            count = this.count(new QueryWrapper<FolderType>().lambda()
                    .eq(FolderType::getParentId, dto.getParentId())
                    .eq(FolderType::getFolderName, dto.getFolderName())
                    .eq(FolderType::getModuleType,dto.getModuleType())
                    .eq(FlowConstant.LONG_ONE == dto.getDataType(),FolderType::getCreateUser, SecurityUtils.getUserId() )
                    .eq(FlowConstant.LONG_TWO == dto.getDataType(), FolderType::getDeptId, SecurityUtils.getUserDeptId())
                    .eq(FolderType::getIsDel, FlowConstant.SHORT_ZERO));
            if(count>0){
                return FlowConstant.NAME_EXISTS;
            }
        }else {
            return FlowConstant.DATATYPE_ERROR;
        }
        FolderType folderType = new FolderType();
        BeanUtils.copyProperties(dto,folderType);
        if(FlowConstant.LONG_TWO == dto.getDataType()){
            folderType.setDeptId(SecurityUtils.getUserDeptId());
        }
        folderType.setIsDel(FlowConstant.SHORT_ZERO);
        return save(folderType);
    }

    /**
     * 修改文件夹信息
     * @param dto 文件夹信息
     * @return
     */
    @Override
    public Object updateFolderTypeDto(UpdateFolderTypeDto dto) {
        FolderType ft = getById(dto.getId());
        if(ft==null){
            return FlowConstant.DATA_NOTHINGNESS;
        }
        if(StringUtils.isNotEmpty(dto.getFolderName())){
            int count = this.count(new QueryWrapper<FolderType>().lambda()
                    .eq(FolderType::getParentId, ft.getParentId())
                    .eq(FolderType::getFolderName, dto.getFolderName())
                    .eq(FolderType::getModuleType,ft.getModuleType())
                    .eq(FlowConstant.LONG_ONE == ft.getDataType(),FolderType::getCreateUser, SecurityUtils.getUserId() )
                    .eq(FlowConstant.LONG_TWO == ft.getDataType(), FolderType::getDeptId, SecurityUtils.getUserDeptId())
                    .eq(FolderType::getIsDel, FlowConstant.SHORT_ZERO).ne(FolderType::getId,dto.getId()));
            if(count>0){
                return FlowConstant.NAME_EXISTS;
            }
        }
        FolderType folderType = new FolderType();
        BeanUtils.copyProperties(dto,folderType);
        return updateById(folderType);
    }

    /**
     * 删除文件
     * @param id 主键
     * @return
     */
    @Override
    public Object removeUpdateById(Long id) {
        int count = folderDocumentService.count(new QueryWrapper<FolderDocument>().lambda()
                .eq(FolderDocument::getFolderId,id)
                .eq(FolderDocument::getIsDel,FlowConstant.SHORT_ZERO));
        if(count>0){
            return FlowConstant.RELATION_ERROR;
        }

        return update(new UpdateWrapper<FolderType>().lambda().set(FolderType::getIsDel,FlowConstant.SHORT_ONE)
                .set(FolderType::getUpdateTime,new Date()).set(FolderType::getUpdateUser, SecurityUtils.getUserId())
                .eq(FolderType::getIsDel,FlowConstant.SHORT_ZERO)
                .eq(FolderType::getId,id));
    }

    /**
     * 获取文件树
     * @param dataType 文件夹类型
     * @param moduleType  模块类型0文档 默认为0;1公告
     * @return
     */
    @Override
    public FolderTypeTreeVo folderTree(Long dataType, Short moduleType) {
        if(moduleType==null){
            moduleType = FlowConstant.SHORT_ZERO;
        }

        List<FolderType> lists=null;
        if(FlowConstant.LONG_ZERO.equals(dataType)){
            lists = this.list(new QueryWrapper<FolderType>().lambda()
                    .eq(FolderType::getModuleType,moduleType)
                    .eq(FolderType::getDataType,dataType).eq(FolderType::getIsDel,FlowConstant.SHORT_ZERO)
                    .orderByAsc(FolderType::getParentId,FolderType::getId));
        }else if(FlowConstant.LONG_ONE.equals(dataType)){
            lists = this.list(new QueryWrapper<FolderType>().lambda()
                    .eq(FolderType::getDataType,dataType)
                    .eq(FolderType::getModuleType,moduleType)
                    .eq(FolderType::getCreateUser,SecurityUtils.getUserId())
                    .eq(FolderType::getIsDel,FlowConstant.SHORT_ZERO)
                    .orderByAsc(FolderType::getParentId,FolderType::getId));
        }else if((FlowConstant.LONG_TWO.equals(dataType))){
            lists = this.list(new QueryWrapper<FolderType>().lambda()
                    .eq(FolderType::getDataType,dataType)
                    .eq(FolderType::getModuleType,moduleType)
                    .eq(FolderType::getDeptId,SecurityUtils.getUserDeptId())
                    .eq(FolderType::getIsDel,FlowConstant.SHORT_ZERO)
                    .orderByAsc(FolderType::getParentId,FolderType::getId));
        }else {
            lists = this.list(new QueryWrapper<FolderType>().lambda()
                    .eq(FolderType::getIsDel,FlowConstant.SHORT_ZERO)
                    .eq(FolderType::getModuleType,moduleType)
                    .and(r->r.ne(FolderType::getDataType,FlowConstant.SHORT_ONE)
                            .or(v->v.eq(FolderType::getDataType,FlowConstant.SHORT_ONE)
                                    .eq(FolderType::getCreateUser,SecurityUtils.getUserId())))
                    .orderByAsc(FolderType::getParentId,FolderType::getId));

        }
        Map<Long,FolderTypeVo> tmap = new HashMap<>();
        Map<Long,FolderTypeVo> map = new HashMap<>();
        if(CollUtil.isNotEmpty(lists)){
            for (FolderType ft:lists) {
                FolderTypeVo vo = new FolderTypeVo();
                BeanUtils.copyProperties(ft,vo);
                vo.setFts(new ArrayList<>());
                FolderTypeVo ftvo;
                if(!tmap.containsKey(ft.getDataType())){
                    ftvo = new FolderTypeVo();
                    ftvo.setDataType(ft.getDataType());
                    if(FlowConstant.LONG_ZERO==ft.getDataType()){
                        ftvo.setFolderName(FlowConstant.COMPANY_DOCUMENT);
                    }else if(FlowConstant.LONG_ONE==ft.getDataType()){
                        ftvo.setFolderName(FlowConstant.MY_DOCUMENT);
                    }else if((FlowConstant.LONG_TWO==ft.getDataType())){
                        ftvo.setFolderName(FlowConstant.DEPARTMENT_DOCUMENT);
                    }
                    ftvo.setFts(new ArrayList<>());

                    tmap.put(ft.getDataType(),ftvo);
                }else {
                    ftvo = tmap.get(ft.getDataType());
                }

                if(map.containsKey(vo.getParentId())){
                    FolderTypeVo fft = map.get(vo.getParentId());
                    fft.getFts().add(vo);
                    map.put(vo.getId(),vo);
                }else {
                    ftvo.getFts().add(vo);
                    map.put(vo.getId(),vo);
                }
            }

        }
        return new FolderTypeTreeVo(new ArrayList<>(tmap.values()));
    }


}
