package com.tofly.flow.util;

import com.tofly.flow.common.FlowConstant;
import com.tofly.flow.config.ForeignConfig;
import com.tofly.flow.custom.service.DocumentService;
import com.tofly.flow.custom.service.ProcessStuffService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * 监听redis队列，消费流程处理结束后发送的消息
 */
@Slf4j
@Service
public class RedisQueueUtil {
    /**
     * redis
     */
    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 配置文件
     */
    @Autowired
    private ForeignConfig foreignConfig;
    /**
     * 流程物资出入库
     */
    @Autowired
    private ProcessStuffService processStuffService;

    /**
     * 文档、公告
     */
    @Autowired
    private DocumentService documentService;

    private volatile boolean flag = true;

    private Long popTimeout = 60l;

    //线程池
    private static ExecutorService executorService = new ThreadPoolExecutor(4, 5,
            60L, TimeUnit.SECONDS,
            new SynchronousQueue<Runnable>());

    @PostConstruct
    public void start(){
        //物资入库
        executorService.execute(()-> {
            try {
                Long message;
                log.info("============启动监听 采购物资入库  {}",flag && !Thread.currentThread().isInterrupted());
                while(flag && !Thread.currentThread().isInterrupted()) {
                    message = (Long) redisTemplate.opsForList().rightPop(FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueRkKey(), popTimeout, TimeUnit.SECONDS);
                    if(message!=null) {
                        log.info("采购物资入库 流程id"+message);
                        processStuffService.synchroStuff(message, FlowConstant.INTEGER_ZERO);
                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage());
            }
        });
        //物资入库
        executorService.execute(()-> {
            try {
                Long message;
                log.info("============启动监听 物资出库  {}",flag && !Thread.currentThread().isInterrupted());
                while(flag && !Thread.currentThread().isInterrupted()) {
                    message = (Long) redisTemplate.opsForList().rightPop(FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueCkKey(), popTimeout, TimeUnit.SECONDS);
                    if(message!=null) {
                        log.info("采购物资出库 流程id"+message);
                        processStuffService.synchroStuff(message, FlowConstant.INTEGER_TWO);
                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage());
            }
        });

        //物资退库
        executorService.execute(()-> {
            try {
                Long message;
                log.info("============启动监听 物资退库  {}",flag && !Thread.currentThread().isInterrupted());
                while(flag && !Thread.currentThread().isInterrupted()) {
                    message = (Long) redisTemplate.opsForList().rightPop(FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueTkKey(), popTimeout, TimeUnit.SECONDS);
                    if(message!=null) {
                        processStuffService.synchroStuff(message, FlowConstant.INTEGER_ONE);
                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage());
            }
        });

        //会议通知
        executorService.execute(()-> {
            try {
                Long message;
                log.info("============启动监听 会议通知  {}",flag && !Thread.currentThread().isInterrupted());
                while(flag && !Thread.currentThread().isInterrupted()) {
                    message = (Long) redisTemplate.opsForList().rightPop(FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueHyKey(), popTimeout, TimeUnit.SECONDS);
                    if(message!=null) {
                        log.info("会议通知 流程ID"+message);
                        documentService.meetingTidings(message);
                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage());
            }
        });

        //收文（阅类）
        //TODO
    }
}
