package com.tofly.watermeter.custom.service.meterBusiness.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.api.UserClent;
import com.tofly.common.core.util.DateUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.InstallConditionDto;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.InstallTableDto;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.ParamDto;
import com.tofly.watermeter.custom.mapper.meterBusiness.TableInstallMapper;
import com.tofly.watermeter.custom.service.meterBusiness.TableInstallService;
import com.tofly.watermeter.custom.service.meterBusiness.export.ExportCommonCodeService;
import com.tofly.watermeter.custom.vo.StationVo;
import com.tofly.watermeter.entity.Bwrksbdatail;
import com.tofly.watermeter.service.BwrksbdatailService;
import com.tofly.watermeter.util.CallAPIUtil;
import com.tofly.watermeter.util.ThreadPoolUtil;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletResponse;
import java.util.*;

@AllArgsConstructor
@Service
public class TableInstallServiceImpl implements TableInstallService {

    private final TableInstallMapper tableInstallMapper;
    private final BwrksbdatailService bwrksbdatailService;
    private final ExportCommonCodeService exportCommonCodeService;
    private final UserClent userClent;

    /**
     * 获取未安装水表客户信息
     *
     * @param dto 客户姓名、地址、电话、编号
     * @return list
     */
    @Override
    public Page<Map> getCustomerInfo(Page page, String dto) {
        return tableInstallMapper.getCustomerInfo(page, dto);
    }

    /**
     * 获取水表信息
     */
    @Override
    public Page<Map> getTableInfo(Page page, String dto, Long sbcj, Long sblx, Long sbkj) {
        return tableInstallMapper.getTableInfo(page, dto, sbcj, sblx, sbkj);
    }

    /**
     * 水表安装
     *
     * @param ids 暂存id
     * @return boolean
     * @throws Exception
     */
    @Override
    public boolean tableInstall(String ids) {

        if (StringUtils.isEmpty(ids)) {
            throw new ToflyDeniedException("暂存信息为空");
        }

        List<String> idList = Arrays.asList(ids.split(","));

        List<Bwrksbdatail> bwrksbdatailList = bwrksbdatailService.list(Wrappers.<Bwrksbdatail>lambdaQuery()
                .in(Bwrksbdatail::getId, idList));

        List<InstallTableDto> dtoList = new ArrayList<>();

        InstallTableDto dto;

        for (Bwrksbdatail rk : bwrksbdatailList) {
            dto = new InstallTableDto();

            dto.setACTION("query")
                    .setYHDZ(rk.getBwKhdz())
                    .setYHXM(rk.getBwKhxm())
                    .setYHBH(rk.getBwKhbh())
                    .setSBQD(rk.getBwSbqd().toString())
                    .setSBLX(rk.getBwSblx().toString())
                    .setSBKJ(rk.getBwSbkj().toString())
                    .setSBCJ(rk.getBwSbcj().toString())
                    .setLXDH(rk.getBwKhdh())
                    .setSFNCYS(rk.getSfncys())
                    .setJYZ(rk.getJyz())
                    .setBZBH(rk.getBzbh())
                    .setGDDH(rk.getGddh())
                    .setSBBH(rk.getBwSbbh())
                    .setAZRQ(DateUtil.getDateFormat(rk.getBwAzrq() == null ? new Date() : rk.getBwAzrq(), "yyyyMMdd"))
                    .setAZRY(rk.getBwAzry().toString())
                    .setAZFS(rk.getAzfs())
                    .setSBAZDZ(rk.getSbazdz());

            dtoList.add(dto);
        }


        ThreadPoolUtil.executorService.execute(() -> {
            Map map = null;
            for (InstallTableDto itDto : dtoList) {
                try {
                    map = CallAPIUtil.callAPIUtil(itDto, "/api/Business/newWater");
                } catch (Exception e) {
                    e.printStackTrace();
                }
                String isSyc = "1";
                String msg = "";
                if (map != null && !map.isEmpty()) {
                    if (!"0000".equals(map.get("Code"))) {
                        isSyc = "2";
                        msg = map.get("message").toString();
                    }else {
                        msg = map.get("data").toString();
                    }

                }
                bwrksbdatailService.update(Wrappers.<Bwrksbdatail>lambdaUpdate()
                        .set(Bwrksbdatail::getIssyc, isSyc)
                        .set(Bwrksbdatail::getMsg, msg)
                        .eq(Bwrksbdatail::getBwSbbh, itDto.getSBBH()));
            }
        });
        return true;
    }

//    @Override
//    @Transactional(rollbackFor = Exception.class)
//    public boolean tableInstall(InstallTableDto dto) throws Exception {
//        if (dto == null) {
//            throw new ToflyDeniedException("传入数据为空");
//        }
//        dto.setACTION("QUERY");
//        Map map = CallAPIUtil.callAPIUtil(dto, "/api/Business/newWater");
//        if ("0000".equals(map.get("code"))) {
//
//            User result = userClent.getById(Long.valueOf(dto.getAZRY())).getResult();
//
//            assert result != null;
//            bwrksbdatailService.update(Wrappers.<Bwrksbdatail>lambdaUpdate().eq(Bwrksbdatail::getBwSbbh, dto.getSBBH())
//                    .set(!StringUtils.isEmpty(dto.getYHBH()), Bwrksbdatail::getBwKhbh, dto.getYHBH())
//                    .set(Bwrksbdatail::getBwKhdz, dto.getYHDZ())
//                    .set(Bwrksbdatail::getAzfs, dto.getAZFS())
//                    .set(Bwrksbdatail::getBwKhxm, dto.getYHXM())
//                    .set(!StringUtils.isEmpty(dto.getLXDH()), Bwrksbdatail::getBwKhdh, dto.getLXDH())
//                    .set(Bwrksbdatail::getBwSbzt, 3)
//                    .set(Bwrksbdatail::getBwAzry, dto.getAZRY())
//                    .set(Bwrksbdatail::getBwSbqd, Long.valueOf(dto.getSBQD()))
//                    .set(!StringUtils.isEmpty(dto.getGDDH()), Bwrksbdatail::getGddh, dto.getGDDH())
//                    .set(Bwrksbdatail::getBwAzrq, new Date())
//                    .set(Bwrksbdatail::getAzbm, result.getDepartmentId()));
//
//            qfkhjbxxService.update(Wrappers.<Qfkhjbxx>lambdaUpdate().eq(Qfkhjbxx::getYhbh, dto.getYHBH())
//                    .set(Qfkhjbxx::getZbry, dto.getAZRY())
//                    .set(Qfkhjbxx::getSfncys, dto.getSFNCYS())
//                    .set(Qfkhjbxx::getJyz, dto.getJYZ())
//                    .set(Qfkhjbxx::getSbbh, dto.getSBBH())
//                    .set(Qfkhjbxx::getSbcj, dto.getSBCJ())
//                    .set(Qfkhjbxx::getSbkj, dto.getSBKJ())
//                    .set(Qfkhjbxx::getSblx, dto.getSBLX())
//                    .set(Qfkhjbxx::getAzfs, dto.getAZFS())
//                    .set(Qfkhjbxx::getZhXgrq, new Date()));
//        } else {
//            throw new ToflyDeniedException(map.get("message").toString());
//        }
//        return true;
//    }

    @Override
    public List<Map> installCount(HttpServletResponse response, InstallConditionDto dto) {

        List<Map> maps = tableInstallMapper.installCount(dto);

        if (dto.getExport() == 1) {
            exportCommonCodeService.exportCode(response, "tableInstallCountService", maps);
        }
        return maps;
    }

    @Override
    public Page<Map> countByType(HttpServletResponse response, Page page, InstallConditionDto dto) {
        if (dto.getExport() == 1) {
            exportCommonCodeService.exportCode(response, "queryByNewTableService", tableInstallMapper.countByNewTable(dto));
        }
        return tableInstallMapper.countByNewTable(page, dto);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object theStagingAbout(Page page, InstallTableDto dto) {
        if (StringUtils.isEmpty(dto.getState())) {
            throw new ToflyDeniedException("state必传");
        }
        //查询
        if ("0".equals(dto.getState())) {
            return tableInstallMapper.theStagingQuery(dto);
            //分页
        } else if ("3".equals(dto.getState())) {
            return tableInstallMapper.theStagingQuery(page, dto);
            //新增或修改
        } else if ("1".equals(dto.getState())) {
            if (StringUtils.isEmpty(dto.getSBBH())) {
                throw new ToflyDeniedException("水表编号不允许为空");
            }

            Long deptId = !StringUtils.isEmpty(dto.getAZRY()) ? userClent.getById(Long.valueOf(dto.getAZRY())).getResult().getDepartmentId() : null;
            Date date = dto.getAZRQ() == null ? null : cn.hutool.core.date.DateUtil.parse(dto.getAZRQ(), "yyyy-MM-dd HH:mm:ss");

            bwrksbdatailService.update(Wrappers.<Bwrksbdatail>lambdaUpdate()
                    .set(!StringUtils.isEmpty(dto.getYHBH()), Bwrksbdatail::getBwKhbh, dto.getYHBH())
                    .set(!StringUtils.isEmpty(dto.getYHXM()), Bwrksbdatail::getBwKhxm, dto.getYHXM())
                    .set(!StringUtils.isEmpty(dto.getLXDH()), Bwrksbdatail::getBwKhdh, dto.getLXDH())
                    .set(!StringUtils.isEmpty(dto.getYHDZ()), Bwrksbdatail::getBwKhdz, dto.getYHDZ())
                    .set(!StringUtils.isEmpty(dto.getGDDH()), Bwrksbdatail::getGddh, dto.getGDDH())
                    .set(!StringUtils.isEmpty(dto.getAZRQ()), Bwrksbdatail::getBwAzrq, date)
                    .set(!StringUtils.isEmpty(dto.getAZRY()), Bwrksbdatail::getBwAzry, dto.getAZRY())
                    .set(!StringUtils.isEmpty(dto.getSBQD()), Bwrksbdatail::getBwSbqd, dto.getSBQD())
                    .set(!StringUtils.isEmpty(dto.getAZFS()), Bwrksbdatail::getAzfs, dto.getAZFS())
                    .set(!StringUtils.isEmpty(dto.getSFNCYS()), Bwrksbdatail::getSfncys, dto.getSFNCYS())
                    .set(!StringUtils.isEmpty(dto.getJYZ()), Bwrksbdatail::getJyz, dto.getJYZ())
                    .set(!StringUtils.isEmpty(dto.getBZBH()), Bwrksbdatail::getBzbh, dto.getBZBH())
                    .set(!StringUtils.isEmpty(dto.getSBAZDZ()), Bwrksbdatail::getSbazdz, dto.getSBAZDZ())
                    .set(!StringUtils.isEmpty(deptId), Bwrksbdatail::getAzbm, deptId)
                    .set(Bwrksbdatail::getIssyc, 0)
                    .set(Bwrksbdatail::getBwSbzt, 3)
                    .eq(Bwrksbdatail::getBwSbbh, dto.getSBBH()));

            return tableInstallMapper.theStagingQuery(null);
            //删除
        } else {
            bwrksbdatailService.update(Wrappers.<Bwrksbdatail>lambdaUpdate()
                    .set(Bwrksbdatail::getIssyc, "")
                    .set(Bwrksbdatail::getBwSbzt, 2)
                    .eq(Bwrksbdatail::getBwSbbh, dto.getSBBH()));

            return tableInstallMapper.theStagingQuery(null);
        }
    }

    @Override
    public List<StationVo> getStation() throws Exception {
        List<StationVo> result = null;
        ParamDto dto = new ParamDto();
        dto.setACTION("query");
        Map map = CallAPIUtil.callAPIUtil(dto, "/api/Business/BOOSTER_STATION");
        if ("0000".equals(map.get("Code"))) {
            result = JSONObject.parseArray(map.get("data").toString(), StationVo.class);
        }
        return result;
    }

}
