package com.tofly.watermeter.custom.service.meterBusiness.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.util.DateUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.ImportTurnOver;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.ImportTurnOverDto;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.TurnOverDto;
import com.tofly.watermeter.custom.mapper.meterBusiness.TurnOverMapper;
import com.tofly.watermeter.custom.service.meterBusiness.TurnOverService;
import com.tofly.watermeter.custom.service.meterBusiness.export.ExportCommonCodeService;
import com.tofly.watermeter.entity.Bwrksbdatail;
import com.tofly.watermeter.entity.Bwsbyjgl;
import com.tofly.watermeter.service.BwrkdetailService;
import com.tofly.watermeter.service.BwsbyjglService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class TurnOverServiceImpl implements TurnOverService {

    @Autowired
    private TurnOverMapper turnOverMapper;
    @Autowired
    private ExportCommonCodeService exportCommonCodeService;
    @Autowired
    private BwsbyjglService bwsbyjglService;
    @Autowired
    private BwrkdetailService bwrkdetailService;

    private final static Integer MAX_INSERT_NUMBER = 1000;

    private final static String TURM_OVER_TABLE = "TF_YWPN_BWSBYJGL_W";

    @Override
    public Page<Map> canTurnOver(Page page, TurnOverDto dto) {
        return turnOverMapper.canTurnOver(page, dto);
    }

    @Override
    public List<Map> count(HttpServletResponse response, TurnOverDto dto) {
        List<Map> count;
        if (dto.getIsTimeCount() == 0){
            count = turnOverMapper.count(dto);
        }else {
            count = turnOverMapper.countByTime(dto);
        }

        if (dto.getExport() == 1) {
            if (dto.getIsTimeCount() == 0){
                exportCommonCodeService.exportCode(response, "turnOverCountService", count);
            }else {
                exportCommonCodeService.exportCode(response, "turnOverCountByTimeService", count);
            }
        }
        return count;
    }

    @Override
    public Page<Map> queryInfo(HttpServletResponse response, Page page, TurnOverDto dto) {
        if (dto.getExport() == 1) {
            exportCommonCodeService.exportCode(response, "turnOverQueryInfoService", turnOverMapper.queryInfo(dto));
        }
        return turnOverMapper.queryInfo(page, dto);
    }

    /**
     * 导入移交信息
     *
     * @param dto dto
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean importTurnOver(ImportTurnOverDto dto) {
        if (dto == null) {
            throw new ToflyDeniedException("传入数据为空");
        }
        List<ImportTurnOver> turnOver = dto.getTurnOver();

        if (turnOver == null || turnOver.isEmpty()) {
            throw new ToflyDeniedException("excel上传内容为空");
        }

        if (dto.getType() == null) {
            throw new ToflyDeniedException("移交类型不能为空");
        }

        //收集水表编号（旧）
        List<String> codeOldList = turnOver.stream()
                .map(ImportTurnOver::getCode)
                .collect(Collectors.toList());
        List<String> codeNewList = turnOver.stream()
                .map(ImportTurnOver::getNewCode)
                .collect(Collectors.toList());
        //移交类型-水表退库
        if (dto.getType() == 1) {
            List<Bwrksbdatail> integers = turnOverMapper.queryTableState(codeOldList, Collections.singletonList("2"));

            if (integers.size() != codeOldList.size()) {
                throw new ToflyDeniedException("水表编号数据异常,请确认数据");
            }
        } else {
//            if (dto.getType() != 4) {
//                List<Bwrksbdatail> integersOld = turnOverMapper.queryTableState(codeOldList, Collections.singletonList("9"));
//                if (integersOld.size() != codeOldList.size()) {
//                    throw new ToflyDeniedException("水表编号数据异常，请确认数据");
//                }
//                List<Bwrksbdatail> integersNew = turnOverMapper.queryTableState(codeNewList, Collections.singletonList("9"));
//                if (integersNew.size() != codeNewList.size()) {
//                    throw new ToflyDeniedException("新表编号数据异常，请确认数据");
//                }
//            }
        }

        //查询厂家类型口径
        List<Bwrksbdatail> rksbdatailOld = turnOverMapper.queryTableState(codeOldList, null);

        //根据编号分组
        Map<String, List<Bwrksbdatail>> groupByCodeOld = rksbdatailOld.stream()
                .collect(Collectors.groupingBy(Bwrksbdatail::getBwSbbh));

        List<Bwrksbdatail> rksbdatailNew;
        Map<String, List<Bwrksbdatail>> groupByCodeNew = null;
        if (dto.getType() != 1) {
            rksbdatailNew = turnOverMapper.queryTableState(codeNewList, null);
            groupByCodeNew = rksbdatailNew.stream()
                    .collect(Collectors.groupingBy(Bwrksbdatail::getBwSbbh));
        }

        Bwsbyjgl yjgl;

        List<Bwsbyjgl> result = new ArrayList<>();

        String dateFormat = DateUtil.getDateFormat(new Date(), "yyyyMMdd-");

        String checkFlag = dto.getType() == 1 ? "2" : "0";

        Long maxId = bwrkdetailService.getMaxId(TURM_OVER_TABLE);

        for (ImportTurnOver it : turnOver) {

            yjgl = new Bwsbyjgl();
            yjgl.setBwLsbh(dateFormat + System.currentTimeMillis() + "");
            if (groupByCodeOld.containsKey(it.getCode())) {
                Bwrksbdatail tail = groupByCodeOld.get(it.getCode()).get(0);
                yjgl.setBwSbcj(tail.getBwSbcj());
                yjgl.setBwSblx(tail.getBwSblx());
                yjgl.setBwSbkj(tail.getBwSbkj());
            }
            yjgl.setSbzsbh(it.getCode());
            yjgl.setBwYjlx(dto.getType());
            yjgl.setBwYjyy(it.getCause());
            if (dto.getType() != 1) {
                yjgl.setUserName(it.getUserName());
                yjgl.setAddress(it.getAddress());
                if (groupByCodeNew != null && groupByCodeNew.containsKey(it.getNewCode())) {
                    Bwrksbdatail tail = groupByCodeNew.get(it.getNewCode()).get(0);
                    yjgl.setSbcj(tail.getBwSbcj());
                    yjgl.setSblx(tail.getBwSblx());
                    yjgl.setSbkj(tail.getBwSbkj());
                    yjgl.setSbbh(it.getNewCode());
                }
                yjgl.setBwJsbm(dto.getAcceptDept());
                yjgl.setBwJsry(dto.getAcceptUser());
            }
            yjgl.setBwYjry(dto.getTurnOverUser());
            yjgl.setBwYjrq(new Date());
            yjgl.setBwYjbm(dto.getTurnOverDept());
            yjgl.setCheckFlag(checkFlag);
            yjgl.setId(maxId);
            yjgl.setCreateUser(SecurityUtils.getUserId());
            maxId++;

//            bwsbyjglService.save(yjgl);
            result.add(yjgl);
        }
        bwsbyjglService.saveBatch(result);
        List<String> tableCodeList = result.stream().map(Bwsbyjgl::getSbzsbh).collect(Collectors.toList());
        if (tableCodeList.size() > 0) {
            //退库
            if (dto.getType() == 1) {
                turnOverMapper.updateTableState(tableCodeList, "8");
            } else {
                //报废
                turnOverMapper.updateTableState(tableCodeList, "7");
            }

        }

        return true;
    }

    /**
     * 获取不存在数据库水表编码
     *
     * @param code code
     * @return list
     */
    @Override
    public List<String> getNonEntityCode(String code, String sbzt) {
        if (StringUtils.isEmpty(code)) {
            throw new ToflyDeniedException("传入数据为空");
        }
        List<String> codeList = Arrays.stream(code.split(",")).distinct().collect(Collectors.toList());

        List<String> collect = turnOverMapper.queryTableState(codeList, Collections.singletonList("2"))
                .stream()
                .distinct().map(Bwrksbdatail::getBwSbbh).collect(Collectors.toList());

        List<String> collect1 = codeList.stream().filter(cl -> !collect.contains(cl)).collect(Collectors.toList());

        return collect1;
    }
}
