/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.reveal.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.reveal.common.RevealConstant;
import com.tofly.reveal.dto.PortalModuleDto;
import com.tofly.reveal.dto.UpdatePortalModuleDto;
import com.tofly.reveal.entity.PortalModule;
import com.tofly.reveal.service.PortalModuleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;

/**
 * 门户模块信息
 *
 * @author xqf
 * @date Fri Mar 11 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/portalmodule")
@Api(tags="门户模块信息接口")
public class PortalModuleController {

    private final  PortalModuleService portalModuleService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param portalModule 门户模块信息
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getPortalModulePage(Page page, @ModelAttribute PortalModule portalModule) {
      return ResultRespone.success(portalModuleService.page(page, Wrappers.query(portalModule)
              .lambda().eq(PortalModule::getIsDel,RevealConstant.SHORT_ZERO)
              .orderByDesc(PortalModule::getUpdateTime)));
    }


    /**
     * 通过id查询门户模块信息
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/info" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(Long id) {
      return ResultRespone.success(portalModuleService.getPortalModuleById(id));
    }

    /**
     * 新增门户模块信息
     * @param dto 门户模块信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增门户模块信息" )
    @ApiOperation(value = "新增门户模块信息")
    @PostMapping
    public ResultRespone save(@Valid PortalModuleDto dto, List<MultipartFile> fileList) {
        if(portalModuleService.savePortalModule(dto,fileList)){
            return ResultRespone.success(true, RevealConstant.SUCCESS);
        }else {
            return ResultRespone.failed(false, RevealConstant.FAILED);
        }
    }

    /**
     * 修改门户模块信息
     * @param dto 门户模块信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改门户模块信息" )
    @ApiOperation(value = "修改门户模块信息")
    @PutMapping
    public ResultRespone updateById(@Valid UpdatePortalModuleDto dto, List<MultipartFile> fileList) {
        Object obj = portalModuleService.updatePortalModule(dto,fileList);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultRespone.success(true, RevealConstant.SUCCESS);
            }else {
                return ResultRespone.failed(false, RevealConstant.FAILED);
            }
        }else {
            return ResultRespone.failed(false, String.valueOf(obj));
        }
    }

    @ToFlyAppLog(title="发布信息" )
    @ApiOperation(value = "发布信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "isRelease", value = "发布，true发布false；取消发布", required = true, dataType = "Boolean")
    })
    @GetMapping("/release")
    public ResultRespone releaseInfo(Long id,Boolean isRelease){
        if (id == null||isRelease==null) {
            return ResultRespone.failed(RevealConstant.ISNOTNUL);
        }
        Object obj = portalModuleService.revealPortalModule(id,isRelease);
        if(obj instanceof String){
            return ResultRespone.failed(String.valueOf(obj));
        }
        return ResultRespone.success(true,String.valueOf(obj));
    }



    /**
     * 通过id删除门户模块信息
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除门户模块信息" )
    @ApiOperation(value = "通过ID删除门户模块信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
        if (id == null) {
            return ResultRespone.failed(RevealConstant.ISNOTNUL);
        }
        Object obj = portalModuleService.removeUpdateModuleService(id);
        if(obj instanceof String){
            return ResultRespone.failed(String.valueOf(obj));
        }
        return ResultRespone.success(true,String.valueOf(obj));
    }

    /**
     * 通过id批量删除门户模块信息
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除门户模块信息" )
    @ApiOperation(value = "通过ID批量删除门户模块信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        if(StringUtils.isEmpty(ids)){
            return ResultRespone.failed(RevealConstant.ISNOTNUL);
        }
        Object obj = portalModuleService.updateRemoveByIds(Arrays.asList(ids.split(",")));
        if(obj instanceof String){
            return ResultRespone.failed(String.valueOf(obj));
        }
        return ResultRespone.success(obj);
    }
}
