/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.reveal.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.reveal.common.RevealConstant;
import com.tofly.reveal.dto.PortalModuleDto;
import com.tofly.reveal.dto.UpdatePortalModuleDto;
import com.tofly.reveal.entity.PortalModule;
import com.tofly.reveal.mapper.PortalModuleMapper;
import com.tofly.reveal.service.FileService;
import com.tofly.reveal.service.PortalModuleService;
import com.tofly.reveal.utils.UploadFiles;
import com.tofly.reveal.vo.PortalModuleVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 门户模块信息
 *
 * @author xqf
 * @date Fri Mar 11 00:00:00 CST 2022
 */
@Slf4j
@Service("portalModuleService")
public class PortalModuleServiceImpl extends ServiceImpl<PortalModuleMapper, PortalModule> implements PortalModuleService {

    @Autowired
    private FileService fileService;

    @Value("${ftp.reveal.door}")
    private String doorfile;

    /**
     * 保存门户模块信息
     * @param dto 发布的内容信息
     * @param fileList 文件
     * @return
     */
    @Override
    public boolean savePortalModule(PortalModuleDto dto, List<MultipartFile> fileList) {
        PortalModule portalModule = new PortalModule();
        BeanUtils.copyProperties(dto,portalModule);
        if(CollUtil.isNotEmpty(fileList)) {
            StringBuilder sb = fileService.saveFileList(fileList,doorfile, RevealConstant.TF_YWPN_PORTAL_MODULE_W);
            if(sb==null){
                return false;
            }
            portalModule.setAnnex(sb.toString());
        }
        portalModule.setIsDel(RevealConstant.SHORT_ZERO);
        if(RevealConstant.SHORT_ONE==portalModule.getIsRelease()){
            portalModule.setReleaseTime(new Date());
        }
        return this.save(portalModule);
    }


    /**
     * 修改门户模块信息
     * @param dto 模块信息
     * @param fileList 文件集合
     * @return
     */
    @Override
    public Object updatePortalModule(UpdatePortalModuleDto dto, List<MultipartFile> fileList) {
        PortalModule pm = this.getOne(new QueryWrapper<PortalModule>().lambda()
                .eq(PortalModule::getIsDel,RevealConstant.SHORT_ZERO).eq(PortalModule::getId,dto.getId()));
        if(pm==null){
            return RevealConstant.DATA_NOTHINGNESS;
        }
        PortalModule portalModule = new PortalModule();
        BeanUtils.copyProperties(dto,portalModule);
        String filepath = null;
        if(CollUtil.isNotEmpty(fileList)) {
            filepath = UploadFiles.saveFiles(fileService, fileList, pm.getAnnex(), dto.getAnnex(), doorfile, RevealConstant.TF_YWPN_PORTAL_MODULE_W);
            if(StringUtils.isEmpty(filepath)){
                this.update(new UpdateWrapper<PortalModule>().lambda()
                        .set(PortalModule::getAnnex,filepath).eq(PortalModule::getId,portalModule.getId()));
            }
            portalModule.setAnnex(filepath);
        }
        if(RevealConstant.SHORT_ONE==portalModule.getIsRelease()){
            portalModule.setReleaseTime(new Date());
        }
        return this.updateById(portalModule);
    }

    /**
     * 根据id删除模块信息
     * @param id 主键id
     * @return
     */
    @Override
    public Object removeUpdateModuleService(Long id) {
        int count = this.count(new QueryWrapper<PortalModule>().lambda()
                .eq(PortalModule::getId,id).eq(PortalModule::getIsDel,RevealConstant.SHORT_ZERO));
        if(count==0){
            return RevealConstant.DATA_NOTHINGNESS;
        }
        return update(new UpdateWrapper<PortalModule>().lambda().set(PortalModule::getIsDel,RevealConstant.SHORT_ONE)
                .set(PortalModule::getUpdateTime,new Date()).set(PortalModule::getUpdateUser, SecurityUtils.getUserId())
                .eq(PortalModule::getIsDel,RevealConstant.SHORT_ZERO)
                .eq(PortalModule::getId,id));
    }

    /**
     * 批量删除模块信息
     * @param asList id集合
     * @return
     */
    @Override
    public Object updateRemoveByIds(List<String> asList) {
        try {
            List<Long> ids= asList.stream().map(st->Long.parseLong(st)).collect(Collectors.toList());
            return this.baseMapper.update(new PortalModule(),new UpdateWrapper<PortalModule>().lambda()
                    .set(PortalModule::getIsDel,RevealConstant.SHORT_ONE)
                    .eq(PortalModule::getIsDel,RevealConstant.SHORT_ZERO)
                    .in(PortalModule::getId,ids));
        }catch (Exception e){
            log.error("异常：",e);
            return e.getMessage();
        }
    }

    /**
     * 模块信息是否发布
     * @param id 模块id
     * @param isRelease 是否发布 true发布 false不发布
     * @return
     */
    @Override
    public Object revealPortalModule(Long id, Boolean isRelease) {
        int count = this.count(new QueryWrapper<PortalModule>().lambda()
                .eq(PortalModule::getId,id).eq(PortalModule::getIsDel,RevealConstant.SHORT_ZERO));
        if(count==0){
            return RevealConstant.DATA_NOTHINGNESS;
        }
        return update(new UpdateWrapper<PortalModule>().lambda()
                .set(isRelease,PortalModule::getIsRelease,RevealConstant.SHORT_ONE)
                .set(!isRelease,PortalModule::getIsRelease,RevealConstant.SHORT_ZERO)
                .set(isRelease,PortalModule::getReleaseTime,new Date())
                .set(PortalModule::getUpdateTime,new Date()).set(PortalModule::getUpdateUser, SecurityUtils.getUserId())
                .eq(PortalModule::getId,id));

    }

    /**
     * 根据id查询模块详细信息
     * @param id 模块id
     * @return
     */
    @Override
    public PortalModuleVo getPortalModuleById(Long id) {
        return this.baseMapper.getPortalModuleById(id);
    }

}
