package com.tofly.trilateral.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.pojo.UserInfoVo;
import com.tofly.pojo.UserNumberVo;
import com.tofly.trilateral.common.BaseController;
import com.tofly.trilateral.common.FlowConstant;
import com.tofly.trilateral.config.ForeignConfig;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 通讯录
 *
 * @author qf
 * @date Thu Apr 21 00:00:00 CST 2022
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/marketing")
@Api(tags="营销第三方接口")
public class MarketingController extends BaseController {

    private final ForeignConfig foreignConfig;


    @PostMapping("/userList")
    @ApiOperation(value = "获取用户信息")
    public ResultRespone getUserList(String lkeyword, Page page){
        Map<String,Object> valueMap = new HashMap<>();
        valueMap.put("lkeyword",lkeyword);
        valueMap.put("page",page.getCurrent());
        valueMap.put("rows",page.getSize());

        Map<String , Object> map = new HashMap<>();
        map.put("token",foreignConfig.getMarketingToken());
        map.put("data",valueMap);

        String result= HttpUtil.post(foreignConfig.getMarketingUrl()+"/service/YhInfo/GetUserList", JSONObject.toJSONString(map));
        if(StringUtils.isEmpty(result)){
            return ResultRespone.failed("获取用户信息异常");
        }
        JSONObject jsonObject = JSONObject.parseObject(result);
        if(jsonObject.containsKey("Code")){
            if(jsonObject.getInteger("Code").equals(FlowConstant.MARKETING_CODE) &&jsonObject.containsKey("Result")){
                JSONObject dataJson = jsonObject.getJSONObject("Result");
                page.setTotal(dataJson.getLong("total"));
                List<UserInfoVo> users = dataJson.getJSONArray("rows").toJavaList(UserInfoVo.class);
                page.setRecords(users);
            }
        }
        return ResultRespone.success(page);
    }



    @PostMapping("/user")
    @ApiOperation(value = "获取用户信息")
    public ResultRespone<UserInfoVo> getUser(@RequestParam("lkeyword")String lkeyword){
        Map<String,Object> valueMap = new HashMap<>();
        valueMap.put("lkeyword",lkeyword);
        valueMap.put("page",0);
        valueMap.put("rows",5);

        Map<String , Object> map = new HashMap<>();
        map.put("token",foreignConfig.getMarketingToken());
        map.put("data",valueMap);

        String result= HttpUtil.post(foreignConfig.getMarketingUrl()+"/service/YhInfo/GetUserList", JSONObject.toJSONString(map));
        if(StringUtils.isEmpty(result)){
            return ResultRespone.failed("获取用户信息异常");
        }
        JSONObject jsonObject = JSONObject.parseObject(result);
        UserInfoVo userInfoVo = null;
        if(jsonObject.containsKey("Code")){
            if(jsonObject.getInteger("Code").equals(FlowConstant.MARKETING_CODE) &&jsonObject.containsKey("Result")){
                JSONObject dataJson = jsonObject.getJSONObject("Result");

                List<UserInfoVo> users = dataJson.getJSONArray("rows").toJavaList(UserInfoVo.class);
                if(CollUtil.isNotEmpty(users)){
                    userInfoVo = users.get(0);
                }

            }
        }
        return ResultRespone.success(userInfoVo);
    }




    @PostMapping("/getUserNumber")
    @ApiOperation(value = "获取用水量信息")
    public ResultRespone getUserNumber(String yhbh, Page page){
        Map<String,Object> valueMap = new HashMap<>();
        valueMap.put("yhbh",yhbh);
        valueMap.put("page",page.getCurrent());
        valueMap.put("rows",page.getSize());

        Map<String , Object> map = new HashMap<>();
        map.put("token",foreignConfig.getMarketingToken());
        map.put("data",valueMap);

        String result= HttpUtil.post(foreignConfig.getMarketingUrl()+"/service/cb/GetUserNumber", JSONObject.toJSONString(map));
        if(StringUtils.isEmpty(result)){
            return ResultRespone.failed("获取用户信息异常");
        }
        JSONObject jsonObject = JSONObject.parseObject(result);
        if(jsonObject.containsKey("Code")){
            if(jsonObject.getInteger("Code").equals(FlowConstant.MARKETING_CODE) &&jsonObject.containsKey("Result")){
                JSONObject dataJson = jsonObject.getJSONObject("Result");
                page.setTotal(dataJson.getLong("total"));
                List<UserNumberVo> users = dataJson.getJSONArray("rows").toJavaList(UserNumberVo.class);
                page.setRecords(users);
            }
        }

        return ResultRespone.success(page);
    }


}
